/**
 * Copyright 2000 Systemcorp - All rights reserved. Made in Canada.
 *

 * Description:
 *
 * Author: Tudor Toma
 * Revision: 42
 * Date: 7/31/00 6:18p
 *
 **/

/*********************************************************************************************************************************************
// search function

  if fname OR lname then ignore the search criteria: cost, org_id, geo_id, pool_id, work_loc,
																				and_roles, and_skills, shw_roles, shw_skills, max_rows

  prj_id =      PROJECT_ID, required

  eunit  =  0 = MINUTE (default), 1= h, 2 = d, 3 = w,           applies to effort
  unit   =  0 = h/d, 3= h/w, 4=h/m, 1= d/w, 2= d/m, 5= w/m,     applies to work
  if unit = 100 then work = percent units
  if effort =   0 than both start  finish date should be notnull

  consider_planned_sched (bits 30 - 31) (host variable --> h_cri)
	= 0 WBS  consider all assignments (consider CRI assignments)
	= 1 CRI  consider all assignments (consider other CRI assignments, WBS assignments are considered by default)
	= 2 WBS  consider consider only WBS assignments (don't consider CRI assignments)
	= 3 CRI  consider consider only current CRI assignments (WBS assignments are considered by default)

  consider_planned_sched values: 1, 2, 3, 4, 5, 6, 7,
						search from project assignments 10  pool_id = resource_id
						search for new profile          12  and_roles = role_id
						duplicate profile               12  pool_id   = profile_id
														inner current project   outer current project   all projects
  scheduled                                                         1                   2                   3
  scheduled  planned                                               4                   5                   6
  NONE(consider all available time)                                 7
  scheduled in projects with priority >= then me                                        13                  14
  scheduled  planned in projects with priority >= then me                              15                  16
  scheduled outer current project    planned all projects          20
  scheduled outer current project    planned inner current project 21
  scheduled all projects    planned outer current project          22
  planned inner current project                                     24

  contour structure: ---------------------------------------------------------------------------------------------------------------------------------------------------------------------
   bitflag  = 0 - ASP
			= 1 - Daily         -- contour3
			= 2 - Weekly        -- contour3{, contur1}, {amount falg}
			= 3 - Monthly(C)    -- contour3{, contur2}
			= 4 - Monthly(B)    -- contour3{, contur2}
			= 5 - Monthly(A)    -- contour3{, contur2 | contour2Countour1}
			= 6 - Yearly(C)     -- contour2, contur4
			= 7 - Yearly(B)     -- contour2, contur4, contur1(only one day)
			= 8 - Yearly(A)     -- contour2, contur4, contur1(many days)
			= 9 - Participant
			=10 - Assign from Project Assignment

  contur1:  DAY:    bits 0-6    0=Su, 1=Mo, 2=Tu, 3=We, 4=Th, 5=Fr, 6=Sa,
  contur2:  WEEK:   bits 7-11   A) 7=week1, 8=week2, 8=week3, 9=week4
								B) 1-First/2-Second/3-Third/4-Fourth/5-Last
								C) The Day 1 to 31
  contour3:     EVRY:   bits 12-16  EVRY 1 to 31 day / week / month
  contour4:     MONTH:  bits 17-28  17 = Jan, 18 = Feb, ... 28 = Dec

  2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1
  8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
												-------------
												S F T W T M S
									  ---------
									  w1 to w4  A(7 days from the 1st)
									  5 4 3 2 1 B(Last, Fourth, Third, Second, First)
									  31 .... 1 C(The Day 6)
							----------
							31 ........ 1 (Day)
							31 ........ 1 (week)
							12 ........ 1 (month)
  -------------------------
  D N O S A J J M A M F J (Month)

  Length (Duration)
  1. Start Date  Finish Date
  2. Start Date  Total Effort
  3. Total Effort  Finish Date

  if effort = 0 then both start  finish dates are required
  if occurrence + contour = 0 then ASAP
  if occurrence > 0 then contur is required

  return the daly contour in the resultset (5*480,2*0,3*400,...)
************************************************************************************************************************************************/

Prompt == FUNCTION SP_SEARCH_2 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_2
(
     prj_id VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , fname VARCHAR2
   , cost001 VARCHAR2
   , org_id001 VARCHAR2
   , geo_id001 VARCHAR2
   , pool_id001 VARCHAR2
   , work_location001 PLS_INTEGER
   , and_roles VARCHAR2
   , and_skills VARCHAR2
   , shw_roles VARCHAR2
   , shw_skills VARCHAR2
   , max_rows PLS_INTEGER
   , p_update PLS_INTEGER
   , work_type PLS_INTEGER
   , p_type PLS_INTEGER
   , last_id VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_temp_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_rs_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_fname VARCHAR2(255);
   h_cost VARCHAR2(10);
   h_and_roles VARCHAR2(8000);
   h_and_skills VARCHAR2(8000);
   h_shw_roles VARCHAR2(8000);
   h_shw_skills VARCHAR2(8000);
   h_and_role VARCHAR2(8000);
   h_and_skill VARCHAR2(8000);
   h_shw_role VARCHAR2(8000);
   h_shw_skill VARCHAR2(8000);
   h_source_ids VARCHAR2(8000);
   h_target_ids VARCHAR2(8000);
   h_res_id VARCHAR2(32);
   h_last_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_dayofyear DATE;
   h_start_date VARCHAR2(26);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_task_start_DT VARCHAR2(26);
   h_task_finish_DT VARCHAR2(26);
   h_work_loc VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_assign_id VARCHAR2(32);
   h_role_id VARCHAR2(32);
   h_skill_id VARCHAR2(32);
   h_full_name VARCHAR2(255);
   h_rec_status VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_exists PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_work_location PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_max_rows PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_res_cost NUMBER := 0.00;
   h_rate_echange NUMBER := 0.00;
   h_total PLS_INTEGER := 0;
   h_pool_type PLS_INTEGER := 0;
   h_currency_id PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_j_efective_date PLS_INTEGER := 0;
   h_j_disconti_date PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_active VARCHAR2(1);
   h_task_type PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
   stwhere VARCHAR2(2999);
   h_temp_stmt VARCHAR2(7999);
   h_table_type PLS_INTEGER := 0;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id CHAR(32);
   h_temp2_security_id CHAR(32);
   h_cri_security_id VARCHAR2(32);
   h_temp_var PLS_INTEGER := 0;
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_continue_01 PLS_INTEGER := 0;
   i PLS_INTEGER NOT NULL := 1;
   ii PLS_INTEGER NOT NULL := 1;
   i_row PLS_INTEGER NOT NULL := 0;
   i_return PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   token VARCHAR2(32);
   i_noskill PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
   len PLS_INTEGER;
   k PLS_INTEGER;
   i_id PLS_INTEGER;
-- Declare Cursors
      CURSOR CSN_PRF IS
         SELECT PROFILE_ID FROM PROFILES
         WHERE PROJECT_ID = h_project_id AND UPPER(PROFILE_NAME) LIKE UPPER(h_fname) || '%';

      CURSOR CSN_RES IS
         SELECT RESOURCE_ID FROM RESOURCES
         WHERE UPPER(FULL_NAME) LIKE UPPER(h_fname) || '%';

      -- Dynamic Cursor Declaration
      C12 PMO_DB2M.REF_CURSOR;

      CURSOR C42 IS
         SELECT PROJECT_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND RESOURCE_ID = h_res_id;

      CURSOR CU42 IS
         SELECT RS_ID, RES_ID, RTRIM(ROLE_ID), COALESCE(ROLE_NAME, ''), RTRIM(SKILL_ID),
         COALESCE(SKILL_NAME, ''), COALESCE(SHOW_ROLE, ''), COALESCE(SHOW_SKILL, '')
         FROM PMO_TMP_SEARCH ORDER BY RES_ID;

      -- Resultset Cursor Declaration
      C775C0 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C775D0 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C775E1 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C775E0 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SEARCH_2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SEARCH_2'), '-- ' || SYSTIMESTAMP, ':r := SP_SEARCH_2(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || '''' || fname || ''''
          || ', ' || '''' || cost001 || ''''
          || ', ' || '''' || org_id001 || ''''
          || ', ' || '''' || geo_id001 || ''''
          || ', ' || '''' || pool_id001 || ''''
          || ', ' || COALESCE(TO_CHAR(work_location001), 'NULL')
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || '''' || shw_roles || ''''
          || ', ' || '''' || shw_skills || ''''
          || ', ' || COALESCE(TO_CHAR(max_rows), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_update), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || '''' || last_id || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 192+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SEARCH_2(193) : cplan.sql(287) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   --to contain work location passed to function when null then 265
   
   
   -- Parse parameters to local variables
   h_project_id := prj_id;
   h_start_date := start_date001;
   h_finish_DT := finish_date001;
   h_fname := fname;
   h_cost := cost001;
   h_and_roles := and_roles;
   h_and_skills := and_skills;
   h_shw_roles := shw_roles;
   h_shw_skills := shw_skills;
   h_rec_user := rec_user001;
   
   h_effort := effort001;
   -- total effort for period
   h_work := work001;
   -- amount for countour
   h_occurrence := occurrence;
   -- occurrence
   h_max_rows := max_rows;
   h_eunit := eunit;
   h_consider := consider_planned_sched;
   h_unit := unit;
   h_type := p_type;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_update := p_update;
   h_work_type := work_type;
   
   h_task_type := 0;
   h_sched_work := 0;
   --local variables
   /*int iii = 0;*/
   --iSF == 0 when finish_date is null
   
   If (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND ((prj_id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 235+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(236) : cplan.sql(330) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 238+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(239) : cplan.sql(333) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((fname IS NULL)) AND ((org_id001 IS NULL)) AND ((geo_id001 IS NULL)) AND ((pool_id001 IS NULL)) 
         AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
      If ((and_roles IS NULL)) AND (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 247+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(248) : cplan.sql(342) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 250+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(251) : cplan.sql(345) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      ElsIf ((and_roles IS NULL)) AND ((and_skills IS NULL)) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 255+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(256) : cplan.sql(350) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 258+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(259) : cplan.sql(353) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
   End If;
   
   If ((finish_date001 IS NOT NULL) AND (Coalesce(Length(h_finish_DT), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 266+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(267) : cplan.sql(361) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 269+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(270) : cplan.sql(364) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL) AND (Coalesce(Length(h_start_date), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 276+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(277) : cplan.sql(371) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 279+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(280) : cplan.sql(374) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   -- Fix: searching for 5 weeks per month
   If Coalesce(h_bitflag, 0) = 5 AND Coalesce(h_unit, 0) = 5 AND Coalesce(h_work, 0) = 5 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 287+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(288) : cplan.sql(382) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 290+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(291) : cplan.sql(385) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --check if work_location is null then 265 otherwise put value passed and use to update resource_assignmnets table
   
   If ((work_location001 IS NULL)) Then
      h_work_location := 265;
   Else
      h_work_location := work_location001;
   End If;
   If (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND ((start_date001 IS NOT NULL)) 
             AND ((finish_date001 IS NOT NULL)) Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 314+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(315) : cplan.sql(409) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_total, 0) < 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 321+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(322) : cplan.sql(416) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 324+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(325) : cplan.sql(419) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, True);
      End If;
      iSF := 1;
   End If;
   
   If ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         SELECT SYSTIMESTAMP INTO h_start_date FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 340+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(341) : cplan.sql(435) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_start_DT := h_start_date;
   --trace_printf("1.a__* h_start_DT = %s\n",h_start_DT);//trace_flush();
   
   h_pool_type := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_type, 0), PMO_CNST.SC_C0000000)), 30);
   -- consider CRI  WBS
   h_type := BITAND(Coalesce(h_type, 0), PMO_CNST.SC_1FFFFFFF);
   
   h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_C0000000)), 30);
   -- consider CRI  WBS
   h_consider := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
   --trace_printf("1__ h_cri = %d, h_pool_type = %d, h_type: %d, h_consider = %d\n",h_cri,h_pool_type,h_type,h_consider);//trace_flush();
   
   If Coalesce(h_type, 0) > 0 Then
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 370+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(371) : cplan.sql(465) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 386+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(387) : cplan.sql(481) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 393+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(394) : cplan.sql(488) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 396+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(397) : cplan.sql(491) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
      
      h_table_type := GetTable(Coalesce(h_type, 0));
      If Coalesce(h_table_type, 0) = 0 Then
         h_type := GetAssnType(Coalesce(h_type, 0));
         If Coalesce(h_type, 0) = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 407+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(408) : cplan.sql(502) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 410+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(411) : cplan.sql(505) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         h_table_type := PMO_CNST.SC_TABLE_WBS;
      End If;
      --trace_printf("22__ h_table_type = %d, h_type = %d\n",h_table_type,h_type);//trace_flush();
      h_task_id := h_project_id;
      
      If Coalesce(h_pool_type, 0) = 0 Then
         If (fname IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 422+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(423) : cplan.sql(517) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 425+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(426) : cplan.sql(520) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
         End If;
         
         BEGIN
            SELECT REPLACE(h_fname,'''','''''') INTO h_fname FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 438+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(439) : cplan.sql(533) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            OPEN CSN_PRF;
            
            FETCH CSN_PRF INTO h_res_id;
            IF CSN_PRF%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               BEGIN
                  CLOSE CSN_PRF ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 462+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(463) : cplan.sql(557) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 468+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(469) : cplan.sql(563) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 471+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(472) : cplan.sql(566) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
            End If;
            
            FETCH CSN_PRF INTO h_uniqueID;
            IF CSN_PRF%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_type := 0;
               
               BEGIN
                  SELECT PROJECT_ID INTO h_project_id FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 494+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(495) : cplan.sql(589) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               CLOSE CSN_PRF ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 508+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(509) : cplan.sql(603) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            OPEN CSN_RES;
            
            FETCH CSN_RES INTO h_res_id;
            IF CSN_RES%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               BEGIN
                  CLOSE CSN_RES ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 531+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(532) : cplan.sql(626) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 536+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(537) : cplan.sql(631) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 539+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(540) : cplan.sql(634) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
            End If;
            
            FETCH CSN_RES INTO h_uniqueID;
            IF CSN_RES%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_type := 0;
               
               BEGIN
                  SELECT PROJECT_ID INTO h_project_id FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 562+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(563) : cplan.sql(657) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               CLOSE CSN_RES ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 576+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(577) : cplan.sql(671) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         h_res_id := pool_id001;
      End If;
      
      --trace_printf("h_type: %d, h_task_id: %d:%s, h_project_id: %d:%s\n", h_type,h_task_id_Ind,h_task_id,h_project_id_Ind,h_project_id);//trace_flush();
      If Coalesce(h_type, 0) > 0 Then
         If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
            h_uniqueID := SYS_GUID();
            
            PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type, h_task_id, h_res_id, h_rec_user
                                    , h_uniqueID);
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 594+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(595) : cplan.sql(689) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 597+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(598) : cplan.sql(692) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, h_uniqueID, True);
         End If;
         
         /*if ( h_cri == 0 )
         			{
         				if(isCRIType(h_type))
         					h_cri = 1;
         				else
         					h_cri = 0;
         			}*/
         --trace_printf("2__ h_table_type = %d, h_type = %d, h_start_DT = %s\n",h_table_type,h_type,h_start_DT);//trace_flush();
         If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
            If isProjectType(Coalesce(h_type, 0)) <> 0 Then
               BEGIN
                  SELECT PROJECT_ID, /*CASE WHEN START_FINISH_DATE > TO_DATE(SubStr(HOST(h_start_DT), 1, 10)) THEN START_DT ELSE TO_TIMESTAMP(HOST(h_start_DT)) END,*/
                  COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
                  INTO h_project_id, /*HOST(h_start_DT), */h_per_day, h_per_week, h_priority
                  FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 623+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(624) : cplan.sql(718) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT B.TASK_TYPE, B.UNRESOLVED_START, B.UNRESOLVED_FINISH, B.PROJECT_ID,
                  COALESCE(A.MINUTE_PER_DAY, 480), COALESCE(A.MINUTE_PER_WEEK, 2400), A.PRIORITY, B.PLANNED_EFFORT + B.SCHED_EFFORT
                  INTO h_task_type, h_task_start_DT, h_task_finish_DT, h_project_id,
                  h_per_day, h_per_week, h_priority, h_sched_work
                  FROM TMT_WBS A, TMT_WBS B WHERE B.ELEMENT_ID = h_task_id AND A.ELEMENT_ID = B.PROJECT_ID;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 640+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(641) : cplan.sql(735) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 648+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(649) : cplan.sql(743) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 651+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(652) : cplan.sql(746) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
            End If;
         Else
            BEGIN
               SELECT PROJECT_ID, TYPE_ID, WBS_ID INTO h_project_id, h_type, h_temp_id
               FROM TMT_CRI WHERE ELEMENT_ID = h_task_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 665+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(666) : cplan.sql(760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 672+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(673) : cplan.sql(767) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 675+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(676) : cplan.sql(770) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
            End If;
            
            BEGIN
               SELECT CASE WHEN START_DT > h_start_DT THEN START_DT ELSE TO_TIMESTAMP(h_start_DT) END,
               COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
               INTO h_start_DT, h_per_day, h_per_week, h_priority
               FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 691+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(692) : cplan.sql(786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 698+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(699) : cplan.sql(793) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 701+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(702) : cplan.sql(796) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_project_id, True);
            ElsIf iSF = 1 Then
               BEGIN
                  SELECT PMO_ORASTD.DAYS(h_start_DT) - PMO_ORASTD.DAYS(h_finish_DT)
                  INTO h_total
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 715+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(716) : cplan.sql(810) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_total, 0) > 0) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 722+1) THEN 
                     LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(723) : cplan.sql(817) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_BEFORE_PROJECT_E), 'NULL'));
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 725+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(726) : cplan.sql(820) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.CANNOT_ASSIGN_BEFORE_PROJECT_E, True);
               End If;
            End If;
            --trace_printf("h_temp_id: %d %s", h_temp_id_Ind,h_temp_id);
            
            If (h_temp_id IS NOT NULL) Then
               h_table_type := PMO_CNST.SC_TABLE_WBS;
               h_type := FC_MIRROR_TYPE(h_type);
            End If;
         End If;
         --trace_printf("a1_* h_task_type: %d, h_table_type: %d, h_type = %d, h_start_DT = %s\n", h_task_type,h_table_type,h_type,h_start_DT);//trace_flush();
         
         If isNotProjectType(Coalesce(h_type, 0)) <> 0 Then
            BEGIN
               SELECT RES_ASSIGNMENT_ID,PARTICIPANT INTO h_uniqueID, h_participant
               FROM RESOURCE_ASSIGNMENTS
               WHERE ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 750+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(751) : cplan.sql(845) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               If (BITAND(Coalesce(h_participant, 0), 4112)) > 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 758+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(759) : cplan.sql(853) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_ALREADY_ASSIGNED), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 761+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(762) : cplan.sql(856) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_ALREADY_ASSIGNED, h_res_id, True);
               Else
                  BEGIN
                     SELECT TSK_ASSIGNMENT_ID INTO h_uniqueID
                     FROM TASK_ASSIGNMENTS
                     WHERE RES_ASSIGNMENT_ID = h_uniqueID AND TASK_ID = h_task_id AND RESOURCE_ID = h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 775+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(776) : cplan.sql(870) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 782+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(783) : cplan.sql(877) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_ALREADY_ASSIGNED), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 785+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(786) : cplan.sql(880) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_ALREADY_ASSIGNED, h_res_id, True);
                  End If;
               End If;
            End If;
            
            BEGIN
               SELECT RES_ASSIGNMENT_ID INTO h_uniqueID FROM RESOURCE_ASSIGNMENTS
               WHERE ELEMENT_ID = h_task_id AND PROFILE_ID = h_res_id AND RESOURCE_TYPE = '1' AND PARTICIPANT = 16;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 801+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(802) : cplan.sql(896) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_res_id := '';
            End If;
         End If;
         
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            h_res_type := '1';
            
            BEGIN
               SELECT ROLE_ID, PROFILE_NAME, REC_STATUS, PROF_NUMBER
               INTO h_role_id, h_full_name, h_rec_status, h_number
               FROM PROFILES WHERE PROFILE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 824+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(825) : cplan.sql(919) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_and_roles := h_role_id;
            ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               h_role_id := SubStr(h_and_roles, 1, 32);
               --h_role_id[32] = '\0';
               
               BEGIN
                  SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number
                  FROM PROFILES WHERE PROJECT_ID=h_project_id AND ROLE_ID=h_role_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 844+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(845) : cplan.sql(939) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')') INTO h_full_name
                  FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP WHERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 858+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(859) : cplan.sql(953) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 865+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(866) : cplan.sql(960) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_role_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 868+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(869) : cplan.sql(963) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_role_id, True);
               End If;
               
               h_res_id := SYS_GUID();
               
               BEGIN
                  INSERT INTO PROFILES(PROFILE_ID,PROJECT_ID,ROLE_ID,PROF_NUMBER, PROFILE_NAME)
                  VALUES(h_res_id, h_project_id, h_role_id, h_number, h_full_name);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 888+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(889) : cplan.sql(983) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 895+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(896) : cplan.sql(990) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 898+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(899) : cplan.sql(993) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
               End If;
            End If;
         Else
            BEGIN
               SELECT NICK_NAME, PMO_ORASTD.JULIAN_DAY(h_start_DT), ACTIVE, COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0),COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
               INTO h_full_name, h_julian_day, h_active, h_j_efective_date, h_j_disconti_date
               FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 914+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(915) : cplan.sql(1009) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 921+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(922) : cplan.sql(1016) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 924+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(925) : cplan.sql(1019) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
            ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               If (SubStr(h_active, 1, 1) = '0') OR ((Coalesce(h_j_disconti_date, 0) > 0) AND (Coalesce(
                     h_julian_day, 0) > Coalesce(h_j_disconti_date, 0))) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 931+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(932) : cplan.sql(1026) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_ACTIVE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 934+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(935) : cplan.sql(1029) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_ACTIVE, h_res_id, True);
               Else
                  h_res_type := '0';
               End If;
            End If;
         End If;
      End If;
   End If;
   
   --trace_printf("2__ h_project_id: %s, h_type: %d, h_consider = %d, h_res_type = %s\n",h_project_id,h_type,h_consider,h_res_type);//trace_flush();
   --trace_printf("1.b__* h_start_DT = %s\n",h_start_DT);//trace_flush();
   If Coalesce(h_type, 0) = 0 Then
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
         -- Clean up the profiles marked as deleted
         BEGIN
            DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID IN (
            SELECT PROFILE_ID FROM PROFILES WHERE PROJECT_ID = h_project_id AND REC_STATUS='D');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 963+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(964) : cplan.sql(1058) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_pool_type, 0) = 1 Then
            h_res_id := pool_id001;
            
            BEGIN
               SELECT ROLE_ID, PROFILE_ID INTO h_role_id, h_res_id FROM PROFILES WHERE PROFILE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 979+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(980) : cplan.sql(1074) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               h_role_id := SubStr(h_and_roles, 1, 32);
               --h_role_id[32] = '\0';
               
               BEGIN
                  SELECT PROFILE_ID INTO h_res_id FROM PROFILES WHERE PROJECT_ID=h_project_id AND ROLE_ID=h_role_id AND
                  PROF_NUMBER = (SELECT MAX(PROF_NUMBER) FROM PROFILES WHERE PROJECT_ID = h_project_id AND ROLE_ID = h_role_id);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 997+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(998) : cplan.sql(1092) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1004+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1005) : cplan.sql(1099) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_role_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1007+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1008) : cplan.sql(1102) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_role_id, True);
               End If;
            End If;
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID IN (SELECT PROFILE_ID FROM PROFILES WHERE PROJECT_ID = h_project_id AND REC_STATUS='D' AND PROFILE_ID <> h_res_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1025+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1026) : cplan.sql(1120) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID IN (SELECT PROFILE_ID FROM PROFILES WHERE PROJECT_ID = h_project_id AND REC_STATUS='D' AND PROFILE_ID <> h_res_id);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1042+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1043) : cplan.sql(1137) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM PROFILES WHERE PROJECT_ID = h_project_id AND REC_STATUS='D' AND PROFILE_ID <> h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1059+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1060) : cplan.sql(1154) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID IN (SELECT PROFILE_ID FROM PROFILES WHERE PROJECT_ID = h_project_id AND REC_STATUS='D');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1076+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1077) : cplan.sql(1171) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID IN (SELECT PROFILE_ID FROM PROFILES WHERE PROJECT_ID = h_project_id AND REC_STATUS='D');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1093+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1094) : cplan.sql(1188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM PROFILES WHERE PROJECT_ID = h_project_id AND REC_STATUS='D';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1110+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1111) : cplan.sql(1205) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         h_res_type := '1';
      Else
         h_res_type := '0';
      End If;
      
      If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE) AND ((and_roles IS NULL)) AND (Coalesce(
             h_pool_type, 0) = 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1123+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1124) : cplan.sql(1218) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_COMPETENCY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1126+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1127) : cplan.sql(1221) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_COMPETENCY, h_project_id
               , True);
      End If;
      
      If Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT Then
         BEGIN
            SELECT PRIORITY, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400)
            INTO h_priority, h_per_day, h_per_week
            FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1143+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1144) : cplan.sql(1238) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT 480, 2400 INTO h_per_day, h_per_week FROM TMT_POOLS WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1156+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1157) : cplan.sql(1251) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1163+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1164) : cplan.sql(1258) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1166+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1167) : cplan.sql(1261) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_project_id, True);
            End If;
         End If;
      End If;
   End If;
   
   If (Coalesce(contour001, 0) = 0) AND (Coalesce(h_unit, 0) <> 100) Then
      h_work := 0;
   End If;
   
   /*  if ( (h_unit == 100)  ((h_work == 10000) || (h_work == 0)) )
   	{
   		h_work = h_per_day;
   		h_unit = 0;
   	}*/
   
   If Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
      If (start_date001 IS NULL) Then
         /*BEGIN_SQL
         				SELECT SC_NOW, SC_NOW INTO HOST(h_start_DT), HOST(h_finish_DT) FROM DUAL;
         			END_SQL;*/
         h_start_DT := '1899-12-31 00:00:00';
         h_finish_DT := '1899-12-31 00:00:00';
      Else
         h_start_DT := start_date001;
         h_finish_DT := start_date001;
      End If;
      
      h_effort := 0;
   Else
      /*if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_type > 0)  (h_sched_work > 0) )
      		{
      			InitializeHost(SC_PARAM(h_start_DT), SC_PARAM(h_task_start_DT));
      			InitializeHost(SC_PARAM(h_finish_DT), SC_PARAM(h_task_finish_DT));
      			if (h_consider != SC_SRCH_PROFILE)
      			{
      				h_consider = 7;
      			}
      			iSF = 1;
      			h_effort   = 0;
      			h_work     = 0;
      		}*/
      -- SEARCH_2
      --trace_printf("h_task_type: %d, iSF = %d, h_unit  =%d, h_work = %d, h_effort = %d\n", h_task_type,iSF,h_unit,h_work,h_effort);//trace_flush();
      --trace_printf("1.c__* h_per_day: %d, h_start_DT = %s\n",h_per_day,h_start_DT);//trace_flush();
      If iSF = 0 Then
         h_start_DT := start_date001;
         h_finish_DT := finish_date001;
         --trace_printf("1.a_* h_start_DT = %s, h_finish_DT: %s\n",h_start_DT,h_finish_DT);//trace_flush();
         
         h_temp_var := FC_CALC_DURATION(h_project_id, h_start_DT, h_finish_DT, h_effort, h_work, h_eunit
                               , h_unit, h_consider, h_bitflag, h_contour, h_occurrence, h_per_day
                               , h_per_week, h_work, h_effort, h_duration);
         
         If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 1224+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(1225) : cplan.sql(1319) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1227+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1228) : cplan.sql(1322) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         ElsIf Coalesce(h_temp_var, 0) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1232+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1233) : cplan.sql(1327) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1235+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1236) : cplan.sql(1330) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
         End If;
         
         h_finish_DT := h_finish_DT;
         If (h_finish_DT IS NULL) Then
            h_finish_DT := '';
         End If;
      ElsIf (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) <> 100) Then
         If (Coalesce(h_unit, 0) = 1) OR (Coalesce(h_unit, 0) = 2) Then
            h_eunit := 2;
         ElsIf Coalesce(h_unit, 0) = 0 Then
            h_eunit := 1;
         ElsIf (Coalesce(h_unit, 0) = 3) OR (Coalesce(h_unit, 0) = 4) Then
            h_eunit := 1;
         ElsIf Coalesce(h_unit, 0) = 5 Then
            h_work := Coalesce(h_per_week, 0) * Coalesce(h_work, 0);
            h_eunit := 3;
         End If;
      End If;
      
      --trace_printf("h_consider: %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n", h_consider,h_start_DT, h_finish_DT,h_duration);//trace_flush();
      If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE) Then
         If FC_CREATE_CDY(h_project_id, h_start_DT, h_finish_DT, h_duration) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1261+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1262) : cplan.sql(1356) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1264+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1265) : cplan.sql(1359) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_project_id
                  , True);
         End If;
         
         If (iSF = 0) AND (Coalesce(h_duration, 0) = 0) Then
            BEGIN
               SELECT COUNT(DAY_YEAR) INTO h_duration FROM PMO_TMP_CDY
               WHERE DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1280+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1281) : cplan.sql(1375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      ElsIf (iSF = 0) AND (Coalesce(h_duration, 0) = 0) AND (Coalesce(Length(h_finish_DT), 0) > 0) Then
         BEGIN
            SELECT COUNT(DAY_YEAR) INTO h_duration FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1295+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1296) : cplan.sql(1390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   --trace_printf("iSF: %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n", iSF,h_start_DT, h_finish_DT,h_duration);//trace_flush();
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   If (Coalesce(h_type, 0) = 0) AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE) 
             AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) Then
      If (last_id IS NOT NULL) Then
         h_last_id := last_id;
      End If;
   End If;
   
   If Coalesce(h_max_rows, 0) = 0 Then
      h_max_rows := PMO_CNST.SC_MAX_ROWS;
   End If;
   
   If ((and_roles IS NULL)) AND ((and_skills IS NULL)) AND ((shw_roles IS NULL)) 
         AND ((shw_skills IS NULL)) Then
      i_noskill := 1;
   End If;
   
   --trace_printf("h_consider: %d, h_type: %d, h_res_id: %s\n", h_consider,h_type,h_res_id);
   stmt := '';
   stwhere := '';
   If Coalesce(h_type, 0) > 0 Then
      If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
         stmt := 'SELECT ''';
         stmt := stmt || h_res_id;
         stmt := stmt || ''' AS ELEMENT_ID FROM TMT_COMPETENCIES r WHERE r.TYPE_ID=73 AND r.ELEMENT_ID = ''';
         stmt := stmt || h_role_id;
         stmt := stmt || '''';
      ElsIf Coalesce(h_consider, 0) = 10 Then
         stmt := 
               'SELECT r.RESOURCE_ID FROM RESOURCE_ASSIGNMENTS ra JOIN TASK_ASSIGNMENTS ta ON ra.ELEMENT_ID=''';
         stmt := stmt || h_project_id;
         stmt := stmt || ''' AND ra.RESOURCE_ID=''';
         stmt := stmt || h_res_id;
         stmt := stmt || ''' AND ra.TSK_ASSIGNMENT_ID=ta.TSK_ASSIGNMENT_ID '
                || 'JOIN RESOURCES r ON ra.RESOURCE_ID=r.RESOURCE_ID';
      Else
         stmt := 
               'SELECT r.RESOURCE_ID FROM RESOURCES r WHERE r.CALENDAR_ID IS NOT NuLL AND r.RESOURCE_ID = ''';
         stmt := stmt || h_res_id;
         stmt := stmt || '''';
      End If;
   Else
      --removed the initailaization of h_total to work_location outsid condiitons
      
      --trace_printf("h_consider: %d, h_res_id: %s\n", h_consider,h_res_id);
      If (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE) AND (Coalesce(
             h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) AND (Coalesce(
             bitflag001, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) Then
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0007';
         PMO_DB2M.SC_STATE := 0;
         
         If (pool_id001 IS NULL) Then
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_007a';
            PMO_DB2M.SC_STATE := 0;
            
            BEGIN
               INSERT INTO PMO_TMP_CPLN_007a
               SELECT ELEMENT_ID, RANK FROM TMT_POOLS WHERE PUBLIC_FLAG = '1'
               UNION ALL
               SELECT P.ELEMENT_ID, P.RANK
               FROM TMT_POOLS P JOIN RESOURCE_ASSIGNMENTS RA ON
               P.ELEMENT_ID = RA.ELEMENT_ID AND
               RA.ELEMENT_TYPE = 11 AND
               RA.RESOURCE_ID = h_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1380+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1381) : cplan.sql(1475) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               INSERT INTO PMO_TMP_CPLN_0007
               SELECT M.RANK
               FROM PMO_TMP_CPLN_007a M
               WHERE NOT EXISTS (
               SELECT B.ELEMENT_ID
               FROM PMO_TMP_CPLN_007a A, PMO_TMP_CPLN_007a B
               WHERE A.RANK < B.RANK
               AND SUBSTR(B.RANK, 1, LENGTH(A.RANK)) = A.RANK
               AND M.ELEMENT_ID = B.ELEMENT_ID);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1405+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1406) : cplan.sql(1500) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_res_id := pool_id001;
            
            BEGIN
               INSERT INTO PMO_TMP_CPLN_0007
               SELECT RANK FROM TMT_POOLS
               WHERE ELEMENT_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1426+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1427) : cplan.sql(1521) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      --trace_printf("*** h_res_id: %s\n", h_res_id);
      If (fname IS NOT NULL) Then
         len := CASE WHEN 
                 Coalesce(Length(fname), 0) > 32767 THEN BITAND(Coalesce(Length(fname), 0), 65535) - 65536 ELSE BITAND(Coalesce(Length(fname), 0), 65535) END;
         k := 0;
         
         WHILE len >= k LOOP
            --              pmo_substr(h_rec_status, h_fname, k, 1);
            h_rec_status := SubStr(h_fname, k + 1, 1);
            If NOT (SubStr(h_rec_status, 1, 1) = '%') AND NOT (SubStr(h_rec_status, 1, 1) = '_') Then
               EXIT;
            End If;
            
            k := k + 1;
         END LOOP;
         
         If k = len Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_2', 1451+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_2(1452) : cplan.sql(1546) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1454+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1455) : cplan.sql(1549) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         BEGIN
            SELECT REPLACE(h_fname,'''','''''')
            INTO h_fname
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1469+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1470) : cplan.sql(1564) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            If Coalesce(h_pool_type, 0) = 0 Then
               stmt := 'SELECT r.ELEMENT_ID FROM TMT_COMPETENCIES r';
               stwhere := ' WHERE r.TYPE_ID=73';
            Else
               stmt := 
                     'SELECT DISTINCT p.PROFILE_ID FROM PROFILES p JOIN TMT_COMPETENCIES r ON p.ROLE_ID = r.ELEMENT_ID';
               stwhere := ' WHERE p.PROFILE_ID=''';
               stwhere := stwhere || h_res_id;
               stwhere := stwhere || '''';
            End If;
         ElsIf Coalesce(h_consider, 0) = 10 Then
            stmt := 
                  'SELECT r.RESOURCE_ID FROM RESOURCES r JOIN RESOURCE_ASSIGNMENTS ra ON BITAND(ra.PARTICIPANT,4096)>0 '
                   || 'AND ra.ELEMENT_ID=''';
            stmt := stmt || h_project_id;
            stmt := stmt || ''' AND ra.RESOURCE_ID=r.RESOURCE_ID';
            stwhere := ' WHERE UPPER(r.FULL_NAME) like UPPER(''';
            stwhere := stwhere || h_fname;
            stwhere := stwhere || '%'')';
         Else
            If Coalesce(bitflag001, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
               If (last_id IS NOT NULL) Then
                  stmt := 'SELECT DISTINCT r.RESOURCE_ID FROM RESOURCES r';
                  stwhere := ' WHERE r.RESOURCE_ID > ''';
                  stwhere := stwhere || h_last_id;
                  stwhere := stwhere || ''' AND r.CALENDAR_ID IS NOT NuLL AND UPPER(r.FULL_NAME) like UPPER(''';
                  stwhere := stwhere || h_fname;
                  stwhere := stwhere || '%'')';
               Else
                  stmt := 'SELECT DISTINCT r.RESOURCE_ID FROM RESOURCES r';
                  stwhere := ' WHERE r.CALENDAR_ID IS NOT NuLL AND UPPER(r.FULL_NAME) like UPPER(''';
                  stwhere := stwhere || h_fname;
                  stwhere := stwhere || '%'')';
               End If;
            Else
               If (last_id IS NOT NULL) Then
                  stmt := stmt || 'SELECT DISTINCT r.RESOURCE_ID FROM TMT_POOLS TP, PMO_TMP_CPLN_0007 MP, RESOURCES r';
                  stwhere := ' WHERE r.RESOURCE_ID > ''';
                  stwhere := stwhere || h_last_id;
                  stwhere := stwhere || ''' AND TP.ELEMENT_ID=R.RESOURCE_ID '
                         || '  AND ( MP.RANK >= SUBSTR(TP.RANK, 1, 8) AND MP.RANK <= TP.RANK '
                         || '        AND substr(TP.RANK, 1, case when length(MP.RANK) > length(TP.RANK) then 1 else length(MP.RANK) end) = MP.RANK) '
                         || '  AND r.CALENDAR_ID IS NOT NuLL '
                         || '  AND UPPER(r.FULL_NAME) like UPPER(''';
                  stwhere := stwhere || h_fname;
                  stwhere := stwhere || '%'')';
               Else
                  stmt := stmt || 'SELECT DISTINCT r.RESOURCE_ID FROM TMT_POOLS TP, PMO_TMP_CPLN_0007 MP, RESOURCES r';
                  stwhere := ' WHERE TP.ELEMENT_ID=R.RESOURCE_ID '
                         || ' AND ( MP.RANK >= SUBSTR(TP.RANK, 1, 8) AND MP.RANK <= TP.RANK '
                         || '       AND substr(TP.RANK, 1, case when length(MP.RANK) > length(TP.RANK) then 1 else length(MP.RANK) end) = MP.RANK) '
                         || ' AND r.CALENDAR_ID IS NOT NuLL '
                         || ' AND UPPER(r.FULL_NAME) like UPPER(''';
                  stwhere := stwhere || h_fname;
                  stwhere := stwhere || '%'')';
               End If;
            End If;
         End If;
      Else
         If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
            If Coalesce(h_pool_type, 0) = 0 Then
               stmt := 'SELECT r.ELEMENT_ID FROM TMT_COMPETENCIES r';
               stwhere := ' WHERE r.TYPE_ID=73';
            Else
               stmt := 'SELECT p.PROFILE_ID FROM PROFILES p JOIN TMT_COMPETENCIES r ON p.ROLE_ID = r.ELEMENT_ID';
               stwhere := ' WHERE p.PROFILE_ID=''';
               stwhere := stwhere || h_res_id;
               stwhere := stwhere || '''';
            End If;
         ElsIf Coalesce(h_consider, 0) = 10 Then
            stmt := 
                  'SELECT r.RESOURCE_ID FROM RESOURCES r JOIN RESOURCE_ASSIGNMENTS ra ON BITAND(ra.PARTICIPANT,4096)>0 AND ra.ELEMENT_ID=''';
            stmt := stmt || h_project_id;
            stmt := stmt || ''' AND ra.RESOURCE_ID=r.RESOURCE_ID';
            stwhere := ' WHERE';
         Else
            If Coalesce(bitflag001, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
               If (last_id IS NOT NULL) Then
                  stmt := 'SELECT DISTINCT r.RESOURCE_ID FROM RESOURCES r';
                  stwhere := ' WHERE r.RESOURCE_ID > ''';
                  stwhere := stwhere || h_last_id;
                  stwhere := stwhere || ''' AND r.CALENDAR_ID IS NOT NuLL';
               Else
                  stmt := 'SELECT DISTINCT r.RESOURCE_ID FROM RESOURCES r';
                  stwhere := ' WHERE r.CALENDAR_ID IS NOT NuLL';
               End If;
            Else
               If (last_id IS NOT NULL) Then
                  stmt := stmt || 'SELECT DISTINCT r.RESOURCE_ID FROM TMT_POOLS TP, PMO_TMP_CPLN_0007 MP, RESOURCES r';
                  stwhere := ' WHERE r.RESOURCE_ID > ''';
                  stwhere := stwhere || h_last_id;
                  stwhere := stwhere || ''' AND TP.ELEMENT_ID=R.RESOURCE_ID '
                         || ' AND ( MP.RANK >= SUBSTR(TP.RANK, 1, 8) AND MP.RANK <= TP.RANK '
                         || ' AND substr(TP.RANK, 1, case when length(MP.RANK) > length(TP.RANK) then 1 else length(MP.RANK) end) = MP.RANK ) '
                         || ' AND r.CALENDAR_ID IS NOT NuLL';
               Else
                  stmt := stmt || 'SELECT DISTINCT r.RESOURCE_ID FROM TMT_POOLS TP, PMO_TMP_CPLN_0007 MP, RESOURCES r';
                  stwhere := ' WHERE TP.ELEMENT_ID=R.RESOURCE_ID '
                         || ' AND (MP.RANK >= SUBSTR(TP.RANK, 1, 8) AND MP.RANK <= TP.RANK '
                         || ' AND  substr(TP.RANK, 1, case when length(MP.RANK) > length(TP.RANK) then 1 else length(MP.RANK) end) = MP.RANK) '
                         || ' AND r.CALENDAR_ID IS NOT NuLL';
               End If;
            End If;
         End If;
      End If;
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
         If (pool_id001 IS NOT NULL) Then
            BEGIN
               SELECT RANK INTO h_rank FROM TMT_POOLS WHERE ELEMENT_ID = pool_id001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1591+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1592) : cplan.sql(1686) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               stmt := stmt || ',TMT_POOLS p';
               If Length(stwhere) > 7 Then
                  stwhere := stwhere || ' AND r.RESOURCE_ID=p.ELEMENT_ID AND SUBSTR(p.RANK, 1, CASE WHEN LENGTH(''';
               Else
                  stwhere := stwhere || ' r.RESOURCE_ID=p.ELEMENT_ID AND SUBSTR(p.RANK, 1, CASE WHEN LENGTH(''';
               End If;
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''') > LENGTH(p.RANK) THEN 1 ELSE LENGTH(''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''') END) = ''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || '''';
               stwhere := stwhere || 'AND (p.RANK LIKE SUBSTR(''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''', 1, 8) || ''%'') ';
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1614+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1615) : cplan.sql(1709) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || pool_id001 || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1617+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1618) : cplan.sql(1712) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, pool_id001, True);
            End If;
         End If;
         
         If (org_id001 IS NOT NULL) Then
            BEGIN
               SELECT RANK INTO h_rank FROM TMT_ORGANIZATIONS WHERE ELEMENT_ID = org_id001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1632+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1633) : cplan.sql(1727) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               stmt := stmt || ',TMT_ORGANIZATIONS o';
               If Length(stwhere) > 7 Then
                  stwhere := stwhere || ' AND r.RESOURCE_ID=o.ELEMENT_ID AND SUBSTR(o.RANK, 1, CASE WHEN LENGTH(''';
               Else
                  stwhere := stwhere || ' r.RESOURCE_ID=o.ELEMENT_ID AND SUBSTR(o.RANK, 1, CASE WHEN LENGTH(''';
               End If;
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''') > LENGTH(o.RANK) THEN 1 ELSE LENGTH(''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''') END) = ''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || '''';
               stwhere := stwhere || ' AND (o.RANK like SUBSTR(''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''', 1, 8) || ''%'') ';
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1655+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1656) : cplan.sql(1750) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || org_id001 || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1658+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1659) : cplan.sql(1753) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, org_id001, True);
            End If;
         End If;
         
         If (geo_id001 IS NOT NULL) Then
            BEGIN
               SELECT RANK INTO h_rank FROM TMT_GEOGRAPHIES WHERE ELEMENT_ID = geo_id001;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1673+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1674) : cplan.sql(1768) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               stmt := stmt || ',TMT_GEOGRAPHIES g';
               If Length(stwhere) > 7 Then
                  stwhere := stwhere || ' AND r.RESOURCE_ID=g.ELEMENT_ID AND SUBSTR(g.RANK, 1, CASE WHEN LENGTH(''';
               Else
                  stwhere := stwhere || ' r.RESOURCE_ID=g.ELEMENT_ID AND SUBSTR(g.RANK, 1, CASE WHEN LENGTH(''';
               End If;
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''') > LENGTH(g.RANK) THEN 1 ELSE LENGTH(''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''') END) = ''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || '''';
               stwhere := stwhere || ' AND (g.RANK LIKE SUBSTR(''';
               stwhere := stwhere || h_rank;
               stwhere := stwhere || ''', 1, 8) || ''%'') ';
            Else
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1696+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1697) : cplan.sql(1791) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || geo_id001 || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1699+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1700) : cplan.sql(1794) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, geo_id001, True);
            End If;
         End If;
         
         If (Coalesce(h_work_location, 0) > 0) AND (Coalesce(h_work_location, 0) <> 265) Then
            If Length(stwhere) > 7 Then
               stwhere := stwhere || ' AND r.WORK_LOCATION=';
            Else
               stwhere := stwhere || ' r.WORK_LOCATION=';
            End If;
            h_work_loc := TO_CHAR(Coalesce(work_location001, 0));
            stwhere := stwhere || h_work_loc;
         End If;
      End If;
      
      token := SubStr(token, 1, PMO_CNST.SC_H_ID - 1);
      --trace_printf("h_and_roles = %d;%s\n",h_and_roles_Ind,h_and_roles);//trace_flush();
      
      --Parse Roles list ( string of IDs, lenght 32, no comma separator: 'ID32ID32ID32')
      If (and_roles IS NOT NULL) Then
         If Coalesce(Length(h_and_roles), 0) < 32 Then
            numItems := 1;
         Else
            numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
         End If;
         
         i := 0;
         WHILE i < numItems LOOP
            -- Get next role_id
            --              pmo_substr(token, h_and_roles, i*32, 32);     // strncpy( token, h_and_roles[i*32], 32);
            token := SubStr(h_and_roles, i * 32 + 1, 32);
            
            If Length(stwhere) > 7 Then
               stwhere := stwhere || ' AND';
            End If;
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               stwhere := stwhere || 
                      ' EXISTS (SELECT t1.ELEMENT_ID FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2 WHERE r.ELEMENT_ID=t1.ELEMENT_ID '
                      || ' AND ( t2.RANK >= SUBSTR(t1.RANK,1,8) AND t2.RANK <= t1.RANK  '
                      || '       AND substr(t1.RANK,1, case when length(t2.RANK) > length(t1.RANK) then 1 else length (t2.RANK) end) = t2.RANK ) '
                      || ' AND t2.ELEMENT_ID=''';
            Else
               stwhere := stwhere || 
                      ' EXISTS (SELECT t1.ELEMENT_ID FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2 WHERE r.RESOURCE_ID=t1.RESOURCE_ID '
                      || ' AND ( t2.RANK >= SUBSTR(t1.RANK, 1, 8) AND t2.RANK <= t1.RANK  '
                      || ' AND    substr(t1.RANK,1, case when length(t2.RANK) > length(t1.RANK) then 1 else length (t2.RANK)end) = t2.RANK ) '
                      || ' AND t2.ELEMENT_ID=''';
            End If;
            stwhere := stwhere || token;
            stwhere := stwhere || ''')';
            
            If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
               EXIT;
            End If;
            i := i + 1;
         END LOOP;
      End If;
      
      If Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE Then
         --Parse Skills list
         If Coalesce(Length(h_and_skills), 0) < 32 Then
            numItems := 0;
         Else
            numItems := Trunc((Coalesce(Length(h_and_skills), 0) + 1) / 32);
         End If;
         
         If numItems > 0 Then
            i := 0;
            WHILE i < numItems LOOP
               -- Get next skill_id
               --                  pmo_substr(token, h_and_skills, i*32, 32);    //strncpy( token, h_and_skills[i*32], 32);
               token := SubStr(h_and_skills, i * 32 + 1, 32);
               
               If Length(stwhere) > 7 Then
                  stwhere := stwhere || ' AND';
               End If;
               
               stwhere := stwhere || 
                      ' EXISTS (SELECT t1.ELEMENT_ID FROM TMT_COMPETENCIES t1,TMT_COMPETENCIES t2 WHERE r.RESOURCE_ID=t1.RESOURCE_ID '
                      || ' AND ( t2.RANK >= SUBSTR(t1.RANK, 1, 8) AND t2.RANK <= t1.RANK '
                      || ' AND substr(t1.RANK,1, case when length(t2.RANK) > length(t1.RANK) then 1 else length (t2.RANK)end) = t2.RANK ) '
                      || ' AND t2.ELEMENT_ID=''';
               stwhere := stwhere || token;
               stwhere := stwhere || ''')';
               
               i := i + 1;
            END LOOP;
         End If;
      End If;
      
      If Length(stwhere) > 7 Then
         stmt := stmt || stwhere;
      End If;
   End If;
   
   h_temp_stmt := ' ORDER BY r.RESOURCE_ID  ';
   stmt := stmt || h_temp_stmt;
   
   --trace_printf("%s\n\n",(char *) stmt); trace_flush();
   
   OPEN C12 FOR stmt;
   
   LOOP
      h_continue_01 := 1;
      -- reset continue variables for this iteration
      
      FETCH C12 INTO h_res_id;
      IF C12%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      --trace_printf("FETCH C12: SC_STATE = %d, h_res_id: %s\n", SC_STATE,h_res_id);//trace_flush();
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      
      If (Coalesce(h_type, 0) = 0) AND ((Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) 
             AND (Coalesce(h_consider, 0) <> PMO_CNST.SC_SRCH_FROM_PROJECT) AND ((Coalesce(
             h_consider, 0) <> PMO_CNST.SC_SRCH_PROFILE) OR ((pool_id001 IS NULL)))) Then
         OPEN C42;
         
         FETCH C42 INTO h_uniqueID;
         IF C42%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("PA_EXISTS: SC_STATE = %d\n", SC_STATE);//trace_flush();
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               CLOSE C42 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1840+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1841) : cplan.sql(1935) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            h_continue_01 := 0;
         Else
            BEGIN
               CLOSE C42 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1854+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1855) : cplan.sql(1949) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      --trace_printf("h_type: %d, h_consider: %d, h_continue_01: %d\n", h_type,h_consider,h_continue_01);//trace_flush();
      If Coalesce(h_continue_01, 0) = 1 Then
         h_and_roles := and_roles;
         h_and_skills := and_skills;
         If Coalesce(h_type, 0) > 0 Then
            If SubStr(h_res_type, 1, 1) = '1' Then
               --Get a unique ID
               h_assign_id := SYS_GUID();
               h_exists := 0;
            Else
               If isProjectType(Coalesce(h_type, 0)) <> 0 Then
                  BEGIN
                     SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS
                     WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1881+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1882) : cplan.sql(1976) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If Coalesce(h_temp_var, 0) > 0 Then
                     BEGIN
                        CLOSE C12 ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1895+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1896) : cplan.sql(1990) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1901+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1902) : cplan.sql(1996) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1904+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1905) : cplan.sql(1999) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                           , True);
                  End If;
               End If;
               
               BEGIN
                  SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_assign_id, h_number FROM RESOURCE_ASSIGNMENTS
                  WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = h_res_type AND RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1920+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1921) : cplan.sql(2015) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  --Get a unique ID
                  h_assign_id := SYS_GUID();
                  h_exists := 0;
               ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (isNotProjectType(Coalesce(h_type, 0))) <> 0 
                     AND ((BITAND(Coalesce(
                     h_number, 0), PMO_CNST.SC_TEAMMEMBER)) = PMO_CNST.SC_TEAMMEMBER) Then
                  BEGIN
                     CLOSE C12 ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1940+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1941) : cplan.sql(2035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 1946+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(1947) : cplan.sql(2041) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_ALREADY_ASSIGNED), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 1949+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(1950) : cplan.sql(2044) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_ALREADY_ASSIGNED, h_res_id, True);
               Else
                  h_exists := 1;
               End If;
            End If;
            
            --trace_printf("*** h_type = %d, h_table_type: %d, h_task_id = %s, h_exists = %d, h_number = %d, h_cri = %d\nh_full_name = %s, h_full_name_Ind = %d\n",h_type,h_table_type,h_task_id,h_exists,h_number,h_cri,h_full_name,h_full_name_Ind);//trace_flush();
            If Coalesce(h_exists, 0) = 0 Then
               If isProjectType(Coalesce(h_type, 0)) <> 0 Then
                  h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  h_temp_participant := PMO_CNST.SC_AFFECTED;
                  h_temp_bitflag := 1;
                  --trace_printf("h_table_type: %d, h_type: %d, h_temp_security_id: %s\n", h_table_type,h_type,h_temp_security_id);//trace_flush();
                  
                  h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
                  
                  If SubStr(h_res_type, 1, 1) = '1' Then
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
                        RESOURCE_TYPE, WORK_LOCATION, PROFILE_ID, PARTICIPANT, SECURITY_ID, REC_USER, REC_DATETIME, TABLE_TYPE)
                        VALUES (h_assign_id, h_project_id, h_task_id, h_type,
                        h_res_type, h_work_location, h_res_id, PMO_CNST.SC_AFFECTED, h_temp_security_id, rec_user001, SYSTIMESTAMP, h_table_type);
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 1985+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(1986) : cplan.sql(2080) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
                        RESOURCE_TYPE, WORK_LOCATION, RESOURCE_ID, PARTICIPANT, SECURITY_ID, REC_USER, REC_DATETIME, TABLE_TYPE)
                        VALUES (h_assign_id, h_project_id, h_task_id, h_type,
                        h_res_type, h_work_location, h_res_id, PMO_CNST.SC_AFFECTED, h_temp_security_id, rec_user001, SYSTIMESTAMP, h_table_type);
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2005+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2006) : cplan.sql(2100) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     --trace_printf("SC_STATE: %d, h_type: %d, h_temp_security_id: %s\n", SC_STATE,h_type,h_temp_security_id);//trace_flush();
                  End If;
                  
                  BEGIN
                     UPDATE TMT_WBS SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2024+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2025) : cplan.sql(2119) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf isTask(Coalesce(h_type, 0)) <> 0 OR isChangeToTask(Coalesce(h_type, 0)) <> 0 Then
                  If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
                     BEGIN
                        UPDATE TMT_WBS SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'  WHERE ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2042+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2043) : cplan.sql(2137) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CRI Then
                     BEGIN
                        UPDATE TMT_CRI SET ASSIGN_NAMES = ASSIGN_NAMES || h_full_name || ';'  WHERE ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2059+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2060) : cplan.sql(2154) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
                  --trace_printf("h_type: %d, h_temp_security_id: %s, h_cri_security_id: %s\n", h_type,h_temp_security_id,h_cri_security_id);
                  
                  h_temp_participant := PMO_CNST.SC_TEAMMEMBER;
                  h_temp_bitflag := 1;
                  
                  h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
                  
                  If SubStr(h_res_type, 1, 1) = '1' Then
                     h_temp_sc_state := FC_GET_CRI_SECURITY_ID(h_task_id, h_type, h_temp_security_id, h_cri_security_id);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
                        RESOURCE_TYPE, WORK_LOCATION, PROFILE_ID, PARTICIPANT,SECURITY_ID, CRI_SECURITY_ID, REC_USER, REC_DATETIME, TABLE_TYPE)
                        VALUES (h_assign_id, h_project_id, h_task_id, h_type,
                        h_res_type, h_work_location, h_res_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, h_cri_security_id, rec_user001, SYSTIMESTAMP, h_table_type);
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2093+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2094) : cplan.sql(2188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     h_temp_sc_state := FC_GET_CRI_SECURITY_ID(h_task_id, h_type, h_temp_security_id, h_cri_security_id);
                     
                     BEGIN
                        INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
                        RESOURCE_TYPE, WORK_LOCATION, RESOURCE_ID, PARTICIPANT,SECURITY_ID, CRI_SECURITY_ID, REC_USER, REC_DATETIME, TABLE_TYPE)
                        VALUES (h_assign_id, h_project_id, h_task_id, h_type,
                        h_res_type, h_work_location, h_res_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, h_cri_security_id, rec_user001, SYSTIMESTAMP, h_table_type);
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2115+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2116) : cplan.sql(2210) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  --trace_printf("SC_STATE: %d, h_temp_security_id: %s, h_cri_security_id: %s\n", SC_STATE,h_temp_security_id,h_cri_security_id);
               Else
                  h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                 , PMO_CNST.SC_PARTICIPANT_GROUP, h_temp_security_id);
                  
                  h_temp_participant := PMO_CNST.SC_PARTICIPANT;
                  h_temp_bitflag := 1;
                  
                  h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
                  
                  BEGIN
                     INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
                     RESOURCE_TYPE, WORK_LOCATION, RESOURCE_ID, PARTICIPANT,SECURITY_ID, REC_USER, REC_DATETIME, TABLE_TYPE)
                     VALUES (h_assign_id, h_project_id, h_task_id, h_type,
                     h_res_type, h_work_location, h_res_id, PMO_CNST.SC_PARTICIPANT, h_temp_security_id, rec_user001, SYSTIMESTAMP, h_table_type);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2146+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2147) : cplan.sql(2241) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               If (isProjectType(Coalesce(h_type, 0))) <> 0 AND ((BITAND(Coalesce(
                     h_number, 0), PMO_CNST.SC_MANAGER)) <> PMO_CNST.SC_MANAGER) Then
                  BEGIN
                     UPDATE TMT_WBS SET ASSIGN_NAMES = CASE WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN REPLACE (ASSIGN_NAMES, h_full_name || ';', '') ELSE ASSIGN_NAMES END
                     WHERE ELEMENT_ID = h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2167+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2168) : cplan.sql(2262) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
               h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                 , PMO_CNST.SC_PARTICIPANT_GROUP, h_temp2_security_id);
               
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET
                  PARTICIPANT = CASE
                  WHEN SECURITY_ID = h_temp2_security_id AND ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) THEN PMO_CNST.SC_AFFECTED
                  WHEN SECURITY_ID = h_temp2_security_id THEN PMO_CNST.SC_TEAMMEMBER
                  WHEN SECURITY_ID = h_temp_security_id THEN PMO_CNST.SC_AFFECTED
                  ELSE PMO_STPU.BIT_OR( PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
                  END,
                  SECURITY_ID = CASE
                  WHEN SECURITY_ID =  h_temp2_security_id THEN  h_temp_security_id
                  ELSE SECURITY_ID
                  END
                  WHERE RES_ASSIGNMENT_ID = h_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2201+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2202) : cplan.sql(2296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_assign_id);
            End If;
            
            i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_and_roles, h_and_skills, h_shw_roles, h_shw_skills, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, Coalesce(h_per_day, 0), 0
                        , 0.00, 0.00);
            --trace_printf("2_i_return = %d, h_res_id = %s\n", i_return,h_res_id);//trace_flush();
            
            If i_return <> 0 Then
               BEGIN
                  CLOSE C12 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2226+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2227) : cplan.sql(2321) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If i_return < 200000 Then
                  i_return := PMO_CNST.TASK_ASSIGNMENT_ERROR;
               End If;
               
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 2236+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(2237) : cplan.sql(2331) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(i_return), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2239+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2240) : cplan.sql(2334) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(i_return, h_assign_id, True);
            ElsIf (SubStr(h_res_type, 1, 1) = '1') AND (SubStr(h_rec_status, 1, 1) = 'D') Then
               BEGIN
                  UPDATE PROFILES SET REC_STATUS='U' WHERE PROFILE_ID = h_res_id AND REC_STATUS='D';
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2255+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2256) : cplan.sql(2350) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            EXIT;
         Else
            --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d, h_res_type = %s\n", h_start_DT,h_finish_DT,h_duration,h_res_type);//trace_flush();
            i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_and_roles, h_and_skills, h_shw_roles, h_shw_skills, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
            --trace_printf("3_i_return = %d, h_res_id = %s\n", i_return,h_res_id);//trace_flush();
            --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n", h_start_DT,h_finish_DT,h_duration);//trace_flush();
            
            /*if ( i_return == 120)
            				{
            					BEGIN_SQL
            						SELECT DATE(MAX(DAY_YEAR)) INTO HOST(h_dayofyear) FROM session.PMO_TMP_CDY ;
            					END_SQL;
            //trace_printf("SC_STATE: %d, h_dayofyear = %s, h_finish_DT = %s\n", SC_STATE,h_dayofyear,h_finish_DT);//trace_flush();
            
            					if ( strncmp(h_finish_DT,h_dayofyear,10) < 0 )
            					{
            						InitializeHost(SC_PARAM(h_finish_DT), SC_PARAM(h_dayofyear));
            
            						i_return =  FC_SEARCH_2( SC_PARAM(h_assign_id),  SC_PARAM(h_project_id),  SC_PARAM(h_task_id),
            										 SC_PARAM(h_res_id),     SC_PARAM(h_res_cost),    SC_PARAM(h_type),
            										 SC_PARAM(h_and_roles),  SC_PARAM(h_and_skills),  SC_PARAM(h_shw_roles),
            										 SC_PARAM(h_shw_skills), SC_PARAM(h_start_DT),    SC_PARAM(h_finish_DT),
            										 SC_PARAM(h_duration),   SC_PARAM(h_effort),      SC_PARAM(h_work),
            										 SC_PARAM(h_eunit),      SC_PARAM(h_unit),        SC_PARAM(h_cri),
            										 SC_PARAM(h_consider),   SC_PARAM(h_res_type),    SC_PARAM(h_work_type),
            										 SC_PARAM(h_bitflag),    SC_PARAM(h_contour),     SC_PARAM(h_occurrence),
            										 SC_PARAM(h_update),     SC_PARAM(h_per_day),     SC_PARAM(h_priority),
            										 SC_PARAM(h_rec_user));
            //trace_printf("120_return = %d, h_res_id = %s\n", i_return,h_res_id);//trace_flush();
            					}
            				}*/
            
            If i_return = 0 Then
               i_row := i_row + 1;
            End If;
            
            If i_row = Coalesce(h_max_rows, 0) Then
               EXIT;
            End If;
         End If;
      End If;
   END LOOP;
   -- end of while
   
   BEGIN
      CLOSE C12 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2317+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2318) : cplan.sql(2412) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (i_return = PMO_CNST.ASSIGN_STRING_TOLONG) OR ((i_return <> 0) AND (Coalesce(
         h_type, 0) > 0)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_2', 2325+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_2(2326) : cplan.sql(2420) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(i_return), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2328+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2329) : cplan.sql(2423) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(i_return, h_assign_id, True);
   End If;
   
   If i_noskill <> 1 Then
      h_rs_id := '';
      h_uniqueID := '';
      h_and_roles := '';
      h_and_skills := '';
      h_shw_roles := '';
      h_shw_skills := '';
      h_source_ids := '';
      -- role ids
      h_target_ids := '';
      -- skill ids
      i_id := 0;
      i := -1;
      
      OPEN CU42;
      
      LOOP
         FETCH CU42 INTO h_task_id, h_res_id, h_role_id, h_and_role, h_skill_id, h_and_skill, h_shw_role, h_shw_skill;
         IF CU42%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         --trace_printf("SC_STATE = %d, RS_ID = %s, h_res_id = %d:%s, h_role_id: %d:%s\n", SC_STATE,h_task_id,h_res_id_Ind,h_res_id,h_role_id_Ind,h_role_id);//trace_flush();
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If ((h_role_id IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_role_id, h_res_id) = 0) 
               AND (Coalesce(Length(h_rs_id), 0) = 0) Then
            h_rs_id := h_task_id;
         End If;
         
         If i = -1 Then
            h_uniqueID := h_res_id;
            i := 0;
         Else
            i_id := PMO_ORASTD.strcmp(h_uniqueID, h_res_id);
            NULL;
         End If;
         
         --trace_printf("i = %d, i_id = %d, h_uniqueID = %s,h_and_role: %s\n", i,i_id,h_uniqueID,h_and_role);//trace_flush();
         If i_id = 0 Then
            BEGIN
               UPDATE PMO_TMP_SEARCH SET ROLE_ID = NuLL, ROLE_NAME = NuLL, SKILL_ID = NuLL, SKILL_NAME = NuLL, SHOW_ROLE = NuLL, SHOW_SKILL = NuLL WHERE RS_ID = h_last_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2388+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2389) : cplan.sql(2483) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(Length(h_and_role), 0) > 0 Then
               If Coalesce(Length(h_and_roles), 0) > 0 Then
                  h_and_roles := h_and_roles || ',';
               End If;
               
               h_and_roles := h_and_roles || h_and_role;
            End If;
            
            If (h_role_id IS NOT NULL) Then
               h_source_ids := h_source_ids || h_role_id;
            End If;
            
            If Coalesce(Length(h_and_skill), 0) > 0 Then
               If Coalesce(Length(h_and_skills), 0) > 0 Then
                  h_and_skills := h_and_skills || ',';
               End If;
               
               h_and_skills := h_and_skills || h_and_skill;
            End If;
            
            If (h_skill_id IS NOT NULL) Then
               h_target_ids := h_target_ids || h_skill_id;
            End If;
            
            If Coalesce(Length(h_shw_role), 0) > 0 Then
               If Coalesce(Length(h_shw_roles), 0) > 0 Then
                  h_shw_roles := h_shw_roles || ',';
               End If;
               
               h_shw_roles := h_shw_roles || h_shw_role;
            End If;
            
            If Coalesce(Length(h_shw_skill), 0) > 0 Then
               If Coalesce(Length(h_shw_skills), 0) > 0 Then
                  h_shw_skills := h_shw_skills || ',';
               End If;
               
               h_shw_skills := h_shw_skills || h_shw_skill;
            End If;
            --trace_printf("i = %d\nROLES = %s\nSKILLS = %s\n\n", i,h_source_ids,h_target_ids);//trace_flush();
            
            i := i + 1;
         Else
            If i > 0 Then
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET ROLE_ID = h_source_ids, ROLE_NAME = h_and_roles, SKILL_ID = h_target_ids, SKILL_NAME = h_and_skills, SHOW_ROLE = h_shw_roles, SHOW_SKILL = h_shw_skills
                  WHERE RS_ID = h_last_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2450+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2451) : cplan.sql(2545) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("RS_ID = %s, SC_STATE: %d\nh_and_roles = %d:%s\nh_and_skills: %d:%s\nh_shw_roles: %d:%s\nh_shw_skills: %d:%s\n", h_last_id,SC_STATE,h_and_roles_Ind,h_and_roles,h_and_skills_Ind,h_and_skills,h_shw_roles_Ind,h_shw_roles,h_shw_skills_Ind,h_shw_skills);//trace_flush();
            End If;
            
            h_source_ids := h_role_id;
            h_and_roles := h_and_role;
            h_target_ids := h_skill_id;
            h_and_skills := h_and_skill;
            h_shw_roles := h_shw_role;
            h_shw_skills := h_shw_skill;
            h_uniqueID := h_res_id;
            
            i := 1;
         End If;
         
         --strcpy(h_last_id, h_task_id); // save the previous RS_ID
         h_last_id := h_task_id;
      END LOOP;
      BEGIN
         CLOSE CU42 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2479+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2480) : cplan.sql(2574) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If i > 0 Then
         BEGIN
            UPDATE PMO_TMP_SEARCH SET ROLE_ID = h_source_ids, ROLE_NAME = h_and_roles, SKILL_ID = h_target_ids, SKILL_NAME = h_and_skills, SHOW_ROLE = h_shw_roles, SHOW_SKILL = h_shw_skills
            WHERE RS_ID = h_last_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2498+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2499) : cplan.sql(2593) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If (finish_date001 IS NOT NULL) Then
      If Coalesce(h_work, 0) = 0 Then
         BEGIN
            SELECT COALESCE(SUM(AVAILABLE_MIN),0) INTO h_effort FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID=h_project_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_DT, 1, 10)) AND TO_DATE(SubStr(h_finish_DT, 1, 10));
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2516+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2517) : cplan.sql(2611) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_duration := 0;
         NULL;
      End If;
   End If;
   
   BEGIN
      SELECT CURRENCY_ID, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4) INTO h_currency_id, h_rate_echange
      FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID
      WHERE tw.ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2536+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2537) : cplan.sql(2631) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("SC_STATE: %d, h_currency_id: %d:%d, h_rate_echange: %d:%.2f\n",SC_STATE,h_currency_id_Ind,h_currency_id,h_rate_echange_Ind,h_rate_echange);//trace_flush();
   --trace_printf("h_consider: %d, h_res_id: %d:%s, h_rs_id: %s\n", h_consider,h_res_id_Ind,h_res_id,h_rs_id);
   
   If Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE Then
      h_and_roles := and_roles;
      h_and_skills := and_skills;
      
      -- Get the first role_id
      --      pmo_substr(h_role_id, h_and_roles, 0, 32);
      h_role_id := SubStr(h_and_roles, 1, 32);
      
      If Coalesce(h_pool_type, 0) = 0 Then
         h_res_id := SYS_GUID();
      End If;
      
      -- create profile
      BEGIN
         SELECT REC_STATUS INTO h_rec_status FROM PROFILES WHERE PROFILE_ID = h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2565+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2566) : cplan.sql(2660) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_rec_status: %s\n", SC_STATE,h_rec_status);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         h_role_id := SubStr(h_and_roles, 1, 32);
         
         BEGIN
            SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number
            FROM PROFILES
            WHERE PROJECT_ID=h_project_id AND ROLE_ID=h_role_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2584+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2585) : cplan.sql(2679) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')') INTO h_full_name
            FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP
            WHERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2599+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2600) : cplan.sql(2694) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PROFILES(PROFILE_ID, PROJECT_ID, ROLE_ID, PROF_NUMBER, PROFILE_NAME, REC_STATUS, POOL_ID, GEO_ID, ORG_ID)
            VALUES(h_res_id, h_project_id, h_role_id, h_number, h_full_name, 'D',
            pool_id001, geo_id001, org_id001);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2618+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2619) : cplan.sql(2713) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf SubStr(h_rec_status, 1, 1) = 'D' Then
         BEGIN
            UPDATE PROFILES SET REC_STATUS = 'U',
            POOL_ID = pool_id001,
            GEO_ID  = geo_id001,
            ORG_ID  = org_id001
            WHERE PROFILE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2639+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2640) : cplan.sql(2734) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_pool_type, 0) = 0 Then
         --trace_printf("Null: h_res_id = %s\n",h_res_id);//trace_flush();
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0005';
         PMO_DB2M.SC_STATE := 0;
         
         BEGIN
            INSERT INTO PMO_TMP_CPLN_0005
            SELECT RES_ID, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE
            FROM PMO_TMP_SEARCH WHERE RS_ID = h_rs_id AND (REGULAR_COST + REGULAR_SALE) > 0;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2665+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2666) : cplan.sql(2760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_role_id = %s\n",SC_STATE,h_role_id);//trace_flush();
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               INSERT INTO PMO_TMP_CPLN_0005
               SELECT RES_ID,                              MAX(REGULAR_COST) AS REGULAR_COST,
               MAX(REGULAR_SALE) AS REGULAR_SALE,   MAX(SPECIAL_COST) AS SPECIAL_COST,
               MAX(SPECIAL_SALE) AS SPECIAL_SALE
               FROM PMO_TMP_SEARCH WHERE RS_ID <> h_rs_id GROUP BY RES_ID;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2688+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2689) : cplan.sql(2783) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               BEGIN
                  INSERT INTO PMO_TMP_CPLN_0005
                  SELECT RES_ID, 0.00, 0.00, 0.00, 0.00
                  FROM PMO_TMP_SEARCH WHERE RS_ID = h_rs_id GROUP BY RES_ID;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2708+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2709) : cplan.sql(2803) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         OPEN C775C0 FOR SELECT h_res_id AS ELEMENT_ID, 41 AS TYPE_ID, p.PROFILE_NAME AS ELEMENT_NAME, rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, h_work_location AS WORK_LOCATION,
         o1.ELEMENT_NAME AS ORGANIZATION,g1.ELEMENT_NAME AS GEOGRAPHY,
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST WHEN rs.RES_SALE_RATE=0 THEN temp.REGULAR_COST * h_rate_echange ELSE a.REGULAR_COST * h_rate_echange END,2) AS REGULAR_COST,
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.REGULAR_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.REGULAR_SALE * h_rate_echange ELSE a.REGULAR_SALE * h_rate_echange END,2) AS REGULAR_SALE,
         ROUND(CASE WHEN pa.SPECIAL_COST>0 THEN pa.SPECIAL_COST WHEN rs.RES_SALE_RATE=0 THEN temp.SPECIAL_COST * h_rate_echange ELSE a.SPECIAL_COST * h_rate_echange END,2) AS SPECIAL_COST,
         ROUND(CASE WHEN pa.SPECIAL_RATE>0 THEN pa.SPECIAL_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.SPECIAL_SALE * h_rate_echange ELSE a.SPECIAL_SALE * h_rate_echange END,2) AS SPECIAL_SALE,
         rs.START_DATE,rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,
         '' AS VERIFIED, 0 AS STATUS2_ID, '' AS FULL_NAME, pp.ELEMENT_NAME AS POOL_NAME, '' AS E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, h_and_roles AS ROLE_ID, h_and_skills AS SKILL_ID, 0 AS ASSIGNED_FLAG,
         'PMOFFICE' AS FS_LABOR_CODE, TO_TIMESTAMP('1899-12-31 00:00:00') AS RES_MODIFIED_DATE
         FROM PMO_TMP_CPLN_0005 temp JOIN PMO_TMP_SEARCH rs ON rs.RES_ID = temp.RES_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID
         JOIN PROFILES p ON p.PROFILE_ID = h_res_id
         JOIN TMT_COMPETENCIES r ON p.ROLE_ID=r.ELEMENT_ID
         LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON p.ORG_ID = o1.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON p.GEO_ID = g1.ELEMENT_ID
         LEFT OUTER JOIN TMT_POOLS pp ON pp.ELEMENT_ID = p.POOL_ID
         WHERE NOT (rs.ROLE_NAME IS NuLL AND rs.SKILL_NAME IS NuLL AND rs.SHOW_ROLE IS NuLL AND rs.SHOW_SKILL IS NuLL);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2734+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2735) : cplan.sql(2829) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C775C0;
      Else
         --trace_printf("POOL ID: h_res_id = %s\n",h_res_id);//trace_flush();
         
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0005';
         PMO_DB2M.SC_STATE := 0;
         
         BEGIN
            INSERT INTO PMO_TMP_CPLN_0005
            SELECT RES_ID, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE
            FROM PMO_TMP_SEARCH WHERE ROLE_ID = h_rs_id AND (REGULAR_COST + REGULAR_SALE) > 0;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2758+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2759) : cplan.sql(2853) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               INSERT INTO PMO_TMP_CPLN_0005
               SELECT RES_ID,                              MAX(REGULAR_COST) AS REGULAR_COST,
               MAX(REGULAR_SALE) AS REGULAR_SALE,   MAX(SPECIAL_COST) AS SPECIAL_COST,
               MAX(SPECIAL_SALE) AS SPECIAL_SALE
               FROM PMO_TMP_SEARCH WHERE RS_ID <> h_rs_id GROUP BY RES_ID;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2780+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2781) : cplan.sql(2875) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               BEGIN
                  INSERT INTO PMO_TMP_CPLN_0005
                  SELECT RES_ID, 0.00, 0.00, 0.00, 0.00
                  FROM PMO_TMP_SEARCH WHERE RS_ID = h_rs_id GROUP BY RES_ID;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2800+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2801) : cplan.sql(2895) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         OPEN C775D0 FOR SELECT h_res_id AS ELEMENT_ID, 41 AS TYPE_ID, p.PROFILE_NAME AS ELEMENT_NAME, rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, h_work_location AS WORK_LOCATION,
         o1.ELEMENT_NAME AS ORGANIZATION,g1.ELEMENT_NAME AS GEOGRAPHY,
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST WHEN rs.RES_SALE_RATE=0 THEN temp.REGULAR_COST * h_rate_echange ELSE a.REGULAR_COST * h_rate_echange END,2) AS REGULAR_COST,
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.REGULAR_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.REGULAR_SALE * h_rate_echange ELSE a.REGULAR_SALE * h_rate_echange END,2) AS REGULAR_SALE,
         ROUND(CASE WHEN pa.SPECIAL_COST>0 THEN pa.SPECIAL_COST WHEN rs.RES_SALE_RATE=0 THEN temp.SPECIAL_COST * h_rate_echange ELSE a.SPECIAL_COST * h_rate_echange END,2) AS SPECIAL_COST,
         ROUND(CASE WHEN pa.SPECIAL_RATE>0 THEN pa.SPECIAL_RATE WHEN rs.RES_SALE_RATE=0 THEN temp.SPECIAL_SALE * h_rate_echange ELSE a.SPECIAL_SALE * h_rate_echange END,2) AS SPECIAL_SALE,
         rs.START_DATE,rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,
         '' AS VERIFIED, 0 AS STATUS2_ID, '' AS FULL_NAME, pp.ELEMENT_NAME AS POOL_NAME, '' AS E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, h_and_roles AS ROLE_ID, /*rs.SKILL_ID*/h_and_skills AS SKILL_ID, 0 AS ASSIGNED_FLAG,
         'PMOFFICE' AS FS_LABOR_CODE, TO_TIMESTAMP('1899-12-31 00:00:00') AS RES_MODIFIED_DATE
         FROM PMO_TMP_CPLN_0005 temp JOIN PMO_TMP_SEARCH rs ON rs.RES_ID = temp.RES_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID
         JOIN PROFILES p ON p.PROFILE_ID=rs.RES_ID
         JOIN TMT_COMPETENCIES r ON p.ROLE_ID=r.ELEMENT_ID
         LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON p.ORG_ID=o1.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON p.GEO_ID=g1.ELEMENT_ID
         LEFT OUTER JOIN TMT_POOLS pp ON pp.ELEMENT_ID=p.POOL_ID
         WHERE NOT (rs.ROLE_NAME IS NuLL AND rs.SKILL_NAME IS NuLL AND rs.SHOW_ROLE IS NuLL AND rs.SHOW_SKILL IS NuLL);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2826+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2827) : cplan.sql(2921) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C775D0;
      End If;
   Else
      If i_noskill = 1 Then
         OPEN C775E1 FOR SELECT rs.RES_ID AS ELEMENT_ID, 13 AS TYPE_ID, r.FULL_NAME AS ELEMENT_NAME, rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, r.WORK_LOCATION,
         o2.ELEMENT_NAME AS ORGANIZATION,g2.ELEMENT_NAME AS GEOGRAPHY,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.REGULAR_COST ELSE rs.REGULAR_COST * h_rate_echange END,2) AS REGULAR_COST,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.REGULAR_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.REGULAR_SALE ELSE rs.REGULAR_SALE * h_rate_echange END,2) AS REGULAR_SALE,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.SPECIAL_COST ELSE rs.SPECIAL_COST * h_rate_echange END,2) AS SPECIAL_COST,
         ROUND(CASE WHEN rs.DEFAULT_FLAG='0' THEN rs.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END WHEN rs.DEFAULT_FLAG='1' THEN rs.SPECIAL_SALE ELSE rs.SPECIAL_SALE * h_rate_echange END,2) AS SPECIAL_SALE,
         rs.START_DATE, rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,
         rv.FULL_NAME AS VERIFIED, r.STATUS2_ID, pp.ELEMENT_NAME AS POOL_NAME, RM.FULL_NAME, RM.E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, rs.ROLE_ID, rs.SKILL_ID, COALESCE(pa.PLANNED_WORK + pa.SCHED_WORK,0) AS ASSIGNED_FLAG,
         r.FS_LABOR_CODE, r.REC_DATETIME AS RES_MODIFIED_DATE
         FROM PMO_TMP_SEARCH rs JOIN TMT_POOLS p ON rs.RES_ID=p.ELEMENT_ID
         JOIN TMT_POOLS pp ON pp.ELEMENT_ID=p.PARENT_ID
         LEFT OUTER JOIN RESOURCES RM ON pp.MANAGER_ID=RM.RESOURCE_ID
         JOIN RESOURCES r ON rs.RES_ID=r.RESOURCE_ID LEFT OUTER JOIN RESOURCES rv ON rv.RESOURCE_ID=r.VERIFIED_1
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON r.RESOURCE_ID=o1.ELEMENT_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o2 ON o1.PARENT_ID=o2.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON r.RESOURCE_ID=g1.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g2 ON g1.PARENT_ID=g2.ELEMENT_ID
         ORDER BY ELEMENT_ID;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2852+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2853) : cplan.sql(2947) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C775E1;
      Else
         -- Temporary Table Declaration has been changed to Truncate
         EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0006';
         PMO_DB2M.SC_STATE := 0;
         
         BEGIN
            INSERT INTO PMO_TMP_CPLN_0006
            SELECT RES_ID, REGULAR_COST, REGULAR_SALE, SPECIAL_COST, SPECIAL_SALE, 1
            FROM PMO_TMP_SEARCH WHERE DEFAULT_FLAG = '1';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2874+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2875) : cplan.sql(2969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO PMO_TMP_CPLN_0006
            SELECT  RES_ID,                             MAX(REGULAR_COST) AS REGULAR_COST,
            MAX(REGULAR_SALE) AS REGULAR_SALE,  MAX(SPECIAL_COST) AS SPECIAL_COST,
            MAX(SPECIAL_SALE) AS SPECIAL_SALE , 0
            FROM PMO_TMP_SEARCH
            WHERE RES_ID NOT IN (SELECT RES_ID
            FROM PMO_TMP_SEARCH WHERE DEFAULT_FLAG = '1')
            GROUP BY RES_ID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2898+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2899) : cplan.sql(2993) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         OPEN C775E0 FOR SELECT rs.RES_ID AS ELEMENT_ID, 13 AS TYPE_ID, r.FULL_NAME AS ELEMENT_NAME, rs.ROLE_NAME, rs.SKILL_NAME, rs.SHOW_ROLE, rs.SHOW_SKILL, r.WORK_LOCATION,
         o2.ELEMENT_NAME AS ORGANIZATION,g2.ELEMENT_NAME AS GEOGRAPHY,
         
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.REGULAR_COST
         WHEN a.REGULAR_COST>0 THEN a.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.REGULAR_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.REGULAR_COST * h_rate_echange
         END,2) AS REGULAR_COST,
         
         ROUND(CASE WHEN pa.REGULAR_COST> 0 THEN pa.REGULAR_RATE
         WHEN a.REGULAR_COST > 0 THEN a.REGULAR_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.REGULAR_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.REGULAR_SALE * h_rate_echange
         END,2) AS REGULAR_SALE,
         
         ROUND(CASE WHEN pa.REGULAR_COST>0 THEN pa.SPECIAL_COST
         WHEN a.REGULAR_COST>0 THEN a.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.SPECIAL_COST * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.SPECIAL_COST * h_rate_echange
         END,2) AS SPECIAL_COST,
         
         ROUND(CASE WHEN pa.REGULAR_RATE>0 THEN pa.SPECIAL_RATE
         WHEN a.REGULAR_SALE>0 THEN a.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         WHEN def.REGULAR_COST>0 THEN def.SPECIAL_SALE * CASE WHEN (r.CURRENCY_ID = h_currency_id) OR (r.CURRENCY_ID * h_currency_id = 0) THEN 1 ELSE (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = h_currency_id) / (SELECT PMO_ORASTD.TODECIMAL(ELEMENT_NAME,20,4) FROM TMT_DATAFIELDS WHERE ELEMENT_ID = r.CURRENCY_ID) END
         else prf.SPECIAL_SALE * h_rate_echange
         END,2) AS SPECIAL_SALE,
         
         rs.START_DATE,rs.FINISH_DATE,rs.ASSIGNABLE,rs.PLANNED,rs.SCHED, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END AS WORK_EFFORT, h_eunit AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS AVAILABLE,
         rv.FULL_NAME AS VERIFIED, r.STATUS2_ID, pp.ELEMENT_NAME AS POOL_NAME, RM.FULL_NAME, RM.E_MAIL_ADDRESS, '' AS UNIQUE_ID, h_per_day AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING,
         rs.ROLE_ID, rs.SKILL_ID, COALESCE(pa.PLANNED_WORK + pa.SCHED_WORK,0) AS ASSIGNED_FLAG,
         r.FS_LABOR_CODE, r.REC_DATETIME AS RES_MODIFIED_DATE
         FROM PMO_TMP_SEARCH rs
         LEFT JOIN PMO_TMP_CPLN_0006 def ON rs.RES_ID=def.RES_ID AND def.DEFAULT_FLAG = 1
         LEFT JOIN PMO_TMP_CPLN_0006 prf ON rs.RES_ID=prf.RES_ID AND prf.DEFAULT_FLAG = 0
         JOIN TMT_POOLS p ON rs.RES_ID=p.ELEMENT_ID
         JOIN TMT_POOLS pp ON pp.ELEMENT_ID=p.PARENT_ID
         LEFT OUTER JOIN RESOURCES RM ON pp.MANAGER_ID=RM.RESOURCE_ID
         JOIN RESOURCES r ON rs.RES_ID=r.RESOURCE_ID LEFT OUTER JOIN RESOURCES rv ON rv.RESOURCE_ID=r.VERIFIED_1
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND pa.RESOURCE_ID=rs.RES_ID
         LEFT OUTER JOIN RESOURCES rr ON r.VERIFIED_1=rr.RESOURCE_ID
         LEFT OUTER JOIN TMT_ORGANIZATIONS o1 ON r.RESOURCE_ID=o1.ELEMENT_ID LEFT OUTER JOIN TMT_ORGANIZATIONS o2 ON o1.PARENT_ID=o2.ELEMENT_ID
         LEFT OUTER JOIN TMT_GEOGRAPHIES g1 ON r.RESOURCE_ID=g1.ELEMENT_ID LEFT OUTER JOIN TMT_GEOGRAPHIES g2 ON g1.PARENT_ID=g2.ELEMENT_ID
         LEFT OUTER JOIN RATES a ON r.RATE_ID=a.RATE_ID
         WHERE NOT (rs.ROLE_NAME IS NuLL AND rs.SKILL_NAME IS NuLL AND rs.SHOW_ROLE IS NuLL AND rs.SHOW_SKILL IS NuLL)
         ORDER BY ELEMENT_ID;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_2', 2949+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_2(2950) : cplan.sql(3044) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C775E0;
      End If;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2959+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2960) : cplan.sql(3054) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_2', 2971+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_2(2972) : cplan.sql(3066) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SEARCH_2;
ALTER FUNCTION SP_SEARCH_2 COMPILE DEBUG;

/*#################################################################################################*/

-- Search - Assign in Assigned tab (Cplan view)

/*#################################################################################################*/

Prompt == FUNCTION SP_SEARCH_1 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_1
(
     prj_id VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , work_type PLS_INTEGER
   , type001 PLS_INTEGER
   , and_roles VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_start_date VARCHAR2(26);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_finish_D DATE;
   h_res_type VARCHAR2(1);
   h_assign_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_shw_role VARCHAR2(8000);
   h_shw_skill VARCHAR2(8000);
   h_full_name VARCHAR2(255);
   h_count PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_julian_day PLS_INTEGER := 0;
   h_j_efective_date PLS_INTEGER := 0;
   h_j_disconti_date PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_active VARCHAR2(1);
   h_rec_status VARCHAR2(1);
   h_res_cost NUMBER := 0.00;
   stmt VARCHAR2(7999);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id CHAR(32);
   h_temp2_security_id CHAR(32);
   h_cri_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_continue_01 PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_task_start_DT VARCHAR2(26);
   h_task_finish_DT VARCHAR2(26);
   h_task_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   id VARCHAR2(32);
   i PLS_INTEGER NOT NULL := 1;
   ii PLS_INTEGER NOT NULL := 1;
   i_return PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SEARCH_1') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SEARCH_1'), '-- ' || SYSTIMESTAMP, ':r := SP_SEARCH_1(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(type001), 'NULL')
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 110+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SEARCH_1(111) : cplan.sql(3197) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   
   -- Parse parameters to local variables
   
   h_project_id := prj_id;
   h_start_date := start_date001;
   h_finish_DT := finish_date001;
   h_rec_user := rec_user001;
   
   h_effort := effort001;
   -- total effort for period
   h_work := work001;
   -- amount for countour
   h_occurrence := occurrence;
   -- occurrence
   h_eunit := eunit;
   h_consider := consider_planned_sched;
   h_unit := unit;
   h_type := type001;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_work_type := work_type;
   
   h_task_id := h_project_id;
   h_dayofyear := '';
   h_prj_assign_id := NULL;
   h_table_type := GetTable(Coalesce(h_type, 0));
   h_task_type := 0;
   
   --local variables
   /*int iii = 0;*/
   -- 0 = SD, 1 = SD + FD
   h_res_cost := 0.00;
   h_update := 0;
   
   If (prj_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 151+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(152) : cplan.sql(3238) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 154+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(155) : cplan.sql(3241) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If (and_roles IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 161+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(162) : cplan.sql(3248) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 164+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(165) : cplan.sql(3251) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((finish_date001 IS NOT NULL) AND (Coalesce(Length(h_finish_DT), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 171+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(172) : cplan.sql(3258) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 174+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(175) : cplan.sql(3261) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL) AND (Coalesce(Length(h_start_date), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 181+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(182) : cplan.sql(3268) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 184+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(185) : cplan.sql(3271) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL)) AND ((finish_date001 IS NOT NULL)) Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 200+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(201) : cplan.sql(3287) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_total, 0) < 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 207+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(208) : cplan.sql(3294) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 210+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(211) : cplan.sql(3297) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, True);
      End If;
      
      iSF := 1;
   ElsIf ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         SELECT SYSTIMESTAMP INTO h_start_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 226+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(227) : cplan.sql(3313) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_start_DT := h_start_date;
   If (finish_date001 IS NULL) Then
      h_finish_DT := '';
   End If;
   
   h_res_id := SubStr(h_res_id, 1, 32);
   h_res_type := SubStr(h_res_type, 1, 1);
   h_and_roles := and_roles;
   
   If Coalesce(h_type, 0) > 0 Then
      -- Get resource_id and resource type
      h_res_id := SubStr(h_and_roles, 1, 32);
      h_res_type := SubStr(h_and_roles, 33, 1);
      
      If isProjectType(Coalesce(h_type, 0)) <> 0 Then
         BEGIN
            SELECT PROJECT_ID, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
            INTO h_project_id, h_per_day, h_per_week, h_priority
            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 257+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(258) : cplan.sql(3344) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT B.TYPE_ID, B.TASK_TYPE, B.UNRESOLVED_START, B.UNRESOLVED_FINISH, B.PROJECT_ID,
            COALESCE(A.MINUTE_PER_DAY, 480), COALESCE(A.MINUTE_PER_WEEK, 2400), A.PRIORITY
            INTO h_type, h_task_type, h_task_start_DT, h_task_finish_DT, h_project_id,
            h_per_day, h_per_week, h_priority FROM TMT_WBS A, TMT_WBS B
            WHERE B.ELEMENT_ID = h_task_id AND B.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND A.ELEMENT_ID = B.PROJECT_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 274+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(275) : cplan.sql(3361) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         BEGIN
            SELECT PROJECT_ID, WBS_ID INTO h_project_id, id FROM TMT_CRI WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 289+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(290) : cplan.sql(3376) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            If (id IS NOT NULL) Then
               h_table_type := 1;
            End If;
            BEGIN
               SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
               INTO h_per_day, h_per_week, h_priority
               FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 308+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(309) : cplan.sql(3395) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 314+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(315) : cplan.sql(3401) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 317+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(318) : cplan.sql(3404) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
         End If;
      Else
         h_table_type := 1;
         
         /*BEGIN_SQL
         				SELECT TYPE_ID INTO HOST(h_type) FROM TMT_CRI WHERE ELEMENT_ID = HOST(h_task_id) ;
         			END_SQL;*/
         
         /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
         			{
         				h_count = 0;
         				BEGIN_SQL
         					SELECT COUNT('X') INTO HOST(h_count) FROM DUAL
         					WHERE EXISTS (SELECT 'X' FROM TASK_ASSIGNMENTS
         						WHERE PROJECT_ID = HOST(h_project_id) AND TASK_ID = HOST(h_project_id) AND RESOURCE_ID = HOST(h_res_id));
         				END_SQL_NOCHECK;
         				if (h_count > 0)
         				{
         					RETURN_ERROR(CANNOT_ASSIGN_FROM_PROJECT_AFFECTED, h_task_id);
         				}
         			}*/
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 354+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(355) : cplan.sql(3441) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 370+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(371) : cplan.sql(3457) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 377+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(378) : cplan.sql(3464) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 380+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(381) : cplan.sql(3467) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
   Else
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
         INTO h_per_day, h_per_week, h_priority
         FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 396+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(397) : cplan.sql(3483) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         BEGIN
            SELECT 480, 2400 INTO h_per_day, h_per_week FROM TMT_POOLS WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 409+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(410) : cplan.sql(3496) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 415+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(416) : cplan.sql(3502) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 418+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(419) : cplan.sql(3505) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_project_id, True);
         End If;
      End If;
   End If;
   
   /*if ( (h_task_type == SC_FD_EFFORT_DRIVEN)  (h_type > 0) )
   	{
   //      InitializeHost(SC_PARAM(h_start_DT), SC_PARAM(h_task_start_DT));
   //      InitializeHost(SC_PARAM(h_finish_DT), SC_PARAM(h_task_finish_DT));
   		if (h_consider != SC_SRCH_PROFILE)
   		{
   			h_consider = 7;
   		}
   		iSF = 1;
   //      h_effort   = 0;
   		h_work     = 0;
   	}
   	else
   	{*/
   If iSF = 0 Then
      h_start_DT := start_date001;
      h_finish_DT := finish_date001;
      --trace_printf("1.a_* h_start_DT = %s, h_finish_DT: %s\n",h_start_DT,h_finish_DT);//trace_flush();
      h_temp_var := FC_CALC_DURATION(h_project_id, h_start_DT, h_finish_DT, h_effort, h_work, h_eunit
                               , h_unit, h_consider, h_bitflag, h_contour, h_occurrence, h_per_day
                               , h_per_week, h_work, h_effort, h_duration);
      
      If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_1', 449+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_1(450) : cplan.sql(3536) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 452+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(453) : cplan.sql(3539) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      ElsIf Coalesce(h_temp_var, 0) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 457+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(458) : cplan.sql(3544) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 460+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(461) : cplan.sql(3547) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
      End If;
      
      h_finish_DT := h_finish_DT;
      If (h_finish_DT IS NULL) Then
         h_finish_DT := '';
      End If;
   ElsIf (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) <> 100) Then
      --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_duration: %d, h_unit: %d, h_work: %d\n",h_start_DT, h_finish_DT,h_duration,h_unit,h_work);
      If (Coalesce(h_unit, 0) = 1) OR (Coalesce(h_unit, 0) = 2) Then
         h_eunit := 2;
      ElsIf Coalesce(h_unit, 0) = 0 Then
         h_eunit := 1;
      ElsIf (Coalesce(h_unit, 0) = 3) OR (Coalesce(h_unit, 0) = 4) Then
         h_eunit := 1;
      ElsIf Coalesce(h_unit, 0) = 5 Then
         h_work := Coalesce(h_per_week, 0) * Coalesce(h_work, 0);
         h_eunit := 3;
      End If;
   End If;
   --}
   
   --trace_printf("1_ h_start_DT = %s, h_finish_DT = %d:%s, h_duration: %d\n",h_start_DT,h_finish_DT_Ind,h_finish_DT,C:\TBS\data);
   -- Check for project calendar
   If FC_CREATE_CDY(h_project_id, h_start_DT, h_finish_DT, h_duration) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 488+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(489) : cplan.sql(3575) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 491+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(492) : cplan.sql(3578) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_project_id
            , True);
   End If;
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   --trace_printf("2_ h_start_DT = %s, h_finish_DT = %d:%s, h_duration = %d\nh_start_date = %s, h_effort = %d\n",h_start_DT,h_finish_DT_Ind,h_finish_DT,h_duration,h_start_date,h_effort);
   --trace_printf("h_work_type = %d, h_type = %d, h_project_id = %s, h_start_date = %s\niSF = %d, h_start_DT = %s, h_finish_DT = %s\n", h_work_type,h_type,h_project_id,h_start_date,iSF,h_start_DT,h_finish_DT);
   
   h_shw_role := '';
   h_shw_skill := '';
   
   --timeLog (file,"2______________*:");
   
   h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_C0000000)), 30);
   -- consider CRI  WBS
   h_consider := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
   --trace_printf("1__ h_cri = %d, h_type: %d, h_consider = %d\n",h_cri,h_type,h_consider);//trace_flush();
   
   If Coalesce(h_type, 0) > 0 Then
      If SubStr(h_res_type, 1, 1) = '0' Then
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_uniqueID, h_participant
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 524+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(525) : cplan.sql(3611) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_uniqueID, h_participant
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND PROFILE_ID = h_res_id AND RESOURCE_TYPE = '1';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 539+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(540) : cplan.sql(3626) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((BITAND(Coalesce(
            h_participant, 0), 16)) > 0) Then
         h_count := 0;
         BEGIN
            SELECT COUNT('X') INTO h_count FROM DUAL WHERE EXISTS (
            SELECT 'X' FROM TASK_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_uniqueID);
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 557+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(558) : cplan.sql(3644) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) > 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 564+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(565) : cplan.sql(3651) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_ALREADY_ASSIGNED), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 567+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(568) : cplan.sql(3654) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_ALREADY_ASSIGNED, h_res_id, True);
         End If;
      End If;
      
      h_start_DT := NULL;
      
      BEGIN
         SELECT MIN(BAK_START_DT), PMO_ORASTD.DAYS(h_start_date) - PMO_ORASTD.DAYS(MIN(BAK_START_DT))
         INTO h_start_DT, h_total FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID=h_project_id AND TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE=h_res_type;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 585+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(586) : cplan.sql(3672) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SD: SC_STATE = %d, h_total = %d, h_start_DT = %s\n", SC_STATE,h_total,h_start_DT);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_start_DT IS NOT NULL)) Then
         h_from_prj := 1;
         If Coalesce(h_total, 0) < 0 Then
            h_start_date := h_start_DT;
         End If;
      End If;
      
      h_work_type := 0;
      h_total := 0;
      If iSF = 1 Then
         If SubStr(h_res_type, 1, 1) = '0' Then
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, SCHED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(SCHED_START, 1, 10)) AND
               TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND SCHED_WORK >= h_effort THEN 0 ELSE 1 END,SCHED_WORK
               INTO h_prj_assign_id, h_start_DT, h_total, h_sched_work
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 614+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(615) : cplan.sql(3701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1.1_SF: SC_STATE = %d, h_total = %d, h_start_DT = %s, h_start_date = %s, h_finish_DT = %s\n", SC_STATE,h_total,h_start_DT,h_start_date,h_finish_DT);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_from_prj := 1;
               If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
                  BEGIN
                     SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                     TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                     INTO h_prj_assign_id, h_start_DT, h_total FROM TASK_ASSIGNMENTS
                     WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 634+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(635) : cplan.sql(3721) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  h_work_type := 2;
               End If;
            ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA OR (Coalesce(h_total, 0) = 1) Then
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                  TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                  INTO h_prj_assign_id, h_start_DT, h_total FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 653+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(654) : cplan.sql(3740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2_SF: SC_STATE = %d, h_total = %d, h_start_DT = %s, h_start_date = %s, h_finish_DT = %s\n", SC_STATE,h_total,h_start_DT,h_start_date,h_finish_DT);
            End If;
         Else
            BEGIN
               SELECT PROJECT_ASSIGNMENT_ID, PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
               TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
               INTO h_prj_assign_id, h_start_DT, h_total FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 671+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(672) : cplan.sql(3758) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If SubStr(h_res_type, 1, 1) = '0' Then
            BEGIN
               SELECT SCHED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND h_effort <= SCHED_WORK THEN 0 ELSE 1 END, SCHED_WORK
               INTO h_start_DT,h_total, h_sched_work FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 688+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(689) : cplan.sql(3775) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("ASS_0: SC_STATE = %d, h_total = %d, h_sched_work = %d, h_start_date = %s, h_start_DT = %s\n", SC_STATE,h_total,h_sched_work,h_start_date,h_start_DT);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_from_prj := 1;
               If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
                  BEGIN
                     SELECT PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                     INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
                     WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 707+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(708) : cplan.sql(3794) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  h_work_type := 2;
                  NULL;
               End If;
            ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               BEGIN
                  SELECT PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                  INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 726+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(727) : cplan.sql(3813) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            BEGIN
               SELECT PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
               INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 742+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(743) : cplan.sql(3829) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      --trace_printf("ASS: SC_STATE = %d, h_res_type = %s, h_total = %d, h_from_prj = %d, h_work_type = %d, h_start_date = %s\nh_prj_assign_id: %s, h_start_DT = %s, h_finish_DT = %s\n", SC_STATE,h_res_type,h_total,h_from_prj,h_work_type,h_start_date,h_prj_assign_id,h_start_DT,h_finish_DT);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         If Coalesce(h_total, 0) = 1 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 753+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(754) : cplan.sql(3840) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 756+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(757) : cplan.sql(3843) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
         ElsIf iSF = 1 Then
            BEGIN
               SELECT MIN(DAY_YEAR) INTO h_dayofyear FROM TASK_DAY_OF_YEAR
               WHERE TSK_ASSIGNMENT_ID=h_prj_assign_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 769+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(770) : cplan.sql(3856) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_ORASTD.strncmp(h_finish_DT, h_dayofyear, 10) < 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 776+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(777) : cplan.sql(3863) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 779+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(780) : cplan.sql(3866) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
            End If;
         End If;
         
         h_from_prj := 1;
      ElsIf (Coalesce(h_from_prj, 0) = 1) AND (isNotProjectType(Coalesce(h_type, 0))) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 788+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(789) : cplan.sql(3875) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 791+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(792) : cplan.sql(3878) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
      End If;
      
      If SubStr(h_res_type, 1, 1) = '0' Then
         h_shw_role := h_shw_role || h_res_id;
         
         BEGIN
            SELECT NICK_NAME, PMO_ORASTD.JULIAN_DAY(h_start_DT), ACTIVE, COALESCE(PMO_ORASTD.JULIAN_DAY(EFFECTIVE_DATE),0),COALESCE(PMO_ORASTD.JULIAN_DAY(DISCONTINUE_DATE),0)
            INTO h_full_name, h_julian_day, h_active, h_j_efective_date, h_j_disconti_date
            FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 809+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(810) : cplan.sql(3896) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 816+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(817) : cplan.sql(3903) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 819+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(820) : cplan.sql(3906) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         If (SubStr(h_active, 1, 1) = '0') OR ((Coalesce(h_j_disconti_date, 0) > 0) AND (Coalesce(
               h_julian_day, 0) > Coalesce(h_j_disconti_date, 0))) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 827+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(828) : cplan.sql(3914) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_ACTIVE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 830+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(831) : cplan.sql(3917) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_ACTIVE, h_res_id, True);
         End If;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_assign_id, h_participant, h_security_id
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '0' AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 845+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(846) : cplan.sql(3932) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         h_shw_skill := h_shw_skill || h_res_id;
         
         BEGIN
            SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 860+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(861) : cplan.sql(3947) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 867+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(868) : cplan.sql(3954) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 870+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(871) : cplan.sql(3957) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         BEGIN
            SELECT BAK_START_DT, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)) AND BAK_WORK >= h_effort THEN 0 ELSE 1 END
            INTO h_start_DT, h_total FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 885+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(886) : cplan.sql(3972) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_total, 0) = 1) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 892+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(893) : cplan.sql(3979) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 895+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(896) : cplan.sql(3982) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
         End If;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_assign_id, h_participant, h_security_id
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '1' AND PROFILE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 910+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(911) : cplan.sql(3997) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("0_ SC_STATE = %d, h_cri = %d\n", SC_STATE,h_cri);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_PARTICIPANT_GROUP
                                 , h_temp2_security_id);
         
         --trace_printf("h_temp_security_id: %d:%s, h_temp2_security_id: %d:%s\n", h_temp_security_id_Ind,h_temp_security_id, h_temp2_security_id_Ind,h_temp2_security_id);
         
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT =
            CASE
            WHEN SECURITY_ID = h_temp2_security_id AND ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) THEN PMO_CNST.SC_AFFECTED
            WHEN SECURITY_ID = h_temp2_security_id THEN PMO_CNST.SC_TEAMMEMBER
            WHEN SECURITY_ID = h_temp_security_id  THEN PMO_CNST.SC_AFFECTED
            ELSE PMO_STPU.BIT_OR( PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
            END, SECURITY_ID =
            CASE
            WHEN SECURITY_ID =  h_temp2_security_id THEN  h_temp_security_id
            ELSE SECURITY_ID
            END
            WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 948+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(949) : cplan.sql(4035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("PARTICIPANT: SC_STATE = %d, h_assign_id = %s, h_participant = %d\n",SC_STATE,h_assign_id,h_participant);
         
         h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_assign_id);
      Else
         --Get a unique ID
         h_assign_id := SYS_GUID();
         
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 980+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(981) : cplan.sql(4067) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               UPDATE TMT_CRI SET ASSIGN_NAMES = CASE
               WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
               ELSE
               CASE
               WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
               ELSE ASSIGN_NAMES || h_full_name || ';'
               END
               END WHERE ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1005+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1006) : cplan.sql(4092) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("1_ SC_STATE = %d, h_type = %d\n", SC_STATE,h_type);
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
         --trace_printf("0_** h_temp_security_id: %d:%s\n", h_temp_security_id_Ind,h_temp_security_id);
         
         h_temp_participant := PMO_CNST.SC_TEAMMEMBER;
         h_temp_bitflag := 1;
         
         h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
         --trace_printf("1_** h_temp_security_id: %d:%s\n", h_temp_security_id_Ind,h_temp_security_id);
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            h_temp_sc_state := FC_GET_CRI_SECURITY_ID(h_task_id, h_type, h_temp_security_id, h_cri_security_id);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE, TABLE_TYPE,
               RESOURCE_TYPE, PROFILE_ID, PARTICIPANT,SECURITY_ID, CRI_SECURITY_ID, REC_USER, REC_DATETIME)
               VALUES (h_assign_id, h_project_id, h_task_id, h_type, h_table_type,
               h_res_type, h_res_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, h_cri_security_id, rec_user001, SYSTIMESTAMP);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1040+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1041) : cplan.sql(4127) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_temp_sc_state := FC_GET_CRI_SECURITY_ID(h_task_id, h_type, h_temp_security_id, h_cri_security_id);
            --trace_printf("2_** h_temp_security_id: %d:%s\n", h_temp_security_id_Ind,h_temp_security_id);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE, TABLE_TYPE,
               RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT,SECURITY_ID, CRI_SECURITY_ID, REC_USER, REC_DATETIME)
               VALUES (h_assign_id, h_project_id, h_task_id, h_type, h_table_type,
               h_res_type, h_res_id, PMO_CNST.SC_TEAMMEMBER, h_temp_security_id, h_cri_security_id, rec_user001, SYSTIMESTAMP);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1063+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1064) : cplan.sql(4150) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("?_ SC_STATE = %d, h_assign_id: %s, h_res_type: %s, h_temp_security_id = %s, h_cri_security_id: %s\n", SC_STATE,h_assign_id,h_res_type,h_temp_security_id,h_cri_security_id);
      End If;
      
      h_start_DT := h_start_DT;
      --trace_printf("A_PA_0: h_from_prj = %d, h_per_day: %d\n", h_from_prj,h_per_day);
      
      If Coalesce(h_from_prj, 0) = 1 Then
         BEGIN
            SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME(h_start_date)) INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1084+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1085) : cplan.sql(4171) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_start_date ELSE h_start_DT END
            INTO h_start_DT
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1099+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1100) : cplan.sql(4186) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_totpr := 10;
         h_finish_DT := finish_date001;
         If Coalesce(h_work_type, 0) = 0 Then
            h_update := 3;
         End If;
         
         i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
         --trace_printf("A_PA: i_return = %d, h_totpr = %d, h_res_id = %s\nh_start_DT = %s, h_finish_DT = %s\n",i_return,h_totpr,h_res_id,h_start_DT, h_finish_DT);
         
         If i_return = 0 Then
            If Coalesce(h_work_type, 0) = 0 Then
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1131+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1132) : cplan.sql(4218) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='1' WHERE RES_ID=h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1148+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1149) : cplan.sql(4235) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         --h_bitflag = (h_bitflag | SC_BIT_30);  // assign from constrained by project (type > 0)
         h_start_DT := h_start_date;
         h_finish_DT := finish_date001;
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            h_totpr := PMO_CNST.SC_SRCH_PROFILE;
         Else
            h_totpr := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
         End If;
         
         --trace_printf("A_TA-1: h_start_DT = %s, h_finish_DT = %s, h_totpr: %d, h_unit: %d, h_work: %d\n",h_start_DT, h_finish_DT,h_totpr,h_unit,h_work);
         i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
         --trace_printf("A_TA-2: i_return = %d, h_totpr = %d, h_res_id = %s\nh_start_DT = %s, h_finish_DT = %s\n",i_return,h_totpr,h_res_id,h_start_DT, h_finish_DT);
         --trace_printf("B_ SC_STATE = %d, h_assign_id: %s, h_temp_security_id = %s, h_cri_security_id: %s\n", SC_STATE,h_assign_id,h_temp_security_id,h_cri_security_id);
         
         If i_return = 0 Then
            BEGIN
               UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1187+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1188) : cplan.sql(4274) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   Else
      --Parse resource_ids list
      
      If Coalesce(Length(h_and_roles), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 33);
      End If;
      --trace_printf("*** iSF = %d, numItems = %d, strlen(h_and_roles): %d\nh_and_roles = %s\n",iSF,numItems,strlen(h_and_roles),h_and_roles);
      If numItems > 0 Then
         h_count := 0;
         WHILE Coalesce(h_count, 0) < numItems LOOP
            h_continue_01 := 1;
            -- Get next resource_id and resource type
            --pmo_substr(h_res_id, h_and_roles, h_count*33, 32);       //strncpy( h_res_id, h_and_roles[h_count*33], 32);
            h_res_id := SubStr(h_and_roles, Coalesce(h_count, 0) * 33 + 1, 32);
            --pmo_substr(h_res_type, h_and_roles, h_count*33+32, 1);   //strncpy( h_res_type, h_and_roles[h_count*33+32], 1);
            h_res_type := SubStr(h_and_roles, Coalesce(h_count, 0) * 33 + 32 + 1, 1);
            
            --trace_printf("\n**************************************************************************\n");
            --trace_printf("******** h_count: %d, h_res_id = %s,h_res_type= %s\n", h_count, h_res_id, h_res_type);//trace_flush();
            
            h_start_date := start_date001;
            h_from_prj := 0;
            h_total := 0;
            h_start_DT := h_start_date;
            
            BEGIN
               SELECT COUNT(*), MIN(PROPOSED_START) INTO h_total, h_start_DT FROM TASK_ASSIGNMENTS
               WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id
               AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = h_res_type;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1230+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1231) : cplan.sql(4317) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE = %d, h_start_DT = %d:%s, h_total = %d\n", SC_STATE, h_start_DT_Ind, h_start_DT, h_total);//trace_flush();
            
            If (Coalesce(h_total, 0) > 0) AND ((h_start_DT IS NOT NULL)) Then
               h_from_prj := 1;
               If PMO_ORASTD.strcmp(h_start_date, h_start_DT) < 0 Then
                  h_start_date := h_start_DT;
               End If;
            End If;
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               h_consider := PMO_CNST.SC_SRCH_PROFILE;
               If Coalesce(Length(h_shw_skill), 0) = 0 Then
                  h_shw_skill := h_res_id;
               Else
                  h_shw_skill := h_shw_skill || ''',''';
                  h_shw_skill := h_shw_skill || h_res_id;
               End If;
            Else
               If Coalesce(Length(h_shw_role), 0) = 0 Then
                  h_shw_role := h_res_id;
               Else
                  h_shw_role := h_shw_role || ''',''';
                  h_shw_role := h_shw_role || h_res_id;
               End If;
            End If;
            
            h_work_type := 0;
            h_total := 0;
            --trace_printf("2____ h_start_date = %d:%s\n", h_start_date_Ind,h_start_date); //trace_flush();
            --trace_printf("*** h_shw_skill: %d %s\n", strlen(h_shw_skill),h_shw_skill); //trace_flush();
            --trace_printf("*** h_shw_role : %d %s\n", strlen(h_shw_role), h_shw_role); //trace_flush();
            
            If iSF = 1 Then
               If SubStr(h_res_type, 1, 1) = '0' Then
                  BEGIN
                     SELECT SCHED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(SCHED_START, 1, 10)) AND
                     TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND SCHED_WORK >= h_effort THEN 0 ELSE 1 END
                     INTO h_start_DT, h_total FROM TASK_ASSIGNMENTS
                     WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                     AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1280+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1281) : cplan.sql(4367) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA OR (Coalesce(h_total, 0) = 1) Then
                     BEGIN
                        SELECT PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                        TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                        INTO h_start_DT, h_total FROM TASK_ASSIGNMENTS
                        WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                        AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1298+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1299) : cplan.sql(4385) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     h_work_type := 2;
                     NULL;
                  End If;
               Else
                  BEGIN
                     SELECT PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) >= TO_DATE(SubStr(PLANNED_START, 1, 10)) AND
                     TO_DATE(SubStr(h_finish_DT, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND PLANNED_WORK >= h_effort THEN 0 ELSE 1 END
                     INTO h_start_DT, h_total FROM TASK_ASSIGNMENTS
                     WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                     AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1319+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1320) : cplan.sql(4406) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               If SubStr(h_res_type, 1, 1) = '0' Then
                  BEGIN
                     SELECT SCHED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(SCHED_START, 1, 10)) AND TO_DATE(SubStr(SCHED_FINISH, 1, 10)) AND h_effort <= SCHED_WORK THEN 0 ELSE 1 END, SCHED_WORK
                     INTO h_start_DT,h_total, h_sched_work FROM TASK_ASSIGNMENTS
                     WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                     AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1337+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1338) : cplan.sql(4424) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     If (Coalesce(h_sched_work, 0) = 0) OR (Coalesce(h_total, 0) = 1) Then
                        BEGIN
                           SELECT PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) <= TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                           INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
                           WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                           AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1355+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1356) : cplan.sql(4442) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        h_work_type := 2;
                        NULL;
                     End If;
                  ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     BEGIN
                        SELECT PLANNED_START, CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                        INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
                        WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                        AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG = '0';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1375+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1376) : cplan.sql(4462) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     SELECT PLANNED_START,CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(PLANNED_START, 1, 10)) AND TO_DATE(SubStr(PLANNED_FINISH, 1, 10)) AND h_effort <= PLANNED_WORK THEN 0 ELSE 1 END
                     INTO h_start_DT,h_total FROM TASK_ASSIGNMENTS
                     WHERE PROJECT_ID = h_project_id AND TASK_ID=h_project_id
                     AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG = '0';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1392+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1393) : cplan.sql(4479) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               If Coalesce(h_total, 0) = 1 Then
                  h_from_prj := 2;
               Else
                  h_from_prj := 1;
               End If;
            ElsIf Coalesce(h_from_prj, 0) = 1 Then
               h_continue_01 := 0;
            Else
               h_from_prj := 0;
            End If;
            
            If Coalesce(h_continue_01, 0) = 1 Then
               h_start_DT := h_start_DT;
               h_update := 0;
               --trace_printf("h_from_prj = %d, h_work_type = %d, h_update = %d\n",h_from_prj,h_work_type,h_update);
               --timeLog (file,"3______________*:");
               
               If Coalesce(h_from_prj, 0) = 1 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_DT, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                     INTO h_start_DT
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1428+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1429) : cplan.sql(4515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_start_date ELSE h_start_DT END
                     INTO h_start_DT
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1443+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1444) : cplan.sql(4530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_totpr := 10;
                  h_finish_DT := finish_date001;
                  If Coalesce(h_work_type, 0) = 0 Then
                     h_update := 3;
                  End If;
                  
                  --trace_printf("S_PA-1: h_effort = %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",h_effort, h_start_DT, h_finish_DT, h_duration);
                  i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
                  --trace_printf("S_PA-2: i_return = %d, h_totpr = %d, h_res_id = %s, h_effort = %d\nh_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",i_return,h_totpr,h_res_id, h_effort, h_start_DT, h_finish_DT, h_duration);
                  
                  If i_return = 0 Then
                     If Coalesce(h_work_type, 0) = 0 Then
                        BEGIN
                           UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1476+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1477) : cplan.sql(4563) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='1' WHERE RES_ID=h_res_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1493+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1494) : cplan.sql(4580) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
               ElsIf Coalesce(h_from_prj, 0) = 0 Then
                  h_start_DT := h_start_date;
                  h_finish_DT := finish_date001;
                  
                  If SubStr(h_res_type, 1, 1) = '1' Then
                     h_totpr := PMO_CNST.SC_SRCH_PROFILE;
                  Else
                     h_totpr := Coalesce(h_consider, 0);
                  End If;
                  
                  --trace_printf("S_TA-1: h_effort = %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d, h_totpr: 5d\n",h_effort, h_start_DT, h_finish_DT, h_duration,h_totpr);
                  i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
                  --trace_printf("S_TA-2: i_return = %d, h_totpr = %d, h_effort = %d, h_start_DT = %s, h_finish_DT = %s, h_duration = %d\n",i_return,h_totpr,h_effort, h_start_DT, h_finish_DT, h_duration);
                  If i_return = 0 Then
                     BEGIN
                        UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1529+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1530) : cplan.sql(4616) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            End If;
            h_count := Coalesce(h_count, 0) + 1;
         END LOOP;
         -- end of while
      End If;
   End If;
   --timeLog (file,"4______________*:");
   
   If (i_return <> 0) AND (Coalesce(h_type, 0) > 0) Then
      If Coalesce(h_participant, 0) = 0 Then
         BEGIN
            DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1557+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1558) : cplan.sql(4644) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS SET SECURITY_ID = h_security_id, PARTICIPANT = h_participant
            WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1575+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1576) : cplan.sql(4662) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("*** SC_STATE = %d, h_participant = %d, h_assign_id = %s, h_security_id: %d:%s\n",SC_STATE,h_participant,h_assign_id,h_security_id_Ind,h_security_id);
      
      If i_return = 100 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1584+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1585) : cplan.sql(4671) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_NOT_ACTIVE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1587+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1588) : cplan.sql(4674) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_NOT_ACTIVE, h_res_id, True);
      ElsIf i_return = PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1592+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1593) : cplan.sql(4679) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1595+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1596) : cplan.sql(4682) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_TO_DEFAULT_CRI, h_res_id
               , True);
      ElsIf i_return = PMO_CNST.CANNOT_CHANGE_OBJECT_TYPE Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1601+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1602) : cplan.sql(4688) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_CHANGE_OBJECT_TYPE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1604+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1605) : cplan.sql(4691) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_CHANGE_OBJECT_TYPE, h_res_id
               , True);
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_1', 1610+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_1(1611) : cplan.sql(4697) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 1613+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(1614) : cplan.sql(4700) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_assign_id, True);
      End If;
   End If;
   --trace_printf("SEARCH_1: h_shw_role = %s, h_shw_skill = %s\n",h_shw_role,h_shw_skill);
   --timeLog (file,"5______________*:");
   
   If (Coalesce(Length(h_shw_role), 0) > 0) AND (Coalesce(Length(h_shw_skill), 0) > 0) Then
      /*stmt.sprintf("WITH RUP AS (SELECT RESOURCE_ID, RESOURCE_TYPE, MIN(PLANNED_START) AS PLANNED_START, MAX(PLANNED_FINISH) AS PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_EFFORT, "
      		"MIN(SCHED_START) AS START_DATE, MAX(SCHED_FINISH) AS FINISH_DATE, SUM(SCHED_WORK) AS WORK, SUM(ETC) AS REMAINING "
      		"FROM TASK_ASSIGNMENTS WHERE PROJECT_ID='%s' AND PROJECT_ID<>TASK_ID AND (RESOURCE_ID IN ('%s') OR RESOURCE_ID IN ('%s')) GROUP BY RESOURCE_ID, RESOURCE_TYPE), "
      		"tb AS (SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID, MIN(PLANNED_START) AS PRJ_PLANNED_START, MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_REMAINING, "
      		"MIN(SCHED_START) AS PRJ_SCHED_START, MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, SUM(SCHED_WORK) AS SCHED_REMAINING "
      		"FROM TASK_ASSIGNMENTS WHERE TASK_ID='%s' AND (RESOURCE_ID IN ('%s') OR RESOURCE_ID IN ('%s')) GROUP BY RESOURCE_ID) "
      		"SELECT DISTINCT r.RESOURCE_ID AS ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, r.FULL_NAME AS ELEMENT_NAME, CASE WHEN ta.TSK_ASSIGNMENT_ID IS NULL THEN 185 ELSE 13 END AS TYPE_ID, "
      		"r.E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING, "
      		"rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK,"
      		"rs.SCHED AS SCHED_WORK, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE %d END AS WORK_EFFORT, %d AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, "
      		"%d AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK,tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING, tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING "
      		"FROM RESOURCES r LEFT OUTER JOIN RUP a ON a.RESOURCE_TYPE='0' AND a.RESOURCE_ID=r.RESOURCE_ID "
      		"LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID='%s' AND r.RESOURCE_ID = ta.RESOURCE_ID "
      		"LEFT OUTER JOIN tb ON r.RESOURCE_ID = tb.PROJECT_ASSIGNMENT_ID "
      		"LEFT OUTER JOIN session.PMO_TMP_SEARCH rs ON r.RESOURCE_ID=rs.RES_ID WHERE r.RESOURCE_ID IN ('%s') "
      		"UNION ALL "
      		"SELECT DISTINCT p.PROFILE_ID AS ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, p.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID, "
      		"'' AS E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING, "
      		"rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, "
      		"rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE %d END AS WORK_EFFORT, %d AS UNIT_EFFORT,"
      		"(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, %d AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING,'' AS OBJECTIVES,'' AS RANK,"
      		"tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING, tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING "
      		"FROM PROFILES p LEFT OUTER JOIN RUP a ON a.RESOURCE_TYPE='1' AND p.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID "
      		"JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID LEFT OUTER JOIN session.PMO_TMP_SEARCH rs ON a.RESOURCE_ID=rs.RES_ID "
      		"LEFT OUTER JOIN tb ON p.PROFILE_ID = tb.PROJECT_ASSIGNMENT_ID "
      		"WHERE p.PROFILE_ID IN ('%s') ORDER BY TYPE_ID,ELEMENT_NAME", h_project_id, h_shw_role, h_shw_skill, h_project_id, h_shw_role, h_shw_skill, h_effort, h_eunit, h_per_day, h_project_id, h_shw_role, h_effort, h_eunit, h_per_day, h_shw_skill);*/
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0003';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0003 SELECT RESOURCE_ID, RESOURCE_TYPE, '
             || ' MIN(PLANNED_START) AS PLANNED_START,  MAX(PLANNED_FINISH) AS PLANNED_FINISH, '
             || ' SUM(PLANNED_WORK) AS PLANNED_EFFORT,  MIN(SCHED_START) AS START_DATE,  '
             || ' MAX(SCHED_FINISH) AS FINISH_DATE,     SUM(SCHED_WORK) AS WORK,  '
             || ' SUM(ETC) AS REMAINING '
             || ' FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=''' || h_project_id || ''' AND '
             || '      (RESOURCE_ID IN (''' || h_shw_role || ''') OR RESOURCE_ID IN (''' || h_shw_skill 
                  || ''')) GROUP BY RESOURCE_ID, RESOURCE_TYPE ';
      --trace_printf("PMO_TMP_CPLN_0003:********************\n%s\n", (char *) stmt);
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1675+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1676) : cplan.sql(4762) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0002';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 
            'INSERT INTO PMO_TMP_CPLN_0002 SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,MIN(PLANNED_START) AS PRJ_PLANNED_START, '
             || ' MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,  SUM(PLANNED_WORK) AS PLANNED_REMAINING, '
             || ' MIN(SCHED_START) AS PRJ_SCHED_START,        MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, '
             || ' SUM(SCHED_WORK) AS SCHED_REMAINING  '
             || ' FROM TASK_ASSIGNMENTS '
             || ' WHERE TASK_ID=''' || h_project_id || ''' AND '
             || ' (RESOURCE_ID IN (''' || h_shw_role || ''') OR RESOURCE_ID IN (''' || h_shw_skill 
                  || ''')) GROUP BY RESOURCE_ID ';
      
      --trace_printf("PMO_TMP_CPLN_0002:********************\n%s\n", (char *) stmt);
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1708+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1709) : cplan.sql(4795) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      --trace_printf("ANI FORST INSERT\n");
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME,RESOURCE_TYPE,TYPE_ID,E_MAIL_ADDRESS,PLANNED_START,
         PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,BEST_START,BEST_FINISH,
         ASSIGNABLE,PLANNED,SCHED,WORK_EFFORT,PERIOD_AVAILABILITY,PROJECT_ASSIGNMENT_ID,PRJ_PLANNED_START,
         PRJ_PLANNED_FINISH,PLANNED_REMAINING,PRJ_SCHED_START,PRJ_SCHED_FINISH,SCHED_REMAINING,REGULAR_COST,
         REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE)
         SELECT DISTINCT r.RESOURCE_ID,r.FULL_NAME AS ELEMENT_NAME,'0',
         CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELSE 13 END AS TYPE_ID,
         r.E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,
         rs.START_DATE, rs.FINISH_DATE , rs.ASSIGNABLE ,rs.PLANNED ,
         rs.SCHED ,
         CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END,
         (rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) ,
         tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,
         tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE
         FROM RESOURCES r LEFT OUTER JOIN PMO_TMP_CPLN_0003 a ON a.RESOURCE_TYPE='0' AND a.RESOURCE_ID=r.RESOURCE_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND r.RESOURCE_ID = pa.RESOURCE_ID
         LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=h_project_id AND r.RESOURCE_ID = ta.RESOURCE_ID
         LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON r.RESOURCE_ID = tb.PROJECT_ASSIGNMENT_ID
         LEFT OUTER JOIN PMO_TMP_SEARCH rs ON r.RESOURCE_ID=rs.RES_ID WHERE r.RESOURCE_ID IN (h_shw_role)
         
         UNION ALL
         SELECT DISTINCT p.PROFILE_ID , p.PROFILE_NAME AS ELEMENT_NAME, '1', 41 AS TYPE_ID,
         '', a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,
         rs.START_DATE , rs.FINISH_DATE, rs.ASSIGNABLE ,
         rs.PLANNED,rs.SCHED ,
         CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE h_effort END ,
         (rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) ,
         tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,
         tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE
         FROM PROFILES p LEFT OUTER JOIN PMO_TMP_CPLN_0003 a ON a.RESOURCE_TYPE='1' AND p.PROFILE_ID=a.RESOURCE_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID= h_project_id AND p.PROFILE_ID = pa.RESOURCE_ID
         JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID LEFT OUTER JOIN PMO_TMP_SEARCH rs ON a.RESOURCE_ID=rs.RES_ID
         LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON p.PROFILE_ID = tb.PROJECT_ASSIGNMENT_ID
         WHERE p.PROFILE_ID IN (h_shw_skill)
         ORDER BY TYPE_ID,ELEMENT_NAME;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1765+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1766) : cplan.sql(4852) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      stmt := 
            'SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME AS ELEMENT_NAME, T.TYPE_ID AS TYPE_ID, '
             || ' T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || ' T.BEST_START, T.BEST_FINISH, T.ASSIGNABLE AS CALENDAR,T.PLANNED AS PLANNED_WORK,'
             || ' T.SCHED AS SCHED_WORK, T.WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,T.PERIOD_AVAILABILITY, '
             || ' ' || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '''' AS OBJECTIVES,'''' AS RANK,T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || 'T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || ' FROM PMO_TMP_CPLN01_TEMP T'
             || ' LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID WHERE T.RESOURCE_TYPE IN (''0'') '
             || ' UNION ALL '
             || ' SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME, T.TYPE_ID, '
             || ' '''' AS E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || ' T.BEST_START, T.BEST_FINISH, T.ASSIGNABLE AS CALENDAR, '
             || ' T.PLANNED AS PLANNED_WORK,T.SCHED AS SCHED_WORK, T.WORK_EFFORT, ' || TO_CHAR(Coalesce(
                  h_eunit, 0)) || ' AS UNIT_EFFORT, '
             || ' T.PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING,'''' AS OBJECTIVES,'''' AS RANK, '
             || ' T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || '  T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || ' FROM PMO_TMP_CPLN01_TEMP T LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID '
             || ' WHERE T.RESOURCE_TYPE IN (''1'') ORDER BY TYPE_ID,ELEMENT_NAME';
   ElsIf Coalesce(Length(h_shw_skill), 0) = 0 Then
      /*stmt.sprintf("WITH RUP AS (SELECT RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, MAX(PLANNED_FINISH) AS PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_EFFORT, "
      		"MIN(SCHED_START) AS START_DATE, MAX(SCHED_FINISH) AS FINISH_DATE, SUM(SCHED_WORK) AS WORK, SUM(ETC) AS REMAINING "
      		"FROM TASK_ASSIGNMENTS WHERE PROJECT_ID='%s' AND RESOURCE_ID IN ('%s') AND PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID), "
      		"tb AS (SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID, MIN(PLANNED_START) AS PRJ_PLANNED_START, MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_REMAINING, "
      		"MIN(SCHED_START) AS PRJ_SCHED_START, MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, SUM(SCHED_WORK) AS SCHED_REMAINING "
      		"FROM TASK_ASSIGNMENTS WHERE TASK_ID='%s' AND RESOURCE_ID IN ('%s') GROUP BY RESOURCE_ID) "
      		"SELECT DISTINCT r.RESOURCE_ID AS ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, r.FULL_NAME AS ELEMENT_NAME, CASE WHEN ta.TSK_ASSIGNMENT_ID IS NULL THEN 185 ELSE 13 END AS TYPE_ID, "
      		"r.E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING, "
      		"rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK,"
      		"rs.SCHED AS SCHED_WORK, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE %d END AS WORK_EFFORT, %d AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, "
      		"%d AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK,tb.PROJECT_ASSIGNMENT_ID, tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING, tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING "
      		"FROM RESOURCES r LEFT OUTER JOIN RUP a ON a.RESOURCE_ID=r.RESOURCE_ID "
      		"LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID='%s' AND r.RESOURCE_ID = ta.RESOURCE_ID "
      		"LEFT OUTER JOIN tb ON r.RESOURCE_ID = tb.PROJECT_ASSIGNMENT_ID "
      		"LEFT OUTER JOIN session.PMO_TMP_SEARCH rs ON r.RESOURCE_ID=rs.RES_ID WHERE r.RESOURCE_ID IN ('%s') "
      		"ORDER BY ELEMENT_NAME", h_project_id, h_shw_role, h_project_id, h_shw_role, h_effort, h_eunit, h_per_day, h_project_id, h_shw_role);*/
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0001';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0001 '
             || 'SELECT  RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, '
             || ' MAX(PLANNED_FINISH) AS PLANNED_FINISH,   SUM(PLANNED_WORK) AS PLANNED_EFFORT, '
             || ' MIN(SCHED_START) AS START_DATE,          MAX(SCHED_FINISH) AS FINISH_DATE, '
             || ' SUM(SCHED_WORK) AS WORK,                 SUM(ETC) AS REMAINING '
             || ' FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=''' || h_project_id || ''' AND '
             || ' RESOURCE_ID IN (''' || h_shw_role || ''') AND PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1835+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1836) : cplan.sql(4922) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0002';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0002 '
             || 'SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,      MIN(PLANNED_START) AS PRJ_PLANNED_START, '
             || ' MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_REMAINING, '
             || ' MIN(SCHED_START) AS PRJ_SCHED_START,       MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, '
             || ' SUM(SCHED_WORK) AS SCHED_REMAINING '
             || ' FROM TASK_ASSIGNMENTS WHERE TASK_ID=''' || h_project_id || ''' AND RESOURCE_ID IN (''' 
                  || h_shw_role || ''') GROUP BY RESOURCE_ID';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1865+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1866) : cplan.sql(4952) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      --trace_printf("ANI SECOND INSERT\n");
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME,RESOURCE_TYPE,TYPE_ID,E_MAIL_ADDRESS,PLANNED_START,
         PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,PROJECT_ASSIGNMENT_ID,PRJ_PLANNED_START,
         PRJ_PLANNED_FINISH,PLANNED_REMAINING,PRJ_SCHED_START,PRJ_SCHED_FINISH,SCHED_REMAINING,REGULAR_COST,
         REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE)
         SELECT DISTINCT r.RESOURCE_ID, r.FULL_NAME,'0',
         CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELSE 13 END ,
         r.E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,
         tb.PROJECT_ASSIGNMENT_ID, tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,
         tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE
         FROM RESOURCES r LEFT OUTER JOIN PMO_TMP_CPLN_0001 a ON a.RESOURCE_ID=r.RESOURCE_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND r.RESOURCE_ID = pa.RESOURCE_ID
         LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=h_project_id AND r.RESOURCE_ID = ta.RESOURCE_ID
         LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON r.RESOURCE_ID = tb.PROJECT_ASSIGNMENT_ID
         WHERE r.RESOURCE_ID IN (h_shw_role);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1898+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1899) : cplan.sql(4985) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      stmt := 
            'SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME AS ELEMENT_NAME, T.TYPE_ID, '
             || 'T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK, '
             || 'rs.SCHED AS SCHED_WORK, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE ' || TO_CHAR(Coalesce(
                  h_effort, 0)) || ' END AS WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, '
             || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '''' AS OBJECTIVES,'''' AS RANK,T.PROJECT_ASSIGNMENT_ID, T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || 'T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || 'FROM PMO_TMP_CPLN01_TEMP T '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID '
             || 'ORDER BY ELEMENT_NAME';
   Else
      /*stmt.sprintf("WITH RUP AS (SELECT RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, MAX(PLANNED_FINISH) AS PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_EFFORT, "
      		"MIN(SCHED_START) AS START_DATE, MAX(SCHED_FINISH) AS FINISH_DATE, SUM(SCHED_WORK) AS WORK, SUM(ETC) AS REMAINING "
      		"FROM TASK_ASSIGNMENTS WHERE PROJECT_ID='%s' AND RESOURCE_ID IN ('%s') AND PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID), "
      		"tb AS (SELECT RESOURCE_ID AS PROJECT_ASSIGNMENT_ID, MIN(PLANNED_START) AS PRJ_PLANNED_START, MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH, SUM(PLANNED_WORK) AS PLANNED_REMAINING, "
      		"MIN(SCHED_START) AS PRJ_SCHED_START, MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, SUM(SCHED_WORK) AS SCHED_REMAINING "
      		"FROM TASK_ASSIGNMENTS WHERE TASK_ID='%s' AND RESOURCE_ID IN ('%s') GROUP BY RESOURCE_ID) "
      		"SELECT DISTINCT p.PROFILE_ID AS ELEMENT_ID, '' AS RES_ASSIGNMENT_ID, p.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID, "
      		"'' AS E_MAIL_ADDRESS, a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING, "
      		"rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, "
      		"rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE %d END AS WORK_EFFORT, %d AS UNIT_EFFORT,"
      		"(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, %d AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK, "
      		"tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING, tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING "
      		"FROM PROFILES p LEFT OUTER JOIN RUP a ON p.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID "
      		"JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID LEFT OUTER JOIN session.PMO_TMP_SEARCH rs ON p.PROFILE_ID=rs.RES_ID "
      		"LEFT OUTER JOIN tb ON p.PROFILE_ID = tb.PROJECT_ASSIGNMENT_ID "
      		"WHERE p.PROFILE_ID IN ('%s') ORDER BY ELEMENT_NAME", h_project_id, h_shw_skill, h_project_id, h_shw_skill, h_effort, h_eunit, h_per_day, h_shw_skill);*/
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0001';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0001 '
             || 'SELECT RESOURCE_ID, MIN(PLANNED_START) AS PLANNED_START, '
             || '       MAX(PLANNED_FINISH) AS PLANNED_FINISH,    SUM(PLANNED_WORK) AS PLANNED_EFFORT, '
             || '       MIN(SCHED_START) AS START_DATE,           MAX(SCHED_FINISH) AS FINISH_DATE, '
             || '       SUM(SCHED_WORK) AS WORK,                  SUM(ETC) AS REMAINING '
             || ' FROM TASK_ASSIGNMENTS '
             || ' WHERE PROJECT_ID=''' || h_project_id || '''        AND '
             || '       RESOURCE_ID IN (''' || h_shw_skill || ''')   AND '
             || '       PROJECT_ID<>TASK_ID GROUP BY RESOURCE_ID ';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1960+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1961) : cplan.sql(5047) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN_0002';
      PMO_DB2M.SC_STATE := 0;
      
      stmt := 'INSERT INTO PMO_TMP_CPLN_0002 '
             || 'SELECT    RESOURCE_ID AS PROJECT_ASSIGNMENT_ID,          MIN(PLANNED_START) AS PRJ_PLANNED_START, '
             || '          MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,     SUM(PLANNED_WORK) AS PLANNED_REMAINING, '
             || '          MIN(SCHED_START) AS PRJ_SCHED_START,           MAX(SCHED_FINISH) AS PRJ_SCHED_FINISH, '
             || '          SUM(SCHED_WORK) AS SCHED_REMAINING  '
             || ' FROM TASK_ASSIGNMENTS WHERE TASK_ID=''' || h_project_id || ''' AND RESOURCE_ID IN (''' 
                  || h_shw_skill || ''') GROUP BY RESOURCE_ID ';
      
      BEGIN
         EXECUTE IMMEDIATE stmt;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 1990+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(1991) : cplan.sql(5077) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               LFILE.FPUT('AllTrace',                         'SQL : ' || stmt);
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      --trace_printf("ANI THIRD INSERT\n");
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME,RESOURCE_TYPE,TYPE_ID,E_MAIL_ADDRESS,PLANNED_START,
         PLANNED_FINISH,PLANNED_EFFORT,START_DATE,FINISH_DATE,WORK,REMAINING,PROJECT_ASSIGNMENT_ID,PRJ_PLANNED_START,
         PRJ_PLANNED_FINISH,PLANNED_REMAINING,PRJ_SCHED_START,PRJ_SCHED_FINISH,SCHED_REMAINING,REGULAR_COST,
         REGULAR_RATE,SPECIAL_COST,SPECIAL_RATE)
         SELECT DISTINCT p.PROFILE_ID , p.PROFILE_NAME ,'1', 41 ,
         '' , a.PLANNED_START, a.PLANNED_FINISH, a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,
         tb.PROJECT_ASSIGNMENT_ID,tb.PRJ_PLANNED_START, tb.PRJ_PLANNED_FINISH, tb.PLANNED_REMAINING,
         tb.PRJ_SCHED_START, tb.PRJ_SCHED_FINISH, tb.SCHED_REMAINING, pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE
         FROM PROFILES p LEFT OUTER JOIN PMO_TMP_CPLN_0001 a ON p.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=p.ROLE_ID
         LEFT OUTER JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID=h_project_id AND p.PROFILE_ID = pa.RESOURCE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID
         LEFT OUTER JOIN PMO_TMP_CPLN_0002 tb ON p.PROFILE_ID = tb.PROJECT_ASSIGNMENT_ID
         WHERE p.PROFILE_ID IN (h_shw_skill);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 2023+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(2024) : cplan.sql(5110) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      stmt := 'SELECT T.ELEMENT_ID, '''' AS RES_ASSIGNMENT_ID, T.ELEMENT_NAME, T.TYPE_ID, '
             || 'T.E_MAIL_ADDRESS, T.PLANNED_START, T.PLANNED_FINISH, T.PLANNED_EFFORT, T.START_DATE, T.FINISH_DATE, T.WORK, T.REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, '
             || 'rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, CASE WHEN rs.PROPOSED > 0 THEN rs.PROPOSED ELSE ' 
                  || TO_CHAR(Coalesce(h_effort, 0)) || ' END AS WORK_EFFORT, ' || TO_CHAR(Coalesce(
                  h_eunit, 0)) || ' AS UNIT_EFFORT, '
             || '(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(
                  h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, '''' AS OBJECTIVES,'''' AS RANK,  '
             || 'T.PROJECT_ASSIGNMENT_ID,T.PRJ_PLANNED_START, T.PRJ_PLANNED_FINISH, T.PLANNED_REMAINING, '
             || 'T.PRJ_SCHED_START, T.PRJ_SCHED_FINISH, T.SCHED_REMAINING, T.REGULAR_COST, T.REGULAR_RATE, T.SPECIAL_COST, T.SPECIAL_RATE '
             || 'FROM PMO_TMP_CPLN01_TEMP T LEFT OUTER JOIN PMO_TMP_SEARCH rs ON T.ELEMENT_ID=rs.RES_ID '
             || 'ORDER BY ELEMENT_NAME';
   End If;
   --trace_printf("%s\n", stmt);
   --timeLog (file,"6______________*:");
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 2046+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(2047) : cplan.sql(5133) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_1', 2049+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_1(2050) : cplan.sql(5136) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(stmt);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 2057+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(2058) : cplan.sql(5144) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_1', 2069+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_1(2070) : cplan.sql(5156) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SEARCH_1;
ALTER FUNCTION SP_SEARCH_1 COMPILE DEBUG;

/*######################################################################################*/

-- Search - Assign in Affected tab (Cplan view)

/*######################################################################################*/

Prompt == FUNCTION SP_SEARCH_3 ==
CREATE OR REPLACE FUNCTION SP_SEARCH_3
(
     prj_id VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , work_type PLS_INTEGER
   , type001 PLS_INTEGER
   , and_roles VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_start_date VARCHAR2(26);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_rec_status VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_pa_timephase VARCHAR2(1);
   h_assign_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_shw_role VARCHAR2(8000);
   h_shw_skill VARCHAR2(8000);
   h_SD DATE;
   h_FD DATE;
   h_full_name VARCHAR2(255);
   h_count PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_totpr PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_week PLS_INTEGER := 0;
   h_res_cost NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id CHAR(32);
   h_temp2_security_id CHAR(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   stmt VARCHAR2(7999);
   h_table_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   i PLS_INTEGER NOT NULL := 1;
   ii PLS_INTEGER NOT NULL := 1;
   i_return PLS_INTEGER NOT NULL := 0;
   iSF PLS_INTEGER NOT NULL := 0;
   numItems PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_SEARCH_3') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_SEARCH_3'), '-- ' || SYSTIMESTAMP, ':r := SP_SEARCH_3(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(type001), 'NULL')
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 103+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_SEARCH_3(104) : cplan.sql(5280) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   
   -- Parse parameters to local variables
   h_project_id := prj_id;
   h_start_date := start_date001;
   h_finish_DT := finish_date001;
   h_rec_user := rec_user001;
   
   h_effort := effort001;
   -- total effort for period
   h_work := work001;
   -- amount for countour
   h_occurrence := occurrence;
   -- occurrence
   h_eunit := eunit;
   h_consider := consider_planned_sched;
   h_unit := unit;
   h_type := type001;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_work_type := work_type;
   
   h_task_id := h_project_id;
   h_table_type := GetTable(Coalesce(h_type, 0));
   
   --local variables
   /*int iii = 0;*/
   -- 0 = SD, 1 = SD + FD
   h_res_cost := 0.00;
   h_update := 0;
   
   If (prj_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 140+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(141) : cplan.sql(5317) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 143+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(144) : cplan.sql(5320) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If ((finish_date001 IS NOT NULL) AND (Coalesce(Length(h_finish_DT), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 150+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(151) : cplan.sql(5327) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 153+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(154) : cplan.sql(5330) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((start_date001 IS NOT NULL) AND (Coalesce(Length(h_start_date), 0) < 14)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 160+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(161) : cplan.sql(5337) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 163+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(164) : cplan.sql(5340) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND ((start_date001 IS NOT NULL)) 
             AND ((finish_date001 IS NOT NULL)) Then
      BEGIN
         SELECT PMO_ORASTD.DAYS(finish_date001) - PMO_ORASTD.DAYS(start_date001)
         INTO h_total
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 180+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(181) : cplan.sql(5357) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_total, 0) < 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 187+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(188) : cplan.sql(5364) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 190+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(191) : cplan.sql(5367) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, True);
      End If;
      
      iSF := 1;
   ElsIf ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         SELECT SYSTIMESTAMP INTO h_start_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 206+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(207) : cplan.sql(5383) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_start_DT := h_start_date;
   
   h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_C0000000)), 30);
   -- consider CRI  WBS
   h_consider := (BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF));
   --trace_printf("h_type: %d, h_consider: %d, h_cri: %d\n",h_type,h_consider,h_cri);
   
   If Coalesce(h_type, 0) = 0 Then
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
         INTO h_per_day, h_per_week, h_priority
         FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 230+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(231) : cplan.sql(5407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         BEGIN
            SELECT 480, 2400, 0 INTO h_per_day, h_per_week, h_priority FROM TMT_POOLS WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 244+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(245) : cplan.sql(5421) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 250+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(251) : cplan.sql(5427) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 253+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(254) : cplan.sql(5430) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_project_id, True);
         End If;
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 270+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(271) : cplan.sql(5447) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 286+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(287) : cplan.sql(5463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 293+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(294) : cplan.sql(5470) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 296+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(297) : cplan.sql(5473) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
   Else
      If (and_roles IS NULL) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 304+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(305) : cplan.sql(5481) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 307+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(308) : cplan.sql(5484) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
      End If;
      
      h_and_roles := and_roles;
      
      --pmo_substr(h_res_id, h_and_roles, 0, 32);   //    strncpy( h_res_id, h_and_roles[0], 32);
      --NullifyStrPos(h_res_id, 32);                //    h_res_id[32] = '\0';
      h_res_id := SubStr(h_and_roles, 1, 32);
      
      --pmo_substr(h_res_type, h_and_roles, 32, 1);    //  strncpy( h_res_type, h_and_roles[32], 1);
      --NullifyStrPos(h_res_type, 1);                  //  h_res_type[1] = '\0';
      h_res_type := SubStr(h_and_roles, 33, 1);
      
      If isProjectType(Coalesce(h_type, 0)) <> 0 Then
         BEGIN
            SELECT PROJECT_ID, COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), PRIORITY
            INTO h_project_id, h_per_day, h_per_week, h_priority
            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3);
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 333+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(334) : cplan.sql(5510) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 340+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(341) : cplan.sql(5517) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 343+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(344) : cplan.sql(5520) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 349+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(350) : cplan.sql(5526) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 352+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(353) : cplan.sql(5529) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
      End If;
   End If;
   --trace_printf("iSF: %d, h_unit: %d, h_effort: %d\n",iSF,h_unit,h_effort);
   
   If (Coalesce(contour001, 0) = 0) AND (Coalesce(h_unit, 0) <> 100) Then
      h_work := 0;
   End If;
   
   If iSF = 0 Then
      h_total := 0;
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 5) Then
         h_work := Coalesce(h_per_week, 0) * Coalesce(h_work, 0);
      End If;
      
      If Coalesce(h_effort, 0) = 0 Then
         If (finish_date001 IS NULL) Then
            If Coalesce(h_occurrence, 0) = 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 373+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(374) : cplan.sql(5550) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 376+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(377) : cplan.sql(5553) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
            ElsIf Coalesce(h_work, 0) = 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 381+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(382) : cplan.sql(5558) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 384+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(385) : cplan.sql(5561) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
            Else
               h_effort := Coalesce(h_occurrence, 0) * Coalesce(h_work, 0);
            End If;
         End If;
      ElsIf Coalesce(h_eunit, 0) > 0 Then
         If Coalesce(h_eunit, 0) = 1 Then
            h_effort := Coalesce(h_effort, 0) * 60;
         ElsIf Coalesce(h_eunit, 0) = 2 Then
            h_effort := Coalesce(h_per_day, 0) * Coalesce(h_effort, 0);
         ElsIf Coalesce(h_eunit, 0) = 3 Then
            h_effort := Coalesce(h_per_week, 0) * Coalesce(h_effort, 0);
         End If;
      End If;
      
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) <> 100) Then
         h_duration := Trunc(Coalesce(h_effort, 0) / Coalesce(h_work, 0));
      ElsIf Coalesce(h_effort, 0) > Coalesce(h_per_day, 0) Then
         h_duration := Trunc(Coalesce(h_effort, 0) / Coalesce(h_per_day, 0));
      Else
         h_duration := 1;
      End If;
   End If;
   
   --trace_printf("h_duration = %d, h_effort = %d, h_work = %d\n", h_duration,h_effort,h_work);
   
   If ((finish_date001 IS NULL)) AND (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) <> 100) Then
      h_every := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_0001F000)), 12);
      If Coalesce(h_bitflag, 0) = 3 Then
         If (Coalesce(h_duration, 0) * Coalesce(h_every, 0)) > 36 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 417+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(418) : cplan.sql(5594) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_SEARCH_ASSIGN), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 420+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(421) : cplan.sql(5597) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_SEARCH_ASSIGN, h_project_id, True);
         End If;
         h_week := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_contour, 0), PMO_CNST.SC_00000F80)), 7);
         
         If Coalesce(h_week, 0) = 0 Then
            h_week := 1;
         End If;
         
         h_duration := Coalesce(h_duration, 0) + 1;
         h_duration := Coalesce(h_duration, 0) * Coalesce(h_every, 0);
         --trace_printf("h_duration = %d, h_start_DT = %d, %s\n", h_duration,h_start_DT_Ind,h_start_DT);
         
         BEGIN
            SELECT DAY_YEAR INTO h_finish_DT FROM (
            SELECT DAY_YEAR, ROW_NUMBER() OVER (ORDER BY DAY_YEAR) AS CNT
            FROM CALENDAR_DAY_OF_YEAR
            WHERE ELEMENT_ID = h_project_id AND
            DAY_YEAR > TO_DATE(SubStr(h_start_DT, 1, 10)) AND
            PMO_ORASTD.DAY(DAY_YEAR) = h_week )  WHERE CNT = h_duration;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 447+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(448) : cplan.sql(5624) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT Add_Months(TO_DATE(SubStr(h_start_DT, 1, 10)), 3 * 12) INTO h_finish_DT  FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 461+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(462) : cplan.sql(5638) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         --BEGIN_SQL
         --  CLOSE CA3c WITH RELEASE;
         --END_SQL;
         h_duration := 0;
      Else
         h_duration := FC_SRCH_DURATION(h_bitflag, h_contour, h_every, h_duration, h_occurrence, h_unit);
      End If;
      
      If Coalesce(h_work, 0) > 0 Then
         If (Coalesce(h_unit, 0) = 1) OR (Coalesce(h_unit, 0) = 2) Then
            h_eunit := 2;
         ElsIf (Coalesce(h_unit, 0) = 0) OR (Coalesce(h_unit, 0) = 3) OR (Coalesce(h_unit, 0) = 4) Then
            h_eunit := 1;
         ElsIf Coalesce(h_unit, 0) = 5 Then
            h_eunit := 3;
         End If;
      End If;
   End If;
   
   --trace_printf("h_work: %d, h_duration = %d, h_effort = %d, h_start_DT = %s\n", h_work,h_duration,h_effort,h_start_DT);
   If Coalesce(h_effort, 0) > 0 Then
      If Coalesce(h_duration, 0) > 5000 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 490+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(491) : cplan.sql(5667) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_SEARCH_ASSIGN), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 493+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(494) : cplan.sql(5670) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_SEARCH_ASSIGN, h_project_id, True);
      End If;
      If Coalesce(h_duration, 0) < 30 Then
         h_duration := 50;
      Else
         h_duration := Trunc(Coalesce(h_duration, 0) * 1.5);
      End If;
   End If;
   
   If FC_CREATE_CDY(h_project_id, h_start_DT, h_finish_DT, h_duration) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 506+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(507) : cplan.sql(5683) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 509+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(510) : cplan.sql(5686) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_project_id
            , True);
   End If;
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   --Parse resource_ids list
   h_shw_role := '';
   h_shw_skill := '';
   h_dayofyear := '';
   h_assign_id := '';
   
   h_res_id := SubStr(h_res_id, 1, 32);
   --h_res_id[32] = '\0';
   h_res_type := SubStr(h_res_type, 1, 1);
   --h_res_type[1] = '\0';
   
   --trace_printf("h_type = %d, h_start_date = %s, h_res_id = %s, h_res_type: %s, h_work_type: %d\n", h_type,h_start_date,h_res_id,h_res_type,h_work_type);
   --trace_printf("h_start_DT = %s, h_finish_DT = %s\n", h_start_DT,h_finish_DT);
   If Coalesce(h_type, 0) > 0 Then
      h_SD := NULL;
      h_start_DT := h_start_date;
      If SubStr(h_res_type, 1, 1) = '1' Then
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
            REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_SD, h_FD, h_billable, h_capital_expense,
            h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1' AND BIT_FLAG='0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 547+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(548) : cplan.sql(5724) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("A_PROF: h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", h_total,h_start_DT,h_finish_DT);
         
         BEGIN
            SELECT PROFILE_NAME INTO h_full_name FROM PROFILES WHERE PROFILE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 561+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(562) : cplan.sql(5738) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 568+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(569) : cplan.sql(5745) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 571+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(572) : cplan.sql(5748) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         h_shw_skill := h_res_id;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_assign_id, h_number
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '1' AND PROFILE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 588+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(589) : cplan.sql(5765) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf SubStr(h_res_type, 1, 1) = '0' Then
         BEGIN
            SELECT NICK_NAME INTO h_full_name FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 601+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(602) : cplan.sql(5778) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 608+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(609) : cplan.sql(5785) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 611+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(612) : cplan.sql(5788) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
         End If;
         
         h_shw_role := h_res_id;
         
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
            REGULAR_COST, REGULAR_RATE
            INTO h_prj_assignment_id, h_SD, h_FD, h_billable, h_capital_expense,
            h_reg_cost, h_reg_sale
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 631+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(632) : cplan.sql(5808) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("A_RES_0: SC_STATE = %d, h_start_DT: %d:%s, h_SD: %d:%s\n", SC_STATE,h_start_DT_Ind,h_start_DT,h_SD_Ind,h_SD);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            /*SELECT CASE WHEN DATE(HOST(h_start_date)) BETWEEN DATE(BAK_START_DT) AND DATE(BAK_FINISH_DT)
            					THEN BAK_START_DT + 1 DAYS ELSE PMO_TOTIMESTAMP(HOST(h_start_date)) END, TSK_ASSIGNMENT_ID
            					INTO HOST(h_start_DT), HOST(h_prj_assignment_id)*/
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
               REGULAR_COST, REGULAR_RATE
               INTO h_prj_assignment_id, h_SD, h_FD, h_billable, h_capital_expense,
               h_reg_cost, h_reg_sale
               FROM TASK_ASSIGNMENTS
               WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 654+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(655) : cplan.sql(5831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("A_RES_2: SC_STATE = %d, h_work_type = %d, h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", SC_STATE,h_work_type,h_total,h_start_DT,h_finish_DT);
         End If;
         
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_assign_id, h_number
            FROM RESOURCE_ASSIGNMENTS
            WHERE ELEMENT_ID = h_task_id AND RESOURCE_TYPE = '0' AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 671+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(672) : cplan.sql(5848) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_SEARCH_3', 677+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_SEARCH_3(678) : cplan.sql(5854) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 680+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(681) : cplan.sql(5857) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
      
      --trace_printf("***: h_shw_role = %s, h_shw_skill: %s\n", h_shw_role,h_shw_skill);
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_PARTICIPANT_GROUP
                                 , h_temp2_security_id);
         
         BEGIN
            UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT =
            CASE
            WHEN SECURITY_ID = h_temp2_security_id AND ELEMENT_TYPE IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) THEN PMO_CNST.SC_AFFECTED
            WHEN SECURITY_ID = h_temp2_security_id THEN PMO_CNST.SC_TEAMMEMBER
            WHEN SECURITY_ID = h_temp_security_id THEN PMO_CNST.SC_AFFECTED
            ELSE PMO_STPU.BIT_OR( PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
            END, SECURITY_ID =
            CASE
            WHEN SECURITY_ID = h_temp2_security_id THEN  h_temp_security_id
            ELSE SECURITY_ID
            END
            WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 715+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(716) : cplan.sql(5892) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_assign_id);
      Else
         --Get a unique ID
         h_assign_id := SYS_GUID();
         
         h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
         
         h_temp_participant := PMO_CNST.SC_AFFECTED;
         h_temp_bitflag := 1;
         
         h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_res_type, h_temp_bitflag);
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE, TABLE_TYPE,
               RESOURCE_TYPE, PROFILE_ID, PARTICIPANT, SECURITY_ID, REC_USER, REC_DATETIME)
               VALUES (h_assign_id, h_project_id, h_task_id, h_type, h_table_type,
               h_res_type, h_res_id, PMO_CNST.SC_AFFECTED, h_temp_security_id, rec_user001, SYSTIMESTAMP);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 750+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(751) : cplan.sql(5927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE, TABLE_TYPE,
               RESOURCE_TYPE, RESOURCE_ID, PARTICIPANT, SECURITY_ID, REC_USER, REC_DATETIME)
               VALUES (h_assign_id, h_project_id, h_task_id, h_type, h_table_type,
               h_res_type, h_res_id, PMO_CNST.SC_AFFECTED, h_temp_security_id, rec_user001, SYSTIMESTAMP);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 770+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(771) : cplan.sql(5947) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      --trace_printf("h_prj_assignment_id: %d:%s, h_SD: %d:%s\n", h_prj_assignment_id_Ind,h_prj_assignment_id,h_SD_Ind,h_SD);//trace_flush();
      If (Coalesce(h_work_type, 0) = 0) AND ((h_SD IS NOT NULL)) Then
         BEGIN
            SELECT TIMEPHASE_PROJECT_ASSIGNMENT INTO h_pa_timephase FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 787+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(788) : cplan.sql(5964) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1_* h_pa_timephase: %s\n", h_pa_timephase);
         
         If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_prj_assignment_id, h_SD
                                            , h_FD, h_reg_cost, h_reg_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
         End If;
      End If;
      
      h_start_DT := h_start_DT;
      
      --h_bitflag = (h_bitflag | SC_BIT_30);  // assign from constrained by project (type > 0)
      --strcpy(h_start_DT, h_start_date);
      
      If SubStr(h_res_type, 1, 1) = '1' Then
         h_totpr := PMO_CNST.SC_SRCH_PROFILE;
      Else
         h_totpr := 4;
      End If;
      --          h_totpr = h_consider;
      --trace_printf("h_start_DT = %s, h_finish_DT = %s, h_effort = %d\n", h_start_DT,h_finish_DT,h_effort);
      
      i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
      
      --trace_printf("h_prj_assignment_id: %s, i_return: %d\n", h_prj_assignment_id,i_return);
      If i_return = 0 Then
         BEGIN
            UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 834+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(835) : cplan.sql(6011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      h_shw_role := '';
      h_shw_skill := '';
      
      h_and_roles := and_roles;
      --Parse resource_ids list
      If Coalesce(Length(h_and_roles), 0) < 32 Then
         numItems := 0;
      Else
         numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 33);
      End If;
      
      If numItems > 0 Then
         h_count := 0;
         WHILE Coalesce(h_count, 0) < numItems LOOP
            -- Get next skill_id
            --pmo_substr(h_res_id,   h_and_roles, h_count*33, 32);      //strncpy( h_res_id, h_and_roles[h_count*33], 32);
            h_res_id := SubStr(h_and_roles, Coalesce(h_count, 0) * 33 + 1, 32);
            --pmo_substr(h_res_type, h_and_roles, h_count*33+32, 1);      //strncpy( h_res_type, h_and_roles[h_count*33+32], 1);
            h_res_type := SubStr(h_and_roles, Coalesce(h_count, 0) * 33 + 32 + 1, 1);
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               h_totpr := PMO_CNST.SC_SRCH_PROFILE;
               
               If Coalesce(Length(h_shw_skill), 0) = 0 Then
                  h_shw_skill := h_res_id;
               Else
                  h_shw_skill := h_shw_skill || ''',''';
                  h_shw_skill := h_shw_skill || h_res_id;
               End If;
            Else
               h_totpr := Coalesce(h_consider, 0);
               
               If Coalesce(Length(h_shw_role), 0) = 0 Then
                  h_shw_role := h_res_id;
               Else
                  h_shw_role := h_shw_role || ''',''';
                  h_shw_role := h_shw_role || h_res_id;
               End If;
            End If;
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               BEGIN
                  SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10))
                  THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END
                  INTO h_start_DT FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='1';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 892+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(893) : cplan.sql(6069) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_total := 0;
               h_work_type := 0;
               --trace_printf("S_PROF: h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", h_total,h_start_DT,h_finish_DT);
            Else
               BEGIN
                  SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10))
                  THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END
                  INTO h_start_DT FROM TASK_ASSIGNMENTS
                  WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 912+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(913) : cplan.sql(6089) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  BEGIN
                     SELECT CASE WHEN TO_DATE(SubStr(h_start_date, 1, 10)) BETWEEN TO_DATE(SubStr(BAK_START_DT, 1, 10)) AND TO_DATE(SubStr(BAK_FINISH_DT, 1, 10))
                     THEN BAK_START_DT + 1      ELSE TO_TIMESTAMP(h_start_date) END
                     INTO h_start_DT FROM TASK_ASSIGNMENTS
                     WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id AND RESOURCE_TYPE='0' AND BIT_FLAG='2';
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 929+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(930) : cplan.sql(6106) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("S_RES: h_work_type = %d, h_total = %d, h_start_DT = %s, h_finish_DT = %s\n", h_work_type,h_total,h_start_DT,h_finish_DT);
            End If;
            
            h_start_DT := h_start_DT;
            
            If SubStr(h_res_type, 1, 1) = '1' Then
               h_totpr := PMO_CNST.SC_SRCH_PROFILE;
            Else
               h_totpr := Coalesce(h_consider, 0);
            End If;
            
            h_start_DT := h_start_date;
            
            i_return := FC_SEARCH_2(h_assign_id, h_project_id, h_task_id, h_res_id, h_res_cost, h_type
                        , h_dayofyear, h_dayofyear, h_dayofyear, h_dayofyear, h_start_DT
                        , h_finish_DT, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri, h_totpr
                        , h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence, h_update
                        , h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
            --trace_printf("SP_SEARCH_3: i_return = %d\n",i_return);
            
            If i_return = 0 Then
               BEGIN
                  UPDATE PMO_TMP_SEARCH SET PLANNED_SCHED_FLAG='0' WHERE RES_ID=h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 967+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(968) : cplan.sql(6144) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_count := Coalesce(h_count, 0) + 1;
         END LOOP;
      End If;
   End If;
   --trace_printf("i_return = %d\n", i_return);
   
   If (i_return <> 0) AND (Coalesce(h_type, 0) > 0) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_SEARCH_3', 981+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_SEARCH_3(982) : cplan.sql(6158) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_FROM_PROJECT_AFF), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 984+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(985) : cplan.sql(6161) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_FROM_PROJECT_AFF, h_assign_id
            , True);
   End If;
   
   If Coalesce(h_type, 0) > 0 Then
      h_project_id := h_task_id;
   End If;
   
   --trace_printf("SEARCH_3: h_shw_role = %s, h_shw_skill = %s\n",h_shw_role,h_shw_skill); //trace_flush();
   
   If (Coalesce(Length(h_shw_role), 0) > 0) AND (Coalesce(Length(h_shw_skill), 0) > 0) Then
      stmt := 
            'SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 13 AS TYPE_ID, '
             || 'p.E_MAIL_ADDRESS,b.PLANNED_START,b.PLANNED_FINISH,b.BEST AS PLANNED_EFFORT, '
             || 'a.SCHED_START AS START_DATE, a.SCHED_FINISH AS FINISH_DATE, a.BEST AS WORK, a.SCHED_WORK AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK,'
             || 'rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) || ' AS WORK_EFFORT, ' 
                  || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY,'
             || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM RESOURCE_ASSIGNMENTS t LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND a.BIT_FLAG=''2'' '
             || 'LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND b.BIT_FLAG=''0'' '
             || 'JOIN RESOURCES p ON t.RESOURCE_ID=p.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON t.RESOURCE_ID=rs.RES_ID '
             || 'WHERE t.ELEMENT_ID=''' || h_project_id 
                  || ''' AND t.RESOURCE_TYPE=''0'' AND t.RESOURCE_ID IN (''' || h_shw_role 
                  || ''') AND BITAND(t.PARTICIPANT, 4112) > 0 '
             || 'UNION ALL '
             || 'SELECT r.PROFILE_ID AS ELEMENT_ID, ta.RES_ASSIGNMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID, '
             || ''''' AS E_MAIL_ADDRESS,ta.PLANNED_START,ta.PLANNED_FINISH,ta.BEST AS PLANNED_EFFORT,'
             || PMO_CNST.STR_1899_12_31_00_00_00 || ' AS START_DATE, ' || PMO_CNST.STR_1899_12_31_00_00_00 
                  || ' AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, '
             || 'rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) 
                  || ' AS WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) || ' AS UNIT_EFFORT,'
             || '(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(
                  h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY , rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM TASK_ASSIGNMENTS ta JOIN PROFILES r ON ta.TASK_ID=''' || h_project_id 
                  || ''' AND ta.RESOURCE_TYPE=''1'' AND ta.RESOURCE_ID IN (''' || h_shw_skill 
                  || ''') AND r.PROFILE_ID=ta.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON ta.RESOURCE_ID=rs.RES_ID '
             || 'ORDER BY TYPE_ID';
   ElsIf Coalesce(Length(h_shw_role), 0) > 0 Then
      stmt := 
            'SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 13 AS TYPE_ID, '
             || 'p.E_MAIL_ADDRESS,b.PLANNED_START,b.PLANNED_FINISH,b.BEST AS PLANNED_EFFORT, '
             || 'a.SCHED_START AS START_DATE, a.SCHED_FINISH AS FINISH_DATE, a.BEST AS WORK, a.SCHED_WORK AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR,rs.PLANNED AS PLANNED_WORK,'
             || 'rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) || ' AS WORK_EFFORT, ' 
                  || TO_CHAR(Coalesce(h_eunit, 0)) 
                  || ' AS UNIT_EFFORT,(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY,'
             || TO_CHAR(Coalesce(h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY, rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM RESOURCE_ASSIGNMENTS t LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND a.BIT_FLAG=''2'' '
             || 'LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND b.BIT_FLAG=''0'' '
             || 'JOIN RESOURCES p ON t.RESOURCE_ID=p.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON t.RESOURCE_ID=rs.RES_ID '
             || 'WHERE t.ELEMENT_ID=''' || h_project_id 
                  || ''' AND t.RESOURCE_TYPE=''0'' AND t.RESOURCE_ID IN (''' || h_shw_role 
                  || ''') AND BITAND(t.PARTICIPANT, 4112) > 0 '
             || 'ORDER BY TYPE_ID';
   ElsIf Coalesce(Length(h_shw_skill), 0) > 0 Then
      stmt := 
            'SELECT r.PROFILE_ID AS ELEMENT_ID, ta.RES_ASSIGNMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID, '
             || ''''' AS E_MAIL_ADDRESS,ta.PLANNED_START,ta.PLANNED_FINISH,ta.BEST AS PLANNED_EFFORT,'
             || PMO_CNST.STR_1899_12_31_00_00_00 || ' AS START_DATE, ' || PMO_CNST.STR_1899_12_31_00_00_00 
                  || ' AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING, '
             || 'rs.START_DATE AS BEST_START, rs.FINISH_DATE AS BEST_FINISH, rs.ASSIGNABLE AS CALENDAR, '
             || 'rs.PLANNED AS PLANNED_WORK,rs.SCHED AS SCHED_WORK, ' || TO_CHAR(Coalesce(h_effort, 0)) 
                  || ' AS WORK_EFFORT, ' || TO_CHAR(Coalesce(h_eunit, 0)) || ' AS UNIT_EFFORT,'
             || '(rs.ASSIGNABLE - rs.PLANNED - rs.SCHED) AS PERIOD_AVAILABILITY, ' || TO_CHAR(Coalesce(
                  h_per_day, 0)) 
                  || ' AS MINUTE_PER_DAY , rs.CONTOUR AS ASSIGN_STRING, rs.PLANNED_SCHED_FLAG, '''' AS OBJECTIVES,'''' AS RANK '
             || 'FROM TASK_ASSIGNMENTS ta JOIN PROFILES r ON ta.TASK_ID=''' || h_project_id 
                  || ''' AND ta.RESOURCE_TYPE=''1'' AND ta.RESOURCE_ID IN (''' || h_shw_skill 
                  || ''') AND r.PROFILE_ID=ta.RESOURCE_ID '
             || 'LEFT OUTER JOIN PMO_TMP_SEARCH rs ON ta.RESOURCE_ID=rs.RES_ID '
             || 'ORDER BY TYPE_ID';
   End If;
   --trace_printf("%s\n", (char *)stmt);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 1070+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(1071) : cplan.sql(6247) : ' || SYSTIMESTAMP);
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_SEARCH_3', 1073+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_SEARCH_3(1074) : cplan.sql(6250) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN PMODEFINES.PMO_SELECT_READ(stmt);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 1081+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(1082) : cplan.sql(6258) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_SEARCH_3', 1093+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_SEARCH_3(1094) : cplan.sql(6270) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_SEARCH_3;
ALTER FUNCTION SP_SEARCH_3 COMPILE DEBUG;

/*######################################################################################*/

/* assign a profile to a task or project, used tables: RESOURCE_ASSIGNMENTS, PROFILES   */

/****************************************************************************************/

/*  RES_ID is profile_id, role_id is role_id in TMT_COMPETENCIES    */

/****************************************************************************************/

Prompt == FUNCTION SP_I_PROFILE ==
CREATE OR REPLACE FUNCTION SP_I_PROFILE
(
     prj_id VARCHAR2
   , and_roles VARCHAR2
   , and_skills VARCHAR2
   , org_id001 VARCHAR2
   , geo_id001 VARCHAR2
   , pool_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_role_id VARCHAR2(32);
   h_full_name VARCHAR2(255);
   h_count PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   numItems PLS_INTEGER;
   numItems001 PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_PROFILE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_PROFILE'), '-- ' || SYSTIMESTAMP, ':r := SP_I_PROFILE(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || '''' || org_id001 || ''''
          || ', ' || '''' || geo_id001 || ''''
          || ', ' || '''' || pool_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_PROFILE', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_PROFILE(35) : cplan.sql(6329) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((prj_id IS NULL)) OR ((and_roles IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_PROFILE', 41+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_PROFILE(42) : cplan.sql(6336) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_PROFILE', 44+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_PROFILE(45) : cplan.sql(6339) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --  InitializeHost(SC_PARAM(h_and_roles), and_roles, ind_i_prof->and_roles);
   h_and_roles := and_roles;
   --pmo_substr(h_role_id, h_and_roles, 0, 32);    //strncpy( h_role_id, h_and_roles[0], 32);
   --NullifyStrPos(h_role_id, 32);   //h_role_id[32] = '\0';
   h_role_id := SubStr(h_and_roles, 1, 32);
   
   BEGIN
      SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number
      FROM PROFILES
      WHERE PROJECT_ID=prj_id/**prj_id*/ AND ROLE_ID=h_role_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 65+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(66) : cplan.sql(6360) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')')
      INTO h_full_name FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP
      WHERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 80+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(81) : cplan.sql(6375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("a_SC_STATE: %d, h_full_name: %s\n", SC_STATE,h_full_name);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_I_PROFILE', 88+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_I_PROFILE(89) : cplan.sql(6383) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_role_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_PROFILE', 91+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_PROFILE(92) : cplan.sql(6386) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_role_id, True);
   End If;
   
   --Get a unique id
   h_uniqueID := SYS_GUID();
   
   BEGIN
      INSERT INTO PROFILES(PROFILE_ID,PROJECT_ID,ROLE_ID,PROF_NUMBER, PROFILE_NAME, POOL_ID, GEO_ID, ORG_ID)
      VALUES(h_uniqueID, prj_id, h_role_id, h_number, h_full_name, pool_id001, geo_id001, org_id001);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 112+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(113) : cplan.sql(6407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      --Parse Roles list
      numItems := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
      If numItems > 1 Then
         h_count := 1;
         WHILE Coalesce(h_count, 0) < numItems LOOP
            -- Get next skill_id
            --strncpy( h_role_id, h_and_roles[h_count*32], 32);
            --pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
            h_role_id := SubStr(h_and_roles, Coalesce(h_count, 0) * 32 + 1, 32);
            
            BEGIN
               INSERT INTO RESOURCE_ASSIGN_ROLES
               SELECT h_uniqueID, ELEMENT_ID, PROFICIENCY_LEVEL_ID, PREFERENCE_LEVEL_ID,
               EXPERIENCE_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
               FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_role_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 143+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(144) : cplan.sql(6438) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            h_count := Coalesce(h_count, 0) + 1;
         END LOOP;
      End If;
      
      --Parse Skills list
      If (and_skills IS NOT NULL) Then
         --          InitializeHost(SC_PARAM(h_and_roles), and_skills, ind_i_prof->and_skills);
         h_and_roles := and_skills;
         numItems001 := Trunc((Coalesce(Length(h_and_roles), 0) + 1) / 32);
         
         If numItems001 > 0 Then
            h_count := 0;
            WHILE Coalesce(h_count, 0) < numItems001 LOOP
               -- Get next skill_id
               --strncpy( h_role_id, h_and_roles[h_count*32], 32);
               --pmo_substr(h_role_id, h_and_roles, h_count*32, 32);
               h_role_id := SubStr(h_and_roles, Coalesce(h_count, 0) * 32 + 1, 32);
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_SKILLS
                  SELECT h_uniqueID, h_role_id, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID,
                  PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, TO_CHAR(h_count)
                  FROM TMT_COMPETENCIES WHERE ELEMENT_ID = h_role_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 180+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(181) : cplan.sql(6475) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_count := Coalesce(h_count, 0) + 1;
            END LOOP;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_PROFILE', 192+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_PROFILE(193) : cplan.sql(6487) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 199+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(200) : cplan.sql(6494) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_PROFILE', 211+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_PROFILE(212) : cplan.sql(6506) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_PROFILE;
ALTER FUNCTION SP_I_PROFILE COMPILE DEBUG;

/*######################################################################################*/

/* assign a profile to a task or project, used tables: RESOURCE_ASSIGNMENTS, PROFILES   */

/****************************************************************************************/

/*  RES_ID is profile_id, role_id is role_id in TMT_COMPETENCIES    */

/****************************************************************************************/

Prompt == FUNCTION SP_A_TSKPROF ==
CREATE OR REPLACE FUNCTION SP_A_TSKPROF
(
     task_id001 VARCHAR2
   , res_id001 VARCHAR2
   , role_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_role_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_full_name VARCHAR2(255);
   h_type PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_number PLS_INTEGER := 0;
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_res_type VARCHAR2(1);
   h_temp_sc_state PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_TSKPROF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_TSKPROF'), '-- ' || SYSTIMESTAMP, ':r := SP_A_TSKPROF(' || 
       '''' || task_id001 || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || role_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 37+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_TSKPROF(38) : cplan.sql(6568) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((task_id001 IS NULL)) OR ((res_id001 IS NULL)) OR ((role_id001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_TSKPROF', 44+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_TSKPROF(45) : cplan.sql(6575) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 47+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(48) : cplan.sql(6578) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   BEGIN
      SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type FROM TMT_WBS WHERE ELEMENT_ID = task_id001;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 60+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(61) : cplan.sql(6591) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type FROM TMT_CRI WHERE ELEMENT_ID = task_id001;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 74+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(75) : cplan.sql(6605) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 81+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(82) : cplan.sql(6612) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || task_id001 || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 84+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(85) : cplan.sql(6615) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, task_id001, True);
      End If;
      h_cri := 1;
      h_table_type := PMO_CNST.SC_TABLE_CRI;
   Else
      h_cri := 0;
      h_table_type := PMO_CNST.SC_TABLE_WBS;
   End If;
   
   h_res_id := res_id001;
   
   -- if it is the profile (res_id) does not exists it will be created on the fly using res_id as role_id
   BEGIN
      SELECT PROFILE_ID, PROFILE_NAME, REC_STATUS INTO h_res_id, h_full_name, h_rec_status
      FROM PROFILES WHERE PROFILE_ID = h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 107+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(108) : cplan.sql(6638) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      h_and_roles := role_id001;
      
      h_role_id := SubStr(h_and_roles, 1, 32);
      
      BEGIN
         SELECT COALESCE(MAX(PROF_NUMBER),0)+1 INTO h_number
         FROM PROFILES
         WHERE PROJECT_ID=h_project_id AND ROLE_ID=h_role_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 127+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(128) : cplan.sql(6658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(TCP.ELEMENT_NAME), ', '), RTRIM(TC.ELEMENT_NAME)), '('), RTRIM(TO_CHAR(h_number))), ')') INTO h_full_name
         FROM TMT_COMPETENCIES TC, TMT_COMPETENCIES TCP
         WHERE TC.ELEMENT_ID = h_role_id AND TCP.ELEMENT_ID = TC.PARENT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 142+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(143) : cplan.sql(6673) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 149+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(150) : cplan.sql(6680) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 152+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(153) : cplan.sql(6683) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
      End If;
      
      BEGIN
         INSERT INTO PROFILES(PROFILE_ID,PROJECT_ID,ROLE_ID,PROF_NUMBER, PROFILE_NAME)
         VALUES(h_res_id, h_project_id, h_role_id, h_number, h_full_name);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 170+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(171) : cplan.sql(6701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 177+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(178) : cplan.sql(6708) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || res_id001 || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 180+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(181) : cplan.sql(6711) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, res_id001, True);
      End If;
   ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If isNotProjectType(Coalesce(h_type, 0)) <> 0 Then
         BEGIN
            SELECT a.RES_ASSIGNMENT_ID INTO h_uniqueID FROM RESOURCE_ASSIGNMENTS a
            WHERE a.ELEMENT_ID = task_id001 AND a.RESOURCE_TYPE = '1' AND a.PROFILE_ID = res_id001;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(196) : cplan.sql(6726) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 202+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(203) : cplan.sql(6733) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_ALREADY_ASSIGNED), 'NULL') || ' ID: ' || '''' || res_id001 || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 205+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(206) : cplan.sql(6736) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_ALREADY_ASSIGNED, res_id001, True);
         End If;
      End If;
      
      If SubStr(h_rec_status, 1, 1) = 'D' Then
         BEGIN
            UPDATE PROFILES SET REC_STATUS = 'U' WHERE PROFILE_ID = h_res_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 224+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(225) : cplan.sql(6755) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_TSKPROF', 231+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_TSKPROF(232) : cplan.sql(6762) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || res_id001 || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKPROF', 234+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKPROF(235) : cplan.sql(6765) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, res_id001, True);
   End If;
   
   h_bitflag := 0;
   BEGIN
      SELECT RES_ASSIGNMENT_ID INTO h_uniqueID FROM RESOURCE_ASSIGNMENTS
      WHERE ELEMENT_ID = task_id001 AND RESOURCE_TYPE = '1' AND PROFILE_ID = h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 249+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(250) : cplan.sql(6780) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1__* SC_STATE: %d, h_type: %d, h_cri: %d\n", SC_STATE, h_type, h_cri);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      If Coalesce(h_cri, 0) = 0 Then
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = task_id001;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 278+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(279) : cplan.sql(6809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_cri, 0) = 1 Then
         BEGIN
            UPDATE TMT_CRI SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE ASSIGN_NAMES || h_full_name || ';'
            END
            END
            WHERE ELEMENT_ID = task_id001;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 304+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(305) : cplan.sql(6835) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --Get a unique ID
      h_uniqueID := SYS_GUID();
      h_temp_sc_state := GetSecurityID(Coalesce(h_table_type, 0), Coalesce(h_type, 0)
                                 , PMO_CNST.SC_TEAMMEMBER_GROUP, h_temp_security_id);
      
      If isProjectType(Coalesce(h_type, 0)) <> 0 Then
         h_temp_participant := PMO_CNST.SC_AFFECTED;
      Else
         h_temp_participant := PMO_CNST.SC_TEAMMEMBER;
      End If;
      
      h_temp_res_type := '1';
      h_temp_bitflag := 1;
      
      h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_temp_res_type, h_temp_bitflag);
      
      BEGIN
         INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
         RESOURCE_TYPE, PROFILE_ID, PARTICIPANT, SECURITY_ID, REC_USER, REC_DATETIME,TABLE_TYPE) VALUES (h_uniqueID, h_project_id,
         task_id001, h_type, '1', h_res_id, h_temp_participant, h_temp_security_id, rec_user001, SYSTIMESTAMP,h_table_type);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 341+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(342) : cplan.sql(6872) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2__* h_temp_participant: %d, h_temp_security_id: %s\n", h_temp_participant,h_temp_security_id);
   ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
         WHERE RES_ASSIGNMENT_ID = h_uniqueID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 360+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(361) : cplan.sql(6891) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_uniqueID);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_TSKPROF', 369+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_TSKPROF(370) : cplan.sql(6900) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 376+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(377) : cplan.sql(6907) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKPROF', 388+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKPROF(389) : cplan.sql(6919) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_TSKPROF;
ALTER FUNCTION SP_A_TSKPROF COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_A_TSKRES ==
CREATE OR REPLACE FUNCTION SP_A_TSKRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_TSKRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_TSKRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_TSKRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKRES', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_TSKRES(26) : cplan.sql(6961) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_TSKRES', 30+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_TSKRES(31) : cplan.sql(6966) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_TSKRES', 33+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_TSKRES(34) : cplan.sql(6969) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   h_table_type := PMO_CNST.SC_TABLE_WBS;
   
   BEGIN
      SELECT TYPE_ID, PROJECT_ID INTO h_type_id, h_project_id
      FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKRES', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKRES(55) : cplan.sql(6990) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_TSKRES', 65+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_TSKRES(66) : cplan.sql(7001) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKRES', 72+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKRES(73) : cplan.sql(7008) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_TSKRES', 84+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_TSKRES(85) : cplan.sql(7020) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_TSKRES;
ALTER FUNCTION SP_A_TSKRES COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_A_CRIRES ==
CREATE OR REPLACE FUNCTION SP_A_CRIRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_CRIRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_CRIRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_CRIRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CRIRES', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_CRIRES(25) : cplan.sql(7061) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_CRIRES', 29+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_CRIRES(30) : cplan.sql(7066) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_CRIRES', 32+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_CRIRES(33) : cplan.sql(7069) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(52) : cplan.sql(7088) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("WBS_SC_STATE: %d, h_type_id: %d, h_element_id: %s\n", SC_STATE,h_type_id,h_element_id);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_CRI WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 66+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(67) : cplan.sql(7103) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("CRI_SC_STATE: %d, h_type_id: %d, h_element_id: %s\n", SC_STATE,h_type_id,h_element_id);
      h_table_type := PMO_CNST.SC_TABLE_CRI;
   Else
      h_table_type := PMO_CNST.SC_TABLE_WBS;
   End If;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_CRIRES', 82+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_CRIRES(83) : cplan.sql(7119) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 89+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(90) : cplan.sql(7126) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_CRIRES', 101+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_CRIRES(102) : cplan.sql(7138) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_CRIRES;
ALTER FUNCTION SP_A_CRIRES COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_A_REQRES ==
CREATE OR REPLACE FUNCTION SP_A_REQRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_REQRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_REQRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_REQRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_REQRES', 23+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_REQRES(24) : cplan.sql(7178) : ' || SYSTIMESTAMP);
   END IF; 

   If (id IS NULL) OR (res_id001 IS NULL) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_REQRES', 28+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_REQRES(29) : cplan.sql(7183) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_REQRES', 31+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_REQRES(32) : cplan.sql(7186) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_WBS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_REQRES', 50+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_REQRES(51) : cplan.sql(7205) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   -- commented by paria because we don't have tmt_requirement table anymore 11/18/2002
   
   h_ra_id := SYS_GUID();
   
   --  SC_STATE = AssignResource( SC_PARAM(h_type_id), SC_PARAM(h_element_id), SC_PARAM(h_resource_id), SC_PARAM(h_rec_user), SC_PARAM(h_ra_id));
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_REQRES', 62+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_REQRES(63) : cplan.sql(7217) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_REQRES', 69+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_REQRES(70) : cplan.sql(7224) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_REQRES', 81+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_REQRES(82) : cplan.sql(7236) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_REQRES;
ALTER FUNCTION SP_A_REQRES COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_A_POOLRES ==
CREATE OR REPLACE FUNCTION SP_A_POOLRES
(
     id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_POOLRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_POOLRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_POOLRES(' || 
       '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_POOLRES', 24+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_POOLRES(25) : cplan.sql(7277) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_POOLRES', 29+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_POOLRES(30) : cplan.sql(7282) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_POOLRES', 32+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_POOLRES(33) : cplan.sql(7285) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   h_table_type := PMO_CNST.SC_TABLE_POL;
   
   BEGIN
      SELECT TYPE_ID INTO h_type_id FROM TMT_POOLS WHERE ELEMENT_ID = h_element_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_POOLRES', 52+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_POOLRES(53) : cplan.sql(7305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_POOLRES', 63+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_POOLRES(64) : cplan.sql(7316) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_POOLRES', 70+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_POOLRES(71) : cplan.sql(7323) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_POOLRES', 82+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_POOLRES(83) : cplan.sql(7335) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_POOLRES;
ALTER FUNCTION SP_A_POOLRES COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_A_DOCRES ==
CREATE OR REPLACE FUNCTION SP_A_DOCRES
(
     table_type001 PLS_INTEGER
   , id VARCHAR2
   , res_id001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_id VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_DOCRES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_DOCRES'), '-- ' || SYSTIMESTAMP, ':r := SP_A_DOCRES(' || 
       COALESCE(TO_CHAR(table_type001), 'NULL')
          || ', ' || '''' || id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_DOCRES', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_DOCRES(27) : cplan.sql(7378) : ' || SYSTIMESTAMP);
   END IF; 

   If ((id IS NULL)) OR ((res_id001 IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_DOCRES', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_DOCRES(32) : cplan.sql(7383) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_DOCRES', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_DOCRES(35) : cplan.sql(7386) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_element_id := id;
   h_resource_id := res_id001;
   h_rec_user := rec_user001;
   h_table_type := table_type001;
   
   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 55+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(56) : cplan.sql(7407) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_SCD Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 68+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(69) : cplan.sql(7420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_POL_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_POOL_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(82) : cplan.sql(7433) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_RES_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_RESOURCE_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 94+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(95) : cplan.sql(7446) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CLN_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_CLIENT_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 107+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(108) : cplan.sql(7459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_AST_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_ASSET_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 120+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(121) : cplan.sql(7472) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_VDR_DOC Then
      BEGIN
         SELECT TYPE_ID INTO h_type_id FROM TMT_VENDOR_DOCUMENTS WHERE ELEMENT_ID = h_element_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 133+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(134) : cplan.sql(7485) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_ra_id := SYS_GUID();
   
   PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_element_id, h_resource_id, h_rec_user
                                    , h_ra_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_DOCRES', 145+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_DOCRES(146) : cplan.sql(7497) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_ra_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 152+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(153) : cplan.sql(7504) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_DOCRES', 164+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_DOCRES(165) : cplan.sql(7516) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_DOCRES;
ALTER FUNCTION SP_A_DOCRES COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION FC_U_CREATETMPTABLE ==
CREATE OR REPLACE FUNCTION FC_U_CREATETMPTABLE RETURN PLS_INTEGER
AS
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_CREATETMPTABLE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_CREATETMPTABLE'), '-- ' || SYSTIMESTAMP, ':r := FC_U_CREATETMPTABLE();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_CREATETMPTABLE', 8+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_CREATETMPTABLE(9) : cplan.sql(7541) : ' || SYSTIMESTAMP);
   END IF; 

   --define temporary table for distinct selection of records in case the select list contains CLOB type of data
   -- Temporary Table Declaration has been changed to Truncate
   EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CPLN01_TEMP';
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_CREATETMPTABLE', 17+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_CREATETMPTABLE(18) : cplan.sql(7550) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_CREATETMPTABLE;
ALTER FUNCTION FC_U_CREATETMPTABLE COMPILE DEBUG;

/************************************************************************************************/

/*****************************************************************************************/

Prompt == FUNCTION FC_U_TSKRES2 ==
CREATE OR REPLACE FUNCTION FC_U_TSKRES2
(
     h_assign_id IN OUT NOCOPY VARCHAR2
   , h_and_roles IN OUT NOCOPY VARCHAR2
   , h_and_skills IN OUT NOCOPY VARCHAR2
   , sh_h_work_location PLS_INTEGER
   , h_start_date IN OUT NOCOPY VARCHAR2
   , h_finish_date IN OUT NOCOPY VARCHAR2
   , sh_h_effort PLS_INTEGER
   , sh_h_work PLS_INTEGER
   , sh_h_eunit PLS_INTEGER
   , sh_h_unit PLS_INTEGER
   , sh_h_work_type PLS_INTEGER
   , sh_h_bitflag PLS_INTEGER
   , sh_h_contour PLS_INTEGER
   , sh_h_contour_type PLS_INTEGER
   , sh_h_occurrence PLS_INTEGER
   , h_assign_string IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_work_location PLS_INTEGER := sh_h_work_location;
   h_effort PLS_INTEGER := sh_h_effort;
   h_work PLS_INTEGER := sh_h_work;
   h_eunit PLS_INTEGER := sh_h_eunit;
   h_unit PLS_INTEGER := sh_h_unit;
   h_work_type PLS_INTEGER := sh_h_work_type;
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_contour PLS_INTEGER := sh_h_contour;
   h_contour_type PLS_INTEGER := sh_h_contour_type;
   h_occurrence PLS_INTEGER := sh_h_occurrence;
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_start_DT VARCHAR2(26);
   h_finish_DT VARCHAR2(26);
   h_dayofyear DATE;
   h_last_submitted DATE;
   h_tsk_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_calendar_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_bit_flag VARCHAR2(1);
   h_full_name VARCHAR2(255);
   h_min_per_day PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_rollup PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp2_security_id VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_TSKRES2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_TSKRES2'), '-- ' || SYSTIMESTAMP, ':r := FC_U_TSKRES2(' || 
       '''' || h_assign_id || ''''
          || ', ' || '''' || h_and_roles || ''''
          || ', ' || '''' || h_and_skills || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_work_location), 'NULL')
          || ', ' || '''' || h_start_date || ''''
          || ', ' || '''' || h_finish_date || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_effort), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_contour_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_occurrence), 'NULL')
          || ', ' || '''' || h_assign_string || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 88+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_TSKRES2(89) : cplan.sql(7651) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   --trace_printf("***** h_bitflag: %d, h_occurrence: %d\n",h_bitflag,h_occurrence);
   If Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_30 Then
      h_cri := PMO_FUNC.ShiftRight((BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_C0000000)), 30);
      -- consider CRI  WBS
      h_consider := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_1FFFFFFF);
      h_bitflag := 0;
      h_rollup := 0;
   ElsIf Coalesce(h_bitflag, 0) >= PMO_CNST.SC_BIT_28 Then
      h_rollup := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_28));
      -- set do not rollup (import from MSP)
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_28));
      -- remove set do not rollup
      h_consider := 0;
   Else
      h_consider := Coalesce(h_bitflag, 0);
      h_bitflag := 0;
      h_rollup := 0;
   End If;
   --trace_printf("h_consider: %d, h_bitflag: %d, h_work_type: %d, h_rollup: %d\n",h_consider,h_bitflag,h_work_type,h_rollup);
   
   h_duration := 0;
   h_res_type := '0';
   h_work_todate := 0;
   h_last_submitted := NULL;
   h_SD := NULL;
   h_FD := NULL;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      h_tsk_assignment_id := h_assign_id;
      BEGIN
         SELECT
         PROJECT_ID,
         TASK_ID,
         RESOURCE_ID,
         RESOURCE_TYPE,
         LAST_SUBMITTED,
         WORK_TODATE,
         TO_DATE(SubStr(SCHED_START, 1, 10)),
         TO_DATE(SubStr(SCHED_FINISH, 1, 10)),
         BILLABLE,
         CAPITAL_EXPENSE,
         REGULAR_COST,
         REGULAR_RATE
         INTO
         h_project_id,
         h_task_id,
         h_res_id,
         h_res_type,
         h_last_submitted,
         h_work_todate,
         h_SD,
         h_FD,
         h_billable,
         h_capital_expense,
         h_reg_cost,
         h_reg_sale
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 157+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(158) : cplan.sql(7720) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("2__* SC_STATE: %d, h_tsk_assignment_id: %d:%s, h_res_type: %s\n",SC_STATE, h_tsk_assignment_id_Ind,h_tsk_assignment_id,h_res_type);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 165+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(166) : cplan.sql(7728) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      End If;
   Else
      BEGIN
         SELECT PROJECT_ID, ELEMENT_ID, CASE WHEN RESOURCE_TYPE = '0' THEN RESOURCE_ID ELSE PROFILE_ID END,
         ELEMENT_TYPE, RESOURCE_TYPE, SECURITY_ID, CASE WHEN CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END
         INTO h_project_id, h_task_id, h_res_id, h_type,
         h_res_type, h_security_id, h_transfed_cri
         FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 182+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(183) : cplan.sql(7745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE = %d, h_assign_id = %d:%s, h_project_id = %s\n", SC_STATE,h_assign_id_Ind,h_assign_id,h_project_id);
      --trace_printf("h_type = %d, h_res_id = %s, h_res_type: %s, h_security_id = %s\n", h_type,h_res_id,h_res_type,h_security_id);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 191+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(192) : cplan.sql(7754) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.RECORD_NOT_FOUND;
      ElsIf isWBSTransferedDeliv(Coalesce(h_type, 0)) <> 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 196+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(197) : cplan.sql(7759) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      End If;
      
      BEGIN
         UPDATE RESOURCE_ASSIGNMENTS SET WORK_LOCATION = h_work_location
         WHERE RES_ASSIGNMENT_ID = h_assign_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 214+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(215) : cplan.sql(7777) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If NOT (isProjectType(Coalesce(h_type, 0))) <> 0 Then
         BEGIN
            SELECT TSK_ASSIGNMENT_ID INTO h_tsk_assignment_id
            FROM TASK_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 229+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(230) : cplan.sql(7792) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("2__* SC_STATE: %d,h_assign_id: %d:%s\n",SC_STATE, h_assign_id_Ind,h_assign_id);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 237+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(238) : cplan.sql(7800) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMO_CNST.RECORD_ALREADY_ASSIGNED;
         End If;
      End If;
      
      h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_assign_id);
   End If;
   --trace_printf("1*** h_bitflag: %d, h_occurrence: %d, h_work_todate: %d, h_last_submitted: %d:%s\n",h_bitflag,h_occurrence,h_work_todate,h_last_submitted_Ind,h_last_submitted);
   --trace_printf("h_res_type: %s, h_SD: %d:%s, h_FD: %d:%s\n",h_res_type,h_SD_Ind,h_SD,h_FD_Ind,h_FD);
   
   /********************************************************************************************************************************/
   h_temp_sc_state := EXTEND_PRCAL(h_res_id, h_start_date, h_finish_date, h_duration
                                , PMO_CNST.SC_RESOURCE);
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY, 480), COALESCE(MINUTE_PER_WEEK, 2400), CALENDAR_ID
      INTO h_min_per_day, h_per_week, h_calendar_id
      FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 262+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(263) : cplan.sql(7825) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_rollup, 0) = 0 Then
      If (Coalesce(h_work, 0) > 0) AND (Coalesce(h_unit, 0) = 5) Then
         h_work := Coalesce(h_min_per_day, 0) * Coalesce(h_work, 0);
      End If;
      
      If Coalesce(h_effort, 0) = 0 Then
         --if ( IsNullSpParam(ind_u_assign, finish_date))
         If (h_finish_date IS NULL) Then
            If Coalesce(h_work, 0) > 0 Then
               h_effort := Coalesce(h_occurrence, 0) * Coalesce(h_work, 0);
            End If;
         Else
            BEGIN
               SELECT SUM(AVAILABLE_MIN) INTO h_effort FROM CALENDAR_DAY_OF_YEAR
               WHERE ELEMENT_ID=h_project_id AND DAY_YEAR BETWEEN TO_DATE(SubStr(h_start_date, 1, 10)) AND TO_DATE(SubStr(h_finish_date, 1, 10));
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 288+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(289) : cplan.sql(7851) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         If Coalesce(h_eunit, 0) = 1 Then
            h_effort := Coalesce(h_effort, 0) * 60;
         ElsIf Coalesce(h_eunit, 0) = 2 Then
            h_effort := Coalesce(h_min_per_day, 0) * Coalesce(h_effort, 0);
         ElsIf Coalesce(h_eunit, 0) = 3 Then
            h_effort := Coalesce(h_per_week, 0) * Coalesce(h_effort, 0);
         End If;
      End If;
      
      If Coalesce(h_effort, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 305+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(306) : cplan.sql(7868) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMO_CNST.TASK_ASSIGNMENT_ERROR;
      End If;
   End If;
   
   --trace_printf("h_bitflag: %d, h_rollup: %d, h_work_type: %d, h_res_type: %s\n",h_bitflag,h_rollup,h_work_type,h_res_type);
   If Coalesce(h_bitflag, 0) = 2 Then
      h_bitflag := (BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_NOT_30));
      -- remove assign from constrained by project (type > 0)
   ElsIf (SubStr(h_res_type, 1, 1) = '0') AND (Coalesce(h_work_type, 0) = 0) Then
      BEGIN
         SELECT TSK_ASSIGNMENT_ID, SCHED_START, SCHED_FINISH, BIT_FLAG
         INTO h_tsk_assignment_id, h_bit_flag, h_start_DT, h_finish_DT
         FROM TASK_ASSIGNMENTS
         WHERE TASK_ID=h_project_id AND RESOURCE_ID=h_res_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_TSKRES2', 327+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_TSKRES2(328) : cplan.sql(7890) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("PMO__SC_STATE: %d, h_tsk_assignment_id: %d:%s\n", SC_STATE,h_tsk_assignment_id_Ind,h_tsk_assignment_id);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_tsk_assignment_id IS NOT NULL)) Then
         If (PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) >= 0) AND (PMO_ORASTD.strncmp(h_finish_DT
               , h_finish_date, 10) >= 0) Then
            If SubStr(h_bit_flag, 1, 1) = '2' Then
               If Coalesce(h_rollup, 0) = 0 Then
                  h_bitflag := 2;
               End If;
               h_work_type := 2;
            End If;
         ElsIf Coalesce(h_bitflag, 0) = 1 Then
            h_consider := (PMO_FUNC.BIT_OR(Coalesce(h_consider, 0), PMO_CNST.SC_BIT_30));
         End If;
      End If;
   End If;
   
   --trace_printf("h_assign_id: %s, h_consider: %d, h_rollup: %d, h_bitflag: %d, h_assign_string: %s\n",h_assign_id,h_consider,h_rollup,h_bitflag,h_assign_string);
   --trace_printf("*** SC_STATE = %d, h_project_id = %s, h_occurrence: %d, h_work_todate: %d\n",SC_STATE, h_project_id,h_occurrence,h_work_todate);
   --trace_printf("h_start_date = %s, h_finish_date = %s, h_effort = %d, h_work_todate = %d\n", h_start_date, h_finish_date, h_effort,h_work_todate);
   
   If (Coalesce(h_bitflag, 0) = 0) OR (Coalesce(h_occurrence, 0) = Coalesce(h_work_todate, 0)) Then
      If Coalesce(h_rollup, 0) <> 0 Then
         h_bitflag := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), Coalesce(h_rollup, 0)));
         -- set do not rollup
      End If;
      
      PMO_DB2M.SC_STATE := U_TSKRES2(h_assign_id, h_project_id, h_task_id, h_res_id, h_and_roles
                               , h_and_skills, h_start_date, h_finish_date, h_effort, h_work
                               , h_eunit, h_unit, h_res_type, h_work_type, h_bitflag, h_contour
                               , h_contour_type, h_occurrence, h_consider, h_assign_string
                               , h_rec_user, h_min_per_day, 0.00, 0.00);
   ElsIf Coalesce(h_bitflag, 0) = 1 Then
      If Coalesce(h_rollup, 0) <> 0 Then
         h_bitflag := (PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), Coalesce(h_rollup, 0)));
         -- set do not rollup
      End If;
      /*
      		if ( (IsNotNullParam(h_SD))  (IsNotNullParam(h_FD)))
      		{
      			InitializeHost(SC_PARAM(h_tph_delete), -1);
      			h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
      						SC_PARAM(h_project_id),
      						SC_PARAM(h_task_id),
      						SC_PARAM(h_tsk_assignment_id),
      						SC_PARAM(h_SD),
      						SC_PARAM(h_FD),
      						SC_PARAM(h_reg_cost),
      						SC_PARAM(h_reg_sale),
      						SC_PARAM(h_tph_delete),
      						SC_PARAM(h_capital_expense),
      						SC_PARAM(h_billable));
      		}
      
      		h_temp_sc_state = FC_DEL_NOACTUAL(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id));
      
      		// Tudor * fix for -803
      		Nullify(SC_PARAM(h_dayofyear));
      		BEGIN_SQL
      			SELECT MAX(DAY_YEAR) INTO HOST(h_dayofyear) FROM TASK_DAY_OF_YEAR
      			WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND ACTUAL_FLAG = 1;
      		END_SQL_NOCHECK;
      //trace_printf("***: h_last_submitted = %s, h_dayofyear: %d:%s\n",h_last_submitted,h_dayofyear_Ind,h_dayofyear);
      
      		if ( (IsNotNullParam(h_dayofyear))  (strncmp(h_last_submitted, h_dayofyear, 10) != 0) )
      		{
      			BEGIN_SQL
      				UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = HOST(h_dayofyear)
      				WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id);
      			END_SQL;
      //trace_printf("***: SC_STATE = %d, h_dayofyear: %d:%s\n",SC_STATE,h_dayofyear_Ind,h_dayofyear);
      
      			BEGIN_SQL
      				SELECT TIMESTAMP(DATE(HOST(h_dayofyear)) + 1 DAYS, TIME('00.00.00')) INTO HOST(h_start_date) FROM DUAL;
      			END_SQL;
      		}
      		else
      		{
      			BEGIN_SQL
      				SELECT TIMESTAMP(DATE(HOST(h_last_submitted)) + 1 DAYS, TIME('00.00.00')) INTO HOST(h_start_date) FROM DUAL;
      			END_SQL;
      		}
      //trace_printf("FC_RESCHEDULE: h_start_date = %s, h_effort: %d\n",h_start_date,h_effort);
      
      		InitializeHost(SC_PARAM(h_effort), h_effort - h_work_todate);
      		InitializeHost(SC_PARAM(h_temp_var), 2);
      		SC_STATE = FC_RESCHEDULE(SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_start_date), SC_PARAM(h_effort), SC_PARAM(h_temp_var), SC_PARAM(h_contour), SC_PARAM(h_res_id));
      */
      PMO_DB2M.SC_STATE := U_TSKRES2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_and_roles
                               , h_and_skills, h_start_date, h_finish_date, h_effort, h_work
                               , h_eunit, h_unit, h_res_type, h_work_type, h_bitflag, h_contour
                               , h_contour_type, h_occurrence, h_consider, h_assign_string
                               , h_rec_user, h_min_per_day, 0.00, 0.00);
   End If;
   --trace_printf("U_TSKRES2: SC_STATE = %d, h_assign_id = %s\n",SC_STATE,h_assign_id);
   
   If PMO_DB2M.SC_STATE = 110 Then
      PMO_DB2M.SC_STATE := PMO_CNST.RESOURCE_UNAVAILABLE;
   ElsIf PMO_DB2M.SC_STATE <> 0 Then
      PMO_DB2M.SC_STATE := PMO_CNST.TASK_ASSIGNMENT_ERROR;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_TSKRES2', 434+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_TSKRES2(435) : cplan.sql(7997) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_TSKRES2;
ALTER FUNCTION FC_U_TSKRES2 COMPILE DEBUG;

/**********************************************************************************************************************************************************
* update resource/profile assignment, used tables: RESOURCE_ASSIGNMENTS, TASK_ASSIGNMENTS, TASK_CAL_OF_YEAR
* updates CALENDAR_DAY_OF_YEAR if scheduled effort
*
* id             = PROJECT_ID, required
* role_id        = COMPETENCY
* and_skills     = list of proficiency level Ids ID (under SKILL), no quotes, no comma separator between the IDs
*
* eunit: 0 = MINUTE (default), 1= h,    2 = d, 3 = w,       applies to effort
* unit : 0 = h/d, 3= h/w, 4=h/m, 1= d/w, 2= d/m, 5= w/m,    applies to work
* if effort = 0 then both start  finish date are required (it has to be a valid date-time value)
*
* work_type = 0 - PLANNED, 2 - SCHEDULE
*
* bitflag = set responsable ( MANAGER = 1, RESPONSIBLE = 4)
* if bitflag    = 0 then insert new assignment (assign_id = RES_ASSIGNMENT_ID)
*               = 1 then update an existing assignment (assign_id = TSK_ASSIGNMENT_ID)
*               = 2 then insert new assignment using project assignment (assign_id = RES_ASSIGNMENT_ID)
*               = 3 then reject a profile replacement, uncommit everything
*               = 9 MSP
*************************************************************************************************************************************************************/

Prompt == FUNCTION SP_U_TSKRES2 ==
CREATE OR REPLACE FUNCTION SP_U_TSKRES2
(
     assign_id VARCHAR2
   , and_roles VARCHAR2
   , and_skills VARCHAR2
   , work_location001 PLS_INTEGER
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , work_type PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , contour001 PLS_INTEGER
   , contour_type001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , assign_string001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_assign_id VARCHAR2(32);
   h_and_roles VARCHAR2(8000);
   h_and_skills VARCHAR2(8000);
   h_work_location PLS_INTEGER := 0;
   h_start_date VARCHAR2(26);
   h_finish_date VARCHAR2(26);
   h_effort PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_rec_user VARCHAR2(32);
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_TSKRES2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_TSKRES2'), '-- ' || SYSTIMESTAMP, ':r := SP_U_TSKRES2(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || and_roles || ''''
          || ', ' || '''' || and_skills || ''''
          || ', ' || COALESCE(TO_CHAR(work_location001), 'NULL')
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour_type001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || '''' || assign_string001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TSKRES2', 64+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_TSKRES2(65) : cplan.sql(8092) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If ((assign_id IS NULL)) OR ((start_date001 IS NULL)) OR ((finish_date001 IS NULL)) 
         OR ((assign_string001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_TSKRES2', 72+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_TSKRES2(73) : cplan.sql(8100) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TSKRES2', 75+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_TSKRES2(76) : cplan.sql(8103) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   h_assign_id := assign_id;
   h_and_roles := and_roles;
   h_and_skills := and_skills;
   h_start_date := start_date001;
   h_finish_date := finish_date001;
   h_assign_string := assign_string001;
   h_rec_user := rec_user001;
   
   h_work_location := work_location001;
   h_effort := effort001;
   h_work := work001;
   h_eunit := eunit;
   h_unit := unit;
   h_work_type := work_type;
   h_bitflag := bitflag001;
   h_contour := contour001;
   h_contour_type := contour_type001;
   h_occurrence := occurrence;
   
   PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_assign_id, h_and_roles, h_and_skills, h_work_location
                                  , h_start_date, h_finish_date, h_effort, h_work, h_eunit, h_unit
                                  , h_work_type, h_bitflag, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_TSKRES2', 105+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_TSKRES2(106) : cplan.sql(8133) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_assign_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_assign_id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TSKRES2', 113+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TSKRES2(114) : cplan.sql(8141) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TSKRES2', 125+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TSKRES2(126) : cplan.sql(8153) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_TSKRES2;
ALTER FUNCTION SP_U_TSKRES2 COMPILE DEBUG;

/***************************************************************************************************/

Prompt == FUNCTION SP_ASSIGN_MSP ==
CREATE OR REPLACE FUNCTION SP_ASSIGN_MSP
(
     id VARCHAR2
   , action_type PLS_INTEGER
   , task_id001 VARCHAR2
   , start_dt001 VARCHAR2
   , finish_dt001 VARCHAR2
   , effort001 PLS_INTEGER
   , assign_string001 VARCHAR2
   , work_todate001 PLS_INTEGER
   , percent_complete001 PLS_INTEGER
   , eetc PLS_INTEGER
   , data_flag PLS_INTEGER
   , units PLS_INTEGER
   , contour_type001 PLS_INTEGER
   , baseline_start001 VARCHAR2
   , baseline_finish001 VARCHAR2
   , baseline_duration001 PLS_INTEGER
   , baseline_work001 PLS_INTEGER
   , actual_start001 VARCHAR2
   , actual_finish001 VARCHAR2
   , actual_effort001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_element_name VARCHAR2(255);
   h_full_name VARCHAR2(255);
   h_message VARCHAR2(16384);
   h_subject VARCHAR2(2048);
   h_revision VARCHAR2(16);
   h_temp_id VARCHAR2(32);
   h_temp_id2 VARCHAR2(32);
   h_wkf_def_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_ra_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_work_location PLS_INTEGER := 0;
   h_zero PLS_INTEGER := 0;
   h_two PLS_INTEGER := 2;
   h_work PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_contour_type PLS_INTEGER := 0;
   h_units PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 480;
   h_impex_rollup PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_data_flag PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_finish_D DATE;
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_null_str VARCHAR2(1);
   h_start_date TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_actual_start TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_rec_DT TIMESTAMP;
   h_lock_DT TIMESTAMP;
   h_assign_string VARCHAR2(8192);
   h_table_type PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_bit_minutes PLS_INTEGER := 0;
   h_action_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_H_WBS IS
         SELECT * FROM (SELECT ELEMENT_NAME,RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
         FROM TMT_WBS
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id
         ORDER BY REC_DATETIME DESC ) WHERE ROWNUM<=1;

      CURSOR CAF IS
         SELECT SCHED_MIN, DAY_YEAR, ACTUAL_FLAG
         FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id
         FOR UPDATE OF ACTUAL_MIN,ACTUAL_FLAG;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ASSIGN_MSP') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ASSIGN_MSP'), '-- ' || SYSTIMESTAMP, ':r := SP_ASSIGN_MSP(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(action_type), 'NULL')
          || ', ' || '''' || task_id001 || ''''
          || ', ' || '''' || start_dt001 || ''''
          || ', ' || '''' || finish_dt001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || '''' || assign_string001 || ''''
          || ', ' || COALESCE(TO_CHAR(work_todate001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(percent_complete001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eetc), 'NULL')
          || ', ' || COALESCE(TO_CHAR(data_flag), 'NULL')
          || ', ' || COALESCE(TO_CHAR(units), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour_type001), 'NULL')
          || ', ' || '''' || baseline_start001 || ''''
          || ', ' || '''' || baseline_finish001 || ''''
          || ', ' || COALESCE(TO_CHAR(baseline_duration001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(baseline_work001), 'NULL')
          || ', ' || '''' || actual_start001 || ''''
          || ', ' || '''' || actual_finish001 || ''''
          || ', ' || COALESCE(TO_CHAR(actual_effort001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 123+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ASSIGN_MSP(124) : cplan.sql(8293) : ' || SYSTIMESTAMP);
   END IF; 

   If ((action_type IS NULL)) OR ((Coalesce(action_type, 0) < 1) OR (Coalesce(
         action_type, 0) > 3)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 129+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(130) : cplan.sql(8299) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 132+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(133) : cplan.sql(8302) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If ((id IS NULL)) OR ((data_flag IS NULL)) OR ((rec_user001 IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 139+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(140) : cplan.sql(8309) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 142+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(143) : cplan.sql(8312) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Local host variable
   
   h_actual_start := NULL;
   h_rec_DT := NULL;
   h_work_location := 265;
   h_start_DT := start_dt001;
   h_finish_DT := finish_dt001;
   h_assign_string := assign_string001;
   h_rec_user := rec_user001;
   h_effort := effort001;
   h_data_flag := data_flag;
   h_action_type := action_type;
   h_units := units;
   h_actual_start := actual_start001;
   h_work_todate := work_todate001;
   h_occurrence := 0;
   --trace_printf("h_effort: %d, h_work_todate = %d, h_action_type: %d, data_flag = %d, h_actual_start: %d:%s\n",h_effort,h_work_todate,h_action_type,h_data_flag,h_actual_start_Ind,h_actual_start);
   --trace_printf("h_start_DT: %d:%s, h_finish_DT: %d:%s, h_assign_string: %s\n",h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_assign_string);
   
   If Coalesce(Length(h_assign_string), 0) < PMO_CNST.SC_ASSIGN_STRING_LENGTH Then
      If (Coalesce(h_action_type, 0) = 1) OR (Coalesce(h_action_type, 0) = 3) Then
         -- insert assignment
         If (task_id001 IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 171+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(172) : cplan.sql(8341) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 174+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(175) : cplan.sql(8344) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         h_resource_id := id;
         h_task_id := task_id001;
         h_rec_user := rec_user001;
         
         BEGIN
            SELECT PROJECT_ID, TYPE_ID INTO h_project_id, h_type_id
            FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 192+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(193) : cplan.sql(8362) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_ra_id := SYS_GUID();
         h_table_type := GetTable(Coalesce(h_type_id, 0));
         
         PMO_DB2M.SC_STATE := AssignResource(h_table_type, h_type_id, h_task_id, h_resource_id, h_rec_user
                                    , h_ra_id);
         --trace_printf("1_3***** SC_STATE = %d,h_ra_id = %d;%s,h_resource_id: %s\n",SC_STATE, h_ra_id_Ind, h_ra_id,h_resource_id);
         
         If (PMO_DB2M.SC_STATE = 0) AND (Coalesce(h_action_type, 0) = 3) AND (Coalesce(h_effort, 0) = 0) Then
            h_temp_var := 0;
            BEGIN
               SELECT COUNT('X') INTO h_temp_var FROM DUAL
               WHERE EXISTS (SELECT 'X' FROM PROFILES
               WHERE PROJECT_ID = h_project_id AND PROFILE_ID = h_resource_id);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 216+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(217) : cplan.sql(8386) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) <> 0 Then
               h_action_type := 1;
               h_effort := 5;
               h_assign_string := '1*5';
               
               If (h_start_DT IS NOT NULL) Then
                  BEGIN
                     SELECT TO_TIMESTAMP(PMO_ORASTD.DB2TIMESTAMP(h_start_DT)) + (5 / 1440.0) INTO h_finish_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 235+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(236) : cplan.sql(8405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("1_* SC_STATE = %d, h_finish_DT = %d;%s\n",SC_STATE, h_finish_DT_Ind, h_finish_DT);
               End If;
            End If;
         End If;
         
         If (Coalesce(h_action_type, 0) = 1) AND (isWBSDeliv(Coalesce(h_type_id, 0))) <> 0 Then
            h_action_type := 3;
         End If;
      Else
         h_ra_id := id;
         
         BEGIN
            SELECT PROJECT_ID, TASK_ID, RESOURCE_ID, WORK_TODATE, PROJECT_ASSIGNMENT_ID,
            UNITS_MSP, C_EFFORT, C_CONTOUR, CONTOUR_TYPE, REC_DATETIME
            INTO h_project_id, h_task_id, h_resource_id, h_work_todate, h_prj_assignment_id,
            h_percent_units, h_work, h_contour, h_contour_type, h_rec_DT
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_ra_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 262+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(263) : cplan.sql(8432) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("***** SC_STATE = %d, h_work = %d, h_contour_type: %d, h_percent_units: %d, h_prj_assignment_id: %d %s\n",SC_STATE, h_work,h_contour_type,h_percent_units,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(work_todate001, 0) <> Coalesce(
               h_work_todate, 0)) Then
            h_actual_start := NULL;
            h_count := -1;
         End If;
      End If;
      
      --trace_printf("h_action_type: %d, h_start_DT: %d:%s, h_finish_DT: %d:%s, h_assign_string: %s\n",h_action_type,h_start_DT_Ind,h_start_DT,h_finish_DT_Ind,h_finish_DT,h_assign_string);
      If Coalesce(h_action_type, 0) <> 3 Then
         If PMO_DB2M.SC_STATE <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ASSIGN_MSP', 279+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ASSIGN_MSP(280) : cplan.sql(8449) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 282+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(283) : cplan.sql(8452) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, h_ra_id, True);
         End If;
         --trace_printf("***** h_ra_id = %s, h_effort: %d, h_assign_string: %s\n",h_ra_id,h_effort,h_assign_string);
         
         If (h_start_DT IS NULL) OR (h_finish_DT IS NULL) OR (h_effort IS NULL) 
               OR (h_assign_string IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_ASSIGN_MSP', 291+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_ASSIGN_MSP(292) : cplan.sql(8461) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 294+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(295) : cplan.sql(8464) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         h_impex_rollup := PMO_CNST.SC_BIT_28;
         --28TH BIT ON -- DON'T ROLLUP
         
         If Coalesce(h_units, 0) > 103079040 Then
            h_units := 103079040;
         End If;
         
         --if ( (h_percent_units == 0) || (h_percent_units == 10000) )
         --      {
         
         BEGIN
            SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_per_day FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 317+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(318) : cplan.sql(8487) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_work, 0) > 0 Then
            --              h_temp_var = int((h_units * h_per_day)/500.0 + 0.5) * 5;    // round up to 5 minutes
            h_temp_var := Trunc(((Coalesce(h_work, 0) / Coalesce(h_per_day, 0)) * 10000) + 0.4);
            --trace_printf("MSP___* h_temp_var: %d, h_units: %d\n",h_temp_var,h_units);
            If Coalesce(h_units, 0) <> Coalesce(h_temp_var, 0) Then
               h_work := 0;
               h_contour := 0;
            Else
               h_units := 0;
            End If;
         End If;
         --      }
         /*
         		else if ( h_percent_units != h_units )
         		{
         			h_work = 0;
         			h_contour = 0;
         			h_contour_type = 0;
         		}*/
         --trace_printf("MSP___* h_temp_var: %d, h_units: %d, h_work: %d, h_contour_type: %d, h_contour: %d, h_effort: %d\n",h_temp_var,h_units,h_work,h_contour_type,h_contour,h_effort);
         
         /*********************************************************************************
         	data_flag ___OLD
         	*********************************************************************************
         	bit 0:  0 = plane,      1 = schedule
         	bit 1:  0 = nothing,    1 = update baseline
         	bit 2:  0 = nothing,    1 = update actual
         	bit 3:  0 = task,       1 = non task (ST, Deliverable, Project, ...)
         	*********************************************************************************
         	data_flag ___NEW
         	*********************************************************************************
         	bit 1(1):   0 - plan
         				1 - sched
         	bit 4(16):  0 = reimport,   ( no baseline, no actual: if there is actual ignore it )
         				1 = cold import ( check for baseline and actual, update actual)
         	**********************************************************************************/
         
         h_null_str := NULL;
         
         If (BITAND(Coalesce(data_flag, 0), 16)) = 16 Then
            If Coalesce(h_action_type, 0) = 1 Then
               If Coalesce(work_todate001, 0) > 0 Then
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_two
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("1___________SC_STATE = %d\n",SC_STATE);
                  
                  If PMO_DB2M.SC_STATE = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                        WHERE TASK_ID = h_task_id AND RESOURCE_ID = h_resource_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 385+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(386) : cplan.sql(8555) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_zero
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("2___________SC_STATE = %d\n",SC_STATE);
               End If;
            Else
               h_impex_rollup := Coalesce(h_impex_rollup, 0) + 1;
               If Coalesce(work_todate001, 0) > 0 Then
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_two
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("3___________SC_STATE = %d\n",SC_STATE);
                  
                  If PMO_DB2M.SC_STATE = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 421+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(422) : cplan.sql(8591) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_zero
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
                  --trace_printf("4___________SC_STATE = %d\n",SC_STATE);
               End If;
            End If;
            
            If (PMO_DB2M.SC_STATE = 0) AND ((BITAND(Coalesce(data_flag, 0), 2)) = 2) Then
               If (Coalesce(work_todate001, 0) = Coalesce(effort001, 0)) AND (Coalesce(eetc, 0) = 0) AND (Coalesce(
                                           percent_complete001, 0) = 10000) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                     UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                     ACTUAL_START = h_start_DT, ACTUAL_FINISH = h_finish_DT,
                     WORK_TODATE = work_todate001,
                     ETC = eetc
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 456+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(457) : cplan.sql(8626) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = SCHED_MIN, ACTUAL_FLAG = 1
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 474+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(475) : cplan.sql(8644) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = h_finish_DT
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 492+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(493) : cplan.sql(8662) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 511+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(512) : cplan.sql(8681) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         Else
            h_data_flag := BITAND(Coalesce(h_data_flag, 0), PMO_CNST.SC_NOT_32);
            --      h_impex_rollup = 536870912; //29TH BIT  -- ROLLUP
            If Coalesce(h_action_type, 0) = 1 Then
               PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_data_flag
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
               --trace_printf("5___________SC_STATE = %d, h_impex_rollup: %d\n",SC_STATE,h_impex_rollup);
               
               If (PMO_DB2M.SC_STATE = 0) AND (Coalesce(data_flag, 0) = 2) AND (Coalesce(h_count, 0) = 0) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                     WHERE TASK_ID = h_task_id AND RESOURCE_ID = h_resource_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 542+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(543) : cplan.sql(8712) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               If (baseline_start001 IS NOT NULL) Then
                  h_lock_DT := baseline_start001;
                  /*BEGIN_SQL
                  						SELECT PMO_TOTIMESTAMP(HOST(h_lock_DT)) INTO HOST(h_lock_DT) FROM DUAL;
                  					END_SQL;*/
                  
                  BEGIN
                     SELECT SUBSTR(h_lock_DT,1,10) || '-' || SUBSTR(h_lock_DT,12,2) || '.' || SUBSTR(h_lock_DT,15,2) || '.' || SUBSTR(h_lock_DT,18,9) INTO h_lock_DT FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 562+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(563) : cplan.sql(8732) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_ORASTD.strncmp(h_lock_DT, h_rec_DT, 19) < 0 Then
                     h_type_id := PMO_CNST.SC_TASK;
                     h_table_type := 1;
                     
                     BEGIN
                        SELECT ELEMENT_ID INTO h_wkf_def_id
                        FROM TMT_WORKFLOW_CYCLES
                        WHERE PARENT_ID = 'NOTIFICATION';
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 581+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(582) : cplan.sql(8751) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     h_temp_sc_state := FC_A_NOTIFICATION(h_table_type, h_task_id, h_wkf_def_id, h_type_id, h_rec_user
                                     , h_uniqueID);
                     
                     If Coalesce(h_temp_sc_state, 0) = 0 Then
                        h_temp_id := NULL;
                        h_temp_id2 := NULL;
                        h_element_name := NULL;
                        h_revision := NULL;
                        h_full_name := NULL;
                        
                        OPEN C_H_WBS;
                        
                        FETCH C_H_WBS INTO h_element_name, h_revision;
                        IF C_H_WBS%FOUND THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                        
                        BEGIN
                           CLOSE C_H_WBS ;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 613+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(614) : cplan.sql(8783) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        BEGIN
                           SELECT FULL_NAME INTO h_full_name FROM RESOURCES
                           WHERE RESOURCE_ID = h_resource_id;
                           PMO_DB2M.SC_STATE := SQLCODE;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 627+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(628) : cplan.sql(8797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        --strcpy(h_subject,"Import of assignment failed.");
                        h_subject := 'Import of assignment failed.';
                        --strcpy(h_message,"Import cannot modify assignment for resource: ");
                        h_message := 'Import cannot modify assignment for resource: ';
                        h_message := h_message || h_full_name;
                        h_message := h_message || ' on the task:';
                        h_message := h_message || h_element_name;
                        h_message := h_message || 
                              '. Assignment was modified in Timesheets during the time of export of the project.';
                        
                        h_temp_sc_state := FC_U_WORKFLOW(h_uniqueID, h_revision, h_zero, h_zero, h_null_str, h_zero
                                 , h_subject, h_message, h_rec_user, h_null_str, h_null_str, h_zero
                                 , h_temp_id, h_temp_id2, h_rec_user);
                        
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ASSIGN_MSP', 647+1) THEN 
                           LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ASSIGN_MSP(648) : cplan.sql(8817) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
                        END IF; 
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 650+1) THEN 
                           LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(651) : cplan.sql(8820) : ' || SYSTIMESTAMP);
                        END IF; 
                        PMODEFINES.PMO_RETURN_ERROR_SP(0, h_ra_id, True);
                     End If;
                  End If;
               End If;
               
               h_impex_rollup := Coalesce(h_impex_rollup, 0) + 1;
               PMO_DB2M.SC_STATE := FC_U_TSKRES2(h_ra_id, h_null_str, h_null_str, h_work_location, h_start_DT
                                  , h_finish_DT, h_effort, h_work, h_zero, h_zero, h_data_flag
                                  , h_impex_rollup, h_contour, h_contour_type, h_occurrence
                                  , h_assign_string, h_rec_user);
               --trace_printf("6___________SC_STATE = %d, h_ra_id: %s\n",SC_STATE,h_ra_id);
               
               --              if( (SC_STATE == 0)  (*data_flag == 2)  (h_count == 0)  IsNullParam(h_actual_start)  (*actual_effort == 0)  (*work_todate == 0) )
               If (PMO_DB2M.SC_STATE = 0) AND (Coalesce(data_flag, 0) = 2) AND (Coalesce(h_count, 0) = 0) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET ETC = eetc,
                     PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 680+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(681) : cplan.sql(8850) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
         End If;
         --trace_printf("SC_STATE = %d, h_count: %d, action_type = %d, data_flag = %d, h_actual_start: %d:%s\n",SC_STATE,h_count,h_action_type,*data_flag,h_actual_start_Ind,h_actual_start);
         
         If PMO_DB2M.SC_STATE <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ASSIGN_MSP', 691+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ASSIGN_MSP(692) : cplan.sql(8861) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_ra_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ASSIGN_MSP', 694+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_ASSIGN_MSP(695) : cplan.sql(8864) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, h_ra_id, True);
         End If;
         
         h_contour_type := contour_type001;
         --      if ( (h_units < 100)  (h_contour_type == 0) )
         --          h_contour_type = 1;
         
         If Coalesce(h_action_type, 0) = 1 Then
            If (BITAND(Coalesce(data_flag, 0), 16)) = 16 Then
               If (baseline_start001 IS NULL) Then
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 726+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(727) : cplan.sql(8896) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units,
                        CONTOUR_TYPE = h_contour_type
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 746+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(747) : cplan.sql(8916) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 789+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(790) : cplan.sql(8959) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     --trace_printf("ACTUAL: SC_STATE: %d, h_ra_id: %d:%s\n", SC_STATE, h_ra_id_Ind,h_ra_id);
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1
                        WHERE RES_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 826+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(827) : cplan.sql(8996) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf Coalesce(h_count, 0) = 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                  UNITS_MSP = h_units,
                  CONTOUR_TYPE = h_contour_type
                  WHERE RES_ASSIGNMENT_ID = h_ra_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 848+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(849) : cplan.sql(9018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         Else
            If (BITAND(Coalesce(data_flag, 0), 16)) = 16 Then
               If (baseline_start001 IS NULL) Then
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 876+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(877) : cplan.sql(9046) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 896+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(897) : cplan.sql(9066) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  --              if( ((*data_flag  1) == 1)  (IsNotNullSpParam(ind_assign_msp, actual_start)))  // update actual
                  If (h_actual_start IS NOT NULL) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1,
                        PROPOSED_START = SCHED_START,PROPOSED_FINISH = SCHED_FINISH,PROPOSED_WORK = SCHED_WORK,
                        ACTUAL_START = actual_start001,
                        ACTUAL_FINISH = actual_finish001,
                        WORK_TODATE = actual_effort001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 939+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(940) : cplan.sql(9109) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001,
                        BASELINE3_START = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_start001) ELSE BASELINE3_START END,
                        BASELINE3_FINISH = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN TO_TIMESTAMP(baseline_finish001) ELSE BASELINE3_FINISH END,
                        BASELINE3_DURATION = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_duration001 ELSE BASELINE3_DURATION END,
                        BASELINE3_WORK = CASE WHEN BASELINE3_SCH_FLAG = 0 THEN baseline_work001 ELSE BASELINE3_WORK END,
                        
                        BASELINE2_START = BASELINE1_START,
                        BASELINE2_FINISH = BASELINE1_FINISH,
                        BASELINE2_DURATION = BASELINE1_DURATION,
                        BASELINE2_WORK = BASELINE1_WORK,
                        
                        BASELINE1_START = baseline_start001,
                        BASELINE1_FINISH = baseline_finish001,
                        BASELINE1_DURATION = baseline_duration001,
                        BASELINE1_WORK = baseline_work001,
                        B_REGULAR_COST = REGULAR_COST, B_REGULAR_RATE = REGULAR_RATE, B_SPECIAL_COST = SPECIAL_COST, B_SPECIAL_RATE = SPECIAL_RATE,
                        
                        BASELINE3_SCH_FLAG = 1
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 976+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(977) : cplan.sql(9146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            ElsIf Coalesce(h_count, 0) = 0 Then
               If (h_prj_assignment_id IS NULL) Then
                  If Coalesce(eetc, 0) = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001,
                        ETC = h_effort - work_todate001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1001+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1002) : cplan.sql(9171) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                        UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                        WORK_TODATE = work_todate001
                        WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1021+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1022) : cplan.sql(9191) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = percent_complete001,
                     UNITS_MSP = h_units, CONTOUR_TYPE = h_contour_type,
                     WORK_TODATE = work_todate001
                     WHERE TSK_ASSIGNMENT_ID = h_ra_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1042+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1043) : cplan.sql(9212) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("A_* SC_STATE = %d, h_ra_id: %s,h_prj_assignment_id: %d %s\n",SC_STATE,h_ra_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
            End If;
         End If;
         
         --  if( ((*data_flag  17) == 17)  (IsNotNullSpParam(ind_assign_msp, actual_start)  (*actual_effort > 0)) || (*work_todate > 0) )
         If ((h_actual_start IS NOT NULL) AND (Coalesce(actual_effort001, 0) > 0)) OR (Coalesce(
               work_todate001, 0) > 0) Then
            h_finish_D := '';
            h_finish_D := NULL;
            h_total := 0;
            h_count := 0;
            
            If Coalesce(h_action_type, 0) = 1 Then
               BEGIN
                  SELECT TSK_ASSIGNMENT_ID, SCHED_WORK, TASK_ID, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                  INTO h_tsk_assignment_id, h_effort, h_task_id, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE RES_ASSIGNMENT_ID = h_ra_id AND TASK_ID = h_task_id AND BIT_FLAG = '2';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1071+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1072) : cplan.sql(9241) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               h_tsk_assignment_id := h_ra_id;
               BEGIN
                  SELECT SCHED_WORK, TASK_ID, TO_DATE(SubStr(SCHED_START, 1, 10)), TO_DATE(SubStr(SCHED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                  INTO h_effort, h_task_id, h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1088+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1089) : cplan.sql(9258) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("SC_STATE = %d, h_work_todate: %d, h_effort = %d, h_tsk_assignment_id = %s, h_ra_id = %s\n",SC_STATE,h_work_todate,h_effort,h_tsk_assignment_id,h_ra_id);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               If (Coalesce(work_todate001, 0) = Coalesce(effort001, 0)) AND (Coalesce(eetc, 0) = 0) AND (Coalesce(
                                           percent_complete001, 0) = 10000) Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = SCHED_MIN, ACTUAL_FLAG = 1
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1111+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1112) : cplan.sql(9281) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET WORK_TODATE = SCHED_WORK,
                     LAST_SUBMITTED = (SELECT MAX(DAY_YEAR) FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id)
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1131+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1132) : cplan.sql(9301) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  /*InitializeHost(SC_PARAM(h_tph_delete), 0);
                  				h_billable = (h_billable | SC_BIT_30);
                  
                  				h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                  							SC_PARAM(h_project_id),
                  							SC_PARAM(h_task_id),
                  							SC_PARAM(h_tsk_assignment_id),
                  							SC_PARAM(h_SD),
                  							SC_PARAM(h_FD),
                  							SC_PARAM(h_reg_cost),
                  							SC_PARAM(h_reg_sale),
                  							SC_PARAM(h_tph_delete),
                  							SC_PARAM(h_capital_expense),
                  							SC_PARAM(h_billable));*/
               Else
                  OPEN CAF;
                  
                  LOOP
                     FETCH CAF INTO h_minutes, h_dayofyear, h_count;
                     IF CAF%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                     If PMO_DB2M.SC_STATE <> 0 Then
                        EXIT;
                     End If;
                     
                     --trace_printf("h_dayofyear = %s, h_total = %d, h_effort: %d, h_minutes = %d, h_count: %d\n", h_dayofyear,h_total,h_effort,h_minutes,h_count);
                     h_total := Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                     
                     If Coalesce(h_count, 0) = 0 Then
                        If Coalesce(h_total, 0) > Coalesce(h_work_todate, 0) Then
                           If Coalesce(h_minutes, 0) >= Coalesce(h_work_todate, 0) Then
                              h_minutes := Coalesce(h_work_todate, 0);
                           Else
                              h_minutes := Coalesce(h_work_todate, 0) - Coalesce(h_total, 0) + Coalesce(h_minutes, 0);
                           End If;
                        End If;
                        
                        BEGIN
                           UPDATE TASK_DAY_OF_YEAR SET ACTUAL_MIN = h_minutes, ACTUAL_FLAG = 1 WHERE CURRENT OF CAF;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1188+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1189) : cplan.sql(9358) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        h_finish_D := h_dayofyear;
                     End If;
                     
                     If Coalesce(h_total, 0) >= Coalesce(h_work_todate, 0) Then
                        EXIT;
                     End If;
                  END LOOP;
                  
                  BEGIN
                     CLOSE CAF ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1209+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1210) : cplan.sql(9379) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("h_total = %d, h_work_todate: %d, h_finish_D = %d:%s, h_tsk_assignment_id = %s\n",h_total,h_work_todate,h_finish_D_Ind,h_finish_D,h_tsk_assignment_id);
                  
                  If (Coalesce(h_total, 0) > 0) AND ((h_finish_D IS NOT NULL)) Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET LAST_SUBMITTED = h_finish_D WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1228+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1229) : cplan.sql(9398) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     /*if (h_total > h_work_todate)
                     					{
                     						InitializeHost(SC_PARAM(h_tph_delete), -1);
                     						h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                     									SC_PARAM(h_project_id),
                     									SC_PARAM(h_task_id),
                     									SC_PARAM(h_tsk_assignment_id),
                     									SC_PARAM(h_SD),
                     									SC_PARAM(h_FD),
                     									SC_PARAM(h_reg_cost),
                     									SC_PARAM(h_reg_sale),
                     									SC_PARAM(h_tph_delete),
                     									SC_PARAM(h_capital_expense),
                     									SC_PARAM(h_billable));
                     
                     						InitializeHost(SC_PARAM(h_bit_minutes), h_minutes - h_total + h_work_todate);
                     						h_temp_sc_state = FC_ACTUAL_DAY( SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_dayofyear), SC_PARAM(h_bit_minutes));
                     
                     						h_temp_sc_state = FC_DEL_NOACTUAL(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id));
                     
                     						InitializeHost(SC_PARAM(h_etc), *eetc, ind_assign_msp->eetc);
                     						BEGIN_SQL
                     							SELECT TIMESTAMP(DATE(HOST(h_dayofyear)) + 1 DAYS, TIME('00.00.00')) INTO HOST(h_start_date) FROM DUAL;
                     						END_SQL;
                     //trace_printf("FC_RESCHEDULE: h_start_date = %s, h_etc: %d\n",h_start_date,h_etc);
                     
                     						InitializeHost(SC_PARAM(h_temp_var), 2);
                     						h_temp_sc_state = FC_RESCHEDULE(SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_start_date), SC_PARAM(h_etc), SC_PARAM(h_temp_var), SC_PARAM(h_contour), SC_PARAM(h_rec_user));
                     
                     						BEGIN_SQL
                     							SELECT TASK_ID, DATE(SCHED_START), DATE(SCHED_FINISH), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                     							INTO HOST(h_task_id), HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
                     							FROM TASK_ASSIGNMENTS
                     							WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
                     						END_SQL;
                     
                     						InitializeHost(SC_PARAM(h_tph_delete), 0);
                     
                     						h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                     									SC_PARAM(h_project_id),
                     									SC_PARAM(h_task_id),
                     									SC_PARAM(h_tsk_assignment_id),
                     									SC_PARAM(h_SD),
                     									SC_PARAM(h_FD),
                     									SC_PARAM(h_reg_cost),
                     									SC_PARAM(h_reg_sale),
                     									SC_PARAM(h_tph_delete),
                     									SC_PARAM(h_capital_expense),
                     									SC_PARAM(h_billable));
                     					}
                     
                     					if ( h_action_type == 1 )
                     					{
                     						InitializeHost(SC_PARAM(h_tph_delete), 0);
                     						h_billable = (h_billable | SC_BIT_30);
                     
                     						h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
                     									SC_PARAM(h_project_id),
                     									SC_PARAM(h_task_id),
                     									SC_PARAM(h_tsk_assignment_id),
                     									SC_PARAM(h_SD),
                     									SC_PARAM(h_FD),
                     									SC_PARAM(h_reg_cost),
                     									SC_PARAM(h_reg_sale),
                     									SC_PARAM(h_tph_delete),
                     									SC_PARAM(h_capital_expense),
                     									SC_PARAM(h_billable));
                     					}*/
                  End If;
               End If;
               
               -- populate timesheets
               If Coalesce(h_action_type, 0) = 1 Then
                  If (BITAND(Coalesce(data_flag, 0), 32)) = 32 Then
                     h_temp_sc_state := FC_GENERATE_TS_ALL(h_tsk_assignment_id, h_rec_user);
                  Else
                     h_temp_sc_state := FC_GENERATE_TS_MSP(h_tsk_assignment_id, h_rec_user);
                  End If;
               End If;
            End If;
         End If;
      End If;
      
      If PMO_DB2M.SC_STATE = 100 Then
         PMO_DB2M.SC_STATE := 0;
      End If;
   End If;
   
   --trace_printf("END: SC_STATE = %d\n",SC_STATE);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ASSIGN_MSP', 1324+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ASSIGN_MSP(1325) : cplan.sql(9494) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1332+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1333) : cplan.sql(9502) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ASSIGN_MSP', 1344+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ASSIGN_MSP(1345) : cplan.sql(9514) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ASSIGN_MSP;
ALTER FUNCTION SP_ASSIGN_MSP COMPILE DEBUG;

/***************************************************************************************************************************************
 Replace a profile with a resource
	prj_id      = wbs element id (project, deliverable or task)
	profile_id  = PROFILE_ID,
	res_id      = RESOURCE_ID,
	work_type   = 1 DON'T REPLACE RATES (for the new resource)
 replace all assignments of this profile in the project, deliverable or task
*****************************************************************************************************************************************
* Leveling algorithm tasks only:
*   1. If there are predecessors start with latest finish date + 1 (assuming SF) if assign PLANNED_START is earlier
*   2. Replace the profile with resource, call Rollup function and get the combinned FINISH_DATE of the current task
*   3. Check the successors START date if it is violated push all successors START date = FINISH_DATE + 1 DAY
*   4. If no more successors go to next replacemnt
*   ??? if CONSTRAINTS ???
******************************************************************************************************************************************/

Prompt == FUNCTION SP_A_RESTOPROF ==
CREATE OR REPLACE FUNCTION SP_A_RESTOPROF
(
     prj_id VARCHAR2
   , profile_id001 VARCHAR2
   , res_id001 VARCHAR2
   , start_date001 VARCHAR2
   , finish_date001 VARCHAR2
   , effort001 PLS_INTEGER
   , work001 PLS_INTEGER
   , bitflag001 PLS_INTEGER
   , eunit PLS_INTEGER
   , unit PLS_INTEGER
   , contour001 PLS_INTEGER
   , occurrence PLS_INTEGER
   , consider_planned_sched PLS_INTEGER
   , work_type PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_assign_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_cur_prj_assign_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_profile_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_task_start_DT TIMESTAMP;
   h_task_finish_DT TIMESTAMP;
   h_null_str VARCHAR2(1);
   h_start_D DATE;
   h_finish_D DATE;
   h_res_type VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_role_id VARCHAR2(32);
   h_full_name VARCHAR2(255);
   h_in VARCHAR2(255);
   h_profile_name VARCHAR2(255);
   h_labor_code VARCHAR2(50);
   h_participant PLS_INTEGER := 0;
   h_every PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_cri PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_occurrence PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_passed PLS_INTEGER := 0;
   h_assigned PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_per_week PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_days PLS_INTEGER := 0;
   h_owner PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_contour_changed PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_spec_cost NUMBER := 0.00;
   h_spec_sale NUMBER := 0.00;
   h_rate_echange NUMBER := 0.00;
   h_currency_id PLS_INTEGER := 0;
   h_donot_replace_bits PLS_INTEGER := 0;
   h_donot_replace_rate PLS_INTEGER := 0;
   h_donot_replace_cost PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_same_labor PLS_INTEGER := 0;
   h_fs_project PLS_INTEGER := 0;
   i_return PLS_INTEGER := 0;
   h_and_roles VARCHAR2(4000);
   h_and_skills VARCHAR2(4000);
   h_temp_res_cost NUMBER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_pa_timephase VARCHAR2(1);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_continue_01 PLS_INTEGER := 0;
   h_SD DATE;
   h_FD DATE;
   h_old_cost NUMBER := 0;
   h_old_sale NUMBER := 0;
   h_pa_count PLS_INTEGER := 0;
   h_ta_count PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_planned_effort PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_timecode_1_id VARCHAR2(32);
   h_timecode_2_id VARCHAR2(32);
   h_old_prj_assign_id VARCHAR2(32);
   i_isTask PLS_INTEGER NOT NULL := (isWBSTask(Coalesce(h_type_id, 0)));
   l_data_flag PLS_INTEGER NOT NULL := 0;
   i_many PLS_INTEGER NOT NULL := 0;
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CRCAL1 IS
         SELECT * FROM (SELECT AVAILABLE_MIN FROM CALENDAR_DAY_OF_YEAR
         WHERE ELEMENT_ID = h_res_id ) WHERE ROWNUM<=1;

      CURSOR CSA1 IS
         SELECT * FROM (SELECT BAK_WORK FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND
         PROJECT_ASSIGNMENT_ID IS NuLL ) WHERE ROWNUM<=1;

      CURSOR C6a IS
         SELECT r.RES_ASSIGNMENT_ID, t.TSK_ASSIGNMENT_ID, t.TASK_ID, r.ELEMENT_TYPE, t.PLANNED_WORK, t.UNITS_MSP,
         TO_DATE(SubStr(t.PLANNED_START, 1, 10)), TO_DATE(SubStr(t.PLANNED_FINISH, 1, 10)),t.PLANNED_START, t.PLANNED_FINISH, CASE WHEN t.PLANNED_DURATION = 0 THEN w.PLANNED_DURATION ELSE t.PLANNED_DURATION END,
         t.PROJECT_ASSIGNMENT_ID, t.BILLABLE, t.CAPITAL_EXPENSE, t.REGULAR_COST, t.REGULAR_RATE,
         t.C_EFFORT, t.T_UNIT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE
         FROM TMT_WBS w, RESOURCE_ASSIGNMENTS r, TASK_ASSIGNMENTS t
         WHERE r.PROJECT_ID = h_project_id AND r.RESOURCE_TYPE = '1' AND r.PROFILE_ID = h_profile_id AND t.RES_ASSIGNMENT_ID = r.RES_ASSIGNMENT_ID AND
         r.ELEMENT_ID = w.ELEMENT_ID AND r.ELEMENT_TYPE IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND w.RANK LIKE h_rank
         ORDER BY t.PLANNED_START;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_RESTOPROF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_RESTOPROF'), '-- ' || SYSTIMESTAMP, ':r := SP_A_RESTOPROF(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || profile_id001 || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || start_date001 || ''''
          || ', ' || '''' || finish_date001 || ''''
          || ', ' || COALESCE(TO_CHAR(effort001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(eunit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(unit), 'NULL')
          || ', ' || COALESCE(TO_CHAR(contour001), 'NULL')
          || ', ' || COALESCE(TO_CHAR(occurrence), 'NULL')
          || ', ' || COALESCE(TO_CHAR(consider_planned_sched), 'NULL')
          || ', ' || COALESCE(TO_CHAR(work_type), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 163+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_RESTOPROF(164) : cplan.sql(9708) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   -- 0 - res is not assigned in
   
   -- Parse parameters to local variables
   h_task_id := prj_id;
   h_profile_id := profile_id001;
   h_res_id := res_id001;
   h_rec_user := rec_user001;
   h_start_DT := start_date001;
   h_consider := consider_planned_sched;
   h_consider := BITAND(Coalesce(h_consider, 0), PMO_CNST.SC_1FFFFFFF);
   If (Coalesce(h_consider, 0) = PMO_CNST.SC_SRCH_PROFILE) OR (Coalesce(
             h_consider, 0) = PMO_CNST.SC_SRCH_FROM_PROJECT) Then
      h_consider := 7;
   End If;
   
   h_finish_DT := h_start_DT;
   h_labor_code := '';
   -- Labor Code
   h_null_str := '';
   h_occurrence := 0;
   -- occurrence
   h_finish_date := '';
   
   h_sched_start := NULL;
   h_sched_finish := NULL;
   
   OPEN CRCAL1;
   
   FETCH CRCAL1 INTO h_count;
   IF CRCAL1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   h_count := PMO_DB2M.SC_STATE;
   
   BEGIN
      CLOSE CRCAL1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 212+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(213) : cplan.sql(9757) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) = 100 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 219+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(220) : cplan.sql(9764) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 222+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(223) : cplan.sql(9767) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
   End If;
   
   BEGIN
      SELECT TYPE_ID, RANK, PROJECT_ID, TIMECODE_1_ID, TIMECODE_2_ID, TASK_TYPE, UNRESOLVED_DURATION
      INTO h_type_id, h_rank, h_project_id, h_timecode_1_id, h_timecode_2_id, h_task_type, h_duration
      FROM TMT_WBS WHERE ELEMENT_ID = h_task_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 237+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(238) : cplan.sql(9782) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_duration: %d\n", SC_STATE,h_duration);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 245+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(246) : cplan.sql(9790) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 248+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(249) : cplan.sql(9793) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
   End If;
   
   h_temp_var := 0;
   BEGIN
      SELECT LEVEL_LOCK INTO h_temp_var
      FROM PROJECT_DETAILS
      WHERE PROJECT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 264+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(265) : cplan.sql(9809) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_temp_var, 0) > 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 271+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(272) : cplan.sql(9816) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_LEVEL), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 274+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(275) : cplan.sql(9819) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_LEVEL, h_project_id, True);
   End If;
   
   BEGIN
      SELECT tw.TYPE_ID, CASE WHEN tw.FS_CODE IS NuLL THEN 0 ELSE 1 END, COALESCE(tw.MINUTE_PER_DAY, 480), COALESCE(tw.MINUTE_PER_WEEK, 2400), tw.PRIORITY, tw.CURRENCY_ID, PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4)
      INTO h_type, h_fs_project, h_per_day, h_per_week, h_priority, h_currency_id, h_rate_echange
      FROM TMT_WBS tw JOIN TMT_DATAFIELDS td on tw.CURRENCY_ID = td.ELEMENT_ID WHERE tw.ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 289+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(290) : cplan.sql(9834) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 296+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(297) : cplan.sql(9841) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 299+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(300) : cplan.sql(9844) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_task_id, True);
   End If;
   
   BEGIN
      SELECT NICK_NAME,FS_LABOR_CODE INTO h_full_name,h_labor_code FROM RESOURCES WHERE RESOURCE_ID = h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 312+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(313) : cplan.sql(9857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 319+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(320) : cplan.sql(9864) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 322+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(323) : cplan.sql(9867) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
   End If;
   
   BEGIN
      SELECT ROLE_ID INTO h_role_id FROM PROFILES WHERE PROFILE_ID = h_profile_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 335+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(336) : cplan.sql(9880) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_ORASTD.strcmp(h_role_id, 'DEFAULTLEVEL') = 0 Then
      h_role_id := '';
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 346+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(347) : cplan.sql(9891) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_profile_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 349+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(350) : cplan.sql(9894) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_profile_id, True);
   End If;
   
   h_donot_replace_bits := work_type;
   
   h_donot_replace_cost := BITAND(Coalesce(h_donot_replace_bits, 0), 4);
   h_from_prj := BITAND(Coalesce(h_donot_replace_bits, 0), 2);
   h_donot_replace_rate := BITAND(Coalesce(h_donot_replace_bits, 0), 1);
   
   /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
   	{
   		h_from_prj = 2;
   		h_assigned = 0;
   	}*/
   
   --trace_printf("***** h_assigned = %d, h_from_prj: %d, i_isTask: %d, h_type_id: %d\n",h_assigned,h_from_prj,i_isTask,h_type_id);
   If i_isTask <> 0 Then
      If Coalesce(h_from_prj, 0) = 2 Then
         BEGIN
            SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND
            RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 379+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(380) : cplan.sql(9924) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_from_prj, 0) = 0 Then
         BEGIN
            SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID <> PROJECT_ID AND
            RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 394+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(395) : cplan.sql(9939) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 402+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(403) : cplan.sql(9947) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 405+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(406) : cplan.sql(9950) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
               , True);
      End If;
   Else
      BEGIN
         SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_ta_count FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND PROJECT_ID <> TASK_ID AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 420+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(421) : cplan.sql(9965) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_pa_count FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND PROJECT_ID = TASK_ID AND RESOURCE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 434+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(435) : cplan.sql(9979) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If (isProjectType(Coalesce(h_type_id, 0))) <> 0 AND ((Coalesce(h_from_prj, 0) = 0) OR (Coalesce(
            h_ta_count, 0) = 0)) Then
         BEGIN
            SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_temp_var FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0'
            AND PROJECT_ASSIGNMENT_ID IS NuLL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 451+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(452) : cplan.sql(9996) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_temp_var, 0) > 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 458+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(459) : cplan.sql(10003) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 461+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(462) : cplan.sql(10006) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                  , True);
         End If;
      End If;
   End If;
   
   /********************************************************************************
   	Salling Rates           Cost Ratess             Client
   	=================================================================================
   	h_donot_replace_rate    h_donot_replace_cost    h_donot_replace_bits
   	=================================================================================
   	Affect this resource to the project             NON = 2, YES = 0
   	=================================================================================
   	1                       1                       2 (2 + 0) 2 = NO affected
   	0                       0                       7 (2 + 5)
   	1                       0                       6 (2 + 4)
   	0                       1                       3 (2 + 1)
   	*********************************************************************************/
   
   h_res_type := '0';
   h_assign_id := '';
   h_prj_assign_id := '';
   h_prj_assignment_id := NULL;
   h_start_date := NULL;
   --trace_printf("*** h_start_DT: %d %s, h_ta_count: %d, h_pa_count: %d, h_from_prj: %d, h_donot_replace_rate: %d, h_donot_replace_cost: %d\n", h_start_DT_Ind,h_start_DT,h_ta_count,h_pa_count,h_from_prj, h_donot_replace_rate, h_donot_replace_cost);
   
   /*
   		if h_assigned = 0, then: res is assigned in the project     -> don't create PA
   		if h_assigned = 1, then: res is affected to the project     -> create/merge PA
   		if h_assigned = 2, then: res is not assigned in the project -> create PA
   	*/
   
   OPEN CSA1;
   
   FETCH CSA1 INTO h_count;
   IF CSA1%FOUND THEN
      PMO_DB2M.SC_STATE := 0;
   ELSE
      PMO_DB2M.SC_STATE := 100;
   END IF;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      h_assigned := 0;
   Else
      h_assigned := 2;
   End If;
   
   BEGIN
      CLOSE CSA1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 518+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(519) : cplan.sql(10063) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("***** h_assigned = %d, h_from_prj: %d, i_isTask: %d\n",h_assigned,h_from_prj,i_isTask);
   If Coalesce(h_assigned, 0) = 2 Then
      BEGIN
         SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_count FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND
         RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND TASK_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 534+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(535) : cplan.sql(10079) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_count, 0) > 0 Then
         h_assigned := 1;
         BEGIN
            SELECT PLANNED_START, PLANNED_FINISH INTO h_sched_start, h_sched_finish
            FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND BIT_FLAG = '0' AND
            RESOURCE_ID = h_res_id AND RESOURCE_TYPE = '0' AND TASK_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 551+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(552) : cplan.sql(10096) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("4_* SC_STATE: %d, h_sched_start: %d:%s, h_sched_finish: %d:%s\n",SC_STATE,h_sched_start_Ind,h_sched_start,h_sched_finish_Ind,h_sched_finish);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 559+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(560) : cplan.sql(10104) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 562+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(563) : cplan.sql(10107) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                  , True);
         ElsIf ((h_sched_start IS NOT NULL)) AND ((h_sched_finish IS NOT NULL)) Then
            If (PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) > 0) OR (PMO_ORASTD.strncmp(h_sched_finish
                  , h_start_date, 10) < 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 570+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(571) : cplan.sql(10115) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 573+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(574) : cplan.sql(10118) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CHILD_TASK_ASSIGNMENT_ALREADY_, h_res_id
                     , True);
            End If;
         End If;
      End If;
      
      --trace_printf("h_assigned = %d, h_count: %d, h_from_prj: %d\n",h_assigned,h_count,h_from_prj);
      If Coalesce(h_from_prj, 0) = 2 Then
         h_assigned := 0;
         h_prj_assignment_id := h_prj_assignment_id;
         --h_prj_assignment_id_Ind = 0;
      Else
         BEGIN
            SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO h_prj_assign_id, h_owner
            FROM RESOURCE_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id AND RESOURCE_TYPE = h_res_type AND RESOURCE_ID = h_res_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 597+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(598) : cplan.sql(10142) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("0__ SC_STATE: %d, h_prj_assign_id: %d:%s, h_owner: %d\n",SC_STATE,h_prj_assign_id_Ind,h_prj_assign_id,h_owner);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            h_prj_assign_id := '';
            h_prj_assign_id := SYS_GUID();
            
            h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
            h_temp_participant := PMO_CNST.SC_AFFECTED;
            h_temp_res_type := '0';
            h_temp_bitflag := 1;
            
            h_temp_sc_state := FC_BEFORE_I_RES_ASSIGN(h_project_id, h_res_id, h_type, h_temp_participant
                                          , h_temp_security_id, h_temp_res_type, h_temp_bitflag);
            
            h_table_type := GetTable(Coalesce(h_type, 0));
            BEGIN
               INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID, PROJECT_ID, ELEMENT_ID, ELEMENT_TYPE,
               RESOURCE_TYPE, RESOURCE_ID, PROFILE_ID, PARTICIPANT, SECURITY_ID, TABLE_TYPE,
               REC_USER, REC_DATETIME)
               VALUES (h_prj_assign_id, h_project_id, h_project_id, h_type,
               h_res_type, h_res_id, h_profile_id, PMO_CNST.SC_AFFECTED, h_temp_security_id, h_table_type,
               rec_user001, SYSTIMESTAMP);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 634+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(635) : cplan.sql(10179) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("RA: SC_STATE = %d, h_type: %d, h_prj_assign_id = %s, h_temp_security_id: %d:%s\n",SC_STATE,h_type,h_prj_assign_id,h_temp_security_id_Ind,h_temp_security_id);
            
            BEGIN
               SELECT PROFILE_NAME INTO h_profile_name FROM PROFILES WHERE PROFILE_ID = h_profile_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 648+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(649) : cplan.sql(10193) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 Then
               h_total := 0;
            Else
               BEGIN
                  SELECT COUNT(TSK_ASSIGNMENT_ID) INTO h_total FROM TASK_ASSIGNMENTS
                  WHERE PROJECT_ID = h_project_id AND TASK_ID <> h_task_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id AND BIT_FLAG = '0';
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 665+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(666) : cplan.sql(10210) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("0.p__ h_total = %d\n",h_total);
            
            If Coalesce(h_total, 0) = 0 Then
               -- first try to replace profile_name with resource_name (if profile_name already exists at the project level)
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name)
                  WHERE ELEMENT_ID = h_project_id AND PROJECT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 687+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(688) : cplan.sql(10232) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               -- if profile has not been assigned to the project level, then add resource name to the list
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
                  ELSE
                  CASE
                  WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
                  ELSE ASSIGN_NAMES || h_full_name || ';'
                  END
                  END
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 714+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(715) : cplan.sql(10259) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               -- if profile is still assigned within the project, then add resource name to the list
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES =
                  CASE
                  WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0 THEN ASSIGN_NAMES
                  ELSE
                  CASE
                  WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
                  ELSE ASSIGN_NAMES || h_full_name || ';'
                  END
                  END
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 741+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(742) : cplan.sql(10286) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            BEGIN
               SELECT TSK_ASSIGNMENT_ID INTO h_prj_assignment_id FROM TASK_ASSIGNMENTS
               WHERE RES_ASSIGNMENT_ID = h_prj_assign_id AND BIT_FLAG = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 756+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(757) : cplan.sql(10301) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("0.2__ SC_STATE = %d, h_owner = %d, h_prj_assignment_id = %d:%s, h_prj_assign_id: %d:%s\n",SC_STATE,h_owner,h_prj_assignment_id_Ind,h_prj_assignment_id,h_prj_assign_id_Ind,h_prj_assign_id);
            
            If Coalesce(h_owner, 0) = PMO_CNST.SC_PARTICIPANT Then
               h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,
                  PARTICIPANT = PMO_CNST.SC_AFFECTED, SECURITY_ID = h_temp_security_id
                  WHERE RES_ASSIGNMENT_ID = h_prj_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 779+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(780) : cplan.sql(10324) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_prj_assign_id);
            ElsIf (BITAND(Coalesce(h_owner, 0), PMO_CNST.SC_AFFECTED)) <> PMO_CNST.SC_AFFECTED Then
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,
                  PARTICIPANT=PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_AFFECTED)
                  WHERE RES_ASSIGNMENT_ID = h_prj_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 800+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(801) : cplan.sql(10345) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_prj_assign_id);
            Else
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id,
                  PARTICIPANT=PMO_STPU.BIT_OR(PARTICIPANT, PMO_CNST.SC_TEAMMEMBER)
                  WHERE RES_ASSIGNMENT_ID = h_prj_assign_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 821+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(822) : cplan.sql(10366) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_temp_sc_state := FC_AFTER_U_RES_ASSIGN(h_prj_assign_id);
            End If;
            --trace_printf("0.3__ SC_STATE = %d, h_temp_security_id = %s\n",SC_STATE,h_temp_security_id);
         End If;
      End If;
   Else
      h_prj_assignment_id := h_prj_assignment_id;
      --h_prj_assignment_id_Ind = 0;
   End If;
   
   -- replace RATES =================================================================
   --trace_printf("SC_STATE = %d, h_res_id = %s, h_currency_id = %d, h_rate_echange = %.2f\n",SC_STATE,h_res_id,h_currency_id,h_rate_echange);
   h_and_roles := '';
   h_and_skills := '';
   h_finish_date := '';
   
   h_temp_sc_state := FC_DECLARE_PMO_TMP_SEARCH();
   
   -- Check for project calendar
   --  if ( FC_CREATE_CDY ( SC_PARAM(h_project_id), SC_PARAM(h_start_DT), SC_PARAM(h_finish_date), SC_PARAM(h_duration) ) != 0)
   --  {
   --      RETURN_ERROR(TASK_ASSIGNMENT_ERROR, h_project_id);
   --  }
   
   If (Coalesce(h_donot_replace_cost, 0) = 0) OR (Coalesce(h_donot_replace_rate, 0) = 0) Then
      h_temp_sc_state := FC_GET_RES_RATES(h_project_id, h_res_id, h_res_type, h_and_roles, h_and_skills
                                    , l_data_flag, h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale);
   End If;
   
   BEGIN
      SELECT CASE WHEN h_currency_id = rs.CURRENCY_ID THEN 1
      ELSE h_rate_echange / PMO_ORASTD.TODECIMAL(td.ELEMENT_NAME,20,4) END
      INTO h_rate_echange
      FROM RESOURCES rs JOIN TMT_DATAFIELDS td ON rs.CURRENCY_ID = td.ELEMENT_ID AND rs.CURRENCY_ID > 0
      WHERE RESOURCE_ID=h_res_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 867+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(868) : cplan.sql(10412) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
      h_rate_echange := 1.00;
   End If;
   
   --trace_printf("1_* h_reg_cost = %.2f, h_spec_cost = %.2f, h_reg_sale = %.2f, h_spec_sale = %.2f\n",h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale);
   --trace_printf("prj_assignment: h_assigned = %d, h_count = %d, h_prj_assignment_id: %d:%s\nh_project_id = %s,h_task_id = %s\n",h_assigned,h_count,h_prj_assignment_id_Ind,h_prj_assignment_id,h_project_id,h_task_id);
   h_passed := 0;
   h_total := 0;
   h_old_prj_assign_id := NULL;
   --trace_printf("A_ prj_assignment: h_prj_assignment_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_assigned,h_from_prj);
   
   BEGIN
      SELECT ASSIGNMENT_BILLABLE_FLAG, TIMEPHASE_PROJECT_ASSIGNMENT INTO h_billable, h_pa_timephase
      FROM PROJECT_DETAILS WHERE PROJECT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 892+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(893) : cplan.sql(10437) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("77:_SC_STATE: %d, h_pa_timephase: %d:%s, h_profile_id: %d:%s\n", SC_STATE,h_pa_timephase_Ind,h_pa_timephase,h_profile_id_Ind,h_profile_id);
   
   If PMO_ORASTD.strcmp(h_project_id, h_task_id) <> 0 Then
      BEGIN
         SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id AND RESOURCE_ID = h_profile_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 908+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(909) : cplan.sql(10453) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("p.a_SC_STATE: %d, h_total: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_total,h_prj_assignment_id_Ind,h_prj_assignment_id);
      
      If Coalesce(h_total, 0) = 0 Then
         --strcpy(h_task_id,h_project_id);
         h_task_id := h_project_id;
      Else
         h_total := 0;
      End If;
   End If;
   
   If Coalesce(h_assigned, 0) > 0 Then
      If PMO_ORASTD.strcmp(h_project_id, h_task_id) = 0 Then
         h_old_prj_assign_id := h_prj_assignment_id;
         --trace_printf("1__* h_old_prj_assign_id: %d:%s\n", h_old_prj_assign_id_Ind,h_old_prj_assign_id);
         
         BEGIN
            SELECT TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,PLANNED_WORK, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
            REGULAR_COST, REGULAR_RATE, TASK_ID, UNITS_MSP, C_EFFORT, T_UNIT, C_UNIT, C_CONTOUR
            INTO h_prj_assignment_id, h_id, h_planned_effort, h_start_D, h_finish_D, h_billable, h_capital_expense,
            h_old_cost, h_old_sale, h_task_id, h_percent_units, h_work, h_eunit, h_unit, h_contour
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID = h_project_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 940+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(941) : cplan.sql(10485) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("p.0_SC_STATE: %d, h_planned_effort: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_planned_effort,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         h_contour_changed := 0;
         If Coalesce(h_pa_count, 0) = 1 Then
            If (Coalesce(h_contour, 0) <> Coalesce(contour001, 0)) OR (Coalesce(h_work, 0) <> Coalesce(
                                       work001, 0)) Then
               If (Coalesce(h_assigned, 0) = 1) AND ((BITAND(Coalesce(
                     bitflag001, 0), PMO_CNST.SC_0000FFFF)) <> 0) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 953+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(954) : cplan.sql(10498) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 956+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(957) : cplan.sql(10501) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_res_id, True);
               Else
                  h_contour_changed := 1;
                  
                  If Coalesce(contour001, 0) = 0 Then
                     h_percent_units := 0;
                  End If;
               End If;
            End If;
            
            h_effort := effort001;
            h_start_date := start_date001;
            h_finish_date := finish_date001;
            h_work := work001;
            -- amount for countour
            h_occurrence := occurrence;
            -- occurrence
            h_eunit := eunit;
            h_unit := unit;
            h_bitflag := bitflag001;
            h_contour := contour001;
         End If;
         --trace_printf("** CONTOUR ** h_bitflag: %d, h_assigned: %d, h_work: %d, h_contour: %d, h_contour_changed: %d\n", h_bitflag,h_assigned,h_work,h_contour,h_contour_changed);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_prj_assignment_id IS NOT NULL)) Then
            If SubStr(h_pa_timephase, 1, 1) = 'Y' Then
               h_tph_delete := -1;
               PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_prj_assignment_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
            End If;
            
            If Coalesce(h_assigned, 0) = 1 Then
               BEGIN
                  DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1003+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1004) : cplan.sql(10548) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("DEL_OLD_TA: SC_STATE: %d, h_prj_assignment_id: %d:%s\n", SC_STATE,h_prj_assignment_id_Ind,h_prj_assignment_id);
               
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1021+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1022) : cplan.sql(10566) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("DEL_OLD_TDY: SC_STATE: %d\n", SC_STATE);
               
               h_prj_assignment_id := h_old_prj_assign_id;
               BEGIN
                  SELECT MAX(DAY_YEAR) INTO h_start_D FROM TASK_DAY_OF_YEAR
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1037+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1038) : cplan.sql(10582) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_ORASTD.strncmp(h_start_date, h_start_D, 10) <= 0 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_start_D, 1, 10)) + 1     , PMO_ORASTD.TIME(h_start_date))
                     INTO h_start_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1053+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1054) : cplan.sql(10598) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               If (Coalesce(h_donot_replace_cost, 0) = 0) AND (Coalesce(h_donot_replace_rate, 0) = 0) Then
                  h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
                  h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
                  
                  If Coalesce(h_billable, 0) = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_COST = h_reg_cost,
                        SPECIAL_COST = h_spec_cost,
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) * h_reg_cost,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1087+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1088) : cplan.sql(10632) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_COST = h_reg_cost,
                        SPECIAL_COST = h_spec_cost,
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) * h_reg_cost,
                        ESTIMATION_LABOR_REVENUE = ((PLANNED_WORK+PLANNED_WORK)/60.00) * h_reg_sale,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1114+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1115) : cplan.sql(10659) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf Coalesce(h_donot_replace_cost, 0) = 0 Then
                  h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                     REGULAR_COST = h_reg_cost,
                     SPECIAL_COST = h_spec_cost,
                     UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                     ESTIMATION_LABOR_COST = ((PLANNED_WORK+SCHED_WORK)/60.00) * h_reg_cost,
                     ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1142+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1143) : cplan.sql(10687) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(h_donot_replace_rate, 0) = 0 Then
                  h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
                  h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
                  
                  If Coalesce(h_billable, 0) = 0 Then
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1169+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1170) : cplan.sql(10714) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                        REGULAR_RATE = h_reg_sale,
                        SPECIAL_RATE = h_spec_sale,
                        UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                        ESTIMATION_LABOR_REVENUE = ((PLANNED_WORK+PLANNED_WORK)/60.00) * h_reg_sale,
                        ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                        WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1193+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1194) : cplan.sql(10738) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                     UNITS_MSP = CASE WHEN h_contour_changed = 0 THEN UNITS_MSP ELSE h_percent_units END,
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0'
                     WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1214+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1215) : cplan.sql(10759) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               BEGIN
                  DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1232+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1233) : cplan.sql(10777) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_SKILLS
                  SELECT h_prj_assignment_id, SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, PRIMARY_SKILL
                  FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1251+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1252) : cplan.sql(10796) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("U_PA_SC_STATE: %d, h_rate_echange: %.2f, h_old_cost: %.2f, h_old_sale: %.2f, h_reg_cost: %.2f, h_reg_sale: %.2f\n", SC_STATE,h_rate_echange,h_old_cost,h_old_sale,h_reg_cost,h_reg_sale);
            
            h_count := 1;
            --trace_printf("0__ prj_assignment: h_pa_count: %d, h_prj_assignment_id: %d %s, h_id = %s\n",h_pa_count,h_prj_assignment_id_Ind,h_prj_assignment_id,h_id);
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1273+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1274) : cplan.sql(10818) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("DEL_OLD_RA: SC_STATE: %d, h_id: %d:%s\n", SC_STATE,h_id_Ind,h_id);
            
            BEGIN
               DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1291+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1292) : cplan.sql(10836) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (Coalesce(h_pa_count, 0) = 1) AND (Coalesce(h_ta_count, 0) = 0) Then
               h_work_type := 0;
               h_cri := 0;
               If PMO_FUNC.AssignLong(h_contour_changed, 1) <> 0 Then
                  h_update := 2;
               Else
                  h_update := 1;
               End If;
               
               If Coalesce(contour001, 0) = 0 Then
                  h_work := 0;
               End If;
               
               h_temp_var := FC_CALC_DURATION(h_project_id, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                               , h_unit, h_consider, h_bitflag, h_contour, h_occurrence, h_per_day
                               , h_per_week, h_work, h_effort, h_duration);
               
               If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RESTOPROF', 1315+1) THEN 
                     LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RESTOPROF(1316) : cplan.sql(10860) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1318+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1319) : cplan.sql(10863) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
               ElsIf Coalesce(h_temp_var, 0) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 1323+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(1324) : cplan.sql(10868) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1326+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1327) : cplan.sql(10871) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
               End If;
               
               --trace_printf("*** h_start_DT = %s, h_finish_date = %s, h_duration = %d, h_effort: %d, h_work: %d\n",h_start_DT,h_finish_date,h_duration,h_effort,h_work);
               
               If Coalesce(h_work, 0) = 0 Then
                  h_bitflag := PMO_CNST.SC_BIT_27;
               Else
                  h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27);
               End If;
               
               --trace_printf("****** Project Assignment ONLY - all effort to PA\n");
               --trace_printf("AA_1: h_bitflag = %d, h_start_date = %s, h_finish_date = %s, h_duration = %d, h_work_type = %d\n", h_bitflag, h_start_date, h_finish_date, h_duration, h_work_type);
               
               -- assign to the project
               h_temp_res_cost := 0.00;
               i_return := FC_SEARCH_2(h_prj_assignment_id, h_project_id, h_project_id, h_res_id, h_temp_res_cost
                        , h_type, h_null_str, h_null_str, h_null_str, h_null_str, h_start_date
                        , h_finish_date, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
               
               --trace_printf("ONLY to the project: i_return = %d, h_count = %d\n", i_return,h_count);
               
               If Coalesce(i_return, 0) = 110 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 1354+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(1355) : cplan.sql(10899) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_UNAVAILABLE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1357+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1358) : cplan.sql(10902) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_UNAVAILABLE, h_res_id, True);
               ElsIf Coalesce(i_return, 0) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 1362+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(1363) : cplan.sql(10907) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 1365+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(1366) : cplan.sql(10910) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                        , True);
               Else
                  h_temp_var := 4;
                  h_temp_var := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, Coalesce(h_per_day, 0)
                              , Coalesce(h_effort, 0));
               End If;
            End If;
         End If;
      ElsIf SubStr(h_pa_timephase, 1, 1) = 'Y' Then
         -- Get the old project assignment id
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_old_prj_assign_id, h_start_D, h_finish_D, h_billable, h_capital_expense, h_old_cost, h_old_sale
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID = h_project_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1389+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1390) : cplan.sql(10934) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("old.1_SC_STATE: %d, h_planned_effort: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_planned_effort,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_old_prj_assign_id IS NOT NULL)) Then
            h_tph_delete := -1;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_old_prj_assign_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
         End If;
      Else
         -- Get the old project assignment id
         BEGIN
            SELECT TSK_ASSIGNMENT_ID, BILLABLE INTO h_old_prj_assign_id, h_billable
            FROM TASK_ASSIGNMENTS
            WHERE TASK_ID = h_project_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1413+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1414) : cplan.sql(10958) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("old.2_SC_STATE: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
      End If;
   ElsIf SubStr(h_pa_timephase, 1, 1) = 'Y' Then
      -- Get the old project assignment id
      BEGIN
         SELECT TSK_ASSIGNMENT_ID, TO_DATE(SubStr(PLANNED_START, 1, 10)), TO_DATE(SubStr(PLANNED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         INTO h_old_prj_assign_id, h_start_D, h_finish_D, h_billable, h_capital_expense, h_old_cost, h_old_sale
         FROM TASK_ASSIGNMENTS
         WHERE TASK_ID = h_project_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1432+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1433) : cplan.sql(10977) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("old.3_SC_STATE: %d, h_planned_effort: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_planned_effort,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_old_prj_assign_id IS NOT NULL)) Then
         h_tph_delete := -1;
         PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_old_prj_assign_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
      End If;
   Else
      -- Get the old project assignment id
      BEGIN
         SELECT TSK_ASSIGNMENT_ID, BILLABLE INTO h_old_prj_assign_id, h_billable
         FROM TASK_ASSIGNMENTS
         WHERE TASK_ID = h_project_id AND RESOURCE_TYPE = '1' AND RESOURCE_ID = h_profile_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1456+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1457) : cplan.sql(11001) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("old.4_SC_STATE: %d, h_old_prj_assign_id: %d:%s\n", SC_STATE,h_old_prj_assign_id_Ind,h_old_prj_assign_id);
   End If;
   
   --trace_printf("1__ prj_assignment: h_prj_assignment_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_assigned,h_from_prj);
   --trace_printf("3_* h_start_D: %d:%s, h_finish_D: %d:%s\n",h_start_D_Ind,h_start_D,h_finish_D_Ind,h_finish_D);
   --trace_printf("5_* h_old_prj_assign_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_old_prj_assign_id_Ind,h_old_prj_assign_id,h_assigned,h_from_prj);
   -- resource is not assigned in the project or affected in schedule mode  create project assignment
   If ((h_prj_assignment_id IS NULL)) AND (Coalesce(h_assigned, 0) > 0) AND (Coalesce(
         h_from_prj, 0) = 0) Then
      h_prj_assignment_id := '';
      h_prj_assignment_id := SYS_GUID();
      --h_prj_assignment_id_Ind = 0;
      --InitializeHost(SC_PARAM(h_total), h_total);  //h_total_Ind = 0;
      
      If Coalesce(unit, 0) = 100 Then
         h_percent_units := Coalesce(h_work, 0);
      End If;
      
      If (Coalesce(h_donot_replace_cost, 0) = 0) OR (Coalesce(h_donot_replace_rate, 0) = 0) Then
         If (Coalesce(h_donot_replace_cost, 0) = 0) AND (Coalesce(h_donot_replace_rate, 0) = 0) Then
            h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
            h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
            h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
            h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
            
            BEGIN
               INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
               PLANNED_START, PLANNED_FINISH, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
               VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
               h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
               h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale,h_timecode_1_id, h_timecode_2_id, h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1502+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1503) : cplan.sql(11047) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_donot_replace_rate, 0) = 0 Then
            h_reg_sale := Coalesce(h_reg_sale, 0) * Coalesce(h_rate_echange, 0);
            h_spec_sale := Coalesce(h_spec_sale, 0) * Coalesce(h_rate_echange, 0);
            
            BEGIN
               INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
               PLANNED_START, PLANNED_FINISH, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
               VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
               h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
               h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1527+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1528) : cplan.sql(11072) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            h_reg_cost := Coalesce(h_reg_cost, 0) * Coalesce(h_rate_echange, 0);
            h_spec_cost := Coalesce(h_spec_cost, 0) * Coalesce(h_rate_echange, 0);
            
            BEGIN
               INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
               PLANNED_START, PLANNED_FINISH, BIT_FLAG, BAK_START_DT, BAK_FINISH_DT, RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
               BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
               VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
               h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
               h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_table_type);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1552+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1553) : cplan.sql(11097) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      Else
         BEGIN
            INSERT INTO TASK_ASSIGNMENTS(TSK_ASSIGNMENT_ID,RES_ASSIGNMENT_ID,TASK_ID,PROJECT_ID,SD,FD,START_DT,FINISH_DT,
            PLANNED_START,PLANNED_FINISH,BIT_FLAG,BAK_START_DT,BAK_FINISH_DT,RESOURCE_ID,RESOURCE_TYPE,PROJECT_ASSIGNMENT_ID,
            BILLABLE, CAPITAL_EXPENSE, REGULAR_COST,SPECIAL_COST,REGULAR_RATE,SPECIAL_RATE, TIMECODE_1_ID, TIMECODE_2_ID, TABLE_TYPE)
            VALUES(h_prj_assignment_id, h_prj_assign_id, h_project_id, h_project_id, PMO_CNST.SC_NS,PMO_CNST.SC_NF,h_start_DT, h_finish_DT,
            h_start_DT, h_finish_DT, '0', h_start_DT, h_finish_DT, h_res_id,h_res_type, h_prj_assignment_id,
            h_billable, h_capital_expense,h_reg_cost,h_spec_cost,h_reg_sale,h_spec_sale, h_timecode_1_id, h_timecode_2_id, h_table_type);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1575+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1576) : cplan.sql(11120) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      --trace_printf("SC_STATE: %d, h_capital_expense: %d, h_billable = %d,h_table_type: %d\n",SC_STATE,h_capital_expense,h_billable,h_table_type);
      --trace_printf("2_* h_start_DT = %d:%s, h_finish_DT = %d:%s\n",h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT);
      
      h_count := 1;
   End If;
   h_rank := h_rank || '%';
   h_tsk_assignment_id := '';
   h_effort := effort001;
   
   --trace_printf("==========: h_prj_assignment_id: %d:%s, h_count = %d, h_assigned: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_count,h_assigned);
   --trace_printf("1_* h_start_DT = %d:%s, h_finish_DT = %d:%s, h_type_id: %d, i_isTask: %d\n",h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT,h_type_id,i_isTask);
   
   -- Replace the profile with Resource
   BEGIN
      SELECT PROFILE_NAME INTO h_profile_name FROM PROFILES WHERE PROFILE_ID = h_profile_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1601+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1602) : cplan.sql(11146) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_old_cost := 0.00;
   h_old_sale := 0.00;
   h_total := 0;
   --  InitializeHost(SC_PARAM(h_days), -1);
   
   If (i_isTask = 0) AND (Coalesce(h_ta_count, 0) = 1) Then
      BEGIN
         SELECT MIN(t.PLANNED_START) INTO h_start_date
         FROM TMT_WBS w, TASK_ASSIGNMENTS t
         WHERE t.PROJECT_ID = h_project_id AND t.RESOURCE_ID = h_profile_id AND
         t.TASK_ID = w.ELEMENT_ID AND w.TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND w.RANK LIKE h_rank;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1623+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1624) : cplan.sql(11168) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      /*BEGIN_SQL
      			SELECT DAYS(HOST(h_start_DT)) - DAYS(HOST(h_start_date)) INTO HOST(h_days) FROM DUAL;
      		END_SQL;*/
   End If;
   --trace_printf("h_start_date: %s, h_start_DT: %s, h_days: %d\n", h_start_date,h_start_DT,h_days);
   
   h_start_date := h_start_DT;
   
   -- replace all instances inside the project_id
   
   OPEN C6a;
   
   LOOP
      FETCH C6a INTO h_assign_id, h_tsk_assignment_id, h_task_id, h_type, h_effort, h_percent_units,
      h_SD,h_FD,h_task_start_DT,h_task_finish_DT,h_duration,
      h_cur_prj_assign_id, h_billable,h_capital_expense,h_old_cost,h_old_sale,
      h_work, h_eunit, h_unit, h_contour, h_bitflag;
      IF C6a%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      --trace_printf("FETCH: h_assign_id: %s, h_tsk_assignment_id: %s\n",h_assign_id,h_tsk_assignment_id);//trace_flush();
      --trace_printf("h_task_id: %s, h_type: %d, h_effort: %d, h_duration: %d\n",h_task_id,h_type,h_effort,h_duration);//trace_flush();
      --trace_printf("h_bitflag: %d, h_contour: %d, h_work: %d, h_percent_units: %d\n",h_contour,h_work,h_percent_units);//trace_flush();
      
      If Coalesce(h_effort, 0) = 0 Then
         h_temp_sc_state := FC_BEFORE_D_RES_ASSIGN(h_assign_id);
         
         BEGIN
            DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_assign_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1673+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1674) : cplan.sql(11218) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1690+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1691) : cplan.sql(11235) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM EFFORT_WEEK WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1707+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1708) : cplan.sql(11252) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1724+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1725) : cplan.sql(11269) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1741+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1742) : cplan.sql(11286) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1758+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1759) : cplan.sql(11303) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            SELECT TSK_ASSIGNMENT_ID INTO h_id FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND TASK_ID = h_task_id AND RESOURCE_ID = h_res_id
            AND RESOURCE_TYPE ='0';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1773+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1774) : cplan.sql(11318) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("1.0__ SC_STATE: %d, h_id: %s, h_assigned: %d, h_tsk_assignment_id: %d:%s\n",SC_STATE,h_id,h_assigned,h_tsk_assignment_id_Ind,h_tsk_assignment_id); trace_flush();
         --trace_printf("A1_** h_prj_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id); trace_flush();
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            If Coalesce(h_assigned, 0) = 0 Then
               h_prj_assignment_id := h_tsk_assignment_id;
            End If;
            
            -- Replace the profile with Resource
            BEGIN
               SELECT RES_ASSIGNMENT_ID, PARTICIPANT, SECURITY_ID INTO h_id, h_participant, h_security_id
               FROM RESOURCE_ASSIGNMENTS WHERE ELEMENT_ID = h_task_id AND RESOURCE_ID = h_res_id AND RESOURCE_TYPE='0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1795+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1796) : cplan.sql(11340) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("1__ SC_STATE = %d, h_id = %s, h_participant = %d, h_security_id = %s\n",SC_STATE,h_id,h_participant,h_security_id);//trace_flush();
            --trace_printf("A2_** h_prj_assignment_id: %d:%s\n",h_prj_assignment_id_Ind,h_prj_assignment_id); trace_flush();
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               If (BITAND(Coalesce(h_participant, 0), PMO_CNST.SC_TEAMMEMBER)) = PMO_CNST.SC_TEAMMEMBER Then
                  h_effort := 0;
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET PROFILE_ID = h_profile_id WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1817+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1818) : cplan.sql(11362) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(h_participant, 0) = PMO_CNST.SC_PARTICIPANT Then
                  h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_CNST.SC_TEAMMEMBER, SECURITY_ID = h_temp_security_id, PROFILE_ID = h_profile_id,
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE='0' WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1838+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1839) : cplan.sql(11383) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_OR(h_participant,PMO_CNST.SC_TEAMMEMBER), SECURITY_ID = h_security_id, PROFILE_ID = h_profile_id,
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE='0' WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1856+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1857) : cplan.sql(11401) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_effort, 0) > 0) Then
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1875+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1876) : cplan.sql(11420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            Else
               BEGIN
                  SELECT ra.RES_ASSIGNMENT_ID, SUBSTR(tw.ASSIGN_NAMES,1,InStr(tw.ASSIGN_NAMES, ';') - 1)
                  INTO h_id,h_in FROM TMT_WBS tw, RESOURCE_ASSIGNMENTS ra
                  WHERE tw.ELEMENT_ID = h_task_id AND tw.ELEMENT_ID = ra.ELEMENT_ID AND ra.PARTICIPANT = 32 AND
                  ra.RESOURCE_ID = tw.CREATED_BY AND NOT EXISTS(SELECT TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta WHERE ta.RES_ASSIGNMENT_ID = ra.RES_ASSIGNMENT_ID);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1892+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1893) : cplan.sql(11437) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --trace_printf("2__ SC_STATE: %d, h_id: %d:%s, h_in: %s\nh_full_name: %s, h_profile_name: %s\n",SC_STATE,h_id_Ind,h_id,h_in,h_full_name,h_profile_name);
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_id IS NOT NULL)) Then
                  h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_RESPONSIBLE_GROUP
                                 , h_temp_security_id);
                  
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1914+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1915) : cplan.sql(11459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET RESOURCE_ID = h_res_id, RESOURCE_TYPE='0',
                     PARTICIPANT = 48, SECURITY_ID=h_temp_security_id
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1933+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1934) : cplan.sql(11478) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_in, h_full_name) WHERE ELEMENT_ID = h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1950+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1951) : cplan.sql(11495) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  h_temp_sc_state := GetSecurityID(0, Coalesce(h_type, 0), PMO_CNST.SC_TEAMMEMBER_GROUP
                                 , h_temp_security_id);
                  
                  BEGIN
                     UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_profile_name, h_full_name) WHERE ELEMENT_ID = h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1970+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1971) : cplan.sql(11515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     UPDATE RESOURCE_ASSIGNMENTS SET RESOURCE_ID = h_res_id, RESOURCE_TYPE = '0',
                     PARTICIPANT = 16, SECURITY_ID=h_temp_security_id
                     WHERE RES_ASSIGNMENT_ID = h_assign_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 1989+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(1990) : cplan.sql(11534) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            End If;
            --trace_printf("2.1__ SC_STATE: %d, h_temp_security_id: %s\n",SC_STATE,h_temp_security_id);
            
            --trace_printf("h_effort = %d, h_assigned: %d, h_tsk_assignment_id: %s\nh_prj_assignment_id: %d:%s\n",h_effort,h_assigned,h_tsk_assignment_id,h_prj_assignment_id_Ind,h_prj_assignment_id);
            If Coalesce(h_effort, 0) > 0 Then
               If (Coalesce(h_fs_project, 0) = 1) AND ((h_labor_code IS NOT NULL)) Then
                  BEGIN
                     SELECT CASE WHEN FS_LABOR_CODE IS NuLL THEN 0 WHEN FS_LABOR_CODE = h_labor_code THEN 1 ELSE 2 END
                     INTO h_same_labor FROM TMT_WBS WHERE ELEMENT_ID=h_task_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2009+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2010) : cplan.sql(11554) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("SC_STATE = %d, h_same_labor = %d,h_labor_code = %s\n", SC_STATE,h_same_labor,h_labor_code);
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_same_labor, 0) = 0) Then
                     BEGIN
                        UPDATE TMT_WBS SET FS_LABOR_CODE = h_labor_code WHERE ELEMENT_ID=h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2028+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2029) : cplan.sql(11573) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_same_labor, 0) = 2) Then
                     BEGIN
                        CLOSE C6a ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2041+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2042) : cplan.sql(11586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2047+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2048) : cplan.sql(11592) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_ASSIGN_LABOR_CODE), 'NULL') || ' ID: ' || '''' || h_task_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2050+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2051) : cplan.sql(11595) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_ASSIGN_LABOR_CODE, h_task_id
                           , True);
                  End If;
               End If;
               
               h_tph_delete := -1;
               PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                            , h_old_cost, h_old_sale, h_tph_delete
                                            , h_capital_expense, h_billable);
               --trace_printf("SC_STATE: %d, h_old_cost: %.2f, h_old_sale: %.2f\n", SC_STATE,h_old_cost,h_old_sale);
               
               /*if (h_task_type == SC_FD_EFFORT_DRIVEN)
               						{
               							BEGIN_SQL
               								DELETE FROM TASK_DAY_OF_YEAR tdy WHERE tdy.DAY_YEAR IN (
               								SELECT ta.DAY_YEAR FROM TASK_DAY_OF_YEAR ta
               								WHERE ta.PROJECT_ID = HOST(h_project_id) AND ta.TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) AND NOT EXISTS (
               								SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR r WHERE r.ELEMENT_ID = HOST(h_res_id) AND r.DAY_YEAR = ta.DAY_YEAR));
               							END_SQL_NOCHECK;
               
               							BEGIN_SQL
               								UPDATE TASK_DAY_OF_YEAR SET RESOURCE_ID = HOST(h_res_id), PARENT_ID = HOST(h_prj_assignment_id)
               								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
               							END_SQL;
               
               							h_remaining = h_effort - h_remaining;
               							h_temp_var = int ((double) h_per_day * ((double) h_effort / (double) h_remaining) + 0.4);
               							if ( (h_percent_units > 0)  (h_percent_units > 0) )
               							{
               								h_temp_var = h_temp_var * (h_percent_units / 10000.00);
               							}
               
               //trace_printf("h_effort: %d, h_remaining: %d, h_per_day: %d, h_percent_units: %d, h_temp_var: %d\n", h_effort,h_remaining,h_per_day,h_percent_units,h_temp_var);
               							h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
               														  SC_PARAM(h_tsk_assignment_id),
               														  SC_PARAM(h_start_DT),
               														  SC_PARAM(h_per_day),
               														  SC_PARAM(h_temp_var));
               
               							BEGIN_SQL
               								SELECT TIMESTAMP(MIN(DAY_YEAR),TIME(HOST(h_task_start_DT))), TIMESTAMP(MAX(DAY_YEAR),TIME(HOST(h_task_finish_DT)))
               								INTO HOST(h_task_start_DT), HOST(h_task_finish_DT)
               								FROM TASK_DAY_OF_YEAR
               								WHERE PROJECT_ID = HOST(h_project_id) AND TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
               							END_SQL;
               
               							BEGIN_SQL
               								UPDATE TASK_ASSIGNMENTS SET PLANNED_START = HOST(h_task_start_DT), PLANNED_FINISH = HOST(h_task_finish_DT),
               								RESOURCE_ID = HOST(h_res_id), RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
               								CASE WHEN HOST(h_assigned) = 0 THEN NuLL ELSE HOST(h_prj_assignment_id) END,
               								REGULAR_COST = CASE WHEN HOST(h_donot_replace_cost) = 0 THEN HOST(h_reg_cost)  ELSE REGULAR_COST END,
               								SPECIAL_COST = CASE WHEN HOST(h_donot_replace_cost) = 0 THEN HOST(h_spec_cost) ELSE SPECIAL_COST END,
               								REGULAR_RATE = CASE WHEN HOST(h_donot_replace_rate) = 0 THEN HOST(h_reg_sale)  ELSE REGULAR_RATE END,
               								SPECIAL_RATE = CASE WHEN HOST(h_donot_replace_rate) = 0 THEN HOST(h_spec_sale) ELSE SPECIAL_RATE END,
               								ROLLUP_BITS  = BIT_OR(ROLLUP_BITS, SC_LABOR_BIT)
               								WHERE TSK_ASSIGNMENT_ID = HOST(h_tsk_assignment_id) ;
               							END_SQL;
               
               							h_temp_var = int((((double) h_effort / (double) h_remaining) * 10000.00) + 0.4) + SC_BIT_30;
               							h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day, h_effort);
               						}
               						else
               						{*/
               If (h_cur_prj_assign_id IS NULL) Then
                  BEGIN
                     DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2128+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2129) : cplan.sql(11673) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  ora_temp_sc_state := PMO_DB2M.SC_STATE;
                  null_return := FC_U_PA_WORK(h_project_id, h_prj_assignment_id, h_tsk_assignment_id, Coalesce(
                            h_per_day, 0), 0);
                  PMO_DB2M.SC_STATE := ora_temp_sc_state;
               End If;
               
               h_finish_date := '';
               --trace_printf("*** h_start_DT = %s, h_finish_date = %s, h_duration = %d, i_isTask: %d\n",h_start_DT,h_finish_date,h_duration,i_isTask);
               
               If i_isTask <> 0 OR (Coalesce(h_pa_count, 0) + Coalesce(h_ta_count, 0) = 1) Then
                  --trace_printf("****** Task Assignment ONLY - all effort to TA bitflag: %d, contour: %d %d, work: %d %d\n", *bitflag,h_contour,*contour,h_work,*work);
                  h_effort := effort001;
                  h_start_date := start_date001;
                  h_finish_date := finish_date001;
                  h_work := work001;
                  -- amount for countour
                  h_occurrence := occurrence;
                  -- occurrence
                  h_eunit := eunit;
                  h_unit := unit;
                  h_bitflag := bitflag001;
                  h_contour := contour001;
                  
                  If (((BITAND(Coalesce(bitflag001, 0), PMO_CNST.SC_0000FFFF)) <> 0) AND ((Coalesce(
                        h_contour, 0) <> Coalesce(contour001, 0)) OR (Coalesce(
                        h_work, 0) <> Coalesce(work001, 0)))) Then
                     If Coalesce(unit, 0) = 100 Then
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                           CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                           REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                           SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                           REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                           SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                           UNITS_MSP    = work001, ETC = h_effort,
                           C_CONTOUR    = contour001,
                           ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2182+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2183) : cplan.sql(11727) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                           CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                           REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                           SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                           REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                           SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                           C_EFFORT     = work001, ETC = h_effort,
                           C_UNIT       = unit,
                           C_CONTOUR    = contour001,
                           ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2210+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2211) : cplan.sql(11755) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     
                     --trace_printf("SC_STATE: %d, h_contour: %d, h_unit: %d, h_work: %d\n", SC_STATE,h_contour,h_unit,h_work);
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                        CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                        REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                        SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                        REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                        SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                        ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT), ETC = h_effort
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2238+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2239) : cplan.sql(11783) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  If (Coalesce(h_contour, 0) = 0) AND (Coalesce(h_unit, 0) <> 100) Then
                     h_work := 0;
                  ElsIf Coalesce(h_unit, 0) = 100 Then
                     h_percent_units := Coalesce(h_work, 0);
                  End If;
                  
                  h_temp_var := FC_CALC_DURATION(h_project_id, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                               , h_unit, h_consider, h_bitflag, h_contour, h_occurrence, h_per_day
                               , h_per_week, h_work, h_effort, h_duration);
                  
                  If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
                     BEGIN
                        CLOSE C6a ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2263+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2264) : cplan.sql(11808) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RESTOPROF', 2269+1) THEN 
                        LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RESTOPROF(2270) : cplan.sql(11814) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2272+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2273) : cplan.sql(11817) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
                  ElsIf Coalesce(h_temp_var, 0) <> 0 Then
                     BEGIN
                        CLOSE C6a ;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2284+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2285) : cplan.sql(11829) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2290+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2291) : cplan.sql(11835) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2293+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2294) : cplan.sql(11838) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
                  End If;
               Else
                  --trace_printf("****** Many Task Assignments ****** h_bitflag: %d, h_duration: %d\n", h_bitflag,h_duration);
                  --InitializeHost(SC_PARAM(h_bitflag),0);
                  If (Coalesce(h_duration, 0) = 0) OR (Coalesce(h_duration, 0) < Coalesce(h_per_day, 0)) Then
                     h_duration := Coalesce(h_per_day, 0);
                  Else
                     h_duration := Trunc(Coalesce(h_duration, 0) / Coalesce(h_per_day, 0));
                  End If;
                  
                  If Coalesce(h_duration, 0) < 30 Then
                     h_duration := Coalesce(h_duration, 0) * 50;
                  Else
                     h_duration := Trunc(Coalesce(h_duration, 0) * 1.5);
                  End If;
                  
                  /*if ( h_days != -1 )
                  								{
                  									BEGIN_SQL
                  										SELECT TIMESTAMP(HOST(h_task_start_DT)) + HOST(h_days) DAYS
                  										INTO HOST(h_start_date)
                  										FROM DUAL;
                  									END_SQL;
                  								}*/
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     RESOURCE_ID = h_res_id, RESOURCE_TYPE='0', PROJECT_ASSIGNMENT_ID =
                     CASE WHEN h_assigned = 0 THEN NuLL ELSE h_prj_assignment_id END,
                     REGULAR_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_cost  ELSE REGULAR_COST END,
                     SPECIAL_COST = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_cost ELSE SPECIAL_COST END,
                     REGULAR_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_sale  ELSE REGULAR_RATE END,
                     SPECIAL_RATE = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_sale ELSE SPECIAL_RATE END,
                     ROLLUP_BITS  = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_LABOR_BIT)
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2341+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2342) : cplan.sql(11886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               h_work_type := 0;
               h_update := 1;
               h_cri := 0;
               
               If Coalesce(h_work, 0) = 0 Then
                  h_bitflag := PMO_CNST.SC_BIT_27;
               ElsIf isProjectType(Coalesce(h_type_id, 0)) <> 0 Then
                  h_bitflag := PMO_FUNC.BIT_OR(1, PMO_CNST.SC_BIT_27);
               Else
                  h_bitflag := PMO_FUNC.BIT_OR(Coalesce(h_bitflag, 0), PMO_CNST.SC_BIT_27);
               End If;
               
               h_occurrence := 0;
               -- occurrence
               --InitializeHost(SC_PARAM(h_consider), 6);
               
               --trace_printf("AA_2: h_bitflag: %d, h_start_date: %s, h_finish_date: %s, h_duration: %d, h_per_day: %d\n", h_bitflag, h_start_date, h_finish_date, h_duration, h_per_day);
               --trace_printf("h_consider: %d, h_contour: %d, h_work: %d, h_unit: %d, h_effort: %d\n", h_consider, h_contour, h_work, h_unit, h_effort);
               -- assign to the project
               h_temp_res_cost := 0.00;
               i_return := FC_SEARCH_2(h_tsk_assignment_id, h_project_id, h_task_id, h_res_id, h_temp_res_cost
                        , h_type, h_null_str, h_null_str, h_null_str, h_null_str, h_start_date
                        , h_finish_date, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
               
               --trace_printf("assign to the task: i_return = %d, h_count = %d\n", i_return,h_count);
               
               If Coalesce(i_return, 0) = 110 Then
                  BEGIN
                     CLOSE C6a ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2384+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2385) : cplan.sql(11929) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2390+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2391) : cplan.sql(11935) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_UNAVAILABLE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2393+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2394) : cplan.sql(11938) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_UNAVAILABLE, h_res_id, True);
               ElsIf Coalesce(i_return, 0) <> 0 Then
                  BEGIN
                     CLOSE C6a ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2405+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2406) : cplan.sql(11950) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2411+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2412) : cplan.sql(11956) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2414+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2415) : cplan.sql(11959) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                        , True);
               End If;
               /*else
               							{
               								InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30 );
               								h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_tsk_assignment_id), SC_PARAM(h_temp_var),h_per_day, h_effort);
               							}*/
               --}
               
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_profile_name || ';','') WHERE ELEMENT_ID=h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2438+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2439) : cplan.sql(11983) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("2.a__ SC_STATE: %d, h_task_id: %s, h_profile_name: %s\n",SC_STATE,h_task_id,h_profile_name);
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_ROLES
                  SELECT h_tsk_assignment_id, c.ELEMENT_ID, c.PROFICIENCY_LEVEL_ID, c.EXPERIENCE_ID, c.PREFERENCE_LEVEL_ID, c.LAST_USED_DATETIME, '0'
                  FROM TMT_COMPETENCIES c JOIN PROFILES p ON c.ELEMENT_ID = p.ROLE_ID WHERE p.PROFILE_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2458+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2459) : cplan.sql(12003) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("h_tsk_assignment_id = %s, h_profile_id = %s\n",h_tsk_assignment_id,h_profile_id);
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_ROLES
                  SELECT h_tsk_assignment_id, ROLE_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, PRIMARY_ROLE
                  FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2478+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2479) : cplan.sql(12023) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  INSERT INTO RESOURCE_ASSIGN_SKILLS
                  SELECT h_tsk_assignment_id, SKILL_ID, PROFICIENCY_LEVEL_ID, EXPERIENCE_ID, PREFERENCE_LEVEL_ID, LAST_USED_DATETIME, PRIMARY_SKILL
                  FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2497+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2498) : cplan.sql(12042) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_passed := Coalesce(h_passed, 0) + 1;
               h_total := Coalesce(h_total, 0) + Coalesce(h_effort, 0);
               
               h_temp_res_type := '1';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_profile_id, h_temp_res_type);
               
               h_temp_res_type := '0';
               h_temp_sc_state := FC_AFTER_REPLACE_RES_ASSIGN(h_project_id, h_res_id, h_temp_res_type);
               
               BEGIN
                  UPDATE EFFORT_WEEK SET RESOURCE_ID = h_res_id
                  WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2524+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2525) : cplan.sql(12069) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --Update SUMMARY_TIMESHEETS
               If (Coalesce(h_donot_replace_cost, 0) = 0) AND (Coalesce(h_donot_replace_rate, 0) = 0) Then
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     ACTUAL_REGULAR_COST = h_reg_cost  ,
                     ACTUAL_SPECIAL_COST = h_spec_cost ,
                     ACTUAL_REGULAR_RATE = h_reg_sale  ,
                     ACTUAL_SPECIAL_RATE = h_spec_sale ,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2549+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2550) : cplan.sql(12094) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               ElsIf Coalesce(h_donot_replace_cost, 0) = 0 Then
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     ACTUAL_REGULAR_COST = h_reg_cost  ,
                     ACTUAL_SPECIAL_COST = h_spec_cost ,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2570+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2571) : cplan.sql(12115) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_reg_sale := Coalesce(h_old_sale, 0);
               ElsIf Coalesce(h_donot_replace_rate, 0) = 0 Then
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     ACTUAL_REGULAR_RATE = h_reg_sale  ,
                     ACTUAL_SPECIAL_RATE = h_spec_sale ,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2593+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2594) : cplan.sql(12138) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_reg_cost := Coalesce(h_old_cost, 0);
               Else
                  BEGIN
                     UPDATE SUMMARY_TIMESHEETS SET RESOURCE_ID = h_res_id,
                     REC_USER = h_rec_user, REC_DATETIME = SYSTIMESTAMP
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2614+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2615) : cplan.sql(12159) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_reg_cost := Coalesce(h_old_cost, 0);
                  h_reg_sale := Coalesce(h_old_sale, 0);
               End If;
               
               /*InitializeHost(SC_PARAM(h_tph_delete), 0);
               						SC_STATE = FC_TPHASE_SCHED_ASSIGN(
               								SC_PARAM(h_project_id),
               								SC_PARAM(h_task_id),
               								SC_PARAM(h_tsk_assignment_id),
               								SC_PARAM(h_SD),
               								SC_PARAM(h_FD),
               								SC_PARAM(h_reg_cost),
               								SC_PARAM(h_reg_sale),
               								SC_PARAM(h_tph_delete),
               								SC_PARAM(h_capital_expense),
               								SC_PARAM(h_billable));*/
               --trace_printf("SC_STATE: %d, h_reg_cost: %.2f, h_reg_sale: %.2f\n", SC_STATE,h_reg_cost,h_reg_sale);
            End If;
         End If;
      End If;
      
      PMO_DB2M.SC_STATE := FC_D_RA_AFTER_REPLACE(h_project_id, h_profile_id, h_rec_user);
   END LOOP;
   -- end of while
   
   BEGIN
      CLOSE C6a ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2652+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2653) : cplan.sql(12197) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   --trace_printf("*** h_passed = %d\n",h_passed);
   h_count := 0;
   If Coalesce(h_passed, 0) > 0 Then
      h_effort := effort001;
      -- total effort for period
      
      --trace_printf("*** h_effort = %d, h_total = %d\n", h_effort, h_total);
      If Coalesce(h_effort, 0) >= Coalesce(h_total, 0) Then
         h_effort := Coalesce(h_effort, 0) - Coalesce(h_total, 0);
         --trace_printf("*** h_effort = %d, h_assigned = %d, h_prj_assignment_id = %d %s\n",h_effort,h_assigned,h_prj_assignment_id_Ind,h_prj_assignment_id);
         
         If (Coalesce(h_effort, 0) > 0) AND (Coalesce(h_assigned, 0) > 0) 
               AND ((h_prj_assignment_id IS NOT NULL)) Then
            h_start_date := start_date001;
            h_finish_date := finish_date001;
            h_work := work001;
            -- amount for countour
            h_occurrence := occurrence;
            -- occurrence
            h_eunit := eunit;
            h_unit := unit;
            h_bitflag := bitflag001;
            h_contour := contour001;
            
            If Coalesce(contour001, 0) = 0 Then
               h_work := 0;
            End If;
            
            h_temp_var := FC_CALC_DURATION(h_project_id, h_start_date, h_finish_date, h_effort, h_work, h_eunit
                               , h_unit, h_consider, h_bitflag, h_contour, h_occurrence, h_per_day
                               , h_per_week, h_work, h_effort, h_duration);
            
            If Coalesce(h_temp_var, 0) = PMO_CNST.INVALID_PARAMS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_RESTOPROF', 2691+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_RESTOPROF(2692) : cplan.sql(12236) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2694+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2695) : cplan.sql(12239) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
            ElsIf Coalesce(h_temp_var, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2699+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2700) : cplan.sql(12244) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2702+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2703) : cplan.sql(12247) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_project_id, True);
            End If;
            
            -- Temporary Table Declaration has been changed to Truncate
            EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PA_SUM';
            PMO_DB2M.SC_STATE := 0;
            
            BEGIN
               INSERT INTO PMO_TMP_PA_SUM SELECT DAY_YEAR, SUM(SCHED_MIN)
               FROM TASK_DAY_OF_YEAR
               WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id
               GROUP BY DAY_YEAR;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2726+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2727) : cplan.sql(12271) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --              if ( IsNullSpParam(ind_a_restoprof, finish_date))
            --strcpy(h_finish_date, "");
            h_finish_date := '';
            --trace_printf("*** h_start_date = %s, h_finish_date = %s, h_duration = %d\n",h_start_date,h_finish_date,h_duration);
            
            h_work_type := 0;
            h_update := 0;
            h_cri := 0;
            h_type := 1;
            
            If Coalesce(h_work, 0) > 0 Then
               h_work := PMO_FUNC.BIT_OR(Coalesce(h_work, 0), PMO_CNST.SC_BIT_30);
               h_bitflag := 1;
            Else
               h_bitflag := 0;
            End If;
            --trace_printf("Remaining to PA: h_bitflag = %d, h_start_date = %s, h_finish_date = %s, h_duration = %d, h_work_type = %d\n", h_bitflag, h_start_date, h_finish_date, h_duration, h_work_type);
            -- assign to the project
            h_temp_res_cost := 0.00;
            i_return := FC_SEARCH_2(h_prj_assign_id, h_project_id, h_project_id, h_res_id, h_temp_res_cost
                        , h_type, h_null_str, h_null_str, h_null_str, h_null_str, h_start_date
                        , h_finish_date, h_duration, h_effort, h_work, h_eunit, h_unit, h_cri
                        , h_consider, h_res_type, h_work_type, h_bitflag, h_contour, h_occurrence
                        , h_update, h_per_day, h_priority, h_rec_user, 0, 0, 0.00, 0.00);
            
            --trace_printf("assign to the project: i_return = %d, h_count = %d\n", i_return,h_count);
            
            If Coalesce(i_return, 0) = 110 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2759+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2760) : cplan.sql(12304) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RESOURCE_UNAVAILABLE), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2762+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2763) : cplan.sql(12307) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RESOURCE_UNAVAILABLE, h_res_id, True);
            ElsIf Coalesce(i_return, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_A_RESTOPROF', 2767+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_A_RESTOPROF(2768) : cplan.sql(12312) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_RESTOPROF', 2770+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_A_RESTOPROF(2771) : cplan.sql(12315) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                     , True);
            ElsIf Coalesce(h_count, 0) = 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET RES_ASSIGNMENT_ID = h_prj_assign_id,
                  RESOURCE_ID  = h_res_id, RESOURCE_TYPE = '0',
                  REGULAR_COST = CASE WHEN h_donot_replace_rate = 0 THEN h_reg_cost  * h_rate_echange ELSE REGULAR_COST END,
                  SPECIAL_COST = CASE WHEN h_donot_replace_rate = 0 THEN h_spec_cost * h_rate_echange ELSE SPECIAL_COST END,
                  REGULAR_RATE = CASE WHEN h_donot_replace_cost = 0 THEN h_reg_sale  * h_rate_echange ELSE REGULAR_RATE END,
                  SPECIAL_RATE = CASE WHEN h_donot_replace_cost = 0 THEN h_spec_sale * h_rate_echange ELSE SPECIAL_RATE END
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2793+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2794) : cplan.sql(12338) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      End If;
      
      BEGIN
         SELECT COUNT(*) INTO h_count FROM RESOURCE_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id AND PROFILE_ID = h_profile_id AND RESOURCE_TYPE = '1';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2810+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2811) : cplan.sql(12355) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_count: %d, h_project_id: %s, h_profile_name: %s\n", SC_STATE,h_count,h_project_id,h_profile_name);
      
      If Coalesce(h_count, 0) = 0 Then
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE(ASSIGN_NAMES,h_profile_name || ';','')
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2830+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2831) : cplan.sql(12375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   h_from_prj := 0;
   
   --trace_printf("7_* h_old_prj_assign_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_old_prj_assign_id_Ind,h_old_prj_assign_id,h_assigned,h_from_prj);
   If (h_old_prj_assign_id IS NOT NULL) Then
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_ASSIGNMENT_BIT) WHERE ELEMENT_ID = h_project_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2853+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2854) : cplan.sql(12398) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("OLD_WBS_SC_LABOR_BIT SC_STATE: %d\n", SC_STATE); //trace_flush();
      
      If Coalesce(h_assigned, 0) = 0 Then
         BEGIN
            SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS
            WHERE PROJECT_ID = h_project_id AND PROJECT_ASSIGNMENT_ID = h_old_prj_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2869+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2870) : cplan.sql(12414) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("OLD_TA_COUNT h_count: %d\n", h_count);
         
         If Coalesce(h_count, 0) = 0 Then
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND
               ELEMENT_ID = h_project_id AND PROFILE_ID = h_profile_id AND RESOURCE_TYPE = '1';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2889+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2890) : cplan.sql(12434) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2906+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2907) : cplan.sql(12451) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      If Coalesce(h_count, 0) > 0 Then
         -- CALL UPDATE assign string function
         h_temp_var := 3;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_old_prj_assign_id, h_temp_var, 0, 0);
         
         BEGIN
            SELECT PLANNED_START, PLANNED_FINISH, BEST, TO_DATE(SubStr(BAK_START_DT, 1, 10)), TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            INTO h_start_DT, h_finish_DT, h_effort, h_start_D, h_finish_D, h_billable, h_capital_expense, h_old_cost, h_old_sale
            FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_old_prj_assign_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2928+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2929) : cplan.sql(12473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("OLD_* SC_STATE: %d, h_start_D = %d:%s, h_finish_D = %d:%s\n", SC_STATE, h_start_D_Ind,h_start_D, h_finish_D_Ind,h_finish_D);
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            If Coalesce(h_billable, 0) = 0 Then
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_START = h_start_DT,
                  ESTIMATED_PLANNED_FINISH = h_finish_DT,
                  ESTIMATED_PLANNED_EFFORT = h_effort,
                  ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_old_cost,
                  ESTIMATED_PLANNED_REVENUE = 0.00
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2954+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2955) : cplan.sql(12499) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE PROJECT_ASSIGNMENTS SET
                  ESTIMATED_PLANNED_START = h_start_DT,
                  ESTIMATED_PLANNED_FINISH = h_finish_DT,
                  ESTIMATED_PLANNED_EFFORT = h_effort,
                  ESTIMATED_PLANNED_COST = (h_effort/60.00) * h_old_cost,
                  ESTIMATED_PLANNED_REVENUE = (h_effort/60.00) * h_old_sale
                  WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_profile_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 2977+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(2978) : cplan.sql(12522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            --trace_printf("OLD_PA SC_STATE: %d, h_profile_id: %s, h_start_DT = %d:%s, h_finish_DT = %d:%s, h_effort: %d\n", SC_STATE, h_profile_id,h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT,h_effort);
            
            If (SubStr(h_pa_timephase, 1, 1) = 'Y') Then
               h_tph_delete := 0;
               PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_old_prj_assign_id
                                            , h_start_D, h_finish_D, h_old_cost, h_old_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
               --trace_printf("OLD_TPHASE_SCHED_ASSIGN SC_STATE = %d, h_old_prj_assign_id = %s, h_total = %d\n",SC_STATE, h_old_prj_assign_id, h_total);
            End If;
         End If;
         
         If ((h_prj_assignment_id IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_old_prj_assign_id
               , h_prj_assignment_id) = 0) Then
            h_from_prj := 1;
         End If;
      End If;
   End If;
   
   --trace_printf("9_* h_prj_assignment_id: %d:%s, h_assigned: %d, h_from_prj: %d\n",h_prj_assignment_id_Ind,h_prj_assignment_id,h_assigned,h_from_prj);
   If (Coalesce(h_assigned, 0) > 0) AND ((h_prj_assignment_id IS NOT NULL)) Then
      -- CALL UPDATE assign string function
      h_temp_var := 3;
      h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
      
      BEGIN
         SELECT PLANNED_START, PLANNED_FINISH, BEST, TO_DATE(SubStr(BAK_START_DT, 1, 10)), TO_DATE(SubStr(BAK_FINISH_DT, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
         INTO h_start_DT, h_finish_DT, h_effort, h_start_D, h_finish_D, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3016+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3017) : cplan.sql(12561) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("NEW_* SC_STATE: %d, h_start_D = %d:%s, h_finish_D = %d:%s, h_billable: %d\n", SC_STATE, h_start_D_Ind,h_start_D, h_finish_D_Ind,h_finish_D,h_billable);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         If Coalesce(h_billable, 0) = 0 Then
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_PLANNED_START  = h_start_DT,
               ESTIMATED_PLANNED_FINISH = h_finish_DT,
               ESTIMATED_PLANNED_EFFORT = h_effort,
               ESTIMATED_PLANNED_COST   = (h_effort/60.00) * h_reg_cost,
               ESTIMATED_PLANNED_REVENUE = 0.00
               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3042+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3043) : cplan.sql(12587) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_PLANNED_START  = h_start_DT,
               ESTIMATED_PLANNED_FINISH = h_finish_DT,
               ESTIMATED_PLANNED_EFFORT = h_effort,
               ESTIMATED_PLANNED_COST   = (h_effort/60.00) * h_reg_cost,
               ESTIMATED_PLANNED_REVENUE = (h_effort/60.00) * h_reg_sale
               WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3065+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3066) : cplan.sql(12610) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --trace_printf("NEW_PA SC_STATE: %d, h_res_id: %s, h_start_DT = %d:%s, h_finish_DT = %d:%s, h_effort: %d\n", SC_STATE, h_res_id,h_start_DT_Ind,h_start_DT, h_finish_DT_Ind,h_finish_DT,h_effort);
         
         If (SubStr(h_pa_timephase, 1, 1) = 'Y') AND (Coalesce(h_from_prj, 0) = 0) Then
            h_tph_delete := 0;
            PMO_DB2M.SC_STATE := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_project_id, h_prj_assignment_id
                                            , h_start_D, h_finish_D, h_reg_cost, h_reg_sale
                                            , h_tph_delete, h_capital_expense, h_billable);
            --trace_printf("NEW_TPHASE_SCHED_ASSIGN SC_STATE = %d, h_prj_assignment_id = %s\n",SC_STATE, h_prj_assignment_id);
         End If;
      End If;
   End If;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   -- New Rollup
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   --trace_printf("RETURN_STATUS: SC_STATE = %d,h_prj_assignment_id = %s\n",SC_STATE,h_prj_assignment_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_RESTOPROF', 3091+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_RESTOPROF(3092) : cplan.sql(12636) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_prj_assignment_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_prj_assignment_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3098+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3099) : cplan.sql(12643) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_RESTOPROF', 3110+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_RESTOPROF(3111) : cplan.sql(12655) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_RESTOPROF;
ALTER FUNCTION SP_A_RESTOPROF COMPILE DEBUG;

/*###########################################*/

Prompt == FUNCTION SP_U_RATES ==
CREATE OR REPLACE FUNCTION SP_U_RATES
(
     id VARCHAR2
   , reg_cost NUMBER
   , spec_cost NUMBER
   , reg_rate NUMBER
   , spec_rate NUMBER
   , rate_adjustment001 PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_reg_cost NUMBER := 0;
   h_reg_rate NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_rate NUMBER := 0;
   h_rate_adjustment PLS_INTEGER := 0;
   h_flag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_RATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_RATES'), '-- ' || SYSTIMESTAMP, ':r := SP_U_RATES(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(reg_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_cost), 'NULL')
          || ', ' || COALESCE(TO_CHAR(reg_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(spec_rate), 'NULL')
          || ', ' || COALESCE(TO_CHAR(rate_adjustment001), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RATES', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_RATES(35) : cplan.sql(12706) : ' || SYSTIMESTAMP);
   END IF; 

   If (id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_RATES', 39+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_RATES(40) : cplan.sql(12711) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RATES', 42+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_RATES(43) : cplan.sql(12714) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
   End If;
   
   h_flag := 1;
   --update rates_edited
   
   h_id := id;
   h_res_id := res_id001;
   h_reg_cost := reg_cost;
   h_spec_cost := spec_cost;
   h_reg_rate := reg_rate;
   h_spec_rate := spec_rate;
   h_rate_adjustment := rate_adjustment001;
   
   PMO_DB2M.SC_STATE := FC_U_RATES(h_id, h_reg_cost, h_spec_cost, h_reg_rate, h_spec_rate
                                , h_rate_adjustment, h_res_id, h_flag);
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_RATES', 66+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_RATES(67) : cplan.sql(12738) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES', 73+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES(74) : cplan.sql(12745) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES', 85+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES(86) : cplan.sql(12757) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_RATES;
ALTER FUNCTION SP_U_RATES COMPILE DEBUG;

/***************************************************************************************/

Prompt == FUNCTION SP_U_RATES_EDITED ==
CREATE OR REPLACE FUNCTION SP_U_RATES_EDITED
(
     id VARCHAR2
   , rates_edited001 PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_resource_id VARCHAR2(32);
   h_reg_cost NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_reg_rate NUMBER := 0;
   h_spec_rate NUMBER := 0;
   h_flag PLS_INTEGER := 0;
   h_old_reg_cost NUMBER := 0;
   h_old_reg_sale NUMBER := 0;
   h_old_spec_cost NUMBER := 0;
   h_old_spec_rate NUMBER := 0;
   h_rate_id VARCHAR2(32);
   h_rate_adjustment PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_RATES_EDITED') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_RATES_EDITED'), '-- ' || SYSTIMESTAMP, ':r := SP_U_RATES_EDITED(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(rates_edited001), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_RATES_EDITED', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_RATES_EDITED(32) : cplan.sql(12805) : ' || SYSTIMESTAMP);
   END IF; 

   h_resource_id := res_id001;
   
   h_flag := 0;
   
   BEGIN
      SELECT REGULAR_COST, REGULAR_RATE,
      SPECIAL_COST, SPECIAL_RATE, RATE_ADJUSTMENT
      INTO   h_old_reg_cost, h_old_reg_sale ,
      h_old_spec_cost,h_old_spec_rate, h_rate_adjustment
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 51+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(52) : cplan.sql(12825) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT RATE_ID INTO h_rate_id
      FROM RESOURCES
      WHERE RESOURCE_ID = res_id001;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 66+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(67) : cplan.sql(12840) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT REGULAR_COST, REGULAR_SALE,
      SPECIAL_COST, SPECIAL_SALE
      INTO   h_reg_cost, h_reg_rate ,
      h_spec_cost,h_spec_rate
      FROM RATES
      WHERE RATE_ID = h_rate_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 84+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(85) : cplan.sql(12858) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE TASK_ASSIGNMENTS
      SET RATES_EDITED = BITAND(RATES_EDITED,rates_edited001)
      WHERE TSK_ASSIGNMENT_ID = id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 103+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(104) : cplan.sql(12877) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (BITAND(Coalesce(rates_edited001, 0), 1)) > 0 Then
      h_old_reg_cost := h_reg_cost;
      h_old_spec_cost := h_spec_cost;
   End If;
   If (BITAND(Coalesce(rates_edited001, 0), 2)) > 0 Then
      h_old_reg_sale := h_reg_rate;
      h_old_spec_rate := h_spec_rate;
   End If;
   
   PMO_DB2M.SC_STATE := FC_U_RATES(h_id, h_old_reg_cost, h_old_spec_cost, h_old_reg_sale
                                , h_old_spec_rate, h_rate_adjustment, h_resource_id, h_flag);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_RATES_EDITED', 121+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_RATES_EDITED(122) : cplan.sql(12895) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 128+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(129) : cplan.sql(12902) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_RATES_EDITED', 140+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_RATES_EDITED(141) : cplan.sql(12914) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_RATES_EDITED;
ALTER FUNCTION SP_U_RATES_EDITED COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_U_TIMECODE2 ==
CREATE OR REPLACE FUNCTION SP_U_TIMECODE2
(
     assign_id VARCHAR2
   , id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_TIMECODE2') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_TIMECODE2'), '-- ' || SYSTIMESTAMP, ':r := SP_U_TIMECODE2(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TIMECODE2', 17+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_TIMECODE2(18) : cplan.sql(12948) : ' || SYSTIMESTAMP);
   END IF; 

   If ((assign_id IS NULL)) OR ((id IS NULL)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_TIMECODE2', 22+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_TIMECODE2(23) : cplan.sql(12953) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_TIMECODE2', 25+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_TIMECODE2(26) : cplan.sql(12956) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   --Update user record
   BEGIN
      UPDATE TASK_ASSIGNMENTS SET TIMECODE2_ID = id WHERE TSK_ASSIGNMENT_ID = assign_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TIMECODE2', 43+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TIMECODE2(44) : cplan.sql(12974) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_TIMECODE2', 49+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_TIMECODE2(50) : cplan.sql(12980) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || assign_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, assign_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TIMECODE2', 56+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TIMECODE2(57) : cplan.sql(12987) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_TIMECODE2', 68+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_TIMECODE2(69) : cplan.sql(12999) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_TIMECODE2;
ALTER FUNCTION SP_U_TIMECODE2 COMPILE DEBUG;

/****************************************************************************************
* id            bitflag res_id      Description
*----------------------------------------------------------------------------------------
* PROJECT_ID    0       -       WBS Project global view: Resources  Profile with Competencies  Skills
* PROJECT_ID    1       -       WBS Project global view: Resources  Profiles
* PROJECT_ID    6       -       CRI Project global view: Resources  Profiles
* ELEMENT_ID    2,3     -       WBS  CRI - Assignments View
* ELEMENT_ID    4 (new) -           WBS  CRI - All assignments with effort (no participants, no project assignments) in the current project
* ELEMENT_ID    5 (new) -           WBS  CRI - All participants (no effort) in the current project
* ELEMENT_ID    8 (new) -           WBS  CRI - All assignments (effort, participants, project assignments) in the current project
* ELEMENT_ID    9 (new) Res/ProfID          WBS  CRI - All assignments for a Resource / Profile under a element (project, deliverable, task)
* PROJECT_ID    7       -       WBS Project Assignments
* ELEMENT_ID    11      -       DOC Assignments View (SC_TABLE_DOC, SC_TABLE_POL_DOC, SC_TABLE_RES_DOC, SC_TABLE_AST_DOC,SC_TABLE_CLN_DOC,SC_TABLE_VDR_DOC)
* ELEMENT_ID    12      -       WBS  CRI - display nickname for All assignments (effort, participants, project assignments) in the current project
* ELEMENT_ID    13      -       Returns nickname for All assignments (effort, participants, project assignments) in the current pool
*****************************************************************************************/

Prompt == FUNCTION SP_S_PROFILE ==
CREATE OR REPLACE FUNCTION SP_S_PROFILE
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
   h_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_in VARCHAR2(255);
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_level_state PLS_INTEGER := 0;
-- Declare Cursors
      -- Resultset Cursor Declaration
      C7751A PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751B PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751C PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751T PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751D PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751E PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751F PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751G PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751H PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751J PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751L PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751M PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C7751N PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C77513N PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      cln_ast_res PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_PROFILE') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_PROFILE'), '-- ' || SYSTIMESTAMP, ':r := SP_S_PROFILE(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 75+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_PROFILE(76) : cplan.sql(13106) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 82+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(83) : cplan.sql(13113) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 85+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(86) : cplan.sql(13116) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
   End If;
   
   -- Parse parameters to local variables
   
   h_id := id;
   h_bitflag := bitflag001;
   h_rec_user := res_id001;
   --trace_printf("h_id: %d %s\n",h_id_Ind,h_id);//trace_flush();
   
   If (Coalesce(h_bitflag, 0) = 2) OR (Coalesce(h_bitflag, 0) = 3) Then
      BEGIN
         SELECT B.PROJECT_ID, B.TYPE_ID, COALESCE(A.MINUTE_PER_DAY, 480), B.LEVEL_STATE
         INTO h_project_id, h_type_id, h_per_day, h_level_state
         FROM TMT_WBS A, TMT_WBS B WHERE B.ELEMENT_ID = h_id AND A.ELEMENT_ID = B.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 108+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(109) : cplan.sql(13139) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("WBS_SC_STATE: %d, h_type_id: %d, h_project_id: %d %s\n",SC_STATE,h_type_id,h_project_id_Ind,h_project_id); trace_flush();
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         BEGIN
            SELECT B.PROJECT_ID, B.TYPE_ID, COALESCE(A.MINUTE_PER_DAY, 480)
            INTO h_project_id, h_type_id, h_per_day
            FROM TMT_WBS A, TMT_CRI B WHERE B.ELEMENT_ID = h_id AND A.ELEMENT_ID = B.PROJECT_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 125+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(126) : cplan.sql(13156) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("CRI_SC_STATE: %d, h_type_id: %d, h_project_id: %d %s\n",SC_STATE,h_type_id,h_project_id_Ind,h_project_id); trace_flush();
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            BEGIN
               SELECT 480 INTO h_per_day FROM TMT_POOLS WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 140+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(141) : cplan.sql(13171) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 147+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(148) : cplan.sql(13178) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || h_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 150+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(151) : cplan.sql(13181) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, h_id, True);
            End If;
         End If;
      End If;
   ElsIf Coalesce(h_bitflag, 0) = 10 Then
      BEGIN
         SELECT PROJECT_ID INTO h_project_id
         FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 166+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(167) : cplan.sql(13197) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 173+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(174) : cplan.sql(13204) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 176+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(177) : cplan.sql(13207) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
      End If;
      
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), LEVEL_STATE INTO h_per_day, h_level_state
         FROM TMT_WBS WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 190+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(191) : cplan.sql(13221) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 13 Then
      BEGIN
         SELECT RANK INTO h_rank FROM TMT_POOLS WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 203+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(204) : cplan.sql(13234) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 210+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(211) : cplan.sql(13241) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 213+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(214) : cplan.sql(13244) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
      End If;
      
      h_per_day := 480;
   ElsIf (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_SRCH_PARTICIPANT) AND (Coalesce(h_bitflag, 0) <> 11) 
                AND (Coalesce(h_bitflag, 0) <> PMO_CNST.SC_ASSET_ASSIGNED_RESOURCES) AND (Coalesce(
                h_bitflag, 0) <> PMO_CNST.SC_CLIENT_ASSIGNED_RESOURCES) Then
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480), LEVEL_STATE INTO h_per_day, h_level_state
         FROM TMT_WBS WHERE ELEMENT_ID = id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 231+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(232) : cplan.sql(13262) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_S_PROFILE', 238+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_S_PROFILE(239) : cplan.sql(13269) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 241+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(242) : cplan.sql(13272) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
      End If;
   End If;
   --trace_printf("SC_STATE: %d, h_type_id: %d, h_project_id: %d %s\n",SC_STATE,h_type_id,h_project_id_Ind,h_project_id); trace_flush();
   --trace_printf("\n 7390 h_bitflag %d, h_rank: %s",h_bitflag,h_rank); trace_flush();
   
   If Coalesce(h_bitflag, 0) = 0 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_ASGNDRES_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_ASGNDRES_INFO
         (
         RESOURCE_ID,
         PARENT_ID,
         RATE_ID,
         PROFICIENCY_LEVEL_ID,
         PREFERENCE_LEVEL_ID,
         EXPERIENCE_ID,
         LAST_USED_DATETIME
         )
         SELECT DISTINCT
         a.RESOURCE_ID,
         r.PARENT_ID,
         r.RATE_ID,
         r.PROFICIENCY_LEVEL_ID,
         r.PREFERENCE_LEVEL_ID,
         r.EXPERIENCE_ID,
         r.LAST_USED_DATETIME
         FROM RESOURCE_ASSIGNMENTS a JOIN TMT_COMPETENCIES r ON a.RESOURCE_ID=r.RESOURCE_ID
         WHERE PROJECT_ID = h_id AND RESOURCE_TYPE = '0';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 285+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(286) : cplan.sql(13316) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PROF_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_PROF_INFO (PROFILE_ID)
         SELECT DISTINCT PROFILE_ID
         FROM RESOURCE_ASSIGNMENTS
         WHERE PROJECT_ID = h_id AND RESOURCE_TYPE='1';
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 309+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(310) : cplan.sql(13340) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751A FOR SELECT CASE WHEN EXISTS(SELECT * FROM TASK_ASSIGNMENTS ta WHERE ta.PROJECT_ID=h_id AND ta.RESOURCE_ID = RES.RESOURCE_ID) THEN 13 ELSE 185 END AS RES_TYPE,
      RES.RESOURCE_ID, p.FULL_NAME AS ELEMENT_NAME, p.STATUS1_ID, p.E_MAIL_ADDRESS, d.TYPE_ID, d.ELEMENT_NAME AS ROLE_NAME,
      RES.PROFICIENCY_LEVEL_ID, RES.PREFERENCE_LEVEL_ID, RES.EXPERIENCE_ID, RES.LAST_USED_DATETIME AS LAST_USED,rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_ASGNDRES_INFO RES JOIN RESOURCES p ON RES.RESOURCE_ID=p.RESOURCE_ID
      JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=RES.PARENT_ID
      JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=RES.RATE_ID
      
      UNION ALL
      
      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID, p.PROFILE_NAME AS ELEMENT_NAME, 0 AS STATUS1_ID, '' AS E_MAIL_ADDRESS,
      c.TYPE_ID, c.ELEMENT_NAME AS ROLE_NAME,
      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE_LEVEL_ID,r.EXPERIENCE_ID,r.LAST_USED_DATETIME AS LAST_USED,
      rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE,'' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_PROF_INFO PRF  JOIN PROFILES p ON PRF.PROFILE_ID=p.PROFILE_ID
      LEFT OUTER JOIN TMT_COMPETENCIES r ON p.ROLE_ID=r.ELEMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID
      
      UNION ALL
      
      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID,p.PROFILE_NAME AS ELEMENT_NAME,0 AS STATUS1_ID,'' AS E_MAIL_ADDRESS,
      c.TYPE_ID, c.ELEMENT_NAME AS ROLE_NAME,
      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE_LEVEL_ID, r.EXPERIENCE_ID, r.LAST_USED_DATETIME AS LAST_USED,
      rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_PROF_INFO PRF JOIN PROFILES p ON PRF.PROFILE_ID=p.PROFILE_ID
      JOIN RESOURCE_ASSIGN_SKILLS s ON p.PROFILE_ID=s.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES r ON s.SKILL_ID=r.ELEMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID
      
      UNION ALL
      
      SELECT 41 AS RES_TYPE,PRF.PROFILE_ID AS RESOURCE_ID,p.PROFILE_NAME AS ELEMENT_NAME,0 AS STATUS1_ID,'' AS E_MAIL_ADDRESS,
      c.TYPE_ID, c.ELEMENT_NAME AS ROLE_NAME,
      r.PROFICIENCY_LEVEL_ID, r.PREFERENCE_LEVEL_ID, r.EXPERIENCE_ID, r.LAST_USED_DATETIME AS LAST_USED,
      rc.REGULAR_COST,rc.REGULAR_SALE,rc.SPECIAL_COST,rc.SPECIAL_SALE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_PROF_INFO PRF JOIN PROFILES p ON PRF.PROFILE_ID=p.PROFILE_ID
      JOIN RESOURCE_ASSIGN_ROLES s ON p.PROFILE_ID=s.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES r ON s.ROLE_ID=r.ELEMENT_ID
      LEFT OUTER JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.PARENT_ID
      LEFT OUTER JOIN RATES rc ON rc.RATE_ID=r.RATE_ID
      ORDER BY RES_TYPE, RESOURCE_ID, TYPE_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 358+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(359) : cplan.sql(13389) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751A;
   ElsIf (Coalesce(h_bitflag, 0) = 1) OR (Coalesce(h_bitflag, 0) = 6) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_DATA';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_DATA (RESOURCE_ID, TYPE_ID)
         SELECT DISTINCT
         ra.RESOURCE_ID,
         CASE WHEN ta.TSK_ASSIGNMENT_ID IS NuLL THEN 185 ELSE 13 END AS TYPE_ID
         FROM RESOURCE_ASSIGNMENTS ra LEFT OUTER JOIN TASK_ASSIGNMENTS ta ON ta.PROJECT_ID=h_id AND ra.RESOURCE_ID = ta.RESOURCE_ID
         WHERE ra.PROJECT_ID=h_id AND ra.RESOURCE_TYPE='0' AND ta.PROJECT_ASSIGNMENT_ID IS NuLL AND
         NOT EXISTS(SELECT * FROM TASK_ASSIGNMENTS t WHERE ra.RESOURCE_ID = t.RESOURCE_ID AND t.TASK_ID=h_id);
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 384+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(385) : cplan.sql(13415) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME)
         SELECT DISTINCT r.PROFILE_ID ,r.PROFILE_NAME
         FROM RESOURCE_ASSIGNMENTS a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.PROFILE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID JOIN TASK_ASSIGNMENTS t ON t.RES_ASSIGNMENT_ID=a.RES_ASSIGNMENT_ID AND t.PROJECT_ASSIGNMENT_ID IS NuLL
         WHERE a.PROJECT_ID=h_id AND a.ELEMENT_TYPE>1;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 407+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(408) : cplan.sql(13438) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751B FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,a.TYPE_ID,p.E_MAIL_ADDRESS,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_START, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_FINISH,0 AS PLANNED_EFFORT,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DATE, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BEST_FINISH,
      0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_RES_DATA a JOIN RESOURCES p ON a.RESOURCE_ID=p.RESOURCE_ID
      
      UNION ALL
      
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_START, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_FINISH,0 AS PLANNED_EFFORT,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DATE, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP
      ORDER BY TYPE_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 427+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(428) : cplan.sql(13458) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751B;
   ElsIf Coalesce(h_bitflag, 0) = 2 Then
      OPEN C7751C FOR SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,
      a.PROFILE_ID, p.PROFILE_NAME AS ELEMENT_NAME,41 AS TYPE_ID,a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.PLANNED_START,t.PLANNED_FINISH,
      t.PLANNED_WORK,t.START_DT,t.FINISH_DT,h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
      COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,'' AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,
      t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID,TC.ELEMENT_NAME AS TIMECODE2, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, BILLABLE,
      t.ASSIGNMENT_DELAY
      FROM RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON t.TASK_ID = h_id AND a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN PROFILES p ON a.PROFILE_ID=p.PROFILE_ID LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
      LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
      WHERE a.ELEMENT_ID = h_id AND a.RESOURCE_TYPE='1' ORDER BY ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 443+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(444) : cplan.sql(13474) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751C;
   ElsIf Coalesce(h_bitflag, 0) = 3 Then
      If isTask(Coalesce(h_type_id, 0)) <> 0 Then
         h_count := 0;
         BEGIN
            SELECT COUNT(*) INTO h_count FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TASK_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 457+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(458) : cplan.sql(13488) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) = 0 Then
            h_bit_flag := '1';
         Else
            BEGIN
               SELECT COUNT('X') INTO h_count FROM DUAL WHERE EXISTS (
               SELECT 'X' FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND TASK_ID = h_id AND BIT_FLAG = '0');
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 474+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(475) : cplan.sql(13505) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) = 0 Then
               BEGIN
                  SELECT COUNT('X') INTO h_count FROM DUAL WHERE EXISTS (
                  SELECT 'X' FROM TASK_ASSIGNMENTS ta JOIN TIMESHEET_APPROVAL_STATUS apv
                  ON apv.ELEMENT_ID = ta.PROJECT_ID AND apv.RESOURCE_ID = ta.RESOURCE_ID
                  WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID = h_id AND
                  (TO_DATE(SubStr(ta.ACTUAL_START, 1, 10)) - 7     ) <= apv.WEEK_OF);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 492+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(493) : cplan.sql(13523) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_count, 0) = 0 Then
                  h_bit_flag := '2';
               Else
                  h_bit_flag := '1';
               End If;
            Else
               h_bit_flag := '1';
            End If;
         End If;
         --trace_printf("h_project_id: %s,h_id: %s, h_bit_flag: %s, h_level_state: %d\n",h_project_id,h_id,h_bit_flag,h_level_state);
         
         --at the task level in staffing view return empty string as security is and name
         --trace_printf("1___SC_WBS_TASK__SC_CRI_TASK__________(h_bitflag == 3)\n");
         
         OPEN C7751T FOR SELECT t.ELEMENT_ID, a.RESOURCE_ID, t.ELEMENT_NAME, t.TYPE_ID, '' AS RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT,t.FINISH_DT,
         t.SD, t.FD, t.START_DT AS PLANNED_START, t.FINISH_DT AS PLANNED_FINISH,
         t.SCHED_EFFORT + t.PLANNED_EFFORT AS PLANNED_WORK, t.SCHED_EFFORT + t.PLANNED_EFFORT - t.WORK_TODATE AS REMAINING,
         0 UNITS_PERCENT, t.MILESTONE_PERCENT,
         CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(t.SCHED_EFFORT + t.PLANNED_EFFORT))*10000) END AS WORK_PERCENT,
         TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_START_DT,TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_FINISH_DT,0 AS BAK_WORK,
         t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
         t.WORK_TODATE AS ACTUAL,t.PROPOSED_START, t.PROPOSED_FINISH, t.PROPOSED_EFFORT AS PROPOSED_WORK,
         h_per_day AS MINUTE_PER_DAY, 0 AS WORK_LOCATION, '' AS SECURITY_ID, '' AS SECURITY_NAME,
         '' AS CRI_SECURITY_ID, '' AS CRI_SECURITY_NAME, 0.00 AS REGULAR_COST,0.00 AS REGULAR_RATE,0.00 AS SPECIAL_COST,0.00 AS SPECIAL_RATE,
         '' AS FS_LABOR_CODE,t.REC_STATUS,'' AS ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_EFFORT AS BASELINE1_WORK,
         '' AS PROJECT_ASSIGNMENT_ID,'' AS TIMECODE2_ID, '' AS TIMECODE2_NAME, 0 AS T_UNIT, 0 AS C_EFFORT, 0 AS C_UNIT, 0 AS C_CONTOUR, 0 AS CONTOUR_TYPE, 0 AS C_OCCURRENCE, 0 AS BILLABLE, 0 AS CAPITAL_EXPENSE,
         0 AS ASSIGNMENT_DELAY, h_bit_flag AS BIT_FLAG, 0 AS RATE_ADJUSTMENT, t.ASSIGN_COUNT, t.UNRESOLVED_DURATION AS DURATION,
         CASE WHEN h_level_state = 1 THEN t.LEVEL_START ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_START,
         CASE WHEN h_level_state = 1 THEN t.LEVEL_FINISH ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_FINISH,
         CASE WHEN h_level_state = 1 THEN t.LEVEL_DURATION ELSE 0 END AS LEVEL_EFFORT, t.WORK_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID, '0' AS LEVEL_AFFECTED_FLAG
         FROM TMT_WBS t LEFT JOIN RESOURCE_ASSIGNMENTS a ON a.PROJECT_ID = t.PROJECT_ID AND a.ELEMENT_ID = t.ELEMENT_ID AND a.RESOURCE_ID = h_rec_user
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
         WHERE t.PROJECT_ID = h_project_id AND t.ELEMENT_ID = h_id
         UNION ALL
         SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,
         CASE WHEN a.RESOURCE_TYPE='1' THEN a.PROFILE_ID ELSE a.RESOURCE_ID END AS RESOURCE_ID,
         CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.PROFILE_ID = p.PROFILE_ID)
         ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
         CASE WHEN a.RESOURCE_TYPE='1' THEN 41 WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN 13 ELSE 185 END AS TYPE_ID,
         a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT, t.UNRESOLVED_FINISH AS FINISH_DT,
         CASE WHEN ((t.SD = PMO_CNST.SC_UFS) OR (t.SD = PMO_CNST.SC_FS)) THEN PMO_CNST.SC_SS ELSE t.SD END AS SD, CASE WHEN t.FD = PMO_CNST.SC_FF THEN PMO_CNST.SC_SF ELSE t.FD END AS FD, t.UNRESOLVED_START AS PLANNED_START, t.UNRESOLVED_FINISH AS PLANNED_FINISH,
         CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END AS PLANNED_WORK,
         CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK - WORK_TODATE ELSE PLANNED_WORK END AS REMAINING,
         CASE WHEN t.ASSIGN_UNITS > t.UNITS_MSP THEN t.ASSIGN_UNITS ELSE t.UNITS_MSP END AS UNITS_PERCENT,
         0 AS MILESTONE_PERCENT,
         CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(PLANNED_WORK+SCHED_WORK))*10000) END AS WORK_PERCENT,
         t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,
         t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
         t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,
         h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
         COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,
         CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
         ELSE '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,
         t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENSE,
         t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUNT, t.UNRESOLVED_DURATION AS DURATION,
         CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_START,t.UNRESOLVED_START) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_START,
         CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_FINISH,t.UNRESOLVED_FINISH) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_FINISH,
         CASE WHEN h_level_state = 1 THEN CASE WHEN t.LEVEL_EFFORT = 0 THEN t.ETC ELSE t.LEVEL_EFFORT END ELSE 0 END AS LEVEL_EFFORT, t.MILESTONE_PERCENT AS PERCENT_DISCRETE, 1 as TOP_ID, '0' AS LEVEL_AFFECTED_FLAG
         FROM RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
         LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
         WHERE a.PROJECT_ID = h_project_id AND a.ELEMENT_ID = h_id AND a.REC_STATUS NOT IN ('L','D')
         ORDER BY TOP_ID, ELEMENT_NAME;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 562+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(563) : cplan.sql(13593) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C7751T;
      Else
         --trace_printf("2_______________(h_bitflag == 3)\n");
         
         OPEN C7751D FOR SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,
         CASE WHEN a.RESOURCE_TYPE='1' THEN a.PROFILE_ID ELSE a.RESOURCE_ID END AS RESOURCE_ID,
         CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.PROFILE_ID = p.PROFILE_ID)
         ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
         CASE WHEN a.RESOURCE_TYPE='1' THEN 41 WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN 13 ELSE 185 END AS TYPE_ID,
         a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT,t.FINISH_DT,
         t.SD, t.FD, t.UNRESOLVED_START AS PLANNED_START, t.UNRESOLVED_FINISH AS PLANNED_FINISH,
         BEST AS PLANNED_WORK, CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END AS REMAINING,
         t.UNITS_MSP AS UNITS_PERCENT, t.MILESTONE_PERCENT, 0 AS WORK_PERCENT,
         t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,
         t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
         t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,
         h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
         COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,
         CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
         ELSE '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,
         t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENSE,
         t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUNT, 0 AS DURATION,
         t.LEVEL_START, t.LEVEL_FINISH, t.LEVEL_EFFORT, t.MILESTONE_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID,
         CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL AND t.PROJECT_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID AND (t.LEVEL_START <> t.UNRESOLVED_START OR t.LEVEL_FINISH <> t.UNRESOLVED_FINISH) THEN '1' ELSE '0' END AS LEVEL_AFFECTED_FLAG
         FROM RESOURCE_ASSIGNMENTS a LEFT OUTER JOIN TASK_ASSIGNMENTS t ON a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
         LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
         LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
         WHERE a.REC_STATUS NOT IN ('L','D') AND a.ELEMENT_ID = h_id
         ORDER BY TYPE_ID, ELEMENT_NAME;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 595+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(596) : cplan.sql(13626) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C7751D;
      End If;
   ElsIf Coalesce(h_bitflag, 0) = 4 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_SET';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_SET
         (
         RESOURCE_ID,
         RESOURCE_TYPE ,
         PLANNED_START ,
         PLANNED_FINISH ,
         PLANNED_EFFORT ,
         START_DATE ,
         FINISH_DATE ,
         WORK ,
         REMAINING
         )
         SELECT
         RESOURCE_ID,
         RESOURCE_TYPE,
         MIN(PLANNED_START) AS PLANNED_START,
         MAX(PLANNED_FINISH) AS PLANNED_FINISH,
         SUM(PLANNED_WORK) AS PLANNED_EFFORT,
         MIN(SCHED_START) AS START_DATE,
         MAX(SCHED_FINISH) AS FINISH_DATE,
         SUM(SCHED_WORK) AS WORK,
         SUM(ETC) AS REMAINING
         FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=h_id AND PROJECT_ASSIGNMENT_ID IS NuLL GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 638+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(639) : cplan.sql(13669) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME, PLANNED_START, PLANNED_FINISH, PLANNED_EFFORT,START_DATE, FINISH_DATE,
         WORK, REMAINING)
         SELECT DISTINCT r.PROFILE_ID,r.PROFILE_NAME,
         a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING
         FROM PMO_TMP_RES_SET a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 662+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(663) : cplan.sql(13693) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751E FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_RES_SET a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      
      UNION ALL
      
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      PLANNED_START, PLANNED_FINISH,PLANNED_EFFORT,   START_DATE, FINISH_DATE, WORK, REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP ORDER BY TYPE_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 679+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(680) : cplan.sql(13710) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751E;
   ElsIf Coalesce(h_bitflag, 0) = 5 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_SET';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_SET
         (
         RESOURCE_ID,
         RESOURCE_TYPE ,
         PLANNED_START ,
         PLANNED_FINISH ,
         PLANNED_EFFORT ,
         START_DATE ,
         FINISH_DATE ,
         WORK ,
         REMAINING
         )
         SELECT
         RESOURCE_ID,
         RESOURCE_TYPE,
         MIN(PLANNED_START) AS PLANNED_START,
         MAX(PLANNED_FINISH) AS PLANNED_FINISH,
         SUM(PLANNED_WORK) AS PLANNED_EFFORT,
         MIN(SCHED_START) AS START_DATE,
         MAX(SCHED_FINISH) AS FINISH_DATE,
         SUM(SCHED_WORK) AS WORK,
         SUM(ETC) AS REMAINING
         FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=h_id AND PROJECT_ASSIGNMENT_ID IS NuLL GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 721+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(722) : cplan.sql(13752) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      PMO_DB2M.SC_STATE := FC_U_CREATETMPTABLE();
      
      BEGIN
         INSERT INTO PMO_TMP_CPLN01_TEMP(ELEMENT_ID,ELEMENT_NAME, RESOURCE_TYPE, TYPE_ID, E_MAIL_ADDRESS, PLANNED_START, PLANNED_FINISH, PLANNED_EFFORT,START_DATE, FINISH_DATE,
         WORK, REMAINING, FS_LABOR_CODE)
         SELECT DISTINCT r.PROFILE_ID ,r.PROFILE_NAME ,'1',41 ,'',
         a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,
         ''
         FROM PMO_TMP_RES_SET a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
         JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID
         UNION ALL
         SELECT DISTINCT ra.RESOURCE_ID ,p.FULL_NAME ,'0',185 ,p.E_MAIL_ADDRESS,
         TO_TIMESTAMP('1899-12-31 00:00:00') , TO_TIMESTAMP('1899-12-31 00:00:00') ,0 ,
         TO_TIMESTAMP('1899-12-31 00:00:00') , TO_TIMESTAMP('1899-12-31 00:00:00') , 0 , 0 ,p.FS_LABOR_CODE
         FROM RESOURCE_ASSIGNMENTS ra, RESOURCES p
         WHERE ra.PROJECT_ID=h_id AND ra.RESOURCE_ID = p.RESOURCE_ID
         AND NOT EXISTS(SELECT ta.TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta
         WHERE ta.RES_ASSIGNMENT_ID IN (SELECT a.RES_ASSIGNMENT_ID
         FROM RESOURCE_ASSIGNMENTS a WHERE a.PROJECT_ID = ra.PROJECT_ID AND a.RESOURCE_ID = ra.RESOURCE_ID));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 755+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(756) : cplan.sql(13786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751F FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_RES_SET a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      UNION ALL
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,TYPE_ID,E_MAIL_ADDRESS,
      PLANNED_START, PLANNED_FINISH,PLANNED_EFFORT,   START_DATE, FINISH_DATE, WORK, REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING, FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP WHERE RESOURCE_TYPE='1' AND TYPE_ID = 41
      UNION ALL
      SELECT ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, ELEMENT_NAME,TYPE_ID,E_MAIL_ADDRESS,
      PLANNED_START, PLANNED_FINISH,PLANNED_EFFORT,
      START_DATE, FINISH_DATE, WORK, REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BEST_FINISH,
      0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK
      FROM PMO_TMP_CPLN01_TEMP
      WHERE RESOURCE_TYPE = '0' AND TYPE_ID = 185
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 778+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(779) : cplan.sql(13809) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751F;
      
      /*BEGIN_SQL
      			DECLARE GLOBAL TEMPORARY TABLE PMO_TMP_RES_SET(
      				RESOURCE_ID             CHAR(32),
      				PROJECT_ASSIGNMENT_ID   CHAR(32),
      				PLANNED_START           TIMESTAMP,
      				PLANNED_FINISH          TIMESTAMP,
      				PLANNED_EFFORT          BIGINT,
      				START_DATE              TIMESTAMP,
      				FINISH_DATE             TIMESTAMP,
      				WORK                    BIGINT,
      				REMAINING               BIGINT
      			) WITH REPLACE ON COMMIT PRESERVE ROWS NOT LOGGED IN USRTS32K;
      		END_SQL_NOCHECK;
      
      		BEGIN_SQL
      			INSERT INTO session.PMO_TMP_RES_SET
      			(
      			  RESOURCE_ID,
      			  PROJECT_ASSIGNMENT_ID,
      			  PLANNED_START,
      			  PLANNED_FINISH,
      			  PLANNED_EFFORT,
      			  START_DATE,
      			  FINISH_DATE,
      			  WORK,
      			  REMAINING
      			)
      				SELECT
      				RESOURCE_ID,
      				PROJECT_ASSIGNMENT_ID,
      				MIN(PLANNED_START) AS PLANNED_START,
      				MAX(PLANNED_FINISH) AS PLANNED_FINISH,
      				SUM(PLANNED_WORK) AS PLANNED_EFFORT,
      				MIN(SCHED_START) AS START_DATE,
      				MAX(SCHED_FINISH) AS FINISH_DATE,
      				SUM(SCHED_WORK) AS WORK,
      				SUM(ETC) AS REMAINING
      				FROM TASK_ASSIGNMENTS WHERE PROJECT_ID=HOST(h_id) AND RESOURCE_TYPE = '0' GROUP BY RESOURCE_ID, PROJECT_ASSIGNMENT_ID;
      		END_SQL;
      
      		BEGIN_SQL
      			DECLARE C7751F CURSOR WITH HOLD FOR
      
      			SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID,p.E_MAIL_ADDRESS,
      			a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,HOST(h_per_day) AS MINUTE_PER_DAY,
      			'' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,
      			'' AS RANK, a.PROJECT_ASSIGNMENT_ID
      			FROM session.PMO_TMP_RES_SET a JOIN RESOURCES p ON a.RESOURCE_ID=p.RESOURCE_ID
      			UNION ALL
      			SELECT DISTINCT ra.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME,185 AS TYPE_ID,p.E_MAIL_ADDRESS,
      			SC_1899_12_31_00_00_00 AS PLANNED_START, SC_1899_12_31_00_00_00 AS PLANNED_FINISH,0 AS PLANNED_EFFORT,
      			SC_1899_12_31_00_00_00 AS START_DATE, SC_1899_12_31_00_00_00 AS FINISH_DATE, 0 AS WORK, 0 AS REMAINING,0 AS T_UNIT,HOST(h_per_day) AS MINUTE_PER_DAY,'' AS BEST_START,'' AS BEST_FINISH,
      			0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,
      			'' AS RANK, '' AS PROJECT_ASSIGNMENT_ID
      			FROM RESOURCE_ASSIGNMENTS ra, RESOURCES p
      			WHERE ra.PROJECT_ID=HOST(h_id) AND ra.RESOURCE_ID = p.RESOURCE_ID
      				AND NOT EXISTS(SELECT ta.TSK_ASSIGNMENT_ID FROM TASK_ASSIGNMENTS ta
      					WHERE ta.RES_ASSIGNMENT_ID IN (SELECT a.RES_ASSIGNMENT_ID
      						FROM RESOURCE_ASSIGNMENTS a WHERE a.PROJECT_ID = ra.PROJECT_ID AND a.RESOURCE_ID = ra.RESOURCE_ID))
      			ORDER BY TYPE_ID DESC, ELEMENT_NAME FOR READ ONLY;
      		END_SQL_NOCHECK;
      
      		BEGIN_SQL
      			RETURN_CURSOR(C7751F);
      		END_SQL;*/
   ElsIf Coalesce(h_bitflag, 0) = 7 Then
      OPEN C7751G FOR SELECT t.RESOURCE_ID AS ELEMENT_ID, t.RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 13 AS TYPE_ID,p.E_MAIL_ADDRESS,
      b.PLANNED_START, b.PLANNED_FINISH, b.BEST AS PLANNED_EFFORT, b.PLANNED_WORK AS PLANNED_REMAINING, a.BAK_START_DT AS START_DATE, a.BAK_FINISH_DT AS FINISH_DATE,
      a.BEST AS WORK, a.SCHED_WORK AS REMAINING, h_per_day AS MINUTE_PER_DAY,'' AS BEST_START, '' AS BEST_FINISH,
      0 AS CALENDAR,0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE, pa.DESCRIPTION,
      pa.LEVEL_PLANNED_START, pa.LEVEL_PLANNED_FINISH, pa.LEVEL_SCHED_START, pa.LEVEL_SCHED_FINISH,
      pa.ESTIMATED_BASELINE1_START, pa.ESTIMATED_BASELINE1_FINISH, pa.ESTIMATED_BASELINE1_EFFORT,
      pa.ESTIMATED_BASELINE3_START, pa.ESTIMATED_BASELINE3_FINISH, pa.ESTIMATED_BASELINE3_EFFORT
      FROM RESOURCE_ASSIGNMENTS t JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID = h_id AND pa.RESOURCE_ID = t.RESOURCE_ID
      LEFT JOIN TASK_ASSIGNMENTS a ON a.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND a.BIT_FLAG='2'
      LEFT JOIN TASK_ASSIGNMENTS b ON b.RES_ASSIGNMENT_ID=t.RES_ASSIGNMENT_ID AND b.BIT_FLAG='0'
      JOIN RESOURCES p ON t.RESOURCE_ID=p.RESOURCE_ID
      WHERE t.ELEMENT_ID=h_id AND t.RESOURCE_TYPE='0' AND (a.TSK_ASSIGNMENT_ID IS NOT NuLL OR b.TSK_ASSIGNMENT_ID IS NOT NuLL)
      UNION ALL
      SELECT a.PROFILE_ID AS ELEMENT_ID, a.RES_ASSIGNMENT_ID, p.PROFILE_NAME AS ELEMENT_NAME, 41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      t.PLANNED_START, t.PLANNED_FINISH,t.BEST AS PLANNED_EFFORT,t.PLANNED_WORK AS PLANNED_REMAINING,TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DATE, TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DATE,
      0 AS WORK,0 AS REMAINING,h_per_day AS MINUTE_PER_DAY,'' AS BEST_START, '' AS BEST_FINISH,
      0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, t.ASSIGN_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      pa.REGULAR_COST, pa.REGULAR_RATE, pa.SPECIAL_COST, pa.SPECIAL_RATE, pa.DESCRIPTION,
      pa.LEVEL_PLANNED_START, pa.LEVEL_PLANNED_FINISH, pa.LEVEL_SCHED_START, pa.LEVEL_SCHED_FINISH,
      pa.ESTIMATED_BASELINE1_START, pa.ESTIMATED_BASELINE1_FINISH, pa.ESTIMATED_BASELINE1_EFFORT,
      pa.ESTIMATED_BASELINE3_START, pa.ESTIMATED_BASELINE3_FINISH, pa.ESTIMATED_BASELINE3_EFFORT
      FROM RESOURCE_ASSIGNMENTS a JOIN PROFILES p ON a.PROFILE_ID=p.PROFILE_ID
      JOIN TASK_ASSIGNMENTS t ON t.RES_ASSIGNMENT_ID=a.RES_ASSIGNMENT_ID
      JOIN PROJECT_ASSIGNMENTS pa ON pa.PROJECT_ID = h_id AND pa.RESOURCE_ID = a.PROFILE_ID
      WHERE a.ELEMENT_ID=h_id AND a.RESOURCE_TYPE='1'
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 876+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(877) : cplan.sql(13907) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751G;
   ElsIf Coalesce(h_bitflag, 0) = 8 Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_PRJ_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_PRJ_INFO
         (RESOURCE_ID,
         RESOURCE_TYPE,
         PRJ_PLANNED_START,
         PRJ_PLANNED_FINISH,
         PLANNED_REMAINING,
         PLANNED_EFFORT,
         PRJ_SCHED_START ,
         PRJ_SCHED_FINISH,
         SCHED_REMAINING,
         WORK,
         TYPE_ID
         )
         SELECT
         RESOURCE_ID,
         RESOURCE_TYPE,
         MIN(PLANNED_START)  AS PRJ_PLANNED_START,
         MAX(PLANNED_FINISH) AS PRJ_PLANNED_FINISH,
         SUM(PLANNED_WORK)   AS PLANNED_REMAINING,
         SUM(CASE WHEN BIT_FLAG='0' THEN BEST ELSE 0 END) AS PLANNED_EFFORT,
         MIN(SCHED_START)    AS PRJ_SCHED_START,
         MAX(SCHED_FINISH)   AS PRJ_SCHED_FINISH,
         SUM(SCHED_WORK)     AS SCHED_REMAINING,
         SUM(CASE WHEN BIT_FLAG='2' THEN BEST ELSE 0 END) AS WORK, 13
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID=h_id AND TASK_ID=h_id
         GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 922+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(923) : cplan.sql(13953) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("I1_* SC_STATE: %d, h_project_id: %s\n", SC_STATE,h_project_id);
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RES_INFO';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RES_INFO
         (
         RESOURCE_ID,
         RESOURCE_TYPE,
         PLANNED_START,
         PLANNED_FINISH,
         PLANNED_EFFORT,
         START_DATE,
         FINISH_DATE,
         WORK,
         REMAINING,
         PRJ_PLANNED_START,
         PRJ_PLANNED_FINISH,
         PLANNED_REMAINING,
         PRJ_SCHED_START,
         PRJ_SCHED_FINISH,
         SCHED_REMAINING,
         LEVEL_PLANNED_START,
         LEVEL_PLANNED_FINISH,
         LEVEL_SCHED_START,
         LEVEL_SCHED_FINISH,
         ESTIMATED_BASELINE1_START,
         ESTIMATED_BASELINE1_FINISH,
         ESTIMATED_BASELINE1_EFFORT,
         ESTIMATED_BASELINE3_START,
         ESTIMATED_BASELINE3_FINISH,
         ESTIMATED_BASELINE3_EFFORT,
         REGULAR_COST,
         REGULAR_RATE,
         SPECIAL_COST,
         SPECIAL_RATE,
         TYPE_ID,
         DESCRIPTION
         )
         SELECT
         a.RESOURCE_ID,      a.RESOURCE_TYPE,        a.PLANNED_START,        a.PLANNED_FINISH,
         a.PLANNED_WORK,     a.SCHED_START,          a.SCHED_FINISH,         a.SCHED_WORK,
         a.ETC,              b.PRJ_PLANNED_START,    b.PRJ_PLANNED_FINISH,   b.PLANNED_REMAINING,
         b.PRJ_SCHED_START,  b.PRJ_SCHED_FINISH,     b.SCHED_REMAINING,
         a.LEVEL_PLANNED_START, a.LEVEL_PLANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,
         a.ESTIMATED_BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,
         a.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATED_BASELINE3_EFFORT,
         a.REGULAR_COST,     a.REGULAR_RATE,         a.SPECIAL_COST,         a.SPECIAL_RATE,
         COALESCE(b.TYPE_ID,CASE WHEN (a.PLANNED_WORK+a.SCHED_WORK) = 0 THEN 185 ELSE 13 END), a.DESCRIPTION
         FROM PROJECT_ASSIGNMENTS a LEFT OUTER JOIN PMO_TMP_PRJ_INFO b ON a.RESOURCE_ID = b.RESOURCE_ID
         WHERE a.PROJECT_ID=h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 989+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(990) : cplan.sql(14020) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("I2_* SC_STATE: %d, h_id: %s\n", SC_STATE,h_id);
      
      BEGIN
         UPDATE PMO_TMP_RES_INFO RI SET TYPE_ID = 13 WHERE TYPE_ID = 185 AND EXISTS(
         SELECT TASK_ID FROM TASK_ASSIGNMENTS T WHERE T.PROJECT_ID=h_id AND T.RESOURCE_ID = RI.RESOURCE_ID AND T.RESOURCE_TYPE = '0');
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1008+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1009) : cplan.sql(14039) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751H FOR SELECT a.RESOURCE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, p.FULL_NAME AS ELEMENT_NAME, a.TYPE_ID,p.E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,'' AS BEST_FINISH, 0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,p.FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      CASE WHEN a.PRJ_PLANNED_START IS NOT NuLL OR a.PRJ_SCHED_START IS NOT NuLL THEN a.RESOURCE_ID ELSE '' END AS PROJECT_ASSIGNMENT_ID, a.PRJ_PLANNED_START, a.PRJ_PLANNED_FINISH, a.PLANNED_REMAINING, a.PRJ_SCHED_START, a.PRJ_SCHED_FINISH, a.SCHED_REMAINING,
      h_id AS PROJECT_ID,
      a.LEVEL_PLANNED_START, a.LEVEL_PLANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,
      a.ESTIMATED_BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,
      a.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATED_BASELINE3_EFFORT,
      a.REGULAR_COST, a.REGULAR_RATE, a.SPECIAL_COST, a.SPECIAL_RATE, a.DESCRIPTION
      FROM PMO_TMP_RES_INFO a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      UNION ALL
      SELECT  r.PROFILE_ID AS ELEMENT_ID,'' AS RES_ASSIGNMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME,41 AS TYPE_ID,'' AS E_MAIL_ADDRESS,
      a.PLANNED_START, a.PLANNED_FINISH,a.PLANNED_EFFORT, a.START_DATE, a.FINISH_DATE, a.WORK, a.REMAINING,0 AS T_UNIT,h_per_day AS MINUTE_PER_DAY,
      '' AS BEST_START,''  AS BEST_FINISH,0 AS CALENDAR, 0 AS PLANNED_WORK, 0 AS PERIOD_AVAILABILITY, '' AS ASSIGN_STRING,'' AS FS_LABOR_CODE, '' AS OBJECTIVES,'' AS RANK,
      CASE WHEN a.PRJ_PLANNED_START IS NOT NuLL OR a.PRJ_SCHED_START IS NOT NuLL THEN a.RESOURCE_ID ELSE '' END AS PROJECT_ASSIGNMENT_ID, a.PRJ_PLANNED_START, a.PRJ_PLANNED_FINISH, a.PLANNED_REMAINING, a.PRJ_SCHED_START, a.PRJ_SCHED_FINISH, a.SCHED_REMAINING,
      h_id AS PROJECT_ID,
      a.LEVEL_PLANNED_START, a.LEVEL_PLANNED_FINISH, a.LEVEL_SCHED_START, a.LEVEL_SCHED_FINISH,
      a.ESTIMATED_BASELINE1_START, a.ESTIMATED_BASELINE1_FINISH, a.ESTIMATED_BASELINE1_EFFORT,
      a.ESTIMATED_BASELINE3_START, a.ESTIMATED_BASELINE3_FINISH, a.ESTIMATED_BASELINE3_EFFORT,
      a.REGULAR_COST, a.REGULAR_RATE, a.SPECIAL_COST, a.SPECIAL_RATE, a.DESCRIPTION
      FROM PMO_TMP_RES_INFO a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID
      JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
      JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1037+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1038) : cplan.sql(14068) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751H;
   ElsIf Coalesce(h_bitflag, 0) = PMO_CNST.SC_SRCH_PARTICIPANT Then
      BEGIN
         SELECT PROFILE_NAME INTO h_in FROM PROFILES WHERE PROFILE_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1049+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1050) : cplan.sql(14080) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         OPEN C7751J FOR SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,30 AS ROLE_TYPE, p.ROLE_ID, h_in AS ROLE_NAME,
         ORG_ID, org.ELEMENT_NAME AS ORG_NAME, GEO_ID, geo.ELEMENT_NAME AS GEO_NAME, POOL_ID, pool.ELEMENT_NAME AS POOL_NAME
         FROM PROFILES p
         LEFT JOIN TMT_ORGANIZATIONS org ON org.ELEMENT_ID  = ORG_ID
         LEFT JOIN TMT_GEOGRAPHIES geo   ON geo.ELEMENT_ID  = GEO_ID
         LEFT JOIN TMT_POOLS pool        ON pool.ELEMENT_ID = POOL_ID
         WHERE p.PROFILE_ID = h_id
         UNION ALL
         SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,ds.TYPE_ID AS ROLE_TYPE,
         cs.ELEMENT_ID AS ROLE_ID, PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(ds.ELEMENT_NAME), ', '), RTRIM(cs.ELEMENT_NAME)) AS ROLE_NAME,
         '' ORG_ID, '' AS ORG_NAME, '' GEO_ID, '' AS GEO_NAME, '' POOL_ID, '' AS POOL_NAME
         FROM RESOURCE_ASSIGN_ROLES r LEFT OUTER JOIN TMT_COMPETENCIES cs ON r.ROLE_ID=cs.ELEMENT_ID
         LEFT OUTER JOIN TMT_COMPETENCIES ds ON ds.ELEMENT_ID=cs.PARENT_ID WHERE r.RES_ASSIGNMENT_ID=h_id
         UNION ALL
         SELECT h_id AS ELEMENT_ID, h_in AS ELEMENT_NAME,41 AS TYPE_ID,ds.TYPE_ID AS ROLE_TYPE,
         cs.ELEMENT_ID AS ROLE_ID, PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(RTRIM(ds.ELEMENT_NAME), ', '), RTRIM(cs.ELEMENT_NAME)) AS ROLE_NAME,
         '' ORG_ID, '' AS ORG_NAME, '' GEO_ID, '' AS GEO_NAME, '' POOL_ID, '' AS POOL_NAME
         FROM RESOURCE_ASSIGN_SKILLS s LEFT OUTER JOIN TMT_COMPETENCIES cs ON s.SKILL_ID=cs.ELEMENT_ID
         LEFT OUTER JOIN TMT_COMPETENCIES ds ON ds.ELEMENT_ID=cs.PARENT_ID WHERE s.RES_ASSIGNMENT_ID=h_id
         ORDER BY ELEMENT_ID,TYPE_ID;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1076+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1077) : cplan.sql(14107) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C7751J;
      End If;
   ElsIf Coalesce(h_bitflag, 0) = 10 Then
      OPEN C7751L FOR SELECT CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN t.TSK_ASSIGNMENT_ID ELSE a.RES_ASSIGNMENT_ID END AS ELEMENT_ID,
      CASE WHEN a.RESOURCE_TYPE='1' THEN a.PROFILE_ID ELSE a.RESOURCE_ID END AS RESOURCE_ID,
      CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.PROFILE_ID = p.PROFILE_ID)
      ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
      CASE WHEN a.RESOURCE_TYPE='1' THEN 41 WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL THEN 13 ELSE 185 END AS TYPE_ID,
      a.RES_ASSIGNMENT_ID, a.PARTICIPANT, t.START_DT,t.FINISH_DT,
      t.SD, t.FD, t.UNRESOLVED_START AS PLANNED_START, t.UNRESOLVED_FINISH AS PLANNED_FINISH,
      CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK ELSE PLANNED_WORK END AS PLANNED_WORK,
      CASE WHEN BIT_FLAG = '2' THEN SCHED_WORK - WORK_TODATE ELSE PLANNED_WORK END AS REMAINING,
      CASE WHEN t.ASSIGN_UNITS > t.UNITS_MSP THEN t.ASSIGN_UNITS ELSE t.UNITS_MSP END AS UNITS_PERCENT,
      0 AS MILESTONE_PERCENT,
      CASE WHEN t.WORK_TODATE = 0 THEN 0 ELSE Trunc((TO_NUMBER(t.WORK_TODATE)/TO_NUMBER(PLANNED_WORK+SCHED_WORK))*10000) END AS WORK_PERCENT,
      t.BAK_START_DT,t.BAK_FINISH_DT,t.BAK_WORK,
      t.FORECAST_START,t.FORECAST_FINISH,t.ETC, t.ACTUAL_START,t.ACTUAL_FINISH,
      t.WORK_TODATE AS ACTUAL,PROPOSED_START, PROPOSED_FINISH, PROPOSED_WORK,
      h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME,
      COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME, t.REGULAR_COST,t.REGULAR_RATE,t.SPECIAL_COST,t.SPECIAL_RATE,
      CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
      ELSE '' END AS FS_LABOR_CODE,a.REC_STATUS,t.ASSIGN_STRING,t.BASELINE1_START,t.BASELINE1_FINISH,t.BASELINE1_WORK,
      t.PROJECT_ASSIGNMENT_ID,t.TIMECODE2_ID, TC.ELEMENT_NAME AS TIMECODE2_NAME, t.T_UNIT, t.C_EFFORT, t.C_UNIT, t.C_CONTOUR, t.CONTOUR_TYPE, t.C_OCCURRENCE, t.BILLABLE, t.CAPITAL_EXPENSE,
      t.ASSIGNMENT_DELAY, CASE WHEN t.BIT_FLAG IS NuLL THEN '' WHEN t.BIT_FLAG = '0' THEN '1' ELSE '2' END AS BIT_FLAG, t.RATE_ADJUSTMENT, 0 AS ASSIGN_COUNT, t.UNRESOLVED_DURATION AS DURATION,
      CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_START,t.UNRESOLVED_START) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_START,
      CASE WHEN h_level_state = 1 THEN COALESCE(t.LEVEL_FINISH,t.UNRESOLVED_FINISH) ELSE TO_TIMESTAMP('1899-12-31 00:00:00') END AS LEVEL_FINISH,
      CASE WHEN h_level_state = 1 THEN CASE WHEN t.LEVEL_EFFORT = 0 THEN t.ETC ELSE t.LEVEL_EFFORT END ELSE 0 END AS LEVEL_EFFORT, t.MILESTONE_PERCENT AS PERCENT_DISCRETE, 0 as TOP_ID,
      CASE WHEN t.TSK_ASSIGNMENT_ID IS NOT NuLL AND t.PROJECT_ASSIGNMENT_ID = t.TSK_ASSIGNMENT_ID AND (t.LEVEL_START <> t.UNRESOLVED_START OR t.LEVEL_FINISH <> t.UNRESOLVED_FINISH) THEN '1' ELSE '0' END AS LEVEL_AFFECTED_FLAG
      FROM RESOURCE_ASSIGNMENTS a JOIN TASK_ASSIGNMENTS t ON t.TSK_ASSIGNMENT_ID = h_id AND a.RES_ASSIGNMENT_ID = t.RES_ASSIGNMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
      LEFT OUTER JOIN TMT_TIMECODES TC ON TC.ELEMENT_ID= t.TIMECODE2_ID
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1112+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1113) : cplan.sql(14143) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751L;
   ElsIf Coalesce(h_bitflag, 0) = 11 Then
      OPEN C7751M FOR SELECT a.RES_ASSIGNMENT_ID AS ELEMENT_ID,
      CASE WHEN a.RESOURCE_TYPE='1' THEN a.PROFILE_ID ELSE a.RESOURCE_ID END AS RESOURCE_ID,
      CASE WHEN a.RESOURCE_TYPE='1' THEN (SELECT p.PROFILE_NAME FROM PROFILES p WHERE a.PROFILE_ID = p.PROFILE_ID)
      ELSE (SELECT r.FULL_NAME FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID) END AS ELEMENT_NAME,
      CASE WHEN a.RESOURCE_TYPE='1' THEN PMO_CNST.SC_PROFILE ELSE 185 END AS TYPE_ID,  a.RES_ASSIGNMENT_ID, a.PARTICIPANT, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_START, TO_TIMESTAMP('1899-12-31 00:00:00') AS PLANNED_FINISH,
      0 AS PLANNED_WORK,TO_TIMESTAMP('1899-12-31 00:00:00') AS START_DT,TO_TIMESTAMP('1899-12-31 00:00:00') AS FINISH_DT,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS SCHED_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS SCHED_FINISH, 0 AS SCHED_WORK,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_START_DT,TO_TIMESTAMP('1899-12-31 00:00:00') AS BAK_FINISH_DT, 0 AS BAK_WORK,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS FORECAST_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS FORECAST_FINISH,0 AS ETC,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS ACTUAL_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS ACTUAL_FINISH,
      0 AS ACTUAL,h_per_day AS MINUTE_PER_DAY, a.WORK_LOCATION, TO_TIMESTAMP('1899-12-31 00:00:00') AS ACCEPTED_DATE,
      a.SECURITY_ID, g.ELEMENT_NAME AS SECURITY_NAME, COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID) AS CRI_SECURITY_ID, g2.ELEMENT_NAME AS CRI_SECURITY_NAME,
      0 AS REGULAR_COST,0 AS REGULAR_RATE,0 AS SPECIAL_COST, 0 AS SPECIAL_RATE,
      CASE WHEN a.RESOURCE_TYPE='0' THEN (SELECT r.FS_LABOR_CODE FROM RESOURCES r WHERE a.RESOURCE_ID = r.RESOURCE_ID)
      ELSE '' END AS FS_LABOR_CODE,
      a.REC_STATUS,'' AS ASSIGN_STRING,
      TO_TIMESTAMP('1899-12-31 00:00:00') AS BASELINE1_START,TO_TIMESTAMP('1899-12-31 00:00:00') AS BASELINE1_FINISH,0 AS BASELINE1_WORK,
      '' AS PROJECT_ASSIGNMENT_ID, '' AS TIMECODE2_ID, '' AS TIMECODE2_NAME, 0 AS T_UNIT,
      0 AS C_EFFORT, 0 AS C_UNIT, 0 AS C_CONTOUR, 0 AS CONTOUR_TYPE, 0 AS C_OCCURRENCE, 0 AS BILLABLE,
      0 AS DELAY
      FROM RESOURCE_ASSIGNMENTS a
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g ON a.SECURITY_ID=g.ELEMENT_ID
      LEFT OUTER JOIN TMT_GLOBAL_SECURITY g2 ON COALESCE(a.CRI_SECURITY_ID,a.SECURITY_ID)=g2.ELEMENT_ID
      WHERE a.REC_STATUS NOT IN ('L','D') AND a.ELEMENT_ID = h_id
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1142+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1143) : cplan.sql(14173) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751M;
   ElsIf Coalesce(h_bitflag, 0) = 12 Then
      -- returns all assignments DISPLAY NICKNAME(effort, participants, project assignments)
      -- in the current project (id = PROJECT_ID)
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RTA';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RTA (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG)
         SELECT RESOURCE_ID, RESOURCE_TYPE, 'N' AS PA_FLAG
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_id AND
         PROJECT_ASSIGNMENT_ID IS NuLL AND
         TABLE_TYPE = PMO_CNST.SC_TABLE_WBS
         GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1170+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1171) : cplan.sql(14201) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_CP_PRJ';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_CP_PRJ (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG)
         SELECT  RESOURCE_ID, RESOURCE_TYPE, 'Y' AS PA_FLAG
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_id AND
         TASK_ID = h_id
         GROUP BY RESOURCE_ID, RESOURCE_TYPE;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1196+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1197) : cplan.sql(14227) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_RESOURCE_DATA';
      PMO_DB2M.SC_STATE := 0;
      
      BEGIN
         INSERT INTO PMO_TMP_RESOURCE_DATA (RESOURCE_ID, RESOURCE_TYPE, PA_FLAG)
         SELECT a.RESOURCE_ID, a.RESOURCE_TYPE, a.PA_FLAG
         FROM PMO_TMP_RTA a LEFT OUTER JOIN PMO_TMP_CP_PRJ b ON a.RESOURCE_ID=b.RESOURCE_ID
         UNION
         SELECT b.RESOURCE_ID, b.RESOURCE_TYPE, b.PA_FLAG
         FROM PMO_TMP_CP_PRJ b LEFT OUTER JOIN PMO_TMP_RTA a ON a.RESOURCE_ID=b.RESOURCE_ID;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1222+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1223) : cplan.sql(14253) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN C7751N FOR SELECT a.RESOURCE_ID AS ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID, p.NICK_NAME, a.PA_FLAG
      FROM PMO_TMP_RESOURCE_DATA a JOIN RESOURCES p ON a.RESOURCE_TYPE = '0' AND a.RESOURCE_ID=p.RESOURCE_ID
      LEFT OUTER JOIN PMO_TMP_CP_PRJ PRJ ON a.RESOURCE_ID=PRJ.RESOURCE_ID
      
      UNION ALL
      
      SELECT DISTINCT r.PROFILE_ID AS ELEMENT_ID, r.PROFILE_NAME AS ELEMENT_NAME,41 AS TYPE_ID, ' ' AS NICK_NAME, a.PA_FLAG
      FROM PMO_TMP_RESOURCE_DATA a JOIN PROFILES r ON a.RESOURCE_TYPE='1' AND r.PROFILE_ID=a.RESOURCE_ID
      JOIN TMT_COMPETENCIES c ON c.ELEMENT_ID=r.ROLE_ID
      JOIN TMT_COMPETENCIES d ON d.ELEMENT_ID=c.PARENT_ID
      LEFT OUTER JOIN PMO_TMP_CP_PRJ PRJ ON a.RESOURCE_ID=PRJ.RESOURCE_ID
      
      UNION ALL
      
      SELECT DISTINCT ra.RESOURCE_ID AS ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME, 185 AS TYPE_ID, p.NICK_NAME, 'N' AS PA_FLAG
      FROM RESOURCE_ASSIGNMENTS ra, RESOURCES p
      WHERE ra.PROJECT_ID=h_id AND
      ra.RESOURCE_ID = p.RESOURCE_ID AND
      NOT EXISTS(SELECT rd.RESOURCE_ID FROM PMO_TMP_RESOURCE_DATA rd WHERE ra.RESOURCE_ID = rd.RESOURCE_ID)
      ORDER BY TYPE_ID DESC, ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1248+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1249) : cplan.sql(14279) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C7751N;
   ElsIf Coalesce(h_bitflag, 0) = 13 Then
      -- returns all assignments DISPLAY NICKNAME(effort, participants, project assignments)
      -- in the current pool (id = ELEMENT_ID)
      
      OPEN C77513N FOR SELECT t.ELEMENT_ID, p.FULL_NAME AS ELEMENT_NAME,13 AS TYPE_ID, p.NICK_NAME, 'N' AS PA_FLAG
      FROM TMT_POOLS t JOIN RESOURCES p
      ON t.ELEMENT_ID = p.RESOURCE_ID
      WHERE t.TYPE_ID = 13
      AND t.RANK LIKE h_rank || '%'
      ORDER BY ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1262+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1263) : cplan.sql(14293) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C77513N;
   ElsIf (Coalesce(h_bitflag, 0) = PMO_CNST.SC_ASSET_ASSIGNED_RESOURCES) OR (Coalesce(
                h_bitflag, 0) = PMO_CNST.SC_CLIENT_ASSIGNED_RESOURCES) Then
      -- returns all resources assigned to an asset(resource can be asssigned indirectly it means
      -- resource who assigned to a document/cri attached to the asset element will be selected in this query)
      
      OPEN cln_ast_res FOR SELECT R.RESOURCE_ID AS ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME,PMO_CNST.SC_RESOURCE AS TYPE_ID, R.NICK_NAME, 'N' AS PA_FLAG
      FROM PROJECT_ASSIGNMENTS RA, RESOURCES R
      WHERE RA.PROJECT_ID = h_id
      AND RA.RESOURCE_ID = R.RESOURCE_ID
      ORDER BY ELEMENT_NAME;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_PROFILE', 1276+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_PROFILE(1277) : cplan.sql(14307) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN cln_ast_res;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_S_PROFILE', 1281+1) THEN 
         LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_S_PROFILE(1282) : cplan.sql(14312) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      PMODEFINES.PMO_RETURN_STATUS(PMO_CNST.INVALID_PARAMS, id, True);
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1290+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1291) : cplan.sql(14321) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_PROFILE', 1302+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_PROFILE(1303) : cplan.sql(14333) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_PROFILE;
ALTER FUNCTION SP_S_PROFILE COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_U_NAMES ==
CREATE OR REPLACE FUNCTION FC_U_NAMES
(
     a_element_id IN OUT NOCOPY VARCHAR2
   , h_full_name IN OUT NOCOPY VARCHAR2
   , a_participant_id IN OUT NOCOPY VARCHAR2
   , sh_h_table_type PLS_INTEGER
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_NAMES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_NAMES'), '-- ' || SYSTIMESTAMP, ':r := FC_U_NAMES(' || 
       '''' || a_element_id || ''''
          || ', ' || '''' || h_full_name || ''''
          || ', ' || '''' || a_participant_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_table_type), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_NAMES', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_NAMES(20) : cplan.sql(14369) : ' || SYSTIMESTAMP);
   END IF; 

   CASE h_table_type
      WHEN PMO_CNST.SC_TABLE_SCD THEN
         BEGIN
            UPDATE TMT_ASSIGNED_SCORECARDS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 46+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(47) : cplan.sql(14396) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_POL THEN
         BEGIN
            UPDATE TMT_POOLS SET MANAGER_ID = a_participant_id, ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 76+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(77) : cplan.sql(14426) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
      
      WHEN PMO_CNST.SC_TABLE_CRI THEN
         BEGIN
            UPDATE TMT_CRI SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 105+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(106) : cplan.sql(14455) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_DOC THEN
         BEGIN
            UPDATE TMT_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 135+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(136) : cplan.sql(14485) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_POL_DOC THEN
         BEGIN
            UPDATE TMT_POOL_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 165+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(166) : cplan.sql(14515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_RES_DOC THEN
         BEGIN
            UPDATE TMT_RESOURCE_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(196) : cplan.sql(14545) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_AST_DOC THEN
         BEGIN
            UPDATE TMT_ASSET_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 225+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(226) : cplan.sql(14575) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_CLN_DOC THEN
         BEGIN
            UPDATE TMT_CLIENT_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 255+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(256) : cplan.sql(14605) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      WHEN PMO_CNST.SC_TABLE_VDR_DOC THEN
         BEGIN
            UPDATE TMT_VENDOR_DOCUMENTS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 285+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(286) : cplan.sql(14635) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         
      
      ELSE
         BEGIN
            UPDATE TMT_WBS SET ASSIGN_NAMES =
            CASE
            WHEN InStr( ASSIGN_NAMES, h_full_name || ';') > 0
            THEN h_full_name || ';' || REPLACE( ASSIGN_NAMES, h_full_name || ';', '')
            ELSE
            CASE
            WHEN LENGTH(h_full_name) + LENGTH(ASSIGN_NAMES) > 2040 THEN ASSIGN_NAMES
            ELSE h_full_name || ';' || ASSIGN_NAMES
            END
            END
            WHERE ELEMENT_ID = a_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_NAMES', 315+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_NAMES(316) : cplan.sql(14665) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
   END CASE;
   --trace_printf("***_ASSIGN_NAMES: SC_STATE: %d, h_full_name: %s, h_table_type: %d\n", SC_STATE,h_full_name,h_table_type);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_NAMES', 323+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_NAMES(324) : cplan.sql(14673) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_NAMES;
ALTER FUNCTION FC_U_NAMES COMPILE DEBUG;

/******************************************************************************************************************
update resource assignments SECURITY

assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_WP_DATES ==
CREATE OR REPLACE FUNCTION SP_U_WP_DATES
(
     assign_id VARCHAR2
   , p_planned_start TIMESTAMP
   , p_planned_finish TIMESTAMP
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_actual_start TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_SD DATE;
   h_FD DATE;
   h_bit_flag VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_last_submitted DATE;
   h_participant PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_new_actual PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_skip PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WP_DATES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WP_DATES'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WP_DATES(' || 
       '''' || assign_id || ''''
          || ', ' || COALESCE(NULLIF('''' || p_planned_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || p_planned_finish || '''',''''''), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 74+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WP_DATES(75) : cplan.sql(14763) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   --Validate res_assignment_id
   h_tsk_assignment_id := assign_id;
   h_rec_user := res_id001;
   
   BEGIN
      SELECT PROJECT_ASSIGNMENT_ID, PROJECT_ID, RESOURCE_ID,
      RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT,
      UNITS_MSP, ACTUAL_START, WORK_TODATE, UNRESOLVED_START, UNRESOLVED_FINISH,
      CASE WHEN BIT_FLAG = '0' THEN PLANNED_WORK ELSE SCHED_WORK END, ASSIGNMENT_DELAY, LAST_SCHED_MIN,
      TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
      REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE,
      B_REGULAR_COST, B_SPECIAL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,
      MILESTONE_PERCENT, TASK_ID
      INTO h_prj_assign_id, h_project_id, h_res_id,
      h_res_type, h_bit_flag, h_last_submitted, h_contour, h_work, h_eunit, h_unit,
      h_percent_units, h_actual_start, h_work_todate, h_start_date, h_finish_date,
      h_effort, h_delay, h_last_day,
      h_SD, h_FD, h_billable, h_capital_expense,
      h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,
      h_b_reg_cost, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,
      h_work_percent, h_task_id
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 108+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(109) : cplan.sql(14797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_SD: %d %s, h_FD: %d %s\n", SC_STATE,h_SD_Ind,h_SD,h_FD_Ind,h_FD);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 116+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(117) : cplan.sql(14805) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(120) : cplan.sql(14808) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_tsk_assignment_id
            , True);
   End If;
   
   If (p_planned_start IS NOT NULL) AND (p_planned_finish IS NOT NULL) Then
      h_sched_start := p_planned_start;
      h_sched_finish := p_planned_finish;
      
      If (PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0) AND (PMO_ORASTD.strncmp(
            h_sched_finish, h_finish_date, 10) <> 0) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 132+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(133) : cplan.sql(14821) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 135+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(136) : cplan.sql(14824) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, h_tsk_assignment_id
               , True);
      End If;
      
      BEGIN
         SELECT PROJECT_ID, MINUTE_PER_DAY
         INTO h_project_id, h_per_day
         FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 151+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(152) : cplan.sql(14840) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_per_day: %d\n", SC_STATE,h_per_day);//trace_flush();
      
      --trace_printf("h_start_date: %s, h_finish_date: %s\n", h_start_date,h_finish_date);
      If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
         h_start_date := h_sched_start;
         h_delay := 0;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_DATES', 163+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_DATES(164) : cplan.sql(14852) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 166+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(167) : cplan.sql(14855) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
   End If;
   
   --trace_printf("**===> h_last_day: %d, h_prj_assign_id: %d %s\n", h_last_day,h_prj_assign_id_Ind,h_prj_assign_id);
   h_tph_delete := -1;
   h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id, h_SD, h_FD
                                          , h_reg_cost, h_reg_sale, h_tph_delete, h_capital_expense
                                          , h_billable);
   
   If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
      h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id
                                   , h_tsk_assignment_id, h_last_submitted, h_last_day, 0);
   End If;
   
   --trace_printf("h_prj_assign_id: %d %s\n", h_prj_assign_id_Ind,h_prj_assign_id);
   If (h_prj_assign_id IS NOT NULL) Then
      h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 2);
   Else
      If SubStr(h_bit_flag, 1, 1) = '2' Then
         h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 0);
      Else
         h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, h_tsk_assignment_id, 1);
      End If;
   End If;
   
   If (h_last_submitted IS NOT NULL) Then
      BEGIN
         SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
         INTO h_start_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 205+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(206) : cplan.sql(14894) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_effort := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
   End If;
   
   --trace_printf("1_FC_DEL_NOACTUAL: h_skip: %d,h_start_date: %s, h_effort: %d, h_percent_units: %d\n",h_skip,h_start_date,h_effort,h_percent_units);
   
   h_temp_var := 2;
   h_temp_sc_state := FC_RESCHEDULE(h_tsk_assignment_id, h_start_date, h_effort, h_temp_var, h_contour
                                 , h_rec_user, Coalesce(h_delay, 0), Coalesce(h_percent_units, 0));
   --trace_printf("2_FC_RESCHEDULE: h_temp_sc_state: %d\n",h_temp_sc_state); //trace_flush();
   
   If Coalesce(h_temp_sc_state, 0) <> 0 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 222+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(223) : cplan.sql(14911) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 225+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(226) : cplan.sql(14914) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_tsk_assignment_id
            , True);
   Else
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT)),
         BAK_START_DT = COALESCE(SCHED_START,PLANNED_START), BAK_FINISH_DT = COALESCE(SCHED_FINISH,PLANNED_FINISH),
         BAK_ASSIGN_STRING = ASSIGN_STRING
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 245+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(246) : cplan.sql(14934) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   h_temp_var := FC_Rollup(h_project_id);
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_DATES', 253+1) THEN 
      LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_DATES(254) : cplan.sql(14942) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
   END IF; 
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_DATES', 256+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_DATES(257) : cplan.sql(14945) : ' || SYSTIMESTAMP);
   END IF; 
   PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_tsk_assignment_id
         , True);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WP_DATES', 262+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WP_DATES(263) : cplan.sql(14951) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_tsk_assignment_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 269+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(270) : cplan.sql(14958) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_DATES', 281+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_DATES(282) : cplan.sql(14970) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WP_DATES;
ALTER FUNCTION SP_U_WP_DATES COMPILE DEBUG;

/******************************************************************************************************************
update Actual Work, %W, %C in Work Packages

assign_id:      TSK_ASSIGNMENT_ID or TASK_ID
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_WP_ACTUAL ==
CREATE OR REPLACE FUNCTION SP_U_WP_ACTUAL
(
     assign_id VARCHAR2
   , p_work_percent PLS_INTEGER
   , p_actual PLS_INTEGER
   , p_units PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_SD DATE;
   h_FD DATE;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_last_submitted DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_rec_status VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_new_actual PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_close_task PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_assign_string VARCHAR2(8192);
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
   Debug_PerfTiming PLS_INTEGER := 0;
   t_total NUMBER := 0;
   s_time_1 PLS_INTEGER := 0;
   s_time_2 PLS_INTEGER := 0;
   s_time_3 PLS_INTEGER := 0;
   s_time_4 PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WP_ACTUAL') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WP_ACTUAL'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WP_ACTUAL(' || 
       '''' || assign_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_actual), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_units), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 80+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WP_ACTUAL(81) : cplan.sql(15071) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_id := assign_id;
   h_rec_user := res_id001;
   
   BEGIN
      SELECT PROJECT_ID, RESOURCE_ID, RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, WORK_TODATE, SCHED_WORK-WORK_TODATE, ETC,
      SCHED_WORK, LAST_SCHED_MIN, PROJECT_ASSIGNMENT_ID, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT, UNITS_MSP,
      UNRESOLVED_START, UNRESOLVED_FINISH, TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
      REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE,
      B_REGULAR_COST, B_SPECIAL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,
      MILESTONE_PERCENT, TASK_ID, ROLLUP_BITS
      INTO h_project_id, h_res_id, h_res_type, h_bit_flag, h_last_submitted, h_work_todate, h_remaining, h_etc,
      h_effort, h_last_day, h_prj_assign_id, h_contour, h_work, h_eunit, h_unit, h_percent_units,
      h_start_date, h_finish_date, h_SD, h_FD, h_billable, h_capital_expense,
      h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,
      h_b_reg_cost, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,
      h_work_percent, h_task_id, h_rollup_bits
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 109+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(110) : cplan.sql(15100) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_id: %s, h_start_date: %d %s\n", SC_STATE,h_id,h_start_date);//trace_flush();
   --trace_printf("h_SD: %d %s, h_FD: %d %s, h_effort: %d\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD,h_effort);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT PROJECT_ID, TASK_TYPE, SCHED_EFFORT, WORK_TODATE, MILESTONE_PERCENT, SCHED_DURATION
         INTO h_project_id, h_task_type, h_effort, h_work_todate, h_work_percent, h_duration
         FROM TMT_WBS WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 127+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(128) : cplan.sql(15118) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_effort: %d, h_work_todate: %d, h_duration: %d\n", SC_STATE,h_effort,h_work_todate,h_duration);//trace_flush();
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 135+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(136) : cplan.sql(15126) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 138+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(139) : cplan.sql(15129) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 154+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(155) : cplan.sql(15145) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 170+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(171) : cplan.sql(15161) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 177+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(178) : cplan.sql(15168) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 180+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(181) : cplan.sql(15171) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
      
      -- varify if there are steps in timesheets
      BEGIN
         SELECT COUNT('X') INTO h_temp_var
         FROM DUAL
         WHERE EXISTS(SELECT A.ELEMENT_ID FROM TASK_ASSIGNMENTS TA, ACTIVITIES A
         WHERE TA.PROJECT_ID = h_project_id
         AND TA.TASK_ID = h_id
         AND TA.TSK_ASSIGNMENT_ID = A.TSK_ASSIGNMENT_ID
         AND A.TSK_ASSIGNMENT_ID <> A.ELEMENT_ID);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 201+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(202) : cplan.sql(15192) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 208+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(209) : cplan.sql(15199) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TIMESHEET_STEP_EXISTS), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 211+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(212) : cplan.sql(15202) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.SC_TIMESHEET_STEP_EXISTS, h_id, True);
      End If;
      
      If Coalesce(h_effort, 0) = 0 Then
         -- TBV
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_ACTUAL', 219+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_ACTUAL(220) : cplan.sql(15210) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 222+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(223) : cplan.sql(15213) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      ElsIf Coalesce(p_actual, 0) <> Coalesce(h_work_todate, 0) Then
         h_close_task := 0;
         If Coalesce(p_actual, 0) > Coalesce(h_effort, 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_ACTUAL', 229+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_ACTUAL(230) : cplan.sql(15220) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TASK_ACTUAL_GRATER_THAN_SCH), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 232+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(233) : cplan.sql(15223) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.SC_TASK_ACTUAL_GRATER_THAN_SCH, True);
         ElsIf Coalesce(p_actual, 0) = Coalesce(h_effort, 0) Then
            h_close_task := 1;
         End If;
         
         h_new_actual := p_actual;
         --trace_printf("p_actual: %d, h_new_actual: %d, h_work_todate: %d\n", *p_actual,h_new_actual,h_work_todate);
         
         h_temp_var := FC_U_TASK_PERCENT(h_project_id, h_id, h_new_actual, h_rec_user, Coalesce(
                                h_close_task, 0));
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_ACTUAL', 246+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_ACTUAL(247) : cplan.sql(15237) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 249+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(250) : cplan.sql(15240) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
   Else
      If Coalesce(h_effort, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_ACTUAL', 256+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_ACTUAL(257) : cplan.sql(15247) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 259+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(260) : cplan.sql(15250) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
      
      h_temp_var := 0;
      BEGIN
         SELECT LEVEL_LOCK INTO h_temp_var
         FROM PROJECT_DETAILS
         WHERE PROJECT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 275+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(276) : cplan.sql(15266) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         BEGIN
            SELECT REC_STATUS INTO h_rec_status
            FROM TMT_WBS
            WHERE ELEMENT_ID = h_project_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 291+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(292) : cplan.sql(15282) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_ORASTD.strcmp(h_rec_status, 'L') = 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 298+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(299) : cplan.sql(15289) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.LEV_LOCKED_BY_COPY), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 301+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(302) : cplan.sql(15292) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.LEV_LOCKED_BY_COPY, h_project_id, True);
         End If;
      End If;
      
      -- varify if there are steps in timesheets
      BEGIN
         SELECT COUNT('X') INTO h_temp_var
         FROM DUAL
         WHERE EXISTS(SELECT ELEMENT_ID FROM ACTIVITIES
         WHERE TSK_ASSIGNMENT_ID = h_id
         AND TSK_ASSIGNMENT_ID <> ELEMENT_ID);
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 320+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(321) : cplan.sql(15311) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_temp_var, 0) > 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 327+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(328) : cplan.sql(15318) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TIMESHEET_STEP_EXISTS), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 330+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(331) : cplan.sql(15321) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.SC_TIMESHEET_STEP_EXISTS, h_id, True);
      End If;
      
      BEGIN
         SELECT PROJECT_ID, MINUTE_PER_DAY
         INTO h_project_id, h_per_day
         FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 345+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(346) : cplan.sql(15336) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_per_day: %d\n", SC_STATE,h_per_day);//trace_flush();
      --trace_printf("h_last_submitted: %d:%s, p_actual: %d, h_work_todate: %d, h_effort = %d\n", h_last_submitted_Ind,h_last_submitted,*p_actual,h_work_todate,h_effort);
      --trace_printf("p_work_percent: %d, h_work_percent: %d\n", *p_work_percent,h_work_percent);
      
      h_sched_effort := Coalesce(h_effort, 0);
      If Coalesce(p_actual, 0) <> Coalesce(h_work_todate, 0) Then
         h_new_actual := p_actual;
         -- new actual work
         If Coalesce(h_new_actual, 0) > Coalesce(h_effort, 0) Then
            --timeLog (file,"1_* START Actual > Schedule:");
            h_tph_delete := -1;
            h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
            
            If Coalesce(h_temp_var, 0) = 0 Then
               --trace_printf("1.a_* h_res_id: %s, h_rec_user: %s\n", h_res_id, h_rec_user);
               --trace_printf("h_new_actual: %d, h_remaining: %d, h_etc: %d, h_work_todate: %d\n", h_new_actual,h_remaining,h_etc,h_work_todate);
               If Coalesce(h_remaining, 0) <> Coalesce(h_etc, 0) Then
                  If PMO_ORASTD.strcmp(h_res_id, h_rec_user) = 0 Then
                     h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_effort, 0);
                     h_work_percent := 10000;
                     
                     If Coalesce(h_new_actual, 0) < (Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0)) Then
                        h_etc := (Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0)) - Coalesce(h_new_actual, 0);
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_new_actual, WORK_TODATE = h_new_actual,
                           ETC = h_etc, MILESTONE_PERCENT = 10000
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 388+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(389) : cplan.sql(15379) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        h_work_percent := PMO_FUNC.BIT_OR(Coalesce(h_work_percent, 0), PMO_CNST.SC_BIT_30);
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_new_actual, WORK_TODATE = h_new_actual, ETC = 0, MILESTONE_PERCENT = 10000
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 409+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(410) : cplan.sql(15400) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     
                     --trace_printf("ME_UPDATE TASK_ASSIGNMENTS: SC_STATE: %d, h_start_date: %s, h_etc: %d\n", SC_STATE,h_start_date,h_etc);//trace_flush();
                  Else
                     If Coalesce(h_new_actual, 0) < (Coalesce(h_work_todate, 0) + Coalesce(h_etc, 0)) Then
                        h_effort := Coalesce(h_etc, 0) - Coalesce(h_effort, 0);
                        h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
                        
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_etc, WORK_TODATE = h_new_actual
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 435+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(436) : cplan.sql(15426) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_effort, 0);
                        h_work_percent := 10000;
                        
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                           SCHED_WORK = h_new_actual, WORK_TODATE = h_new_actual, ETC = 0, MILESTONE_PERCENT = 10000
                           WHERE TSK_ASSIGNMENT_ID = h_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 457+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(458) : cplan.sql(15448) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     --trace_printf("PM_UPDATE TASK_ASSIGNMENTS: SC_STATE: %d, h_start_date: %s\n", SC_STATE,h_start_date);//trace_flush();
                  End If;
               Else
                  h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_work_todate, 0);
                  h_work_percent := 10000;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                     SCHED_WORK = h_new_actual, WORK_TODATE = h_new_actual, ETC = 0, MILESTONE_PERCENT = 10000
                     WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 482+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(483) : cplan.sql(15473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If (h_last_submitted IS NOT NULL) Then
                  h_week_of := NULL;
                  BEGIN
                     SELECT MAX(WEEK_OF) INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS
                     WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 499+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(500) : cplan.sql(15490) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("TIMESHEET_APPROVAL_STATUS: SC_STATE: %d, h_start_date: %s, h_week_of: %s\n", SC_STATE,h_start_date,h_week_of);//trace_flush();
                  
                  If (h_week_of IS NOT NULL) Then
                     BEGIN
                        SELECT TO_DATE(SubStr(h_week_of, 1, 10)) + 7      INTO h_week_of FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 514+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(515) : cplan.sql(15505) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If PMO_ORASTD.strncmp(h_last_submitted, h_week_of, 10) < 0 Then
                        h_start_date := h_week_of;
                     End If;
                  ElsIf Coalesce(h_contour, 0) > 0 Then
                     h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
                     
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 535+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(536) : cplan.sql(15526) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 550+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(551) : cplan.sql(15541) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  --trace_printf("*** h_start_date: %s\n", h_start_date);//trace_flush();
               End If;
               
               --trace_printf("*** h_work_percent: %d, h_start_date: %s, h_effort: %d\n", h_work_percent,h_start_date,h_effort);//trace_flush();
               If (h_prj_assign_id IS NOT NULL) Then
                  h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 2);
               Else
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 0);
                  Else
                     h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 1);
                  End If;
               End If;
               
               h_temp_var := 2;
               h_temp_var := FC_RESCHEDULE(h_id, h_start_date, h_effort, h_temp_var, h_contour, h_rec_user, 0
                            , Coalesce(h_percent_units, 0));
               
               If Coalesce(h_temp_var, 0) <> 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 575+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(576) : cplan.sql(15566) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 578+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(579) : cplan.sql(15569) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_id, True);
               End If;
               
               h_effort := h_new_actual;
            End If;
         Else
            --timeLog (file,"2_* START Actual <= Schedule:");
            h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
         End If;
         
         --trace_printf("h_work_percent: %d, h_last_day: %d, h_new_actual: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_last_day,h_new_actual,h_work_todate,h_effort); //trace_flush();
         --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD); //trace_flush();
         
         h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, h_id, h_last_submitted
                                  , h_bit_flag, h_effort, h_new_actual, h_sched_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, Coalesce(h_per_day, 0));
         
         --trace_printf("*_FC_U_ASSIGN_PERCENT: h_temp_var: %d\n", h_temp_var); //trace_flush();
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 602+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(603) : cplan.sql(15593) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 605+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(606) : cplan.sql(15596) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), h_id, True);
      ElsIf Coalesce(p_work_percent, 0) <> Coalesce(h_work_percent, 0) Then
         /*if (*p_work_percent == 10000)
         			{
         				BEGIN_SQL
         					UPDATE TASK_ASSIGNMENTS SET
         						SCHED_WORK = WORK_TODATE,
         						ETC = 0, FD = SC_AF,
         						MILESTONE_PERCENT = 10000
         					WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
         				END_SQL;
         
         				if( SC_SUCCESS )
         				{
         					InitializeHost(SC_PARAM(h_temp_var), 1);
         					h_temp_var = FC_ETC_SCHEDULE(SC_PARAM(h_id), SC_PARAM(h_temp_var), SC_PARAM(h_rec_user));
         				}
         			}
         			else if ( h_work_percent == 10000 )
         			{
         				BEGIN_SQL
         					UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_wp),
         					ETC = SCHED_WORK - WORK_TODATE, FD = SC_SF, ACTUAL_FINISH = SC_NULL, ACTUAL_DURATION = 0
         					WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
         				END_SQL;
         			}
         			else
         			{
         				BEGIN_SQL
         					UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_wp),
         					ETC = SCHED_WORK - WORK_TODATE
         					WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
         				END_SQL;
         			}*/
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = p_work_percent,
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
            WHERE TSK_ASSIGNMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 656+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(657) : cplan.sql(15647) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         /*
         			BEGIN_SQL
         				UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = HOST(h_rollup_bits) WHERE TSK_ASSIGNMENT_ID = HOST(h_id);
         			END_SQL;
         */
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
            WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 678+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(679) : cplan.sql(15669) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --RETURN_ERROR(SC_STATE, h_id);
      ElsIf Coalesce(p_units, 0) <> Coalesce(h_percent_units, 0) Then
         If Coalesce(p_units, 0) = 0 Then
            h_percent_units := 100;
         Else
            If (Coalesce(p_units, 0) < 1) OR (Coalesce(p_units, 0) > 103079040) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_ACTUAL', 690+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_ACTUAL(691) : cplan.sql(15681) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 693+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(694) : cplan.sql(15684) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
            End If;
            
            h_percent_units := p_units;
         End If;
         
         If (h_prj_assign_id IS NOT NULL) Then
            BEGIN
               SELECT UNITS_MSP INTO h_temp_var
               FROM TASK_ASSIGNMENTS
               WHERE TSK_ASSIGNMENT_ID = h_prj_assign_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 712+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(713) : cplan.sql(15703) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("SC_STATE: %d, h_temp_var: %d\n", SC_STATE,h_temp_var);//trace_flush();
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_percent_units, 0) > Coalesce(
                  h_temp_var, 0)) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 721+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(722) : cplan.sql(15712) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 724+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(725) : cplan.sql(15715) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                     , True);
            End If;
         End If;
         
         /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
         			{
         				InitializeHost(SC_PARAM(h_tph_delete), -1);
         				h_temp_var = FC_TPHASE_SCHED_ASSIGN(
         							SC_PARAM(h_project_id),
         							SC_PARAM(h_task_id),
         							SC_PARAM(h_id),
         							SC_PARAM(h_SD),
         							SC_PARAM(h_FD),
         							SC_PARAM(h_reg_cost),
         							SC_PARAM(h_reg_sale),
         							SC_PARAM(h_tph_delete),
         							SC_PARAM(h_capital_expense),
         							SC_PARAM(h_billable));
         
         				if ( (h_last_day > 0)  (IsNotNullParam(h_last_submitted)))
         				{
         					h_temp_var = FC_U_TDY_LAST_ACTUAL(SC_PARAM(h_project_id),
         									SC_PARAM(h_task_id),
         									SC_PARAM(h_res_id),
         									SC_PARAM(h_prj_assign_id),
         									SC_PARAM(h_id),
         									SC_PARAM(h_last_submitted),
         									SC_PARAM(h_last_day) );
         
         				}
         
         				h_minutes = int(((h_percent_units * h_per_day) / 10000.00) + 0.4);
         				h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
         										  SC_PARAM(h_id),
         										  SC_PARAM(h_start_date),
         										  SC_PARAM(h_per_day),
         										  SC_PARAM(h_minutes));
         
         				InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30 );
         				h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(h_id), SC_PARAM(h_temp_var));
         
         				BEGIN_SQL
         					SELECT DATE(UNRESOLVED_START), DATE(UNRESOLVED_FINISH) INTO HOST(h_SD), HOST(h_FD)
         					FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = HOST(h_id) ;
         				END_SQL;
         
         				InitializeHost(SC_PARAM(h_tph_delete), 0);
         				h_temp_var = FC_TPHASE_SCHED_ASSIGN(
         							SC_PARAM(h_project_id),
         							SC_PARAM(h_task_id),
         							SC_PARAM(h_id),
         							SC_PARAM(h_SD),
         							SC_PARAM(h_FD),
         							SC_PARAM(h_reg_cost),
         							SC_PARAM(h_reg_sale),
         							SC_PARAM(h_tph_delete),
         							SC_PARAM(h_capital_expense),
         							SC_PARAM(h_billable));
         			}
         			else
         			{*/
         h_tph_delete := -1;
         h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, h_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
         
         If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
            h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id, h_id
                                   , h_last_submitted, h_last_day, 0);
         End If;
         
         --trace_printf("h_prj_assign_id: %d %s\n", h_prj_assign_id_Ind,h_prj_assign_id);
         If (h_prj_assign_id IS NOT NULL) Then
            h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 2);
         Else
            If SubStr(h_bit_flag, 1, 1) = '2' Then
               h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 0);
            Else
               h_temp_var := FC_DEL_NOACTUAL(h_project_id, h_id, 1);
            End If;
         End If;
         
         If (h_last_submitted IS NOT NULL) Then
            BEGIN
               SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
               INTO h_start_date
               FROM DUAL;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 819+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(820) : cplan.sql(15810) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            h_effort := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
         End If;
         
         --trace_printf("1_WP_FC_DEL_NOACTUAL: h_start_date: %s, h_effort: %d, h_percent_units: %d\n", h_start_date,h_effort,h_percent_units);
         
         h_contour := 0;
         h_temp_var := 2;
         h_temp_var := FC_RESCHEDULE(h_id, h_start_date, h_effort, h_temp_var, h_contour, h_rec_user, 0
                            , Coalesce(h_percent_units, 0));
         --trace_printf("2_WP_FC_RESCHEDULE: h_temp_var: %d\n",h_temp_var); //trace_flush();
         
         If Coalesce(h_temp_var, 0) <> 0 Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_ACTUAL', 837+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_ACTUAL(838) : cplan.sql(15828) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_ACTUAL', 840+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_ACTUAL(841) : cplan.sql(15831) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_id, True);
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, UNITS_MSP = h_percent_units,
               C_EFFORT = 0, C_UNIT = 0, C_CONTOUR = 0, CONTOUR_TYPE = 0,
               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 859+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(860) : cplan.sql(15850) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         --}
      End If;
      --timeLog (file,"1.1_* FC_Rollup:");
   End If;
   
   PMO_DB2M.SC_STATE := FC_Rollup(h_project_id);
   --trace_printf("**_FC_Rollup_** SC_STATE: %d\n",SC_STATE);
   
   --meLog (file,"1.2_* FC_Rollup:");
   
   --trace_printf("**_SP_U_WP_ACTUAL_** SC_STATE: %d\n",SC_STATE);
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WP_ACTUAL', 876+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WP_ACTUAL(877) : cplan.sql(15867) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 884+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(885) : cplan.sql(15875) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_ACTUAL', 896+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_ACTUAL(897) : cplan.sql(15887) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WP_ACTUAL;
ALTER FUNCTION SP_U_WP_ACTUAL COMPILE DEBUG;

/******************************************************************************************************************
update resource assignments SECURITY

assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_PARTICIPANT ==
CREATE OR REPLACE FUNCTION SP_U_PARTICIPANT
(
     assign_id VARCHAR2
   , security_id001 VARCHAR2
   , p_planned_start TIMESTAMP
   , p_planned_finish TIMESTAMP
   , p_delay PLS_INTEGER
   , p_units PLS_INTEGER
   , p_work_percent PLS_INTEGER
   , p_actual PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_security_id VARCHAR2(32);
   a_element_id VARCHAR2(32);
   a_participant_id VARCHAR2(255);
   a_participant_name VARCHAR2(255);
   a_security_name VARCHAR2(32);
   h_full_name VARCHAR2(255);
   a_res_assignment_id VARCHAR2(32);
   a_security_id VARCHAR2(32);
   a_temp_id VARCHAR2(32);
   h_prj_assign_id VARCHAR2(32);
   h_created_by VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_cri_security_id VARCHAR2(32);
   h_actual_start TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_SD DATE;
   h_FD DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_res_type VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_dayofyear DATE;
   h_last_submitted DATE;
   a_assignment_count PLS_INTEGER := 0;
   a_element_type PLS_INTEGER := 0;
   h_responsible PLS_INTEGER := 0;
   a_participant PLS_INTEGER := 0;
   a_resource_type PLS_INTEGER := 0;
   a_security_value PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_delay PLS_INTEGER := 0;
   h_percent_units PLS_INTEGER := 0;
   h_work_percent PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_new_actual PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_last_day PLS_INTEGER := 0;
   h_minutes PLS_INTEGER := 0;
   h_task_type PLS_INTEGER := 0;
   h_per_day PLS_INTEGER := 0;
   h_work_type PLS_INTEGER := 0;
   h_assign_count PLS_INTEGER := 0;
   h_duration PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_consider PLS_INTEGER := 0;
   h_eunit PLS_INTEGER := 0;
   h_unit PLS_INTEGER := 0;
   h_sched_effort PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_close_task PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0;
   h_reg_sale NUMBER := 0;
   h_spec_cost NUMBER := 0;
   h_spec_sale NUMBER := 0;
   h_b_reg_cost NUMBER := 0;
   h_b_reg_sale NUMBER := 0;
   h_b_spec_cost NUMBER := 0;
   h_b_spec_sale NUMBER := 0;
   stmt VARCHAR2(7999);
   i_resched PLS_INTEGER;
   i_bitflag PLS_INTEGER;
-- Declare Cursors
      -- Dynamic Cursor Declaration
      C_P_DELAY PMO_DB2M.REF_CURSOR;

      -- Dynamic Cursor Declaration
      C_R_DELAY PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_PARTICIPANT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_PARTICIPANT'), '-- ' || SYSTIMESTAMP, ':r := SP_U_PARTICIPANT(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || security_id001 || ''''
          || ', ' || COALESCE(NULLIF('''' || p_planned_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || p_planned_finish || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_units), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_actual), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 119+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_PARTICIPANT(120) : cplan.sql(16028) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_task_type := 0;
   h_created_by := '';
   h_rec_user := res_id001;
   a_security_id := security_id001;
   
   --Validate res_assignment_id
   a_temp_id := assign_id;
   
   BEGIN
      SELECT ra.RES_ASSIGNMENT_ID, ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE,
      CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,
      CASE WHEN ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,
      CASE WHEN ra.RESOURCE_TYPE = '0' THEN ra.RESOURCE_ID ELSE ra.PROFILE_ID END,
      ra.PROJECT_ID, ra.TABLE_TYPE
      INTO a_res_assignment_id, a_participant, a_element_id, a_element_type,
      a_resource_type,
      h_transfed_cri,
      a_participant_id,
      h_project_id, h_table_type
      FROM RESOURCE_ASSIGNMENTS ra
      WHERE ra.RES_ASSIGNMENT_ID = a_temp_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 151+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(152) : cplan.sql(16060) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("1_ SC_STATE: %d, a_temp_id: %s\n",SC_STATE,a_temp_id);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT RES_ASSIGNMENT_ID, PROJECT_ASSIGNMENT_ID, PROJECT_ID, RESOURCE_ID,
         RESOURCE_TYPE, BIT_FLAG, LAST_SUBMITTED, C_CONTOUR, C_EFFORT, T_UNIT, C_UNIT,
         CASE WHEN ASSIGN_UNITS > UNITS_MSP THEN ASSIGN_UNITS ELSE UNITS_MSP END, ACTUAL_START, WORK_TODATE, UNRESOLVED_START, UNRESOLVED_FINISH,
         CASE WHEN BIT_FLAG = '0' THEN PLANNED_WORK ELSE SCHED_WORK END, ASSIGNMENT_DELAY,LAST_SCHED_MIN,
         TO_DATE(SubStr(UNRESOLVED_START, 1, 10)), TO_DATE(SubStr(UNRESOLVED_FINISH, 1, 10)), BILLABLE, CAPITAL_EXPENSE,
         REGULAR_COST, SPECIAL_COST, REGULAR_RATE, SPECIAL_RATE,
         B_REGULAR_COST, B_SPECIAL_COST, B_REGULAR_RATE, B_SPECIAL_RATE,
         MILESTONE_PERCENT, TASK_ID, ROLLUP_BITS
         INTO a_res_assignment_id, h_prj_assign_id, h_project_id, h_res_id,
         h_res_type, h_bit_flag, h_last_submitted, h_contour, h_work, h_eunit, h_unit,
         h_percent_units, h_actual_start, h_work_todate, h_start_date, h_finish_date,
         h_effort, h_delay, h_last_day,
         h_SD, h_FD, h_billable, h_capital_expense,
         h_reg_cost, h_spec_cost, h_reg_sale, h_spec_sale,
         h_b_reg_cost, h_b_spec_cost, h_b_reg_sale, h_b_spec_sale,
         h_work_percent, h_task_id, h_rollup_bits
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = a_temp_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 183+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(184) : cplan.sql(16092) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, a_temp_id: %s\n", SC_STATE,a_temp_id);//trace_flush();
      --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD);
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         BEGIN
            SELECT PROJECT_ID, TASK_TYPE, MINUTE_PER_DAY,/*UNRESOLVED_START, UNRESOLVED_FINISH*/ START_DT, FINISH_DT, UNRESOLVED_DURATION,
            ASSIGN_COUNT, SD, WORK_TODATE, PLANNED_EFFORT + SCHED_EFFORT, WORK_PERCENT, ETC, COALESCE(FORECAST_START,FORECAST_FINISH)
            INTO h_project_id, h_task_type, h_per_day, h_start_date, h_finish_date, h_duration,
            h_assign_count, h_work_type, h_work_todate, h_effort, h_work_percent, h_etc, h_start_DT
            FROM TMT_WBS WHERE ELEMENT_ID = a_temp_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 203+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(204) : cplan.sql(16112) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_task_type: %d, h_work_percent: %d, h_start_date: %s, h_finish_date: %s\n", SC_STATE,h_task_type,h_work_percent,h_start_date,h_finish_date);//trace_flush();
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 211+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(212) : cplan.sql(16120) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 214+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(215) : cplan.sql(16123) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, a_temp_id, True);
         End If;
         
         h_sched_start := p_planned_start;
         h_sched_finish := p_planned_finish;
         If (PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0) AND (PMO_ORASTD.strncmp(
               h_sched_finish, h_finish_date, 10) <> 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 224+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(225) : cplan.sql(16133) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 227+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(228) : cplan.sql(16136) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, a_temp_id
                  , True);
         End If;
         --trace_printf("p_work_percent: %d, h_sched_start: %s, h_sched_finish: %s\n", *p_work_percent,h_sched_start,h_sched_finish);//trace_flush();
         --trace_printf("p_actual: %d, h_effort: %d, h_work_todate: %d\n", *p_actual,h_effort,h_work_todate);//trace_flush();
         
         If ((h_start_DT IS NOT NULL)) OR ((Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0)) <> Coalesce(
               h_etc, 0)) Then
            --trace_printf("ACCEPT_REJECT: SC_STATE: %d, h_start_DT: %d %s, h_etc: %d\n", SC_STATE,h_start_DT_Ind,h_start_DT,h_etc);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 239+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(240) : cplan.sql(16148) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_ASSIGN_ACCEPT_RE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 242+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(243) : cplan.sql(16151) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_ASSIGN_ACCEPT_RE, a_temp_id
                  , True);
         End If;
         
         If (PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0) OR (PMO_ORASTD.strncmp(h_sched_finish
               , h_finish_date, 10) <> 0) Then
            If Coalesce(h_assign_count, 0) = 0 Then
               If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_start
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_start, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 263+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(264) : cplan.sql(16172) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_start, 1, 10)), PMO_ORASTD.TIME('08:00')) INTO h_sched_start
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 278+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(279) : cplan.sql(16187) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  If PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0 Then
                     h_sched_finish := NULL;
                  End If;
               Else
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_finish
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_finish, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 299+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(300) : cplan.sql(16208) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_ORASTD.TIME('17:00')) INTO h_sched_finish
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 314+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(315) : cplan.sql(16223) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  
                  h_duration := 0;
               End If;
               
               If Coalesce(h_work_type, 0) = PMO_CNST.SC_SS Then
                  BEGIN
                     UPDATE TMT_WBS SET SCHED_START = h_sched_start, SCHED_FINISH = h_sched_finish, SCHED_DURATION = h_duration
                     WHERE ELEMENT_ID = a_temp_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 337+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(338) : cplan.sql(16246) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE TMT_WBS SET PLANNED_START = h_sched_start, PLANNED_FINISH = h_sched_finish, PLANNED_DURATION = h_duration
                     WHERE ELEMENT_ID = a_temp_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 355+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(356) : cplan.sql(16264) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_temp_var := FC_Rollup(h_project_id);
               End If;
            Else
               BEGIN
                  SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (
                  SELECT 'X' FROM TASK_ASSIGNMENTS
                  WHERE PROJECT_ID = h_project_id AND TASK_ID = a_temp_id
                  AND ACTUAL_START IS NOT NuLL);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 376+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(377) : cplan.sql(16285) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("1_* SC_STATE: %d, h_temp_var: %d\n", SC_STATE,h_temp_var);
               
               If Coalesce(h_temp_var, 0) > 0 Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 384+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(385) : cplan.sql(16293) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 387+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(388) : cplan.sql(16296) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL, a_temp_id
                        , True);
               End If;
               
               If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_start
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_start, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 405+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(406) : cplan.sql(16314) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_start, 1, 10)), PMO_ORASTD.TIME('08:00')) INTO h_sched_start
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 420+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(421) : cplan.sql(16329) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               Else
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_sched_finish
                     FROM CALENDAR_DAY_OF_YEAR
                     WHERE ELEMENT_ID = h_project_id
                     AND DAY_YEAR = TO_DATE(SubStr(h_sched_finish, 1, 10));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 437+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(438) : cplan.sql(16346) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_ORASTD.TIME('17:00')) INTO h_sched_finish
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 452+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(453) : cplan.sql(16361) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               h_temp_var := FC_CHANGE_TA_DATES(h_project_id, a_temp_id, h_start_date, h_finish_date, h_sched_start
                                 , h_sched_finish, h_per_day, h_rec_user, Coalesce(h_task_type, 0)
                                 , 0);
               --trace_printf("***** FC_CHANGE_TA_DATES h_temp_var: %d\n", h_temp_var);
               If Coalesce(h_temp_var, 0) = 0 Then
                  h_temp_var := FC_Rollup(h_project_id);
               Else
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 467+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(468) : cplan.sql(16376) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 470+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(471) : cplan.sql(16379) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
               End If;
            End If;
         ElsIf (Coalesce(p_actual, 0) <> Coalesce(h_work_todate, 0)) AND (Coalesce(h_effort, 0) > 0) Then
            h_close_task := 0;
            If Coalesce(p_actual, 0) > Coalesce(h_effort, 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_PARTICIPANT', 479+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_PARTICIPANT(480) : cplan.sql(16388) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TASK_ACTUAL_GRATER_THAN_SCH), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 482+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(483) : cplan.sql(16391) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.SC_TASK_ACTUAL_GRATER_THAN_SCH, True);
            ElsIf Coalesce(p_actual, 0) = Coalesce(h_effort, 0) Then
               h_close_task := 1;
            End If;
            
            -- varify if there are steps in timesheets
            h_temp_var := 0;
            BEGIN
               SELECT COUNT('X') INTO h_temp_var
               FROM DUAL
               WHERE EXISTS(SELECT A.ELEMENT_ID FROM TASK_ASSIGNMENTS TA, ACTIVITIES A
               WHERE TA.PROJECT_ID = h_project_id
               AND TA.TASK_ID = a_temp_id
               AND TA.TSK_ASSIGNMENT_ID = A.TSK_ASSIGNMENT_ID
               AND A.TSK_ASSIGNMENT_ID <> A.ELEMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 505+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(506) : cplan.sql(16414) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) > 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 512+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(513) : cplan.sql(16421) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TIMESHEET_STEP_EXISTS), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 515+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(516) : cplan.sql(16424) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.SC_TIMESHEET_STEP_EXISTS, a_temp_id
                     , True);
            End If;
            
            -- varify if there are approved timesheets
            BEGIN
               SELECT COUNT('X') INTO h_temp_var FROM DUAL WHERE EXISTS (
               SELECT 'X' FROM TASK_ASSIGNMENTS ta JOIN TIMESHEET_APPROVAL_STATUS apv
               ON apv.ELEMENT_ID = ta.PROJECT_ID AND apv.RESOURCE_ID = ta.RESOURCE_ID
               WHERE ta.PROJECT_ID = h_project_id AND ta.TASK_ID = a_temp_id AND
               (TO_DATE(SubStr(ta.ACTUAL_START, 1, 10)) - 7     ) <= apv.WEEK_OF);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 534+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(535) : cplan.sql(16443) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("*** h_temp_var: %d\n", h_temp_var);
            
            If Coalesce(h_temp_var, 0) > 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 542+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(543) : cplan.sql(16451) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TS_APPROVAL_ERROR), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 545+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(546) : cplan.sql(16454) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TS_APPROVAL_ERROR, a_temp_id, True);
            End If;
            
            h_new_actual := p_actual;
            --InitializeHost(SC_PARAM(h_work_percent), int(((h_new_actual / h_effort) * 10000.00) + 0.4));      // new actual work
            --h_billable = (h_billable | SC_BIT_30);
            --trace_printf("h_work_percent: %d, h_new_actual: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_new_actual,h_work_todate,h_effort);
            --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD);
            
            h_temp_var := FC_U_TASK_PERCENT(h_project_id, a_temp_id, h_new_actual, h_rec_user, Coalesce(
                                h_close_task, 0));
         End If;
         
         --trace_printf("***** FC_U_TASK_PERCENT h_temp_var: %d\n", h_temp_var);
         If Coalesce(h_temp_var, 0) = 0 Then
            h_temp_var := FC_Rollup(h_project_id);
         End If;
         
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 566+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(567) : cplan.sql(16475) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 569+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(570) : cplan.sql(16478) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
      ElsIf (p_planned_start IS NOT NULL) AND (p_planned_finish IS NOT NULL) Then
         h_sched_start := p_planned_start;
         h_sched_finish := p_planned_finish;
         
         If (PMO_ORASTD.strncmp(h_sched_start, h_sched_finish, 10) > 0) AND (PMO_ORASTD.strncmp(
               h_sched_finish, h_finish_date, 10) <> 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 579+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(580) : cplan.sql(16488) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 582+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(583) : cplan.sql(16491) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, a_temp_id
                  , True);
         End If;
         
         BEGIN
            SELECT PROJECT_ID, TASK_TYPE, MINUTE_PER_DAY
            INTO h_project_id, h_task_type, h_per_day
            FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 598+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(599) : cplan.sql(16507) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("SC_STATE: %d, h_task_type: %d, h_per_day: %d\n", SC_STATE,h_task_type,h_per_day);//trace_flush();
         
         If SubStr(h_res_type, 1, 1) = '1' Then
            a_resource_type := 1;
         Else
            a_resource_type := 0;
         End If;
         
         i_resched := 0;
         i_bitflag := 0;
         
         --trace_printf("SC_STATE: %d, h_percent_units: %d, h_sched_start: %s, h_sched_finish: %s\n", SC_STATE,h_percent_units,h_sched_start,h_sched_finish);
         --trace_printf("h_start_date: %s, h_finish_date: %s\n", h_start_date,h_finish_date);
         --trace_printf("h_last_submitted: %d:%s, h_work_todate: %d, h_effort = %d\n", h_last_submitted_Ind,h_last_submitted,h_work_todate,h_effort);
         --trace_printf("p_units: %d, h_percent_units: %d\n", *p_units,h_percent_units);
         --trace_printf("p_delay: %d, h_delay: %d\n", *p_delay,h_delay);
         --trace_printf("p_work_percent: %d, h_work_percent: %d\n", *p_work_percent,h_work_percent);
         If (PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0) OR (PMO_ORASTD.strncmp(h_sched_finish
               , h_finish_date, 10) <> 0) OR (Coalesce(p_delay, 0) <> Coalesce(h_delay, 0)) Then
            If PMO_ORASTD.strncmp(h_sched_start, h_start_date, 10) <> 0 Then
               BEGIN
                  SELECT TASK_TYPE, MINUTE_PER_DAY INTO h_task_type, h_per_day
                  FROM TMT_WBS
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 632+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(633) : cplan.sql(16541) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_task_type: %d, h_task_id: %s\n", SC_STATE,h_task_type,h_task_id);//trace_flush();
               
               /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
               					{
               						BEGIN_SQL
               							SELECT TIMESTAMP(DATE(HOST(h_sched_start)), TIME(HOST(h_start_date)))
               							INTO HOST(h_sched_start)
               							FROM DUAL;
               						END_SQL;
               
               						if (checkFlagStatus(h_bit_flag, '2'))
               						{
               							InitializeHost(SC_PARAM(h_work_type), 2);
               						}
               						else
               						{
               							InitializeHost(SC_PARAM(h_work_type), 0);
               						}
               
               						i_bitflag = strcmp(h_sched_start,h_start_date); // new date , old date
               						InitializeHost(SC_PARAM(h_tph_delete), -1);
               						h_temp_var = FC_TPHASE_SCHED_ASSIGN(
               									SC_PARAM(h_project_id),
               									SC_PARAM(h_task_id),
               									SC_PARAM(a_temp_id),
               									SC_PARAM(h_SD),
               									SC_PARAM(h_FD),
               									SC_PARAM(h_reg_cost),
               									SC_PARAM(h_reg_sale),
               									SC_PARAM(h_tph_delete),
               									SC_PARAM(h_capital_expense),
               									SC_PARAM(h_billable));
               
               						if ( (h_last_day > 0)  (IsNotNullParam(h_last_submitted)))
               						{
               							BEGIN_SQL
               								UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
               								WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id) AND DAY_YEAR = DATE(HOST(h_last_submitted));
               							END_SQL;
               						}
               
               						h_temp_var = FC_FD_CTD_TDY( SC_PARAM(h_project_id),
               													SC_PARAM(h_task_id),
               													SC_PARAM(a_temp_id),
               													SC_PARAM(h_res_id),
               													SC_PARAM(h_res_type),
               													SC_PARAM(h_SD),             // old task assign dates
               													SC_PARAM(h_FD),
               													SC_PARAM(h_sched_start),    // new task dates
               													SC_PARAM(h_finish_date),
               													SC_PARAM(h_work_type),
               													SC_PARAM(h_per_day),
               													SC_PARAM(h_percent_units),
               													SC_PARAM(h_rec_user), 1, i_bitflag );
               
               						BEGIN_SQL
               							SELECT DATE(HOST(h_sched_start)), DATE(HOST(h_finish_date)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
               							INTO HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
               							FROM TASK_ASSIGNMENTS
               							WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id) ;
               						END_SQL;
               
               						InitializeHost(SC_PARAM(h_tph_delete), 0);
               						h_temp_var = FC_TPHASE_SCHED_ASSIGN(
               									SC_PARAM(h_project_id),
               									SC_PARAM(h_task_id),
               									SC_PARAM(a_temp_id),
               									SC_PARAM(h_SD),
               									SC_PARAM(h_FD),
               									SC_PARAM(h_reg_cost),
               									SC_PARAM(h_reg_sale),
               									SC_PARAM(h_tph_delete),
               									SC_PARAM(h_capital_expense),
               									SC_PARAM(h_billable));
               
               						h_temp_var = FC_Rollup(h_project_id);
               						RETURN_STATUS(h_temp_var, a_res_assignment_id);
               					}
               					else
               					{*/
               h_start_date := h_sched_start;
               h_delay := 0;
               i_resched := 1;
               --}
            ElsIf PMO_ORASTD.strncmp(h_sched_finish, h_finish_date, 10) <> 0 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_sched_finish, 1, 10)), PMO_ORASTD.TIME(h_finish_date))
                  INTO h_sched_finish
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 731+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(732) : cplan.sql(16640) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               i_bitflag := PMO_ORASTD.strcmp(h_sched_finish, h_finish_date);
               -- new date , old date
               h_tph_delete := -1;
               h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
               
               If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
                     WHERE TSK_ASSIGNMENT_ID = a_temp_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 756+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(757) : cplan.sql(16665) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               If SubStr(h_bit_flag, 1, 1) = '2' Then
                  h_work_type := 2;
               Else
                  h_work_type := 0;
               End If;
               
               h_temp_var := FC_FD_CTD_TDY(h_project_id, h_task_id, a_temp_id, h_res_id, h_res_type, h_SD, h_FD
                            , h_start_date, h_sched_finish, h_work_type, h_per_day, h_percent_units
                            , h_rec_user, 2, i_bitflag, 0);
               
               BEGIN
                  SELECT TO_DATE(SubStr(h_start_date, 1, 10)), TO_DATE(SubStr(h_sched_finish, 1, 10)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
                  INTO h_SD, h_FD, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = a_temp_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 783+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(784) : cplan.sql(16692) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               h_tph_delete := 0;
               h_temp_var := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                     , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
               
               h_temp_var := FC_Rollup(h_project_id);
               
               PMO_DB2M.SC_STATE := 0;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_PARTICIPANT', 796+1) THEN 
                  LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_PARTICIPANT(797) : cplan.sql(16705) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_res_assignment_id || '''');
               END IF; 
               PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, a_res_assignment_id, True);
            ElsIf Coalesce(h_delay, 0) < Coalesce(p_delay, 0) Then
               h_delay := Coalesce(p_delay, 0) - Coalesce(h_delay, 0);
               i_resched := 1;
            ElsIf Coalesce(p_delay, 0) = 0 Then
               h_delay := 0;
               i_resched := 1;
               
               BEGIN
                  SELECT START_DT INTO h_start_date
                  FROM TMT_WBS
                  WHERE ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 816+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(817) : cplan.sql(16725) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            ElsIf Coalesce(h_delay, 0) > Coalesce(p_delay, 0) Then
               h_delay := Coalesce(h_delay, 0) - Coalesce(p_delay, 0);
               h_dayofyear := NULL;
               stmt := '';
               
               If Coalesce(a_resource_type, 0) = 1 Then
                  stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                         || 'WHERE ELEMENT_ID = ''' || h_project_id || ''' AND DAY_YEAR < TO_DATE(SubStr(''' 
                              || h_start_date || ''', 1, 10)) '
                         || 'ORDER BY DAY_YEAR DESC) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
                  
                  OPEN C_P_DELAY FOR stmt;
               Else
                  stmt := 'SELECT * FROM (SELECT DAY_YEAR FROM CALENDAR_DAY_OF_YEAR '
                         || 'WHERE ELEMENT_ID = ''' || h_res_id || ''' AND DAY_YEAR < TO_DATE(SubStr(''' || h_start_date 
                              || ''', 1, 10)) '
                         || 'ORDER BY DAY_YEAR DESC) WHERE ROWNUM<=' || TO_CHAR(Coalesce(h_delay, 0));
                  
                  OPEN C_R_DELAY FOR stmt;
               End If;
               
               LOOP
                  If Coalesce(a_resource_type, 0) = 1 Then
                     FETCH C_P_DELAY INTO h_dayofyear;
                     IF C_P_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  Else
                     FETCH C_R_DELAY INTO h_dayofyear;
                     IF C_R_DELAY%FOUND THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  End If;
                  
                  If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                     EXIT;
                  End If;
               END LOOP;
               
               If Coalesce(a_resource_type, 0) = 1 Then
                  BEGIN
                     CLOSE C_P_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 872+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(873) : cplan.sql(16781) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     CLOSE C_R_DELAY ;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 885+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(886) : cplan.sql(16794) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               --trace_printf("PPPPPPP000: h_dayofyear: %d, %s\n%s\n", h_dayofyear_Ind, h_dayofyear,stmt);
               If (h_dayofyear IS NULL) Then
                  If Coalesce(a_resource_type, 0) = 1 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 895+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(896) : cplan.sql(16804) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_project_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 898+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(899) : cplan.sql(16807) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_project_id, True);
                  Else
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 903+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(904) : cplan.sql(16812) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 906+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(907) : cplan.sql(16815) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_res_id, True);
                  End If;
               End If;
               
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_dayofyear, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                  INTO h_start_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 922+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(923) : cplan.sql(16831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("PPPPPPP0001: h_start_date: %d, %s\n", h_start_date_Ind, h_start_date);
               h_delay := 0;
               i_resched := 1;
            End If;
         ElsIf Coalesce(p_units, 0) <> Coalesce(h_percent_units, 0) Then
            If Coalesce(h_work_todate, 0) = Coalesce(h_effort, 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 933+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(934) : cplan.sql(16842) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 936+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(937) : cplan.sql(16845) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, a_temp_id, True);
            End If;
            
            If Coalesce(p_units, 0) = 0 Then
               h_percent_units := 100;
            Else
               If (Coalesce(p_units, 0) < 1) OR (Coalesce(p_units, 0) > 103079040) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_PARTICIPANT', 946+1) THEN 
                     LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_PARTICIPANT(947) : cplan.sql(16855) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 949+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(950) : cplan.sql(16858) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
               End If;
               
               h_percent_units := p_units;
            End If;
            
            If (h_prj_assign_id IS NOT NULL) Then
               BEGIN
                  SELECT UNITS_MSP INTO h_temp_var
                  FROM TASK_ASSIGNMENTS
                  WHERE TSK_ASSIGNMENT_ID = h_prj_assign_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 968+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(969) : cplan.sql(16877) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_temp_var: %d\n", SC_STATE,h_temp_var);//trace_flush();
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_percent_units, 0) > Coalesce(
                     h_temp_var, 0)) Then
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 977+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(978) : cplan.sql(16886) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_prj_assign_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 980+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(981) : cplan.sql(16889) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_prj_assign_id
                        , True);
               End If;
            End If;
            
            /*if ( h_task_type == SC_FD_EFFORT_DRIVEN )
            				{
            					InitializeHost(SC_PARAM(h_tph_delete), -1);
            					h_temp_sc_state = FC_TPHASE_SCHED_ASSIGN(
            								SC_PARAM(h_project_id),
            								SC_PARAM(h_task_id),
            								SC_PARAM(a_temp_id),
            								SC_PARAM(h_SD),
            								SC_PARAM(h_FD),
            								SC_PARAM(h_reg_cost),
            								SC_PARAM(h_reg_sale),
            								SC_PARAM(h_tph_delete),
            								SC_PARAM(h_capital_expense),
            								SC_PARAM(h_billable));
            
            					if ( (h_last_day > 0)  (IsNotNullParam(h_last_submitted)))
            					{
            						BEGIN_SQL
            							UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = ACTUAL_MIN
            							WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id) AND DAY_YEAR = DATE(HOST(h_last_submitted));
            						END_SQL;
            					}
            
            					h_minutes = int(((h_percent_units * h_per_day) / 10000.00) + 0.4);
            					h_temp_var = FC_FD_U_TDY (SC_PARAM(h_project_id),
            											  SC_PARAM(a_temp_id),
            											  SC_PARAM(h_start_date),
            											  SC_PARAM(h_per_day),
            											  SC_PARAM(h_minutes));
            
            					InitializeHost(SC_PARAM(h_temp_var), h_percent_units | SC_BIT_30 );
            					h_temp_var = U_ASSIGN_STRING(SC_PARAM(h_project_id), SC_PARAM(a_temp_id), SC_PARAM(h_temp_var));
            
            					BEGIN_SQL
            						SELECT DATE(HOST(h_start_date)), DATE(HOST(h_sched_finish)), BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
            						INTO HOST(h_SD), HOST(h_FD), HOST(h_billable), HOST(h_capital_expense), HOST(h_reg_cost), HOST(h_reg_sale)
            						FROM TASK_ASSIGNMENTS
            						WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id) ;
            					END_SQL;
            
            					InitializeHost(SC_PARAM(h_tph_delete), 0);
            					h_temp_var = FC_TPHASE_SCHED_ASSIGN(
            								SC_PARAM(h_project_id),
            								SC_PARAM(h_task_id),
            								SC_PARAM(a_temp_id),
            								SC_PARAM(h_SD),
            								SC_PARAM(h_FD),
            								SC_PARAM(h_reg_cost),
            								SC_PARAM(h_reg_sale),
            								SC_PARAM(h_tph_delete),
            								SC_PARAM(h_capital_expense),
            								SC_PARAM(h_billable));
            
            					h_temp_var = FC_Rollup(h_project_id);
            					RETURN_ERROR(h_temp_var, a_temp_id);
            				}
            				else
            				{*/
            h_delay := 0;
            i_resched := 2;
            --move the update of task assignments tables when I-resched = 2 since this is the only case i_resched is initilaized to 2
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, UNITS_MSP = h_percent_units,
               C_EFFORT = 0, C_UNIT = 0, C_CONTOUR = 0, CONTOUR_TYPE = 0,
               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT))
               WHERE TSK_ASSIGNMENT_ID = a_temp_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1063+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1064) : cplan.sql(16972) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --also if resource was assigned with contour but user changes %units then we can't keep the contour, it has to reschedule as ASAP
            --contour contradicts with the change of %units
            
            If Coalesce(h_contour, 0) <> 0 Then
               h_contour := 0;
            End If;
            --}
            --trace_printf("**: h_task_type: %d, h_minutes: %d, h_percent_units: %d\n", h_task_type,h_minutes,h_percent_units);
         End If;
         
         --trace_printf("**===> i_resched: %d, h_last_day: %d, h_prj_assign_id: %d %s\n", i_resched,h_last_day,h_prj_assign_id_Ind,h_prj_assign_id);
         If i_resched > 0 Then
            h_tph_delete := -1;
            h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                          , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
            
            If (Coalesce(h_last_day, 0) > 0) AND ((h_last_submitted IS NOT NULL)) Then
               h_temp_var := FC_U_TDY_LAST_ACTUAL(h_project_id, h_task_id, h_res_id, h_prj_assign_id, a_temp_id
                                   , h_last_submitted, h_last_day, 0);
            End If;
            
            --trace_printf("h_prj_assign_id: %d %s\n", h_prj_assign_id_Ind,h_prj_assign_id);
            If (h_prj_assign_id IS NOT NULL) Then
               h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 2);
            Else
               If SubStr(h_bit_flag, 1, 1) = '2' Then
                  h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 0);
               Else
                  h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 1);
               End If;
            End If;
            
            If (h_last_submitted IS NOT NULL) Then
               If Coalesce(h_contour, 0) > 0 Then
                  h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
                  
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                     INTO h_start_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1114+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1115) : cplan.sql(17023) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
                     INTO h_start_date
                     FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1129+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1130) : cplan.sql(17038) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               h_effort := Coalesce(h_effort, 0) - Coalesce(h_work_todate, 0);
            End If;
            
            --trace_printf("1_FC_DEL_NOACTUAL: h_start_date: %s, h_effort: %d, h_percent_units: %d\n",h_start_date,h_effort,h_percent_units);
            
            h_temp_bitflag := 2;
            h_temp_sc_state := FC_RESCHEDULE(a_temp_id, h_start_date, h_effort, h_temp_bitflag, h_contour
                                 , h_rec_user, Coalesce(h_delay, 0), Coalesce(h_percent_units, 0));
            --trace_printf("2_FC_RESCHEDULE: h_temp_sc_state: %d, i_resched: %d\n",h_temp_sc_state,i_resched); //trace_flush();
            
            If Coalesce(h_temp_sc_state, 0) <> 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1147+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1148) : cplan.sql(17056) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1150+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1151) : cplan.sql(17059) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, a_temp_id, True);
            ElsIf i_resched = 1 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,(PMO_CNST.SC_SCHED_BIT+PMO_CNST.SC_PLANNED_BIT)),
                  BAK_START_DT = COALESCE(SCHED_START,PLANNED_START), BAK_FINISH_DT = COALESCE(SCHED_FINISH,PLANNED_FINISH),
                  BAK_ASSIGN_STRING = ASSIGN_STRING
                  WHERE TSK_ASSIGNMENT_ID = a_temp_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1169+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1170) : cplan.sql(17078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            h_temp_var := FC_Rollup(h_project_id);
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1177+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1178) : cplan.sql(17086) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1180+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1181) : cplan.sql(17089) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
         End If;
         
         If Coalesce(p_actual, 0) <> Coalesce(h_work_todate, 0) Then
            -- varify if there are steps in timesheets
            BEGIN
               SELECT COUNT('X') INTO h_temp_var
               FROM DUAL
               WHERE EXISTS(SELECT ELEMENT_ID FROM ACTIVITIES
               WHERE TSK_ASSIGNMENT_ID = a_temp_id
               AND TSK_ASSIGNMENT_ID <> ELEMENT_ID);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1199+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1200) : cplan.sql(17108) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_temp_var, 0) > 0 Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1206+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1207) : cplan.sql(17115) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.SC_TIMESHEET_STEP_EXISTS), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1209+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1210) : cplan.sql(17118) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.SC_TIMESHEET_STEP_EXISTS, a_temp_id
                     , True);
            End If;
            
            h_new_actual := p_actual;
            -- new actual work
            h_sched_effort := Coalesce(h_effort, 0);
            
            If Coalesce(h_new_actual, 0) > Coalesce(h_effort, 0) Then
               h_tph_delete := -1;
               h_temp_sc_state := FC_TPHASE_SCHED_ASSIGN(h_project_id, h_task_id, a_temp_id, h_SD, h_FD, h_reg_cost
                                          , h_reg_sale, h_tph_delete, h_capital_expense, h_billable);
               
               If (h_prj_assign_id IS NOT NULL) Then
                  h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 2);
               Else
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 0);
                  Else
                     h_temp_sc_state := FC_DEL_NOACTUAL(h_project_id, a_temp_id, 1);
                  End If;
               End If;
               
               h_minutes := 0;
               If (h_last_submitted IS NOT NULL) Then
                  If Coalesce(h_contour, 0) > 0 Then
                     h_contour := (PMO_FUNC.BIT_OR(Coalesce(h_contour, 0), PMO_CNST.SC_BIT_30));
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date))
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1248+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1249) : cplan.sql(17157) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        SELECT PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(h_last_submitted, 1, 10)), PMO_ORASTD.TIME(h_start_date)) + 1     
                        INTO h_start_date
                        FROM DUAL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1263+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1264) : cplan.sql(17172) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  If Coalesce(h_last_day, 0) > 0 Then
                     BEGIN
                        SELECT SCHED_MIN - ACTUAL_MIN INTO h_minutes
                        FROM TASK_DAY_OF_YEAR 
                        WHERE TSK_ASSIGNMENT_ID = a_temp_id AND DAY_YEAR = TO_DATE(SubStr(h_last_submitted, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1279+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1280) : cplan.sql(17188) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
               
               h_effort := Coalesce(h_new_actual, 0) - Coalesce(h_work_todate, 0) - Coalesce(h_minutes, 0);
               
               h_work_percent := 10000;
               
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET CONSIDER = 7, ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_SCHED_BIT),
                  SCHED_WORK = h_new_actual, WORK_TODATE = h_new_actual, ETC = 0, MILESTONE_PERCENT = 10000,
                  FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0
                  WHERE TSK_ASSIGNMENT_ID = a_temp_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1305+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1306) : cplan.sql(17214) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  --trace_printf("*** h_contour: %d, h_new_actual: %d, h_start_date: %s, h_effort: %d, h_work_todate: %d\n",h_contour,h_new_actual,h_start_date,h_effort,h_work_todate);
                  h_temp_bitflag := 2;
                  h_temp_var := FC_RESCHEDULE(a_temp_id, h_start_date, h_effort, h_temp_bitflag, h_contour, h_rec_user
                            , 0, Coalesce(h_percent_units, 0));
                  --trace_printf("3_FC_RESCHEDULE: h_temp_sc_state: %d, i_resched: %d\n",h_temp_sc_state,i_resched); //trace_flush();
                  
                  If Coalesce(h_temp_var, 0) <> 0 Then
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1319+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1320) : cplan.sql(17228) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1322+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1323) : cplan.sql(17231) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
                  End If;
               End If;
               
               h_effort := Coalesce(h_new_actual, 0);
            Else
               h_work_percent := Trunc(((Coalesce(h_new_actual, 0) / Coalesce(h_effort, 0)) * 10000.00) + 0.4);
            End If;
            
            --trace_printf("h_work_percent: %d, h_last_day: %d, h_new_actual: %d, h_work_todate: %d, h_effort = %d\n", h_work_percent,h_last_day,h_new_actual,h_work_todate,h_effort); //trace_flush();
            --trace_printf("h_SD: %d %s, h_FD: %d %s\n", h_SD_Ind,h_SD,h_FD_Ind,h_FD); //trace_flush();
            
            h_temp_var := FC_U_ASSIGN_PERCENT(h_project_id, h_task_id, h_res_id, a_temp_id, h_last_submitted
                                  , h_bit_flag, h_effort, h_new_actual, h_sched_effort
                                  , h_work_todate, h_work_percent, h_reg_cost, h_spec_cost
                                  , h_reg_sale, h_spec_sale, h_b_reg_cost, h_b_spec_cost
                                  , h_b_reg_sale, h_b_spec_sale, h_capital_expense, h_billable
                                  , h_last_day, h_rec_user, Coalesce(h_per_day, 0));
            
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1344+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1345) : cplan.sql(17253) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1347+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1348) : cplan.sql(17256) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
         ElsIf Coalesce(p_work_percent, 0) <> Coalesce(h_work_percent, 0) Then
            If (Coalesce(h_work_todate, 0) > 0) AND (Coalesce(p_work_percent, 0) = 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1353+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1354) : cplan.sql(17262) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1356+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1357) : cplan.sql(17265) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, a_temp_id, True);
            End If;
            
            /*if (*p_work_percent == 10000)
            				{
            					BEGIN_SQL
            						UPDATE TASK_ASSIGNMENTS SET
            							SCHED_WORK = WORK_TODATE,
            							ETC = 0, FD = SC_AF,
            							MILESTONE_PERCENT = 10000
            						WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
            					END_SQL;
            
            					if( SC_SUCCESS )
            					{
            						InitializeHost(SC_PARAM(h_temp_var), 1);
            						h_temp_var = FC_ETC_SCHEDULE(SC_PARAM(a_temp_id), SC_PARAM(h_temp_var), SC_PARAM(h_rec_user));
            					}
            				}
            				else if ( h_work_percent == 10000 )
            				{
            					BEGIN_SQL
            						UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_security),
            						ETC = SCHED_WORK - WORK_TODATE, FD = SC_SF, ACTUAL_FINISH = SC_NULL, ACTUAL_DURATION = 0
            						WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
            					END_SQL;
            				}
            				else
            				{
            					BEGIN_SQL
            						UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = HOST_DB(p_work_percent,ind_security),
            						ETC = SCHED_WORK - WORK_TODATE
            						WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
            					END_SQL;
            				}*/
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET MILESTONE_PERCENT = p_work_percent,
               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
               WHERE TSK_ASSIGNMENT_ID = a_temp_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1408+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1409) : cplan.sql(17317) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            /*
            				BEGIN_SQL
            					UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = HOST(h_rollup_bits) WHERE TSK_ASSIGNMENT_ID = HOST(a_temp_id);
            				END_SQL;
            */
            BEGIN
               UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1430+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1431) : cplan.sql(17339) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --RETURN_ERROR(SC_STATE, a_temp_id);
         End If;
         
         h_temp_var := FC_Rollup(h_project_id);
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1440+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1441) : cplan.sql(17349) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_var), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1443+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1444) : cplan.sql(17352) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_var, 0), a_temp_id, True);
      Else
         BEGIN
            SELECT ra.SECURITY_ID, ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE,
            CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,
            CASE WHEN ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,
            CASE WHEN ra.RESOURCE_TYPE = '0' THEN ra.RESOURCE_ID ELSE ra.PROFILE_ID END,
            ra.PROJECT_ID, ra.TABLE_TYPE
            INTO h_security_id, a_participant, a_element_id, a_element_type,
            a_resource_type,
            h_transfed_cri,
            a_participant_id,
            h_project_id, h_table_type
            FROM RESOURCE_ASSIGNMENTS ra WHERE ra.RES_ASSIGNMENT_ID = a_res_assignment_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 1465+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(1466) : cplan.sql(17374) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         --trace_printf("3_ SC_STATE: %d, a_temp_id: %s\n",SC_STATE,a_temp_id);
         
         If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1473+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1474) : cplan.sql(17382) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || a_res_assignment_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1476+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1477) : cplan.sql(17385) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, a_res_assignment_id
                  , True);
         End If;
         
         If PMO_ORASTD.strcmp(a_security_id, h_security_id) = 0 Then
            PMO_DB2M.SC_STATE := 0;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_PARTICIPANT', 1485+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_PARTICIPANT(1486) : cplan.sql(17394) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_res_assignment_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_PARTICIPANT', 1488+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_PARTICIPANT(1489) : cplan.sql(17397) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_DB2M.SC_STATE, a_res_assignment_id, True);
         End If;
      End If;
   End If;
   
   a_temp_id := assign_id;
   a_security_id := security_id001;
   h_rec_user := res_id001;
   
   h_table_type := PMO_CNST.SC_TABLE_WBS;
   PMO_DB2M.SC_STATE := FC_U_ASSIGNED_SECURITY(h_table_type, a_temp_id, a_security_id, h_rec_user);
   /*
   	if ( (IsNullParam(h_table_type)) || (h_table_type == 0) )
   	{
   		InitializeHost(SC_PARAM(h_table_type), GetTable(a_element_type));
   	}
   
   	//Retrive participant name for updating AssignNames field
   	if( a_resource_type == 0)
   	{
   		BEGIN_SQL
   			SELECT NICK_NAME INTO HOST(a_participant_name) FROM RESOURCES WHERE RESOURCE_ID = :a_participant_id;
   		END_SQL;
   //trace_printf("4_ SC_STATE: %d, a_temp_id: %s\n",SC_STATE,a_temp_id);
   		if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_participant_id); }
   	}
   	else if( a_resource_type == 1)
   	{
   		BEGIN_SQL
   			SELECT PROFILE_NAME INTO HOST(a_participant_name) FROM PROFILES WHERE PROFILE_ID = :a_participant_id;
   		END_SQL;
   		if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_participant_id); }
   	}
   	if( ( a_resource_type > 1) || (IsNullParam(a_participant_name))) {InitializeHost(SC_PARAM(a_participant_name),""); }
   //trace_printf("* h_table_type: %d, a_element_id: %s\n",h_table_type,a_element_id);
   
   	switch( h_table_type)
   	{
   		case SC_TABLE_SCD :
   			BEGIN_SQL
   				SELECT CREATED_BY INTO HOST(h_created_by) FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   			break;
   
   		case SC_TABLE_POL :
   			BEGIN_SQL
   				SELECT CREATED_BY INTO HOST(h_created_by) FROM TMT_POOLS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		case SC_TABLE_CRI:
   			BEGIN_SQL
   				SELECT CREATED_BY INTO HOST(h_created_by) FROM TMT_CRI WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		case SC_TABLE_DOC:
   			BEGIN_SQL
   				SELECT AUTHOR_ID INTO HOST(h_created_by) FROM TMT_DOCUMENTS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		case SC_TABLE_POL_DOC:
   			BEGIN_SQL
   				SELECT AUTHOR_ID INTO HOST(h_created_by) FROM TMT_POOL_DOCUMENTS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		case SC_TABLE_RES_DOC:
   			BEGIN_SQL
   				SELECT AUTHOR_ID INTO HOST(h_created_by) FROM TMT_RESOURCE_DOCUMENTS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		case SC_TABLE_AST_DOC:
   			BEGIN_SQL
   				SELECT AUTHOR_ID INTO HOST(h_created_by) FROM TMT_ASSET_DOCUMENTS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		case SC_TABLE_CLN_DOC:
   			BEGIN_SQL
   				SELECT AUTHOR_ID INTO HOST(h_created_by) FROM TMT_CLIENT_DOCUMENTS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		case SC_TABLE_VDR_DOC:
   			BEGIN_SQL
   				SELECT AUTHOR_ID INTO HOST(h_created_by) FROM TMT_VENDOR_DOCUMENTS WHERE ELEMENT_ID = HOST(a_element_id);
   			END_SQL;
   
   			break;
   		default :
   			BEGIN_SQL
   				SELECT CREATED_BY INTO HOST(h_created_by) FROM TMT_WBS WHERE ELEMENT_ID = :a_element_id;
   			END_SQL;
   
   			break;
   	}
   
   //trace_printf("5_ SC_STATE: %d, h_created_by: %s\n",SC_STATE,h_created_by);
   	if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_element_id); }
   
   //trace_printf("a_element_id = %s, a_element_type: %d, a_participant_id = %s\nh_created_by = %s\n", a_element_id, a_element_type, a_participant_id,h_created_by);
   	//Validate security_id
   	InitializeHost(SC_PARAM(a_security_id), security_id, ind_security->security_id);
   	BEGIN_SQL
   		SELECT SECURITY_GROUP INTO HOST(a_security_value)
   		FROM TMT_GLOBAL_SECURITY WHERE ELEMENT_ID = HOST(a_security_id);
   	END_SQL;
   
   	if ( ((a_security_value  SC_TEAMMEMBER) == SC_TEAMMEMBER)  ((a_participant  SC_TEAMMEMBER) != SC_TEAMMEMBER) )
   		a_security_value = SC_PARTICIPANT;
   
   	if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_security_id); }
   //trace_printf("a_participant = %d, a_security_value = %d, a_security_id: %s\n", a_participant,a_security_value,a_security_id);
   
   	if ( (a_participant  SC_TEAMMEMBER) == SC_TEAMMEMBER )
   		a_security_value = a_security_value | SC_TEAMMEMBER;
   
   	//Do not remove the only responsible
   	if( ((a_participant  SC_WBS_RESPONSIBLE) > 0)    ((a_security_value  SC_WBS_RESPONSIBLE) == 0) )
   	{
   
   //trace_printf("\n 8163 a_participant %d",a_participant);//trace_flush();
   //trace_printf("\n 8163 a_security_value %d",a_security_value);//trace_flush();
   		if ( strcmp(h_created_by,a_participant_id) == 0 )
   		{
   			RETURN_ERROR(CANNOT_DELETE_MANAGER, a_participant_id);
   		}
   		else
   		{
   			h_temp_sc_state = GetSecurityID(h_table_type, a_element_type, SC_RESPONSIBLE_GROUP, SC_PARAM(h_temp_security_id));
   //trace_printf("\n 0__8182 a_element_type: %d, h_temp_security_id: %s\n",a_element_type,h_temp_security_id);//trace_flush();
   
   			// Lock for creator
   			BEGIN_SQL
   				SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO HOST(a_temp_id), HOST(h_participant)
   				FROM RESOURCE_ASSIGNMENTS
   				WHERE ELEMENT_ID = HOST(a_element_id) AND RESOURCE_ID = HOST(h_created_by);
   			END_SQL;
   
   			if( SC_SUCCESS)
   			{
   
   //trace_printf("\n 1__8182 a_element_id %s",a_element_id);//trace_flush();
   				if ( isProjectType(a_element_type) )
   				{
   					if ( (h_participant  SC_AFFECTED) == SC_AFFECTED )
   						h_participant = SC_AFFECTED | SC_MANAGER;
   					else
   						h_participant = SC_MANAGER;
   
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS
   						SET PARTICIPANT = HOST(h_participant),
   						SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id);
   					END_SQL;
   				}
   				else if (  (a_element_type == SC_TASK) || (a_element_type == SC_MILESTONE))
   				{
   					if ( (h_participant  SC_TEAMMEMBER) == SC_TEAMMEMBER )
   						h_participant = SC_TEAMMEMBER | SC_TASK_RESPONSIBLE;
   					else
   						h_participant = SC_TASK_RESPONSIBLE;
   
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS
   						SET PARTICIPANT = HOST(h_participant),
   						SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id) ;
   					END_SQL;
   //trace_printf("W1__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id);
   				}
   				else if (( a_security_value  SC_RESOURCE_MANAGER) == SC_RESOURCE_MANAGER)
   				{
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS
   						SET PARTICIPANT = SC_RESOURCE_MANAGER,
   						SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id);
   					END_SQL;
   				}
   				else
   				{
   //                  h_responsible = a_participant  SC_CLEAR_TEAMMEMBER_VISITOR;
   //                    GetSecurityName(h_responsible,a_security_name);
   
   //trace_printf("\n 8213 a_security_name %s",a_security_name);//trace_flush();
   
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS
   						SET PARTICIPANT = HOST(a_participant),
   						SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id);
   					END_SQL;
   //trace_printf("W2__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id);
   				}
   
   				h_temp_sc_state = FC_AFTER_U_RES_ASSIGN(SC_PARAM(a_temp_id));
   				BEGIN_SQL
   					SELECT NICK_NAME INTO HOST(h_full_name) FROM RESOURCES WHERE RESOURCE_ID = :h_created_by;
   				END_SQL;
   				if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, h_created_by); }
   
   //trace_printf("\na_element_type: %d, h_created_by: %s, h_full_name: %s, h_table_type: %d:%d\n",a_element_type,h_created_by,h_full_name,h_table_type_Ind,h_table_type);
   				if ( strlen(h_full_name) > 0 )
   				{
   					h_temp_sc_state = FC_U_NAMES( SC_PARAM(a_element_id), SC_PARAM(h_full_name), SC_PARAM(a_participant_id), SC_PARAM(h_table_type) );
   				}
   			}
   			else
   			{
   //trace_printf("\n 2__8182 a_element_id %s",a_element_id);//trace_flush();
   				if ( isProjectType(a_element_type) )
   				{
   					InitializeHost(SC_PARAM(h_temp_participant), SC_MANAGER);
   					InitializeHost(SC_PARAM(h_temp_res_type), "0");
   					InitializeHost(SC_PARAM(h_temp_bitflag), 1);
   
   					h_temp_sc_state = FC_BEFORE_I_RES_ASSIGN(  SC_PARAM(h_project_id),       SC_PARAM(h_created_by),        SC_PARAM(a_element_type),
   															   SC_PARAM(h_temp_participant), SC_PARAM(h_temp_security_id),  SC_PARAM(h_temp_res_type),
   															   SC_PARAM(h_temp_bitflag));
   
   					BEGIN_SQL
   						INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   							RESOURCE_ID,PARTICIPANT,SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   						VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   							HOST(h_created_by), SC_MANAGER,HOST(h_temp_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type), SC_NOW);
   					END_SQL;
   				}
   				else if ( (a_element_type == SC_TASK) || (a_element_type == SC_MILESTONE) || (a_element_type == SC_CHANGE_RESPONSE_TASK) )
   				{
   					InitializeHost(SC_PARAM(h_temp_participant), SC_TASK_RESPONSIBLE);
   					InitializeHost(SC_PARAM(h_temp_res_type), "0");
   					InitializeHost(SC_PARAM(h_temp_bitflag), 1);
   
   					h_temp_sc_state = FC_BEFORE_I_RES_ASSIGN(  SC_PARAM(h_project_id),       SC_PARAM(h_created_by),       SC_PARAM(a_element_type),
   															   SC_PARAM(h_temp_participant), SC_PARAM(h_temp_security_id), SC_PARAM(h_temp_res_type),
   															   SC_PARAM(h_temp_bitflag));
   
   					if (h_transfed_cri == 0)
   					{
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(h_temp_participant), HOST(h_temp_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   					else
   					{
   						h_temp_sc_state = GetSecurityID(SC_TABLE_CRI, FC_MIRROR_TYPE(SC_PARAM(a_element_type)), SC_PARTICIPANT_GROUP, SC_PARAM(h_cri_security_id));
   						h_temp_participant = h_temp_participant | SC_PARTICIPANT;
   
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,CRI_SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(h_temp_participant),HOST(h_temp_security_id), HOST(h_cri_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   //trace_printf("W3__* SC_STATE: %d, h_temp_participant: %d,h_temp_security_id: %s, h_cri_security_id: %s\n", SC_STATE,h_temp_participant,h_temp_security_id,h_cri_security_id);
   				}
   				else if (a_element_type == SC_POOLS)
   				{
   					InitializeHost(SC_PARAM(h_temp_participant), SC_RESOURCE_MANAGER);
   					InitializeHost(SC_PARAM(h_temp_res_type), "0");
   					InitializeHost(SC_PARAM(h_temp_bitflag), 1);
   
   					h_temp_sc_state = FC_BEFORE_I_RES_ASSIGN(SC_PARAM(h_project_id),SC_PARAM(h_created_by),SC_PARAM(a_element_type),
   										   SC_PARAM(h_temp_participant), SC_PARAM(h_temp_security_id), SC_PARAM(h_temp_res_type),
   										   SC_PARAM(h_temp_bitflag));
   
   					BEGIN_SQL
   						INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   							RESOURCE_ID,PARTICIPANT,SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   						VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   							HOST(h_created_by), SC_RESOURCE_MANAGER, HOST(h_temp_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   					END_SQL;
   				}
   				else
   				{
   //trace_printf("\n 8281 a_participant %d",a_participant);//trace_flush();
   					h_temp_participant = a_participant  SC_CLEAR_PARTICIPANT;
   //                    GetSecurityName(h_temp_participant,h_temp_security_id);
   
   //trace_printf("\n 8281 h_temp_participant: %d, h_temp_security_id %s",h_temp_participant,h_temp_security_id);//trace_flush();
   
   //                  InitializeHost(SC_PARAM(h_temp_participant), SC_DLV_MANAGER);
   //                  InitializeHost(SC_PARAM(h_temp_security_id), "SCDELIVERABLEMANAGER");
   					InitializeHost(SC_PARAM(h_temp_res_type), "0");
   					InitializeHost(SC_PARAM(h_temp_bitflag), 1);
   
   					h_temp_sc_state = FC_BEFORE_I_RES_ASSIGN(SC_PARAM(h_project_id),SC_PARAM(h_created_by),SC_PARAM(a_element_type),
   										   SC_PARAM(h_temp_participant), SC_PARAM(h_temp_security_id), SC_PARAM(h_temp_res_type),
   										   SC_PARAM(h_temp_bitflag));
   
   					if (h_transfed_cri == 0)
   					{
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(a_participant), HOST(h_temp_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   					else
   					{
   						h_temp_sc_state=GetSecurityID(SC_TABLE_CRI, FC_MIRROR_TYPE(SC_PARAM(a_element_type)), SC_PARTICIPANT_GROUP, SC_PARAM(h_cri_security_id));
   						h_temp_participant = h_temp_participant | SC_PARTICIPANT;
   
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,CRI_SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(h_temp_participant),HOST(h_temp_security_id), HOST(h_cri_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   //trace_printf("W4__* SC_STATE: %d, h_temp_participant: %d,h_temp_security_id: %s, h_cri_security_id: %s\n", SC_STATE,h_temp_participant,h_temp_security_id,h_cri_security_id);
   				}
   
   				BEGIN_SQL
   					SELECT NICK_NAME INTO HOST(h_full_name) FROM RESOURCES WHERE RESOURCE_ID = :h_created_by;
   				END_SQL;
   				if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, h_created_by); }
   
   //trace_printf("a_element_type = %d, h_created_by = %s, h_full_name = %s\n",a_element_type,h_created_by,h_full_name);
   				if ( strlen(h_full_name) > 0 )
   				{
   					h_temp_sc_state = FC_U_NAMES( SC_PARAM(a_element_id), SC_PARAM(h_full_name), SC_PARAM(a_participant_id), SC_PARAM(h_table_type) );
   				}
   			}
   		}
   	}
   
   //trace_printf("\n 1__ a_security_value = %d",a_security_value);
   	//If re-assign manager then update existing manager
   	if( (a_security_value  SC_WBS_RESPONSIBLE) > 0)
   	{
   		BEGIN_SQL
   			SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO HOST(a_temp_id), HOST(h_temp_participant)
   			FROM RESOURCE_ASSIGNMENTS
   			WHERE ELEMENT_ID = HOST(a_element_id)
   				AND BIT_AND(PARTICIPANT, SC_WBS_RESPONSIBLE) > 0
   				AND RES_ASSIGNMENT_ID <> HOST(a_res_assignment_id);
   		END_SQL;
   
   //trace_printf("\n 8390 SC_STATE: %d, a_element_id %s, h_temp_participant: %d",SC_STATE,a_element_id,h_temp_participant);//trace_flush();
   		if( ( SC_SUCCESS)  ( IsNotNullParam(a_temp_id)))   // delete the old responsible
   		{
   			BEGIN_SQL
   				SELECT COUNT(*) INTO HOST(a_assignment_count)
   				   FROM TASK_ASSIGNMENTS
   					  WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id);
   			END_SQL;
   
   //trace_printf("\n 9325 ");//trace_flush();
   			if( a_assignment_count > 0)
   			{
   				h_temp_sc_state=GetSecurityID(h_table_type, a_element_type, SC_TEAMMEMBER_GROUP, SC_PARAM(h_temp_security_id));
   //trace_printf("\n 0__9325 a_temp_id: %s, a_element_type: %d, h_temp_security_id: %s\n",a_temp_id,a_element_type,h_temp_security_id);//trace_flush();
   
   				// if there is assignment, then update the old responsible
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET
   						PARTICIPANT = CASE WHEN HOST(a_element_type) IN (SC_PROJECT_GROUP_TYPES) THEN SC_AFFECTED ELSE SC_TEAMMEMBER END,
   						SECURITY_ID = HOST(h_temp_security_id)
   					WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   				END_SQL;
   			}
   			else if ( a_assignment_count == 0)
   			{
   
   //trace_printf("\n 9341 ");//trace_flush();
   				h_temp_sc_state = GetSecurityID(h_table_type, a_element_type, SC_PARTICIPANT_GROUP, SC_PARAM(h_temp_security_id));
   				if ( isProjectType(a_element_type) )
   				{
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS SET
   							PARTICIPANT = SC_CO_MANAGER,
   							SECURITY_ID = 'SCCOMANAGER'
   						 WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   					END_SQL;
   				}
   				else if ( (a_security_value  SC_RESOURCE_MANAGER) == SC_RESOURCE_MANAGER )
   				{
   //trace_printf("\n 1__9325 a_element_type: %d, h_temp_security_id: %s\n",a_element_type,h_temp_security_id);//trace_flush();
   
   					// if no assignment, then delete the old responsible
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS SET
   							PARTICIPANT = SC_PARTICIPANT,
   							SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   					END_SQL;
   //trace_printf("\n SCPOOLPARTICIPANT SC_STATE: %d",SC_STATE);//trace_flush();
   				}
   				else
   				{
   //trace_printf("\n 2__9325 a_element_type: %d, h_temp_participant: %d, h_temp_security_id: %s\n",a_element_type,h_temp_participant,h_temp_security_id);//trace_flush();
   					h_temp_participant = SC_PARTICIPANT | (h_temp_participant  SC_CLEAR_WBS_RESPONSIBLE);
   
   					// if no assignment, then delete the old responsible
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS SET
   							PARTICIPANT = HOST(h_temp_participant),
   							SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   					END_SQL;
   //trace_printf("\n SCPARTICIPANTS SC_STATE: %d",SC_STATE);//trace_flush();
   //trace_printf("W6.2__* SC_STATE: %d, h_temp_participant: %d, h_temp_security_id: %s\n", SC_STATE,h_temp_participant,h_temp_security_id);
   				}
   			}
   
   			h_temp_sc_state = FC_AFTER_U_RES_ASSIGN(SC_PARAM(a_temp_id));
   		}
   	}
   
   //trace_printf("\n 2__ a_participant = %d",a_participant);
   	//If current type is any type of manager
   	if( (a_participant  SC_WBS_RESPONSIBLE) > 0)
   	{
   		a_security_value = a_security_value | (a_participant  SC_CLEAR_WBS_RESPONSIBLE);
   //trace_printf("\n 8440 a_res_assignment_id %s",a_res_assignment_id);//trace_flush();
   
   		BEGIN_SQL
   			UPDATE RESOURCE_ASSIGNMENTS SET
   				PARTICIPANT = HOST(a_security_value),
   				SECURITY_ID = HOST(a_security_id),
   				REC_STATUS = 'U',
   				REC_USER = HOST_DB(res_id,ind_security),
   				REC_DATETIME = SC_NOW
   			WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   		END_SQL;
   //trace_printf("W7__* SC_STATE: %d, a_security_value: %d,a_security_id: %s\n", SC_STATE,a_security_value,a_security_id);
   	}
   	else
   	{
   //trace_printf("\n 8450 a_participant %d",a_participant);//trace_flush();
   //trace_printf("\n 8450 a_security_value %d",a_security_value);//trace_flush();
   		if( (a_security_value  SC_WBS_RESPONSIBLE) > 0)
   		{
   			if ( (a_participant  SC_AFFECTED) > 0 )
   			{
   				a_security_value = SC_AFFECTED | a_security_value;
   			}
   			else if ( (a_participant  SC_TEAMMEMBER) > 0 )
   			{
   				a_security_value = (SC_TEAMMEMBER | a_security_value) | (a_participant  SC_CLEAR_WBS_RESPONSIBLE);
   			}
   //          else
   //          {
   //              a_security_value = a_security_value | (a_participant  SC_CLEAR_WBS_RESPONSIBLE);
   //          }
   
   //trace_printf("\n 8462 a_res_assignment_id %s, a_security_id: %s, a_security_value: %d\n",a_res_assignment_id,a_security_id,a_security_value);//trace_flush();
   
   			BEGIN_SQL
   				UPDATE RESOURCE_ASSIGNMENTS SET
   					PARTICIPANT = :a_security_value,
   					SECURITY_ID = :a_security_id,
   					REC_STATUS = 'U',
   					REC_USER = HOST_DB(res_id,ind_security),
   					REC_DATETIME = SC_NOW
   				WHERE RES_ASSIGNMENT_ID = :a_res_assignment_id;
   			END_SQL;
   //trace_printf("a_element_type = %d, a_element_id = %s, a_participant_name = %s\n",a_element_type,a_element_id,a_participant_name);
   //trace_printf("W8__* SC_STATE: %d, a_security_value: %d,a_security_id: %s\n", SC_STATE,a_security_value,a_security_id);
   
   			h_temp_sc_state = FC_U_NAMES( SC_PARAM(a_element_id), SC_PARAM(a_participant_name), SC_PARAM(a_participant_id), SC_PARAM(h_table_type) );
   		}
   		else if( (a_participant  SC_TEAMMEMBER) > 0)
   		{
   			a_security_value = a_security_value | (a_participant  SC_CLEAR_WBS_RESPONSIBLE);
   
   			BEGIN_SQL
   				UPDATE RESOURCE_ASSIGNMENTS SET
   					PARTICIPANT = HOST(a_security_value),
   					SECURITY_ID = HOST(a_security_id),
   					REC_STATUS = 'U',
   					REC_USER = HOST_DB(res_id,ind_security),
   					REC_DATETIME = SC_NOW
   				WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   			END_SQL;
   			h_update = 0;
   //trace_printf("W9__* SC_STATE: %d, a_security_value: %d,a_security_id: %s\n", SC_STATE,a_security_value,a_security_id);
   		}
   		else
   		{
   //          a_security_value = a_security_value | (a_participant  SC_CLEAR_WBS_RESPONSIBLE);
   
   //trace_printf("\n 8754 a_participant %d",a_participant);//trace_flush();
   //trace_printf("\n 8754 a_security_value %d",a_security_value);//trace_flush();
   //trace_printf("\n 8754 a_element_id: %s, a_element_type: %d",a_element_id,a_element_type);//trace_flush();
   
   			BEGIN_SQL
   				UPDATE RESOURCE_ASSIGNMENTS SET
   					PARTICIPANT = HOST(a_security_value),
   					SECURITY_ID = HOST(a_security_id),
   					REC_STATUS = 'U',
   					REC_USER = HOST_DB(res_id,ind_security),
   					REC_DATETIME = SC_NOW
   				WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   			END_SQL;
   //trace_printf("W10__* SC_STATE: %d, a_security_value: %d,a_security_id: %s\n", SC_STATE,a_security_value,a_security_id);
   		}
   	}
   
   	SC_STATE = FC_AFTER_U_RES_ASSIGN(SC_PARAM(a_res_assignment_id));
   */
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_PARTICIPANT', 2003+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_PARTICIPANT(2004) : cplan.sql(17912) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, a_temp_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 2010+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(2011) : cplan.sql(17919) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_PARTICIPANT', 2022+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_PARTICIPANT(2023) : cplan.sql(17931) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_PARTICIPANT;
ALTER FUNCTION SP_U_PARTICIPANT COMPILE DEBUG;

/******************************************************************************************************************
update resource assignments SECURITY

assign_id:      RES_ASSIGNMENT_ID or TSK_ASSIGNMENT_ID
security_id:    SECURITY_ID in TMT_GLOBAL_SECURITY
*******************************************************************************************************************/

Prompt == FUNCTION SP_U_CRI_SECURITY ==
CREATE OR REPLACE FUNCTION SP_U_CRI_SECURITY
(
     assign_id VARCHAR2
   , security_id001 VARCHAR2
   , p_planned_start TIMESTAMP
   , p_planned_finish TIMESTAMP
   , p_delay PLS_INTEGER
   , p_units PLS_INTEGER
   , p_work_percent PLS_INTEGER
   , p_actual PLS_INTEGER
   , res_id001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   a_element_id VARCHAR2(32);
   a_participant_id VARCHAR2(255);
   a_participant_name VARCHAR2(255);
   a_security_name VARCHAR2(32);
   h_full_name VARCHAR2(255);
   a_res_assignment_id VARCHAR2(32);
   a_security_id VARCHAR2(32);
   a_temp_id VARCHAR2(32);
   h_created_by VARCHAR2(32);
   h_project_id VARCHAR2(32);
   a_assignment_count PLS_INTEGER := 0;
   a_element_type PLS_INTEGER := 0;
   h_responsible PLS_INTEGER := 0;
   a_participant PLS_INTEGER := 0;
   a_resource_type PLS_INTEGER := 0;
   a_security_value PLS_INTEGER := 0;
   h_participant PLS_INTEGER := 0;
   h_update PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   h_assign_string VARCHAR2(8192);
   h_temp_res_type VARCHAR2(1);
   h_temp_participant PLS_INTEGER := 0;
   h_temp_security_id VARCHAR2(32);
   h_wbs_security_id VARCHAR2(32);
   h_temp_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_transfed_cri PLS_INTEGER := 0;
   h_cri_table PLS_INTEGER := 0;
   h_cri_element_type PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_CRI_SECURITY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_CRI_SECURITY'), '-- ' || SYSTIMESTAMP, ':r := SP_U_CRI_SECURITY(' || 
       '''' || assign_id || ''''
          || ', ' || '''' || security_id001 || ''''
          || ', ' || COALESCE(NULLIF('''' || p_planned_start || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || p_planned_finish || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_delay), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_units), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_work_percent), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_actual), 'NULL')
          || ', ' || '''' || res_id001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_CRI_SECURITY', 60+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_CRI_SECURITY(61) : cplan.sql(18013) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_created_by := '';
   
   --Validate res_assignment_id
   a_temp_id := assign_id;
   a_security_id := security_id001;
   h_rec_user := res_id001;
   
   h_table_type := PMO_CNST.SC_TABLE_CRI;
   PMO_DB2M.SC_STATE := FC_U_ASSIGNED_SECURITY(h_table_type, a_temp_id, a_security_id, h_rec_user);
   
   /*  BEGIN_SQL
   		SELECT ra.RES_ASSIGNMENT_ID, ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE, CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,
   		CASE WHEN ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,
   		CASE WHEN ra.RESOURCE_TYPE = '0' THEN ra.RESOURCE_ID ELSE ra.PROFILE_ID END, ra.PROJECT_ID, ra.TABLE_TYPE
   		INTO HOST(a_res_assignment_id), HOST(a_participant), HOST(a_element_id), HOST(a_element_type), HOST(a_resource_type),
   		HOST(h_transfed_cri), HOST(a_participant_id), HOST(h_project_id), HOST(h_table_type)
   		FROM RESOURCE_ASSIGNMENTS ra WHERE ra.RES_ASSIGNMENT_ID = HOST(a_temp_id);
   	END_SQL;
   //trace_printf("1_ SC_STATE: %d, a_temp_id: %s, a_res_assignment_id: %s\n",SC_STATE,a_temp_id,a_res_assignment_id);
   
   	if( SC_NODATA)
   	{
   		//Validate tsk_assignment_id
   		BEGIN_SQL
   			SELECT TA.RES_ASSIGNMENT_ID INTO HOST(a_res_assignment_id) FROM TASK_ASSIGNMENTS TA
   			WHERE TA.TSK_ASSIGNMENT_ID = HOST(a_temp_id) AND EXISTS( SELECT * FROM RESOURCE_ASSIGNMENTS WHERE RES_ASSIGNMENT_ID = TA.RES_ASSIGNMENT_ID);
   		END_SQL;
   //trace_printf("2_ SC_STATE: %d, a_temp_id: %s, a_res_assignment_id: %s\n",SC_STATE,a_temp_id,a_res_assignment_id);
   
   		if( SC_NODATA){ RETURN_ERROR(RECORD_NOT_FOUND, a_temp_id); }
   
   		BEGIN_SQL
   			SELECT ra.PARTICIPANT, ra.ELEMENT_ID, ra.ELEMENT_TYPE, CASE WHEN ra.RESOURCE_TYPE = '0' THEN 0 ELSE 1 END,
   			CASE WHEN ra.CRI_SECURITY_ID IS NuLL THEN 0 ELSE 1 END,
   			CASE WHEN ra.RESOURCE_TYPE = '0' THEN ra.RESOURCE_ID ELSE ra.PROFILE_ID END, ra.PROJECT_ID, ra.TABLE_TYPE
   			INTO HOST(a_participant), HOST(a_element_id), HOST(a_element_type), HOST(a_resource_type),
   			HOST(h_transfed_cri), HOST(a_participant_id), HOST(h_project_id), HOST(h_table_type)
   			FROM RESOURCE_ASSIGNMENTS ra WHERE ra.RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   		END_SQL;
   //trace_printf("3_ SC_STATE: %d, a_temp_id: %s\n",SC_STATE,a_temp_id);
   
   		if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_res_assignment_id); }
   	}
   
   	if ( (IsNullParam(h_table_type)) || (h_table_type == 0) )
   	{
   		InitializeHost(SC_PARAM(h_table_type), GetTable(a_element_type));
   	}
   
   	//Retrive participant name for updating AssignNames field
   	if( a_resource_type == 0)
   	{
   		BEGIN_SQL
   			SELECT NICK_NAME INTO HOST(a_participant_name) FROM RESOURCES WHERE RESOURCE_ID = :a_participant_id;
   		END_SQL;
   //trace_printf("4_ SC_STATE: %d, a_temp_id: %s\n",SC_STATE,a_temp_id);
   		if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_participant_id); }
   	}
   	else if( a_resource_type == 1)
   	{
   		BEGIN_SQL
   			SELECT PROFILE_NAME INTO HOST(a_participant_name) FROM PROFILES WHERE PROFILE_ID = :a_participant_id;
   		END_SQL;
   		if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_participant_id); }
   	}
   	if( ( a_resource_type > 1) || (IsNullParam(a_participant_name))) { InitializeHost(SC_PARAM(a_participant_name),""); }
   //trace_printf("* h_transfed_cri: %d, h_table_type: %d, a_participant_name: %s\n",h_transfed_cri,h_table_type,a_participant_name);
   
   	BEGIN_SQL
   		SELECT CREATED_BY INTO HOST(h_created_by) FROM TMT_CRI WHERE ELEMENT_ID = HOST(a_element_id);
   	END_SQL;
   
   //trace_printf("5_ SC_STATE: %d, h_created_by: %s\n",SC_STATE,h_created_by);
   	if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_element_id); }
   
   //trace_printf("a_element_id = %s, a_participant_id = %s, h_created_by = %s\n", a_element_id, a_participant_id,h_created_by);
   	//Validate security_id
   	InitializeHost(SC_PARAM(a_security_id), security_id, ind_security->security_id);
   
   	BEGIN_SQL
   		SELECT SECURITY_GROUP INTO HOST(a_security_value)
   		FROM TMT_GLOBAL_SECURITY WHERE ELEMENT_ID = HOST(a_security_id);
   	END_SQL;
   
   	if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, a_security_id); }
   //trace_printf("a_participant = %d, a_security_value = %d, a_security_id: %s\n", a_participant,a_security_value,a_security_id);
   
   	if ( (a_participant  SC_TEAMMEMBER) == SC_TEAMMEMBER )
   		a_security_value = a_security_value | SC_TEAMMEMBER;
   
   	//Do not remove the only responsible
   	if( ((a_participant  SC_CRI_RESPONSIBLE) > 0)    ((a_security_value  SC_CRI_RESPONSIBLE) == 0) )
   	{
   
   //trace_printf("\n 8163 a_participant %d",a_participant);//trace_flush();
   //trace_printf("\n 8163 a_security_value %d",a_security_value);//trace_flush();
   		if ( strcmp(h_created_by,a_participant_id) == 0 )
   		{
   			RETURN_ERROR(CANNOT_DELETE_MANAGER, a_participant_id);
   		}
   		else
   		{
   			if (h_transfed_cri == 0)
   			{
   				h_temp_sc_state = GetSecurityID(h_table_type, a_element_type, SC_RESPONSIBLE_GROUP, SC_PARAM(h_temp_security_id));
   			}
   			else
   			{
   				h_cri_table = SC_TABLE_CRI;
   				h_cri_element_type = FC_MIRROR_TYPE(SC_PARAM(a_element_type));
   
   				h_temp_sc_state = GetSecurityID(h_cri_table, h_cri_element_type, SC_RESPONSIBLE_GROUP, SC_PARAM(h_temp_security_id));
   			}
   //trace_printf("\n 0__8182 a_element_type: %d, h_temp_security_id: %s\n",a_element_type,h_temp_security_id);//trace_flush();
   
   			// Lock for creator
   			BEGIN_SQL
   				SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO HOST(a_temp_id), HOST(h_participant)
   				FROM RESOURCE_ASSIGNMENTS
   				WHERE ELEMENT_ID = HOST(a_element_id) AND RESOURCE_ID = HOST(h_created_by);
   			END_SQL;
   
   			if( SC_SUCCESS) // creator is already assigned
   			{
   //trace_printf("\n 1__8182 a_element_id %s",a_element_id);//trace_flush();
   				if (  (a_element_type == SC_TASK) || (a_element_type == SC_MILESTONE) || (a_element_type == SC_CHANGE_RESPONSE_TASK))
   				{
   					if ( (h_participant  SC_TEAMMEMBER) == SC_TEAMMEMBER )
   						h_participant = SC_TEAMMEMBER | SC_TASK_RESPONSIBLE;
   					else
   						h_participant = SC_TASK_RESPONSIBLE;
   
   					if (h_transfed_cri == 0)
   					{
   						BEGIN_SQL
   							UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(h_participant), SECURITY_ID = HOST(h_temp_security_id)
   							WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id) ;
   						END_SQL;
   					}
   					else
   					{
   						BEGIN_SQL
   							UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(h_participant), CRI_SECURITY_ID = HOST(h_temp_security_id)
   							WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id) ;
   						END_SQL;
   					}
   //trace_printf("A1__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id);
   				}
   				else
   				{
   //                  h_responsible = a_participant  SC_CLEAR_TEAMMEMBER_VISITOR;
   //                    GetSecurityName(h_responsible,a_security_name);
   
   //trace_printf("\n 8213 a_participant %d, h_temp_security_id: %s\n",a_participant,h_temp_security_id);//trace_flush();
   					if (h_transfed_cri == 0)
   					{
   						BEGIN_SQL
   							UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(a_participant), SECURITY_ID = HOST(h_temp_security_id)
   							WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id);
   						END_SQL;
   					}
   					else
   					{
   						BEGIN_SQL
   							UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(a_participant), CRI_SECURITY_ID = HOST(h_temp_security_id)
   							WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id);
   						END_SQL;
   					}
   //trace_printf("A2__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id);
   				}
   
   				h_temp_sc_state = FC_AFTER_U_RES_ASSIGN(SC_PARAM(a_temp_id));
   				BEGIN_SQL
   					SELECT NICK_NAME INTO HOST(h_full_name) FROM RESOURCES WHERE RESOURCE_ID = :h_created_by;
   				END_SQL;
   				if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, h_created_by); }
   
   //trace_printf("\na_element_type: %d, h_created_by: %s, h_full_name: %s, h_table_type: %d:%d\n",a_element_type,h_created_by,h_full_name,h_table_type_Ind,h_table_type);
   				if ( strlen(h_full_name) > 0 )
   				{
   					h_temp_sc_state = FC_U_NAMES( SC_PARAM(a_element_id), SC_PARAM(h_full_name), SC_PARAM(a_participant_id), SC_PARAM(h_table_type) );
   				}
   			}
   			else
   			{
   //trace_printf("\n 2__8182 a_element_id %s",a_element_id);//trace_flush();
   				if ( (a_element_type == SC_TASK) || (a_element_type == SC_MILESTONE) || (a_element_type == SC_CHANGE_RESPONSE_TASK) )
   				{
   					InitializeHost(SC_PARAM(h_temp_participant), SC_TASK_RESPONSIBLE);
   //                  InitializeHost(SC_PARAM(h_temp_security_id), "SCTASKRESPONSIBLE");
   					InitializeHost(SC_PARAM(h_temp_res_type), "0");
   					InitializeHost(SC_PARAM(h_temp_bitflag), 1);
   
   					h_temp_sc_state = FC_BEFORE_I_RES_ASSIGN(  SC_PARAM(h_project_id),       SC_PARAM(h_created_by),       SC_PARAM(a_element_type),
   															   SC_PARAM(h_temp_participant), SC_PARAM(h_temp_security_id), SC_PARAM(h_temp_res_type),
   															   SC_PARAM(h_temp_bitflag));
   					// INSERT creator --> task responsible
   					if (h_transfed_cri == 0)
   					{
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(h_temp_participant),HOST(h_temp_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   					else
   					{
   						h_cri_table = SC_TABLE_CRI;
   						h_cri_element_type = FC_MIRROR_TYPE(SC_PARAM(a_element_type));
   
   						h_temp_sc_state = GetSecurityID(h_cri_table, h_cri_element_type, SC_PARTICIPANT_GROUP, SC_PARAM(h_wbs_security_id));
   						h_temp_participant = h_temp_participant | SC_PARTICIPANT;
   
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,CRI_SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(h_temp_participant),HOST(h_temp_security_id), HOST(h_wbs_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   //trace_printf("A3__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\nh_temp_participant: %d, h_wbs_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id,h_temp_participant,h_wbs_security_id);
   				}
   				else
   				{
   //trace_printf("\n 8281 a_participant %d",a_participant);//trace_flush();
   					h_temp_participant = a_participant  SC_CLEAR_PARTICIPANT;
   					InitializeHost(SC_PARAM(h_temp_res_type), "0");
   					InitializeHost(SC_PARAM(h_temp_bitflag), 1);
   
   					h_temp_sc_state = FC_BEFORE_I_RES_ASSIGN(SC_PARAM(h_project_id),       SC_PARAM(h_created_by),       SC_PARAM(a_element_type),
   										   SC_PARAM(h_temp_participant), SC_PARAM(h_temp_security_id), SC_PARAM(h_temp_res_type),
   										   SC_PARAM(h_temp_bitflag));
   
   					if (h_transfed_cri == 0)
   					{
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(a_participant), HOST(h_temp_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   					else
   					{
   						h_temp_sc_state = GetSecurityID(SC_TABLE_WBS, FC_MIRROR_TYPE(SC_PARAM(a_element_type)), SC_PARTICIPANT_GROUP, SC_PARAM(h_wbs_security_id));
   						h_temp_participant = h_temp_participant | SC_PARTICIPANT;
   
   						BEGIN_SQL
   							INSERT INTO RESOURCE_ASSIGNMENTS(RES_ASSIGNMENT_ID,PROJECT_ID,ELEMENT_ID,ELEMENT_TYPE,RESOURCE_TYPE,
   								RESOURCE_ID,PARTICIPANT,SECURITY_ID,CRI_SECURITY_ID,REC_USER,TABLE_TYPE,REC_DATETIME)
   							VALUES (GET_UNIQUE_ID(),HOST(h_project_id),HOST(a_element_id),HOST(a_element_type),'0',
   								HOST(h_created_by), HOST(a_participant), HOST(h_temp_security_id), HOST(h_wbs_security_id), HOST_DB(res_id,ind_security), HOST(h_table_type),SC_NOW);
   						END_SQL;
   					}
   //trace_printf("A4__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\nh_temp_participant: %d, h_wbs_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id,h_temp_participant,h_wbs_security_id);
   				}
   
   				BEGIN_SQL
   					SELECT NICK_NAME INTO HOST(h_full_name) FROM RESOURCES WHERE RESOURCE_ID = :h_created_by;
   				END_SQL;
   				if( !SC_SUCCESS){ RETURN_ERROR(RECORD_NOT_FOUND, h_created_by); }
   
   //trace_printf("a_element_type = %d, h_created_by = %s, h_full_name = %s\n",a_element_type,h_created_by,h_full_name);
   				if ( strlen(h_full_name) > 0 )
   				{
   					h_temp_sc_state = FC_U_NAMES( SC_PARAM(a_element_id), SC_PARAM(h_full_name), SC_PARAM(a_participant_id), SC_PARAM(h_table_type) );
   				}
   			}
   		}
   	}
   
   	//If re-assign manager then update existing manager
   	if( (a_security_value  SC_CRI_RESPONSIBLE) > 0)
   	{
   		BEGIN_SQL
   			SELECT RES_ASSIGNMENT_ID, PARTICIPANT INTO HOST(a_temp_id), HOST(h_temp_participant)
   			FROM RESOURCE_ASSIGNMENTS
   			WHERE ELEMENT_ID = HOST(a_element_id)
   				AND BIT_AND( PARTICIPANT, SC_CRI_RESPONSIBLE) > 0
   				AND RES_ASSIGNMENT_ID <> HOST(a_res_assignment_id);
   		END_SQL;
   
   //trace_printf("\n 8390 SC_STATE: %d, a_element_id %s",SC_STATE,a_element_id);//trace_flush();
   		if( ( SC_SUCCESS)  ( IsNotNullParam(a_temp_id)))   // delete the old responsible
   		{
   			BEGIN_SQL
   				SELECT COUNT(*) INTO HOST(a_assignment_count)
   				   FROM TASK_ASSIGNMENTS
   					  WHERE RES_ASSIGNMENT_ID = HOST(a_temp_id);
   			END_SQL;
   
   //trace_printf("\n 9325 ");//trace_flush();
   			if( a_assignment_count > 0)
   			{
   				h_temp_sc_state = GetSecurityID(h_table_type, a_element_type, SC_TEAMMEMBER_GROUP, SC_PARAM(h_temp_security_id));
   //trace_printf("\n 0__9325 a_temp_id: %s, a_element_type: %d, h_temp_security_id: %s\n",a_temp_id,a_element_type,h_temp_security_id);//trace_flush();
   
   				// if there is assignment, then update the old responsible
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET
   						PARTICIPANT = CASE WHEN HOST(a_element_type) IN (SC_PROJECT_GROUP_TYPES) THEN SC_AFFECTED ELSE SC_TEAMMEMBER END,
   						SECURITY_ID = HOST(h_temp_security_id)
   					WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   				END_SQL;
   			}
   			else if ( a_assignment_count == 0)
   			{
   //trace_printf("\n 2__9325 a_security_value: %d, a_participant: %d, h_temp_participant: %d\n",a_security_value,a_participant,h_temp_participant);//trace_flush();
   
   				// if no assignment, then delete the old responsible
   				if (h_transfed_cri == 0)
   				{
   					h_temp_sc_state = GetSecurityID(h_table_type, a_element_type, SC_PARTICIPANT_GROUP, SC_PARAM(h_temp_security_id));
   					h_temp_participant = SC_PARTICIPANT;
   
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS SET
   							PARTICIPANT = HOST(h_temp_participant), SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   					END_SQL;
   				}
   				else
   				{
   					h_cri_table = SC_TABLE_CRI;
   					h_cri_element_type = FC_MIRROR_TYPE(SC_PARAM(a_element_type));
   
   					// if no assignment, then delete the old responsible
   					h_temp_sc_state = GetSecurityID(h_cri_table, h_cri_element_type, SC_PARTICIPANT_GROUP, SC_PARAM(h_temp_security_id));
   
   					//h_temp_sc_state = GetSecurityID(h_table_type, a_element_type, SC_PARTICIPANT_GROUP, SC_PARAM(h_temp_security_id));
   					if (isCRIwbsElement(a_element_type))
   						h_temp_participant = SC_PARTICIPANT;
   					else
   						h_temp_participant = SC_PARTICIPANT | (h_temp_participant  SC_CLEAR_CRI_RESPONSIBLE);
   
   					BEGIN_SQL
   						UPDATE RESOURCE_ASSIGNMENTS SET
   							PARTICIPANT = HOST(h_temp_participant), CRI_SECURITY_ID = HOST(h_temp_security_id)
   						WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   					END_SQL;
   				}
   //trace_printf("A5__* SC_STATE: %d, h_temp_participant: %d, h_temp_security_id: %s\n", SC_STATE,h_temp_participant,h_temp_security_id);//trace_flush();
   			}
   
   			h_temp_sc_state = FC_AFTER_U_RES_ASSIGN(SC_PARAM(a_temp_id));
   		}
   		else
   		{
   			if (h_transfed_cri == 0)
   			{
   				h_temp_sc_state = GetSecurityID(h_table_type, a_element_type, SC_PARTICIPANT_GROUP, SC_PARAM(h_temp_security_id));
   			}
   			else
   			{
   				h_cri_table = SC_TABLE_CRI;
   				h_cri_element_type = FC_MIRROR_TYPE(SC_PARAM(a_element_type));
   
   				h_temp_sc_state = GetSecurityID(h_cri_table, h_cri_element_type, SC_PARTICIPANT_GROUP, SC_PARAM(h_temp_security_id));
   			}
   			if ( (h_transfed_cri == 0) || (isCRIwbsElement(a_element_type)) )
   			{
   				InitializeHost(SC_PARAM(h_temp_participant), SC_PARTICIPANT);
   			}
   			else
   			{
   				h_temp_participant = SC_PARTICIPANT | (h_temp_participant  SC_CLEAR_CRI_RESPONSIBLE);
   			}
   //trace_printf("\n 0__932d a_temp_id: %s, a_element_type: %d, h_temp_security_id: %s\n",a_temp_id,a_element_type,h_temp_security_id);//trace_flush();
   
   			if (h_transfed_cri == 0)
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(h_temp_participant), SECURITY_ID = HOST(h_temp_security_id)
   					WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   				END_SQL;
   			}
   			else
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(h_temp_participant), CRI_SECURITY_ID = HOST(h_temp_security_id)
   					WHERE RES_ASSIGNMENT_ID = :a_temp_id;
   				END_SQL;
   			}
   		}
   	}
   
   //trace_printf("\n 2__ a_participant = %d",a_participant);
   	//If current type is any type of manager
   	if( (a_participant  SC_CRI_RESPONSIBLE) > 0)
   	{
   
   //trace_printf("\n 8440 a_res_assignment_id %s",a_res_assignment_id);//trace_flush();
   		if (h_transfed_cri == 0)
   		{
   			BEGIN_SQL
   				UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(a_security_value), SECURITY_ID = HOST(a_security_id),
   					REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   				WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   			END_SQL;
   		}
   		else
   		{
   			BEGIN_SQL
   				UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(a_security_value), CRI_SECURITY_ID = HOST(a_security_id),
   					REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   				WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   			END_SQL;
   		}
   
   //trace_printf("A6__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id);//trace_flush();
   	}
   	else
   	{
   //trace_printf("\n 8450 a_participant %d",a_participant);//trace_flush();
   //trace_printf("\n 8450 a_security_value %d",a_security_value);//trace_flush();
   		if ( !(isCRIwbsElement(a_element_type)) )
   			a_security_value = a_security_value | (a_participant  SC_CLEAR_CRI_RESPONSIBLE);
   
   		if( (a_security_value  SC_CRI_RESPONSIBLE) > 0)
   		{
   			if ( (a_participant  SC_TEAMMEMBER) > 0 )
   			{
   				a_security_value = SC_TEAMMEMBER | a_security_value;
   			}
   
   //trace_printf("\n 8462 a_res_assignment_id %s, a_security_id: %s, a_security_value: %d\n",a_res_assignment_id,a_security_id,a_security_value);//trace_flush();
   			if (h_transfed_cri == 0)
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = :a_security_value, SECURITY_ID = :a_security_id,
   						REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   					WHERE RES_ASSIGNMENT_ID = :a_res_assignment_id;
   				END_SQL;
   			}
   			else
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = :a_security_value, CRI_SECURITY_ID = :a_security_id,
   						REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   					WHERE RES_ASSIGNMENT_ID = :a_res_assignment_id;
   				END_SQL;
   			}
   //trace_printf("A7__* SC_STATE: %d, a_security_value: %d,a_security_id: %s\n", SC_STATE,a_security_value,a_security_id);//trace_flush();
   //trace_printf("a_element_type = %d, a_element_id = %s\n",a_element_type,a_element_id);
   
   //trace_printf("h_table_type = %d, a_participant_id = %s, a_participant_name = %s\n",h_table_type,a_participant_id,a_participant_name);
   			if ( strlen(a_participant_name) > 0 )
   			{
   				h_temp_sc_state = FC_U_NAMES( SC_PARAM(a_element_id), SC_PARAM(a_participant_name), SC_PARAM(a_participant_id), SC_PARAM(h_table_type) );
   			}
   		}
   		else if( (a_participant  SC_TEAMMEMBER) > 0)
   		{
   			if (h_transfed_cri == 0)
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = BIT_OR( SC_TEAMMEMBER, HOST(a_security_value)), SECURITY_ID = HOST(a_security_id),
   						REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   					WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   				END_SQL;
   			}
   			else
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = BIT_OR( SC_TEAMMEMBER, HOST(a_security_value)), CRI_SECURITY_ID = HOST(a_security_id),
   						REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   					WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   				END_SQL;
   			}
   //trace_printf("A8__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id);//trace_flush();
   		}
   		else
   		{
   //trace_printf("\n 8754 a_participant %d",a_participant);//trace_flush();
   //trace_printf("\n 8754 a_security_value %d",a_security_value);//trace_flush();
   
   			if (h_transfed_cri == 0)
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(a_security_value), SECURITY_ID = HOST(a_security_id),
   						REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   					WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   				END_SQL;
   			}
   			else
   			{
   				BEGIN_SQL
   					UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = HOST(a_security_value), CRI_SECURITY_ID = HOST(a_security_id),
   						REC_STATUS = 'U', REC_USER = HOST_DB(res_id,ind_security), REC_DATETIME = SC_NOW
   					WHERE RES_ASSIGNMENT_ID = HOST(a_res_assignment_id);
   				END_SQL;
   			}
   
   //trace_printf("A9__* SC_STATE: %d, a_participant: %d,h_temp_security_id: %s\n", SC_STATE,a_participant,h_temp_security_id);//trace_flush();
   		}
   	}
   
   	SC_STATE = FC_AFTER_U_RES_ASSIGN(SC_PARAM(a_res_assignment_id));
   */
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_CRI_SECURITY', 566+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_CRI_SECURITY(567) : cplan.sql(18519) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || a_temp_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, a_temp_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CRI_SECURITY', 573+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CRI_SECURITY(574) : cplan.sql(18526) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_CRI_SECURITY', 585+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_CRI_SECURITY(586) : cplan.sql(18538) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_CRI_SECURITY;
ALTER FUNCTION SP_U_CRI_SECURITY COMPILE DEBUG;

/****************************************************************************************
 id = TSK_ASSIGN_ID
 bitflag:
		1 = Copy Schedule to Baseline (includes budgets)
		2 = Copy Planned to Schedule
		3 = Copy Proposed to Schedule
		4 = Copy Forecast to Schedule (includes budgets)
		5 = Copy Forecast  Scheduled to Planned

		6 = Move Planned to Schedule
		7 = Append Planned to Schedule

 bussiness rules:
	1. Do not copy date for completed tasks (MILESTONE_PERCENT = 100)
	2. If the task has ACTUAL do not change ETC, WORK_TODATE, SD, FD
	3. When copy Forecast Date to ..., ETC is Forecast Effort, and clear Forcast dates after copy
	4. If both Forcast dates exists ignor forcast start, consider only forcast finish and ETC
	5. Calculated Forecast Start should not be in the past (is >= NOW)
	6. When copy Forcast to and assignments exists forcast rollup wins, and clear Forcast dates after copy
	7. Actual Finish date does not rollup from Assignments to Task
	8. When copy Planned to Schedule clear Planned Dates, Duration  Effort
****************************************************************************************/

Prompt == FUNCTION SP_COMMIT_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_COMMIT_ASSIGN
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_bitID RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_bitID1 RAW(36) NOT NULL := PMO_CNST.SC_ZERO_BIT;
   h_project_id VARCHAR2(32);
   h_dayofyear DATE;
   h_res_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_sched_work PLS_INTEGER := 0;
   h_billable PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_day_min PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_min_per_day PLS_INTEGER := 0;
   h_change_flag PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_from_prj PLS_INTEGER := 0;
   h_tsk_prj PLS_INTEGER := 0;
   h_work PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   stmt VARCHAR2(7999);
-- Declare Cursors
      CURSOR C270 IS
         SELECT TA.TSK_ASSIGNMENT_ID, TA.RESOURCE_ID
         FROM TASK_ASSIGNMENTS TA JOIN TMT_WBS W
         ON W.PROJECT_ID = TA.PROJECT_ID AND W.ELEMENT_ID = TA.TASK_ID
         WHERE W.PROJECT_ID = h_project_id AND TA.BIT_FLAG = '0'
         AND TA.RESOURCE_TYPE = '0' AND TA.PROJECT_ASSIGNMENT_ID IS Null;

      -- Dynamic Cursor Declaration
      CMM1 PMO_DB2M.REF_CURSOR;

      CURSOR C27 IS
         SELECT DAY_YEAR, SCHED_MIN, BIT_SCHED FROM TASK_DAY_OF_YEAR
         WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND BIT_FLAG = '0';

      CURSOR C54 IS
         SELECT TASK_ID, TSK_ASSIGNMENT_ID
         FROM TASK_ASSIGNMENTS
         WHERE BIT_FLAG = '0' AND PROJECT_ASSIGNMENT_ID = h_id;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_COMMIT_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_COMMIT_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_COMMIT_ASSIGN(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_COMMIT_ASSIGN', 62+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_COMMIT_ASSIGN(63) : cplan.sql(18638) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   h_id := id;
   
   BEGIN
      SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_min_per_day FROM TMT_WBS WHERE ELEMENT_ID = h_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 79+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(80) : cplan.sql(18655) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_bitflag = %d, h_id = %s\n",SC_STATE, h_bitflag, h_id);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         SELECT PROJECT_ID, CASE WHEN PROJECT_ID = TASK_ID THEN 1 ELSE 0 END, CASE WHEN PROJECT_ASSIGNMENT_ID IS NuLL THEN 0 ELSE 1 END
         INTO h_project_id,h_tsk_prj,h_from_prj
         FROM TASK_ASSIGNMENTS WHERE MILESTONE_PERCENT < 10000 AND RESOURCE_TYPE='0' AND TSK_ASSIGNMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 96+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(97) : cplan.sql(18672) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_COMMIT_ASSIGN', 103+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_COMMIT_ASSIGN(104) : cplan.sql(18679) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_COMMIT_ASSIGN', 106+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_COMMIT_ASSIGN(107) : cplan.sql(18682) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.RECORD_NOT_FOUND, True);
      ElsIf (Coalesce(h_tsk_prj, 0) = 0) AND (Coalesce(h_from_prj, 0) = 1) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_COMMIT_ASSIGN', 111+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_COMMIT_ASSIGN(112) : cplan.sql(18687) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_EXISTS), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_COMMIT_ASSIGN', 114+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_COMMIT_ASSIGN(115) : cplan.sql(18690) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_EXISTS, h_id, True);
      End If;
      
      BEGIN
         SELECT COALESCE(MINUTE_PER_DAY, 480) INTO h_min_per_day FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 127+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(128) : cplan.sql(18703) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      stmt := 'SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, TASK_ID, C_EFFORT FROM TASK_ASSIGNMENTS '
             || 'WHERE TSK_ASSIGNMENT_ID = ''' || h_id 
                  || ''' AND MILESTONE_PERCENT < 10000 AND RESOURCE_TYPE=''0'' AND BIT_FLAG = ''0'' ';
   Else
      h_project_id := h_id;
      
      --UPDATING CALENDAR_DAY_OF_YEAR  TASK_DAY_OF_YEAR
      If Coalesce(h_bitflag, 0) = 3 Then
         --wbs changes
         BEGIN
            UPDATE TMT_WBS SET
            TYPE_ID         = CASE WHEN TYPE_ID = PMO_CNST.SC_PROPOSAL THEN PMO_CNST.SC_PROJECT ELSE TYPE_ID END,
            SCHED_START     = COALESCE(PLANNED_START, SCHED_START),
            SCHED_FINISH    = COALESCE(PLANNED_FINISH, SCHED_FINISH),
            SCHED_DURATION  = CASE WHEN PLANNED_DURATION = 0 THEN SCHED_DURATION ELSE PLANNED_DURATION END,
            SCHED_EFFORT    = PLANNED_EFFORT,
            PLANNED_START   = Null,
            PLANNED_FINISH  = Null,
            PLANNED_DURATION = 0,
            FORECAST_START  = Null,
            FORECAST_FINISH = Null,
            FORECAST_EFFORT = PLANNED_EFFORT,
            ETC             = PLANNED_EFFORT,
            FORECAST_DURATION = 0
            WHERE PROJECT_ID = h_project_id AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK) AND ASSIGN_COUNT = 0 AND WORK_TODATE = 0 AND SCHED_EFFORT = 0;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 167+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(168) : cplan.sql(18743) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         OPEN C270;
         
         LOOP
            FETCH C270 INTO h_tsk_assignment_id, h_res_id;
            IF C270%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               BIT_FLAG = '2',
               SCHED_START = COALESCE(PLANNED_START, SCHED_START),
               SCHED_FINISH = COALESCE(PLANNED_FINISH, SCHED_FINISH),
               SCHED_DURATION = CASE WHEN PLANNED_DURATION > 0 THEN PLANNED_DURATION ELSE SCHED_DURATION END,
               SCHED_WORK = PLANNED_WORK,
               ETC = PLANNED_WORK,
               BAK_START_DT = COALESCE(PLANNED_START, SCHED_START),
               BAK_FINISH_DT = COALESCE(PLANNED_FINISH, SCHED_FINISH),
               BAK_WORK = PLANNED_WORK,
               SCHED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,
               SCHED_LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_REVENUE END,
               FORECAST_LABOR_COST = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_COST) + ACTUAL_LABOR_COST END,
               FORECAST_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTUAL_LABOR_REVENUE END,
               PLANNED_START = Null, PLANNED_FINISH = Null, PLANNED_DURATION = 0, PLANNED_WORK = 0,
               FORECAST_START = Null, FORECAST_FINISH = Null, FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 213+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(214) : cplan.sql(18789) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET BIT_FLAG = '2'
               WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 231+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(232) : cplan.sql(18807) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         --end of while
         
         BEGIN
            CLOSE C270 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 246+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(247) : cplan.sql(18822) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT
         WHERE PROJECT_ID = h_project_id AND ASSIGN_COUNT > 0 AND WORK_TODATE = 0 AND SCHED_EFFORT = 0;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 265+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(266) : cplan.sql(18841) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_tsk_prj := 1;
      h_bitflag := 2;
      
      stmt := 
            'SELECT TSK_ASSIGNMENT_ID, RESOURCE_ID, TASK_ID, C_EFFORT FROM TASK_ASSIGNMENTS WHERE PROJECT_ID = TASK_ID AND '
             || 'PROJECT_ID = ''' || h_id 
                  || ''' AND MILESTONE_PERCENT < 10000 AND RESOURCE_TYPE=''0'' AND BIT_FLAG = ''0'' ';
   End If;
   
   --TODO When Planned To Schedule merge to old schedule to the Project Assignments
   -- Copy Planned To Schedule for a task assignment if from Constrained by Project Assignment is not allowed
   --trace_printf("%s\n", (char *) stmt);
   
   OPEN CMM1 FOR stmt;
   
   LOOP
      FETCH CMM1 INTO h_id, h_res_id, h_task_id, h_work;
      IF CMM1%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      --trace_printf("h_bitflag = %d, h_id = %s, h_work = %d\nh_res_id = %s, h_task_id = %s\n",h_bitflag, h_id, h_work, h_res_id, h_task_id);
      
      CASE h_bitflag
         
         WHEN 1 THEN
            --11111111111111111111111111111111111111111111111111111111111111111111111111111
            --1 = Schedule to Baseline (includes budgets)
            --budget changes
            BEGIN
               UPDATE BUDGETS B SET
               B.ESTIMATED_BASELINE3 = B.ESTIMATED_BASELINE2,
               B.ESTIMATED_BASELINE2 = B.ESTIMATED_BASELINE1,
               B.ESTIMATED_BASELINE1 = B.ESTIMATED_BUDGET,
               B.CONTINGENCY_BASELINE3 = B.CONTINGENCY_BASELINE2,
               B.CONTINGENCY_BASELINE2 = B.CONTINGENCY_BASELINE1,
               B.CONTINGENCY_BASELINE1 = B.CONTINGENCY_BUDGET,
               B.BUDGET_BASELINE3 = B.BUDGET_BASELINE2,
               B.BUDGET_BASELINE2 = B.BUDGET_BASELINE1,
               B.BUDGET_BASELINE1 = B.BUDGET
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 326+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(327) : cplan.sql(18902) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --assignment changes
            BEGIN
               UPDATE TASK_ASSIGNMENTS TA SET
               TA.BASELINE3_START = TA.BASELINE2_START, TA.BASELINE3_FINISH = TA.BASELINE2_FINISH, TA.BASELINE3_DURATION = TA.BASELINE2_DURATION, TA.BASELINE3_WORK = TA.BASELINE2_WORK,
               TA.BASELINE3_LABOR_COST = TA.BASELINE2_LABOR_COST, TA.BASELINE3_LABOR_REVENUE = TA.BASELINE2_LABOR_REVENUE,
               TA.BASELINE2_START = TA.BASELINE1_START, TA.BASELINE2_FINISH = TA.BASELINE1_FINISH, TA.BASELINE2_DURATION = TA.BASELINE1_DURATION, TA.BASELINE2_WORK = TA.BASELINE1_WORK,
               TA.BASELINE2_LABOR_COST = TA.BASELINE1_LABOR_COST, TA.BASELINE2_LABOR_REVENUE = TA.BASELINE1_LABOR_REVENUE,
               TA.BASELINE1_START = TA.UNRESOLVED_START, TA.BASELINE1_FINISH = TA.UNRESOLVED_FINISH, TA.BASELINE1_DURATION = TA.UNRESOLVED_DURATION, TA.BASELINE1_WORK = TA.SCHED_WORK + TA.PLANNED_WORK,
               TA.BASELINE1_LABOR_COST = TA.SCHED_LABOR_COST + TA.PLANNED_LABOR_COST, TA.BASELINE1_LABOR_REVENUE = TA.SCHED_LABOR_REVENUE + TA.PLANNED_LABOR_REVENUE,
               TA.B_REGULAR_COST = TA.REGULAR_COST, TA.B_REGULAR_RATE = TA.REGULAR_RATE, TA.B_SPECIAL_COST = TA.SPECIAL_COST, TA.B_SPECIAL_RATE = TA.SPECIAL_RATE
               WHERE TSK_ASSIGNMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 352+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(353) : cplan.sql(18928) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE PROJECT_ASSIGNMENTS SET
               ESTIMATED_BASELINE3_START = ESTIMATED_BASELINE2_START,
               ESTIMATED_BASELINE2_START = ESTIMATED_BASELINE1_START,
               ESTIMATED_BASELINE1_START = CASE WHEN ESTIMATED_SCHED_START IS NOT NULL THEN ESTIMATED_SCHED_START ELSE ESTIMATED_PLANNED_START END,
               ESTIMATED_BASELINE3_FINISH = ESTIMATED_BASELINE2_FINISH,
               ESTIMATED_BASELINE2_FINISH = ESTIMATED_BASELINE1_FINISH,
               ESTIMATED_BASELINE1_FINISH = CASE WHEN ESTIMATED_SCHED_FINISH IS NOT NULL THEN ESTIMATED_SCHED_FINISH ELSE ESTIMATED_PLANNED_FINISH END,
               ESTIMATED_BASELINE3_EFFORT = ESTIMATED_BASELINE2_EFFORT,
               ESTIMATED_BASELINE2_EFFORT = ESTIMATED_BASELINE1_EFFORT,
               ESTIMATED_BASELINE1_EFFORT = CASE WHEN ESTIMATED_SCHED_EFFORT > 0 THEN ESTIMATED_SCHED_EFFORT ELSE ESTIMATED_PLANNED_EFFORT END,
               ESTIMATED_BASELINE3_COST = ESTIMATED_BASELINE2_COST,
               ESTIMATED_BASELINE2_COST = ESTIMATED_BASELINE1_COST,
               ESTIMATED_BASELINE1_COST = CASE WHEN ESTIMATED_SCHED_COST > 0.00 THEN ESTIMATED_SCHED_COST ELSE ESTIMATED_PLANNED_COST END,
               ESTIMATED_BASELINE3_REVENUE = ESTIMATED_BASELINE2_REVENUE,
               ESTIMATED_BASELINE2_REVENUE = ESTIMATED_BASELINE1_REVENUE,
               ESTIMATED_BASELINE1_REVENUE = CASE WHEN ESTIMATED_SCHED_REVENUE > 0.00 THEN ESTIMATED_SCHED_REVENUE ELSE ESTIMATED_PLANNED_REVENUE END,
               BASELINE_START = UNRESOLVED_START,
               BASELINE_FINISH = UNRESOLVED_FINISH,
               BASELINE_DURATION = UNRESOLVED_DURATION,
               BASELINE_WORK = SCHED_WORK + PLANNED_WORK,
               BASELINE_LABOR_COST = SCHED_LABOR_COST + PLANNED_LABOR_COST,
               BASELINE_LABOR_REVENUE = SCHED_LABOR_REVENUE + PLANNED_LABOR_REVENUE
               WHERE PROJECT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 391+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(392) : cplan.sql(18967) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            
         
         WHEN 2 THEN
            --22222222222222222222222222222222222222222222222222222222222222222222222222222
            --2 = Planned to Schedule
            -- check if there is assignment in planned
            BEGIN
               SELECT PLANNED_WORK INTO h_day_min FROM TASK_ASSIGNMENTS WHERE BIT_FLAG = '0' AND TSK_ASSIGNMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 410+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(411) : cplan.sql(18986) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS OR ((h_day_min IS NULL)) Then
               GOTO CaseEnd0000;
            End If;
            
            BEGIN
               SELECT TSK_ASSIGNMENT_ID, CASE WHEN C_EFFORT = 0 THEN 0 ELSE C_EFFORT + h_work END
               INTO h_prj_assignment_id, h_work
               FROM TASK_ASSIGNMENTS WHERE BIT_FLAG = '2' AND TASK_ID = h_project_id AND RESOURCE_ID = h_res_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 429+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(430) : cplan.sql(19005) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("h_tsk_prj = %d, h_id = %s, h_prj_assignment_id = %s, h_work = %d\n",h_tsk_prj, h_id, h_prj_assignment_id, h_work);
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_prj_assignment_id IS NOT NULL)) Then
               If Coalesce(h_tsk_prj, 0) = 0 Then
                  h_total := 2;
               Else
                  -- delete the project assignment in planned
                  BEGIN
                     DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 452+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(453) : cplan.sql(19028) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM EFFORT_WEEK WHERE TSK_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 469+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(470) : cplan.sql(19045) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGN_ROLES WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 486+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(487) : cplan.sql(19062) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  BEGIN
                     DELETE FROM RESOURCE_ASSIGN_SKILLS WHERE RES_ASSIGNMENT_ID = h_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 503+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(504) : cplan.sql(19079) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  h_total := 1;
               End If;
            Else
               h_prj_assignment_id := h_id;
               h_total := 0;
            End If;
            
            h_day_min := 0;
            --trace_printf("h_total = %d, h_id = %s, h_prj_assignment_id= %s\n",h_total,h_id,h_prj_assignment_id);
            
            --Merge task day of year assignments
            
            OPEN C27;
            
            LOOP
               FETCH C27 INTO h_dayofyear, h_day_min, h_bitID;
               IF C27%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               --trace_printf("h_dayofyear = %s, h_day_min = %d\n",h_dayofyear,h_day_min);
               
               If Coalesce(h_total, 0) = 1 Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET SCHED_MIN = SCHED_MIN + h_day_min,
                     BIT_SCHED = UTL_RAW.BIT_OR(BIT_SCHED, h_bitID)
                     WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_prj_assignment_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 550+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(551) : cplan.sql(19126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("1__ SC_STATE = %d\n",SC_STATE);
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     BEGIN
                        DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 569+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(570) : cplan.sql(19145) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 586+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(587) : cplan.sql(19162) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf Coalesce(h_total, 0) = 2 Then
                  BEGIN
                     UPDATE TASK_DAY_OF_YEAR SET TSK_ASSIGNMENT_ID = h_prj_assignment_id WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_id AND DAY_YEAR = h_dayofyear;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 604+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(605) : cplan.sql(19180) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            END LOOP;
            BEGIN
               CLOSE C27 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 618+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(619) : cplan.sql(19194) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TASK_DAY_OF_YEAR SET BIT_FLAG = '2', PARENT_ID = h_prj_assignment_id
               WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 636+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(637) : cplan.sql(19212) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_tsk_prj, 0) = 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET
                  BIT_FLAG = '2',
                  SD = PMO_CNST.SC_SS, FD = PMO_CNST.SC_SF,
                  SCHED_START = PLANNED_START,
                  SCHED_FINISH = PLANNED_FINISH,
                  SCHED_DURATION = PLANNED_DURATION,
                  SCHED_WORK = PLANNED_WORK,
                  BAK_START_DT = PLANNED_START,
                  BAK_FINISH_DT = PLANNED_FINISH,
                  BAK_WORK = PLANNED_WORK,
                  SCHED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,
                  SCHED_LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_REVENUE END,
                  ETC = PLANNED_WORK,
                  FORECAST_LABOR_COST = ((ETC+PLANNED_WORK/60.00) * REGULAR_COST) + ACTUAL_LABOR_COST,
                  FORECAST_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTUAL_LABOR_REVENUE END,
                  PLANNED_START = NuLL, PLANNED_FINISH = NuLL, PLANNED_DURATION = 0, PLANNED_WORK = 0,
                  FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
                  PROJECT_ASSIGNMENT_ID = NuLL
                  WHERE TSK_ASSIGNMENT_ID = h_id AND SCHED_WORK = 0 AND PLANNED_WORK > 0;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 672+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(673) : cplan.sql(19248) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_total, 0) = 2 Then
                  -- CALL UPDATE assign string function
                  h_temp_var := 3;
                  h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
               End If;
            Else
               -- update task assignments
               
               OPEN C54;
               
               LOOP
                  FETCH C54 INTO h_task_id, h_tsk_assignment_id;
                  IF C54%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                     EXIT;
                  End If;
                  
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET
                     BIT_FLAG = '2',
                     SD = PMO_CNST.SC_SS,
                     FD = PMO_CNST.SC_SF,
                     SCHED_START = PLANNED_START,
                     SCHED_FINISH = PLANNED_FINISH,
                     SCHED_DURATION = PLANNED_DURATION,
                     SCHED_WORK = PLANNED_WORK,
                     BAK_START_DT = PLANNED_START,
                     BAK_FINISH_DT = PLANNED_FINISH,
                     BAK_WORK = PLANNED_WORK,
                     SCHED_LABOR_COST = CASE WHEN PLANNED_LABOR_COST > 0.00 THEN PLANNED_LABOR_COST ELSE SCHED_LABOR_COST END,
                     SCHED_LABOR_REVENUE = CASE WHEN PLANNED_LABOR_REVENUE > 0.00 THEN PLANNED_LABOR_REVENUE ELSE SCHED_LABOR_REVENUE END,
                     ETC = PLANNED_WORK,
                     FORECAST_LABOR_COST = ((ETC+PLANNED_WORK/60.00) * REGULAR_COST) + ACTUAL_LABOR_COST,
                     FORECAST_LABOR_REVENUE = CASE WHEN BILLABLE = 0 THEN 0.00 ELSE ((ETC+PLANNED_WORK/60.00) * REGULAR_RATE) + ACTUAL_LABOR_REVENUE END,
                     PLANNED_START = NuLL, PLANNED_FINISH = NuLL, PLANNED_DURATION = 0, PLANNED_WORK = 0,
                     FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
                     PROJECT_ASSIGNMENT_ID = h_prj_assignment_id
                     WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 729+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(730) : cplan.sql(19305) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  --trace_printf("SC_STATE = %d, h_tsk_assignment_id = %s, h_prj_assignment_id = %s\n",SC_STATE,h_tsk_assignment_id,h_prj_assignment_id);
                  
                  -- Flag ROLLUP_BITS at the task
                  BEGIN
                     UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT WHERE ELEMENT_ID = h_task_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 748+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(749) : cplan.sql(19324) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               END LOOP;
               
               BEGIN
                  CLOSE C54 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 762+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(763) : cplan.sql(19338) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If (Coalesce(h_tsk_prj, 0) = 1) AND (Coalesce(h_work, 0) > 0) Then
                  BEGIN
                     UPDATE TASK_ASSIGNMENTS SET C_EFFORT = h_work WHERE TSK_ASSIGNMENT_ID = h_prj_assignment_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 780+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(781) : cplan.sql(19356) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               -- CALL UPDATE assign string function
               h_temp_var := 3;
               h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
               
               BEGIN
                  SELECT PROJECT_ASSIGNMENT_ID, BEST, SCHED_START, SCHED_FINISH, BILLABLE, REGULAR_COST, REGULAR_RATE
                  INTO h_prj_assignment_id, h_sched_work, h_start_date, h_finish_date, h_billable, h_reg_cost, h_reg_sale
                  FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID   = h_prj_assignment_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 800+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(801) : cplan.sql(19376) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_start_date = %s, h_finish_date: %s, h_billable: %d, h_sched_work: %d\n", SC_STATE,h_start_date,h_finish_date,h_billable,h_sched_work);//trace_flush();
               
               If Coalesce(h_billable, 0) = 0 Then
                  BEGIN
                     UPDATE PROJECT_ASSIGNMENTS SET
                     ESTIMATED_PLANNED_START = NuLL,
                     ESTIMATED_PLANNED_FINISH = NuLL,
                     ESTIMATED_PLANNED_EFFORT = 0,
                     ESTIMATED_PLANNED_COST = 0.00,
                     ESTIMATED_PLANNED_REVENUE = 0.00,
                     ESTIMATED_SCHED_START = h_start_date,
                     ESTIMATED_SCHED_FINISH = h_finish_date,
                     ESTIMATED_SCHED_EFFORT = h_sched_work,
                     ESTIMATED_SCHED_COST = (h_sched_work / 60.0) * h_reg_cost,
                     ESTIMATED_SCHED_REVENUE = 0.00
                     WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 830+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(831) : cplan.sql(19406) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     UPDATE PROJECT_ASSIGNMENTS SET
                     ESTIMATED_PLANNED_START = NuLL,
                     ESTIMATED_PLANNED_FINISH = NuLL,
                     ESTIMATED_PLANNED_EFFORT = 0,
                     ESTIMATED_PLANNED_COST = 0.00,
                     ESTIMATED_PLANNED_REVENUE = 0.00,
                     ESTIMATED_SCHED_START = h_start_date,
                     ESTIMATED_SCHED_FINISH = h_finish_date,
                     ESTIMATED_SCHED_EFFORT = h_sched_work,
                     ESTIMATED_SCHED_COST = (h_sched_work / 60.0) * h_reg_cost,
                     ESTIMATED_SCHED_REVENUE = (h_sched_work / 60.0) * h_reg_sale
                     WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 858+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(859) : cplan.sql(19434) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               --trace_printf("SC_STATE: %d, h_reg_cost: %.2f, h_reg_sale: %2f\n", SC_STATE,h_reg_cost,h_reg_sale);
            End If;
            
            <<CaseEnd0000>>
            NULL;
         ELSE
            NULL;
      END CASE;
      
      -- Flag ROLLUP_BITS at the project/task
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_ASSIGNMENT_BIT WHERE ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 885+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(886) : cplan.sql(19461) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("ROLLUP_BITS: SC_STATE = %d, h_task_id = %s, h_project_id = %s\n",SC_STATE,h_task_id,h_project_id);
   END LOOP;
   -- end of while
   
   BEGIN
      CLOSE CMM1 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 901+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(902) : cplan.sql(19477) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   -- New Rollup
   
   BEGIN
      SELECT CASE WHEN (SD <> PMO_CNST.SC_NS) AND (FD <> PMO_CNST.SC_NF) AND (TYPE_ID = PMO_CNST.SC_PROPOSAL) THEN 1 ELSE 0 END
      INTO h_change_flag
      FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 919+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(920) : cplan.sql(19495) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_change_flag, 0) = 1 Then
      h_type_id := PMO_CNST.SC_PROPOSAL;
      PMO_DB2M.SC_STATE := FC_CHANGE_PROJ_TYPE(h_project_id, h_change_flag, h_type_id);
   End If;
   
   BEGIN
      UPDATE RESOURCE_ASSIGNMENTS SET
      ELEMENT_TYPE  = (SELECT TYPE_ID FROM TMT_WBS WHERE ELEMENT_ID = h_project_id)
      WHERE ELEMENT_ID = h_project_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 943+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(944) : cplan.sql(19519) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_COMMIT_ASSIGN', 953+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_COMMIT_ASSIGN(954) : cplan.sql(19529) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 960+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(961) : cplan.sql(19536) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_COMMIT_ASSIGN', 972+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_COMMIT_ASSIGN(973) : cplan.sql(19548) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_COMMIT_ASSIGN;
ALTER FUNCTION SP_COMMIT_ASSIGN COMPILE DEBUG;

/****************************************************************************************
Accept ETC and/or FORECAST_START, Assign level
  Bitflag =
	1 - ETC
	2 - FORECAST_START
	3 - ETC + FORECAST_START
****************************************************************************************/

Prompt == FUNCTION SP_ACPT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_ACPT_ETC_ASSIGN
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_tsk_assignment_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_finish_D DATE;
   h_last_submitted DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_constr_DT TIMESTAMP;
   h_dayofyear DATE;
   h_finish_time VARCHAR2(8);
   h_minute PLS_INTEGER := 0;
   h_constr_type PLS_INTEGER := 0;
   h_FF_flag PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_SD DATE;
   h_FD DATE;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   h_projaffected_start DATE;
   h_work_todate PLS_INTEGER := 0;
   h_passed PLS_INTEGER := 0;
   h_not_effort_PA PLS_INTEGER := 0;
   i_percent_complete PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ACPT_ETC_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ACPT_ETC_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_ACPT_ETC_ASSIGN(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_ASSIGN', 61+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ACPT_ETC_ASSIGN(62) : cplan.sql(19632) : ' || SYSTIMESTAMP);
   END IF; 

   --project assignmnet start date
   
   -- Parse parameters to local variables
   
   h_bitflag := bitflag001;
   h_tsk_assignment_id := id;
   h_rec_user := rec_user001;
   --trace_printf("h_bitflag = %d, h_tsk_assignment_id = %s\n", h_bitflag,h_tsk_assignment_id);//trace_flush();
   
   -- Process Effort Task
   h_start_date := NULL;
   -- forecast start
   h_finish_date := NULL;
   -- forecast finish
   
   BEGIN
      SELECT
      PROJECT_ID,
      TASK_ID,
      PROJECT_ASSIGNMENT_ID,
      SCHED_WORK+PLANNED_WORK,
      FORECAST_START,
      FORECAST_FINISH,
      CASE WHEN h_bitflag = 2 AND FORECAST_START IS NuLL THEN 0 ELSE ETC END,
      SCHED_WORK+PLANNED_WORK - WORK_TODATE,
      ETC,
      C_CONTOUR,
      LAST_SUBMITTED,
      CASE WHEN FORECAST_FINISH IS NOT NuLL AND FORECAST_START IS NuLL AND FORECAST_FINISH BETWEEN COALESCE(PLANNED_START, SCHED_START) AND COALESCE(PLANNED_FINISH, SCHED_FINISH) THEN 1 ELSE 0 END,
      UNRESOLVED_START,
      UNRESOLVED_FINISH,
      BILLABLE,
      CAPITAL_EXPENSE,
      REGULAR_COST,
      REGULAR_RATE,
      BIT_FLAG,
      MILESTONE_PERCENT,
      RESOURCE_ID,
      RESOURCE_TYPE,
      WORK_TODATE
      INTO
      h_project_id,
      h_task_id,
      h_prj_assignment_id,
      h_sched_work,
      h_start_date,
      h_finish_date,
      h_effort,
      h_remaining,
      h_etc,
      h_contour,
      h_last_submitted,
      h_FF_flag,
      h_ta_start_DT,
      h_ta_finish_DT,
      h_billable,
      h_capital_expense,
      h_reg_cost,
      h_reg_sale,
      h_bit_flag,
      h_mlst_percent,
      h_res_id,
      h_res_type,
      h_work_todate
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
      AND (((SCHED_WORK+PLANNED_WORK) > 0
      AND (SCHED_WORK+PLANNED_WORK - WORK_TODATE) <> ETC)
      OR FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 139+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(140) : cplan.sql(19710) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_finish_date: %d %s\n", SC_STATE,h_finish_date_Ind,h_finish_date);//trace_flush();
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ACPT_ETC_ASSIGN', 147+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ACPT_ETC_ASSIGN(148) : cplan.sql(19718) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_ASSIGN', 150+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ACPT_ETC_ASSIGN(151) : cplan.sql(19721) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_tsk_assignment_id
            , True);
   End If;
   
   h_temp_sc_state := FC_ACPT_EFFORT(h_project_id, h_task_id, h_tsk_assignment_id, h_prj_assignment_id
                                  , h_sched_work, h_start_date, h_finish_date, h_effort, h_remaining
                                  , h_etc, h_contour, h_last_submitted, h_FF_flag, h_ta_start_DT
                                  , h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost
                                  , h_reg_sale, h_bit_flag, h_mlst_percent, h_res_id, h_res_type
                                  , h_work_todate, h_bitflag, h_rec_user);
   
   --trace_printf("h_temp_sc_state: %d\n", h_temp_sc_state);
   
   If Coalesce(h_temp_sc_state, 0) > 200000 Then
      --trace_printf("it comes here--before returning the error\n");
      --RETURN_ERROR(CANNOT_ASSIGN_PROJ_AFFECT_DATES, h_tsk_assignment_id);
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_ACPT_ETC_ASSIGN', 169+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_ACPT_ETC_ASSIGN(170) : cplan.sql(19740) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(h_temp_sc_state), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_ASSIGN', 172+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_ACPT_ETC_ASSIGN(173) : cplan.sql(19743) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(Coalesce(h_temp_sc_state, 0), h_tsk_assignment_id
            , True);
   Else
      If ((BITAND(Coalesce(h_bitflag, 0), 1)) = 1) AND (Coalesce(h_mlst_percent, 0) > 0) Then
         i_percent_complete := Trunc(((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00) 
                                       - Coalesce(h_mlst_percent, 0));
         If (i_percent_complete > 1) OR i_percent_complete < -1 Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT)
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 195+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(196) : cplan.sql(19766) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_temp_var := 2;
      h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
      
      If (h_prj_assignment_id IS NOT NULL) Then
         h_temp_var := 3;
         h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
      End If;
      
      h_total := 0;
      
      --get the remaining after reschedule and compared with etc if less the update etc to be accepted/rejected
      
      BEGIN
         SELECT ETC INTO h_total
         FROM TASK_ASSIGNMENTS
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 224+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(225) : cplan.sql(19795) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("AFTER FC_U_ETC_ASSIGN--->remaining: %d, h_tsk_assignment_id: %s, h_etc: %d\n", h_total, h_tsk_assignment_id, h_etc);
      If Coalesce(h_total, 0) > Coalesce(h_etc, 0) Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS
            SET ETC  = h_total
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 244+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(245) : cplan.sql(19815) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      h_temp_sc_state := FC_Rollup(h_project_id);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ACPT_ETC_ASSIGN', 254+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ACPT_ETC_ASSIGN(255) : cplan.sql(19825) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 261+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(262) : cplan.sql(19832) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_ASSIGN', 273+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_ASSIGN(274) : cplan.sql(19844) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ACPT_ETC_ASSIGN;
ALTER FUNCTION SP_ACPT_ETC_ASSIGN COMPILE DEBUG;

/****************************************************************************************
Accept ETC and/or FORECAST_START, WBS level
  Bitflag =
	1 - ETC
	2 - FORECAST DATES
	3 - ETC + FORECAST DATES
****************************************************************************************/

Prompt == FUNCTION SP_ACPT_ETC_WBS ==
CREATE OR REPLACE FUNCTION SP_ACPT_ETC_WBS
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_prj_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rec_user VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_finish_D DATE;
   h_week_of DATE;
   h_last_submitted DATE;
   h_task_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_passed PLS_INTEGER := 0;
   h_constr_DT TIMESTAMP;
   h_dayofyear DATE;
   h_finish_time VARCHAR2(8);
   h_minute PLS_INTEGER := 0;
   h_constr_type PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_sched_start TIMESTAMP;
   h_sched_finish TIMESTAMP;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_SD DATE;
   h_FD DATE;
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_tph_delete PLS_INTEGER := 0;
   h_not_effort_PA PLS_INTEGER := 0;
   h_FF_flag PLS_INTEGER := 0;
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
      CURSOR CAWTSK IS
         SELECT PROJECT_ID, ELEMENT_ID, SCHED_EFFORT+PLANNED_EFFORT,
         CASE WHEN FORECAST_START IS NULL THEN COALESCE(PLANNED_START, SCHED_START) ELSE FORECAST_START END,
         CASE WHEN FORECAST_FINISH IS NULL THEN COALESCE(PLANNED_FINISH, SCHED_FINISH) ELSE FORECAST_FINISH END
         FROM TMT_WBS WHERE RANK LIKE h_rank AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND (((SCHED_EFFORT+PLANNED_EFFORT) > 0 AND ((SCHED_EFFORT+PLANNED_EFFORT-WORK_TODATE) <> ETC)) OR
         (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL)) ORDER BY COALESCE(PLANNED_START, SCHED_START);

      CURSOR CAWASS IS
         SELECT
         TSK_ASSIGNMENT_ID,
         PROJECT_ASSIGNMENT_ID,
         SCHED_WORK+PLANNED_WORK,
         FORECAST_START,
         FORECAST_FINISH,
         CASE WHEN h_bitflag = 2 AND FORECAST_START IS NuLL THEN 0 ELSE ETC END,
         SCHED_WORK+PLANNED_WORK - WORK_TODATE,
         ETC,
         C_CONTOUR,
         LAST_SUBMITTED,
         CASE WHEN FORECAST_FINISH IS NOT NuLL AND FORECAST_START IS NuLL AND FORECAST_FINISH BETWEEN COALESCE(PLANNED_START, SCHED_START) AND COALESCE(PLANNED_FINISH, SCHED_FINISH) THEN 1 ELSE 0 END,
         UNRESOLVED_START,
         UNRESOLVED_FINISH,
         BILLABLE,
         CAPITAL_EXPENSE,
         REGULAR_COST,
         REGULAR_RATE,
         BIT_FLAG,
         MILESTONE_PERCENT,
         RESOURCE_ID,
         RESOURCE_TYPE,
         WORK_TODATE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND TASK_ID = h_task_id
         AND (((SCHED_WORK+PLANNED_WORK) > 0
         AND ((SCHED_WORK + PLANNED_WORK - WORK_TODATE) <> ETC))
         OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL));

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_ACPT_ETC_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_ACPT_ETC_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_ACPT_ETC_WBS(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_ACPT_ETC_WBS', 106+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_ACPT_ETC_WBS(107) : cplan.sql(19973) : ' || SYSTIMESTAMP);
   END IF; 

   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   h_rec_user := rec_user001;
   h_not_effort_PA := 0;
   
   BEGIN
      SELECT RTRIM(RANK), TYPE_ID INTO h_rank, h_type
      FROM TMT_WBS WHERE ELEMENT_ID = id/*HOST(h_id)*/;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 123+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(124) : cplan.sql(19990) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_type IS NOT NULL)) Then
      h_rank := SubStr(h_rank || '%', 1, 200);
      h_effort := -1;
      
      --as we can change forecast dates in proposed mode we need to fetch proposed dates also
      
      OPEN CAWTSK;
      
      LOOP
         FETCH CAWTSK INTO h_project_id, h_task_id, h_effort, h_start_date, h_finish_date;
         IF CAWTSK%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         If Coalesce(h_effort, 0) = 0 Then
            -- Process Duration Task
            If Coalesce(h_bitflag, 0) = 2 Then
               -- Accept FORECAST Dates
               ora_temp_sc_state := PMO_DB2M.SC_STATE;
               null_return := FC_ACPT_DURATION(h_project_id, h_task_id, h_start_date, h_finish_date, h_rec_user);
               PMO_DB2M.SC_STATE := ora_temp_sc_state;
               h_passed := Coalesce(h_passed, 0) + 1;
            End If;
         Else
            -- Process Effort Task
            h_start_date := NULL;
            -- forecast start
            h_finish_date := NULL;
            -- forecast finish
            
            OPEN CAWASS;
            
            LOOP
               FETCH CAWASS INTO
               h_tsk_assignment_id,
               h_prj_assignment_id,
               h_sched_work,
               h_start_date,
               h_finish_date,
               h_effort,
               h_remaining,
               h_etc,
               h_contour,
               h_last_submitted,
               h_FF_flag,
               h_ta_start_DT,
               h_ta_finish_DT,
               h_billable,
               h_capital_expense,
               h_reg_cost,
               h_reg_sale,
               h_bit_flag,
               h_mlst_percent,
               h_res_id,
               h_res_type,
               h_work_todate;
               IF CAWASS%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               
               h_temp_sc_state := FC_ACPT_EFFORT(h_project_id, h_task_id, h_tsk_assignment_id, h_prj_assignment_id
                                  , h_sched_work, h_start_date, h_finish_date, h_effort, h_remaining
                                  , h_etc, h_contour, h_last_submitted, h_FF_flag, h_ta_start_DT
                                  , h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost
                                  , h_reg_sale, h_bit_flag, h_mlst_percent, h_res_id, h_res_type
                                  , h_work_todate, h_bitflag, h_rec_user);
               
               If Coalesce(h_temp_sc_state, 0) = 0 Then
                  h_passed := Coalesce(h_passed, 0) + 1;
                  
               ElsIf Coalesce(h_temp_sc_state, 0) = PMO_CNST.CANNOT_ACCEPT_FROM_PROJECT_AFF Then
                  h_not_effort_PA := Coalesce(h_not_effort_PA, 0) + 1;
               End If;
            END LOOP;
            -- end of while
            
            BEGIN
               CLOSE CAWASS ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 224+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(225) : cplan.sql(20091) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("h_passed: %d, h_temp_sc_state: %d, h_constr_type: %d, h_constr_DT: %s\n", h_passed, h_temp_sc_state,h_constr_type,h_constr_DT);
            --trace_printf("h_start_DT_Ind = %d, h_finish_DT_Ind = %d, h_start_DT = %s, h_finish_DT = %s\n", h_start_DT_Ind,h_finish_DT_Ind,h_start_DT,h_finish_DT);//trace_flush();
            
            If (Coalesce(h_bitflag, 0) <> 1) AND (Coalesce(h_passed, 0) > 0) Then
               h_start_DT := NULL;
               h_finish_DT := NULL;
               
               BEGIN
                  SELECT CONSTRAINT_DT, CONSTRAINT_TYPE, FORECAST_START, FORECAST_FINISH
                  
                  
                  INTO h_constr_DT, h_constr_type, h_start_DT, h_finish_DT
                  FROM TMT_WBS WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 248+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(249) : cplan.sql(20115) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("h_start_DT_Ind = %d, h_finish_DT_Ind = %d, h_start_DT = %s, h_finish_DT = %s\n", h_start_DT_Ind,h_finish_DT_Ind,h_start_DT,h_finish_DT);//trace_flush();
               
               If (h_finish_DT IS NOT NULL) Then
                  -- fix constraint date, forecast finish
                  If (h_constr_DT IS NOT NULL) Then
                     BEGIN
                        UPDATE TMT_WBS SET
                        CONSTRAINT_DT =
                        CASE
                        WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_DT, 1, 10)) THEN h_finish_DT
                        WHEN h_constr_type IN (0,5,6,7) THEN h_finish_DT
                        ELSE h_constr_DT
                        END,
                        CONSTRAINT_TYPE =
                        CASE
                        WHEN h_constr_type IN (1,2,3,4) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) < TO_DATE(SubStr(h_finish_DT, 1, 10)) THEN 2
                        WHEN h_constr_type IN (0,5,6,7) THEN 2
                        ELSE h_constr_type
                        END
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 282+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(283) : cplan.sql(20149) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TMT_WBS SET CONSTRAINT_DT = h_finish_DT, CONSTRAINT_TYPE = 2
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 300+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(301) : cplan.sql(20167) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               ElsIf ((h_start_DT IS NOT NULL)) AND ((h_finish_DT IS NULL)) Then
                  -- fix constraint date, forecast start
                  If (h_constr_DT IS NOT NULL) Then
                     BEGIN
                        UPDATE TMT_WBS SET
                        CONSTRAINT_DT =
                        CASE
                        WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN h_start_DT
                        WHEN h_constr_type IN (1,2,3,4) THEN h_start_DT
                        ELSE h_constr_DT
                        END,
                        CONSTRAINT_TYPE =
                        CASE
                        WHEN h_constr_type IN (0,5,6,7) AND TO_DATE(SubStr(h_constr_DT, 1, 10)) > TO_DATE(SubStr(h_start_DT, 1, 10)) THEN 6
                        WHEN h_constr_type IN (1,2,3,4) THEN 6
                        ELSE h_constr_type
                        END
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 333+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(334) : cplan.sql(20200) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     BEGIN
                        UPDATE TMT_WBS SET CONSTRAINT_DT = h_start_DT, CONSTRAINT_TYPE = 6
                        WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 351+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(352) : cplan.sql(20218) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            End If;
         End If;
      END LOOP;
      -- end of while
      
      BEGIN
         CLOSE CAWTSK ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 370+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(371) : cplan.sql(20237) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("END: h_passed = %d, h_effort: %d, h_temp_sc_state: %d\n",h_passed,h_effort,h_temp_sc_state);
      --trace_printf("END: h_not_effort_PA: %d\n", h_not_effort_PA);
      
      If (Coalesce(h_passed, 0) > 0) AND (Coalesce(h_effort, 0) > -1) Then
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_rank), 0); 
         		If 1 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
         	End If; 
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PROPOSED_BIT)
            WHERE PROJECT_ID = h_project_id AND RANK LIKE h_rank;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 405+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(406) : cplan.sql(20272) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_temp_sc_state := FC_Rollup(h_project_id);
         -- New Rollup
      End If;
   End If;
   
   If Coalesce(h_not_effort_PA, 0) > 0 Then
      PMO_DB2M.SC_STATE := PMO_CNST.CANNOT_ACCEPT_FROM_PROJECT_AFF;
   ElsIf Coalesce(h_temp_sc_state, 0) > 100 Then
      PMO_DB2M.SC_STATE := Coalesce(h_temp_sc_state, 0);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_ACPT_ETC_WBS', 422+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_ACPT_ETC_WBS(423) : cplan.sql(20289) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 429+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(430) : cplan.sql(20296) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_ACPT_ETC_WBS', 441+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_ACPT_ETC_WBS(442) : cplan.sql(20308) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_ACPT_ETC_WBS;
ALTER FUNCTION SP_ACPT_ETC_WBS COMPILE DEBUG;

/****************************************************************************************
Reject ETC and/or FORECAST_START, WBS level
  Bitflag =
	1 - ETC
	2 - FORECAST_START
	3 - ETC + FORECAST_START
****************************************************************************************/

Prompt == FUNCTION SP_RJCT_ETC_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_RJCT_ETC_ASSIGN
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_tsk_assignment_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_steps_exist PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   i_percent_complete PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_RJCT_ETC_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_RJCT_ETC_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_RJCT_ETC_ASSIGN(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_RJCT_ETC_ASSIGN', 30+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_RJCT_ETC_ASSIGN(31) : cplan.sql(20361) : ' || SYSTIMESTAMP);
   END IF; 

   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   h_tsk_assignment_id := id;
   
   BEGIN
      SELECT PROJECT_ID, TASK_ID, BIT_FLAG, SCHED_WORK+PLANNED_WORK, WORK_TODATE, MILESTONE_PERCENT
      INTO h_project_id, h_task_id, h_bit_flag, h_sched_work, h_work_todate, h_mlst_percent
      FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 47+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(48) : cplan.sql(20378) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      If SubStr(h_bit_flag, 1, 1) = '2' Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ETC = SCHED_WORK - WORK_TODATE--, MILESTONE_PERCENT = (WORK_TODATE * 10000) / SCHED_WORK
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 67+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(68) : cplan.sql(20398) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE ACTIVITIES
            SET ETC_MODIFIED = 'N'
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
            AND TSK_ASSIGNMENT_ID <> ELEMENT_ID;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 87+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(88) : cplan.sql(20418) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ETC = PLANNED_WORK, WORK_TODATE = 0, MILESTONE_PERCENT = 0
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 105+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(106) : cplan.sql(20436) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      BEGIN
         UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_SCHED_BIT) WHERE ELEMENT_ID = h_task_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 123+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(124) : cplan.sql(20454) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      If Coalesce(h_bitflag, 0) = 2 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 142+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(143) : cplan.sql(20473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_SCHED_BIT) WHERE ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 159+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(160) : cplan.sql(20490) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         If SubStr(h_bit_flag, 1, 1) = '2' Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
               ETC = SCHED_WORK - WORK_TODATE--, MILESTONE_PERCENT = (WORK_TODATE * 10000) / SCHED_WORK
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 179+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(180) : cplan.sql(20510) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE ACTIVITIES
               SET ETC_MODIFIED = 'N'
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
               AND TSK_ASSIGNMENT_ID <> ELEMENT_ID;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 199+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(200) : cplan.sql(20530) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET FORECAST_START = NuLL, FORECAST_FINISH = NuLL, FORECAST_DURATION = 0,
               ETC = PLANNED_WORK, WORK_TODATE = 0, MILESTONE_PERCENT = 0
               WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 218+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(219) : cplan.sql(20549) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS, PMO_CNST.SC_SCHED_BIT) WHERE ELEMENT_ID = h_task_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 236+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(237) : cplan.sql(20567) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   If ((BITAND(Coalesce(h_bitflag, 0), 1)) = 1) AND (Coalesce(h_mlst_percent, 0) > 0) Then
      i_percent_complete := Trunc(((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00) 
                                       - Coalesce(h_mlst_percent, 0));
      If (i_percent_complete > 1) OR i_percent_complete < -1 Then
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT),
            ACTUAL_FINISH = CASE
            WHEN ACTUAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH
            ELSE ACTUAL_FINISH
            END
            WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 265+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(266) : cplan.sql(20596) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   Else
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         ACTUAL_FINISH = CASE
         WHEN ACTUAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH
         ELSE ACTUAL_FINISH
         END
         WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 288+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(289) : cplan.sql(20619) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND (Coalesce(h_bitflag, 0) <> 2) Then
      -- Call Fix_Summary_TS and Effort_Week
      h_temp_sc_state := FC_FIX_SUMMARY_TS(h_project_id, h_tsk_assignment_id);
   End If;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   -- New Rollup
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_RJCT_ETC_ASSIGN', 303+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_RJCT_ETC_ASSIGN(304) : cplan.sql(20634) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 310+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(311) : cplan.sql(20641) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_ASSIGN', 322+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_ASSIGN(323) : cplan.sql(20653) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_RJCT_ETC_ASSIGN;
ALTER FUNCTION SP_RJCT_ETC_ASSIGN COMPILE DEBUG;

/****************************************************************************************
Reject ETC and/or FORECAST_START, WBS level
  Bitflag =
	1 - ETC
	2 - FORECAST_START
	3 - ETC + FORECAST_START
****************************************************************************************/

Prompt == FUNCTION SP_RJCT_ETC_WBS ==
CREATE OR REPLACE FUNCTION SP_RJCT_ETC_WBS
(
     id VARCHAR2
   , bitflag001 PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_finish_date TIMESTAMP;
   h_project_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_start_date TIMESTAMP;
   h_task_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_bit_flag VARCHAR2(1);
   h_bitflag PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_start_D DATE;
   h_temp_sc_state PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_continue_01 PLS_INTEGER := 0;
   h_temp_sc_nodata PLS_INTEGER := 0;
   h_steps_exist PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_work_todate PLS_INTEGER := 0;
   h_mlst_percent PLS_INTEGER := 0;
   h_temp_sc_state_alert PLS_INTEGER := 0;
   h_alert_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_calltype PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
   i_steps_exist PLS_INTEGER NOT NULL := 0;
   i_percent_complete PLS_INTEGER;
-- Declare Cursors
      CURSOR CAWTSKR IS
         SELECT ELEMENT_ID, SCHED_EFFORT+PLANNED_EFFORT,
         CASE WHEN FORECAST_START IS NULL THEN SCHED_START ELSE FORECAST_START END,
         CASE WHEN FORECAST_FINISH IS NULL THEN SCHED_FINISH ELSE FORECAST_FINISH END
         FROM TMT_WBS
         WHERE PROJECT_ID = h_project_id AND RANK LIKE h_rank AND TYPE_ID IN (PMO_CNST.SC_TASK,PMO_CNST.SC_TASK_TYPE1,PMO_CNST.SC_TASK_TYPE2,PMO_CNST.SC_TASK_TYPE3,PMO_CNST.SC_MILESTONE,PMO_CNST.SC_PERSONAL_TASK,PMO_CNST.SC_ACTIVE_ADM_TASK,PMO_CNST.SC_INACTIVE_ADM_TASK,PMO_CNST.SC_WBS_CHANGE_REQUEST_TASK,PMO_CNST.SC_WBS_ACTION_ITEM_TASK,PMO_CNST.SC_WBS_RISK_TASK,PMO_CNST.SC_WBS_ISSUE_TASK,PMO_CNST.SC_WBS_REQUIREMENT_TASK,PMO_CNST.SC_WBS_DEFECT_TASK,PMO_CNST.SC_WBS_SERVICE_REQUEST_TASK)
         AND (((SCHED_EFFORT+PLANNED_EFFORT) > 0 AND ((SCHED_EFFORT+PLANNED_EFFORT-WORK_TODATE) <> ETC))
         OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL));

      CURSOR CAWASSR IS
         SELECT TSK_ASSIGNMENT_ID, BIT_FLAG, BEST, SCHED_WORK+PLANNED_WORK, WORK_TODATE, MILESTONE_PERCENT
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id
         AND TASK_ID = h_task_id
         AND (((SCHED_WORK+PLANNED_WORK) > 0 AND ((SCHED_WORK+PLANNED_WORK-WORK_TODATE) <> ETC))
         OR (FORECAST_START IS NOT NULL OR FORECAST_FINISH IS NOT NULL));

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_RJCT_ETC_WBS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_RJCT_ETC_WBS'), '-- ' || SYSTIMESTAMP, ':r := SP_RJCT_ETC_WBS(' || 
       '''' || id || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_RJCT_ETC_WBS', 61+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_RJCT_ETC_WBS(62) : cplan.sql(20737) : ' || SYSTIMESTAMP);
   END IF; 

   /**** Added to issue alert  ****/
   
   -- Parse parameters to local variables
   h_bitflag := bitflag001;
   
   /**** Added to issue alert  ****/
   h_rec_user := rec_user001;
   h_calltype := 3;
   h_alert_id := 'SCETCFORCASTREJECTED';
   
   BEGIN
      SELECT PROJECT_ID, RTRIM(RANK), TYPE_ID
      INTO h_project_id, h_rank, h_type
      FROM TMT_WBS WHERE ELEMENT_ID = id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 84+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(85) : cplan.sql(20760) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE = %d, h_bitflag: %d, h_rank = %s\n", SC_STATE,h_bitflag,h_rank);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS AND ((h_type IS NOT NULL)) Then
      h_rank := h_rank || '%';
      h_effort := -1;
      
      BEGIN
         SELECT START_FINISH_DATE INTO h_start_D FROM TMT_WBS WHERE ELEMENT_ID = h_project_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 102+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(103) : cplan.sql(20778) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN CAWTSKR;
      
      LOOP
         h_continue_01 := 1;
         -- Initialize continue flag
         
         FETCH CAWTSKR INTO h_task_id, h_effort, h_start_date, h_finish_date;
         IF CAWTSKR%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         --trace_printf("h_effort = %d, h_task_id = %s\n", h_effort,h_task_id);
         
         If Coalesce(h_effort, 0) = 0 Then
            -- Process Duration based Task
            If Coalesce(h_bitflag, 0) = 2 Then
               -- Reject FORECAST
               BEGIN
                  UPDATE TMT_WBS SET
                  FORECAST_START = NULL,
                  FORECAST_FINISH = NULL,
                  FORECAST_DURATION = 0
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 145+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(146) : cplan.sql(20821) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               /**** Added to issue alert  ****/
               h_table_type := 1;
               h_temp_sc_state_alert := FC_ISSUE_ALERT(h_alert_id, h_task_id, h_rec_user, h_calltype, Coalesce(
                                        h_table_type, 0));
            Else
               h_continue_01 := 0;
               -- start the next iteration and dont continue the rest statements
            End If;
         Else
            -- Process Effort based Task
            
            OPEN CAWASSR;
            
            LOOP
               FETCH CAWASSR INTO h_tsk_assignment_id, h_bit_flag, h_temp_var, h_sched_work, h_work_todate, h_mlst_percent;
               IF CAWASSR%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               
               If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
                  EXIT;
               End If;
               --trace_printf("h_tsk_assignment_id = %s, h_bitflag = %d, h_temp_var: %d\n", h_tsk_assignment_id,h_bitflag,h_temp_var);
               
               h_temp_sc_nodata := 0;
               If Coalesce(h_bitflag, 0) = 1 Then
                  If SubStr(h_bit_flag, 1, 1) = '2' Then
                     BEGIN
                        UPDATE ACTIVITIES
                        SET ETC_MODIFIED = 'N'
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                        AND TSK_ASSIGNMENT_ID <> ELEMENT_ID;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 194+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(195) : cplan.sql(20870) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If Coalesce(h_temp_var, 0) = -1 Then
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           PROPOSED_START      = SCHED_START,
                           PROPOSED_FINISH     = SCHED_FINISH,
                           PROPOSED_WORK       = SCHED_WORK,
                           BAK_START_DT        = SCHED_START,
                           BAK_FINISH_DT       = SCHED_FINISH,
                           BAK_WORK            = SCHED_WORK,
                           BAK_ASSIGN_STRING   = ASSIGN_STRING,
                           BEST                = SCHED_WORK,
                           ETC                 = SCHED_WORK - WORK_TODATE,
                           ACTUAL_FINISH       = NULL
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 223+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(224) : cplan.sql(20899) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           SCHED_WORK = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK ELSE SCHED_WORK END,
                           ETC = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK - WORK_TODATE ELSE 0 END,
                           ACTUAL_FINISH = NULL/*,
                           MILESTONE_PERCENT = (WORK_TODATE * 10000.00) / SCHED_WORK*/
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 245+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(246) : cplan.sql(20921) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  Else
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        ETC = PLANNED_WORK, MILESTONE_PERCENT = 0 ,
                        ACTUAL_FINISH = NULL, ACTUAL_DURATION = 0
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 266+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(267) : cplan.sql(20942) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
                  --trace_printf("SC_STATE = %d, h_tsk_assignment_id = %s\n", SC_STATE,h_tsk_assignment_id);
               Else
                  If Coalesce(h_bitflag, 0) = 2 Then
                     -- Reject FORECAST dates
                     BEGIN
                        UPDATE TASK_ASSIGNMENTS SET
                        FORECAST_START = NULL,
                        FORECAST_FINISH = NULL,
                        FORECAST_DURATION = 0
                        WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 291+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(292) : cplan.sql(20967) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  Else
                     -- Reject ETC + FORECAST_START
                     If SubStr(h_bit_flag, 1, 1) = '2' Then
                        BEGIN
                           UPDATE ACTIVITIES
                           SET ETC_MODIFIED = 'N'
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id
                           AND TSK_ASSIGNMENT_ID <> ELEMENT_ID;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 313+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(314) : cplan.sql(20989) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                        
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           FORECAST_START = NULL,
                           FORECAST_FINISH = NULL,
                           FORECAST_DURATION = 0,
                           SCHED_WORK = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK ELSE SCHED_WORK END,
                           ETC = CASE WHEN PROPOSED_WORK > WORK_TODATE THEN PROPOSED_WORK - WORK_TODATE ELSE 0 END/*,
                           MILESTONE_PERCENT = (WORK_TODATE * 10000) / SCHED_WORK*/
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 337+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(338) : cplan.sql(21013) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           FORECAST_START = NULL,
                           FORECAST_FINISH = NULL,
                           FORECAST_DURATION = 0,
                           ETC = PLANNED_WORK, MILESTONE_PERCENT = 0
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 359+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(360) : cplan.sql(21035) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
               End If;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  h_temp_sc_nodata := 1;
               End If;
               
               --trace_printf("h_temp_sc_nodata: %d, h_bitflag: %d, h_mlst_percent: %d\n", h_temp_sc_nodata,h_bitflag,h_mlst_percent);
               If Coalesce(h_temp_sc_nodata, 0) = 0 Then
                  If ((BITAND(Coalesce(h_bitflag, 0), 1)) = 1) AND (Coalesce(h_mlst_percent, 0) > 0) Then
                     i_percent_complete := Trunc(((Coalesce(h_work_todate, 0) / Coalesce(h_sched_work, 0)) * 10000.00) 
                                       - Coalesce(h_mlst_percent, 0));
                     If (i_percent_complete > 1) OR i_percent_complete < -1 Then
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           ROLLUP_BITS = PMO_STPU.BIT_OR(ROLLUP_BITS,PMO_CNST.SC_PERCENT_COMPLETE_BIT),
                           ACTUAL_FINISH = CASE
                           WHEN ACTUAL_FINISH IS NULL AND ETC = 0 THEN SCHED_FINISH
                           ELSE ACTUAL_FINISH
                           END
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 395+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(396) : cplan.sql(21071) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     Else
                        BEGIN
                           UPDATE TASK_ASSIGNMENTS SET
                           ACTUAL_FINISH = CASE WHEN (ACTUAL_FINISH IS NULL) AND ETC = 0 THEN SCHED_FINISH ELSE ACTUAL_FINISH END
                           WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 414+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(415) : cplan.sql(21090) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                     --trace_printf("i_percent_complete: %d, h_work_todate: %d, h_sched_work: %d\n", i_percent_complete,h_work_todate,h_sched_work);
                     
                     -- Reject ETC and/or FORECAST_START
                     -- Call Fix_Summary_TS and Effort_Week
                     h_temp_sc_state := FC_FIX_SUMMARY_TS(h_project_id, h_tsk_assignment_id);
                  End If;
               End If;
               
               /**** Added to issue alert  ****/
               h_table_type := 0;
               h_temp_sc_state_alert := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_calltype
                                        , Coalesce(h_table_type, 0));
            END LOOP;
            -- end of while
            
            BEGIN
               CLOSE CAWASSR ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 442+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(443) : cplan.sql(21118) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         -- flag the task, for later rollup
         If Coalesce(h_continue_01, 0) = 1 Then
            BEGIN
               UPDATE TMT_WBS SET
               CONSTRAINT_TYPE =
               CASE
               WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) <> TO_DATE(SubStr(h_start_D, 1, 10)) THEN 6
               ELSE 0
               END,
               CONSTRAINT_DT =
               CASE
               WHEN TO_DATE(SubStr(SCHED_START, 1, 10)) <> TO_DATE(SubStr(h_start_D, 1, 10)) THEN SCHED_START
               ELSE NULL
               END
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_task_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 473+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(474) : cplan.sql(21149) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      -- end of while
      
      BEGIN
         CLOSE CAWTSKR ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 489+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(490) : cplan.sql(21165) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_effort, 0) > -1 Then
         h_temp_sc_state := FC_Rollup(h_project_id);
         -- New Rollup
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_RJCT_ETC_WBS', 501+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_RJCT_ETC_WBS(502) : cplan.sql(21177) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 508+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(509) : cplan.sql(21184) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_RJCT_ETC_WBS', 520+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_RJCT_ETC_WBS(521) : cplan.sql(21196) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_RJCT_ETC_WBS;
ALTER FUNCTION SP_RJCT_ETC_WBS COMPILE DEBUG;

--******************************************************************************************

Prompt == FUNCTION SP_U_WP_FORECAST ==
CREATE OR REPLACE FUNCTION SP_U_WP_FORECAST
(
     id VARCHAR2
   , start_date001 TIMESTAMP
   , finish_date001 TIMESTAMP
   , acc_rej_bitflag PLS_INTEGER
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_project_id VARCHAR2(32);
   h_task_id VARCHAR2(32);
   h_prj_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_start_date TIMESTAMP;
   h_finish_date TIMESTAMP;
   h_start_DT TIMESTAMP;
   h_finish_DT TIMESTAMP;
   h_forecast_start TIMESTAMP;
   h_forecast_finish TIMESTAMP;
   h_actual_start TIMESTAMP;
   h_rank VARCHAR2(200);
   h_mlst_percent PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_reg_cost NUMBER := 0.00;
   h_reg_sale NUMBER := 0.00;
   h_ta_start_DT TIMESTAMP;
   h_ta_finish_DT TIMESTAMP;
   h_last_submitted DATE;
   h_projaffected_start DATE;
   h_dayofyear DATE;
   h_week_of DATE;
   h_bit_flag VARCHAR2(1);
   h_billable PLS_INTEGER := 0;
   h_capital_expense PLS_INTEGER := 0;
   h_etc PLS_INTEGER := 0;
   h_total PLS_INTEGER := 0;
   h_sched_work PLS_INTEGER := 0;
   h_continue PLS_INTEGER := 0;
   h_effort PLS_INTEGER := 0;
   h_contour PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   h_acc_rej_bitflag PLS_INTEGER := 0;
   h_remaining PLS_INTEGER := 0;
   h_FF PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_alert_flag PLS_INTEGER := 0;
   h_alert_id VARCHAR2(32);
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WP_FORECAST') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WP_FORECAST'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WP_FORECAST(' || 
       '''' || id || ''''
          || ', ' || COALESCE(NULLIF('''' || start_date001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || finish_date001 || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(acc_rej_bitflag), 'NULL')
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 67+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WP_FORECAST(68) : cplan.sql(21280) : ' || SYSTIMESTAMP);
   END IF; 

   --bitflag to indicate the right to accept/reject forecast
   
   --clear the dates before the initilization
   h_start_date := '';
   h_finish_date := '';
   
   h_continue := 1;
   --to let users enter forecast dates at the begining
   h_start_date := start_date001;
   h_finish_date := finish_date001;
   h_tsk_assignment_id := id;
   h_acc_rej_bitflag := acc_rej_bitflag;
   
   If (Coalesce(h_acc_rej_bitflag, 0) = 1) Then
      h_acc_rej_bitflag := PMO_CNST.SC_ACCEPT_REJECT_FORECAST;
   End If;
   
   BEGIN
      SELECT PROJECT_ID, RES_ASSIGNMENT_ID, PROJECT_ASSIGNMENT_ID, ACTUAL_START, MILESTONE_PERCENT, BIT_FLAG,
      COALESCE(FORECAST_START,SCHED_START), COALESCE(FORECAST_FINISH,SCHED_FINISH), FORECAST_START, FORECAST_FINISH, TASK_ID, RESOURCE_ID, SCHED_WORK, (PLANNED_WORK + SCHED_WORK - WORK_TODATE), ETC,
      C_CONTOUR, LAST_SUBMITTED, UNRESOLVED_START, UNRESOLVED_FINISH, BILLABLE, CAPITAL_EXPENSE, REGULAR_COST, REGULAR_RATE
      INTO h_project_id, h_res_assignment_id, h_prj_assignment_id, h_actual_start, h_mlst_percent, h_bit_flag,
      h_start_DT, h_finish_DT, h_forecast_start, h_forecast_finish, h_task_id, h_res_id, h_sched_work,h_remaining, h_etc,
      h_contour, h_last_submitted, h_ta_start_DT, h_ta_finish_DT, h_billable, h_capital_expense, h_reg_cost, h_reg_sale
      FROM TASK_ASSIGNMENTS
      WHERE TSK_ASSIGNMENT_ID = id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 102+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(103) : cplan.sql(21315) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("SC_STATE: %d, h_forecast_start: %s, h_forecast_finish: %s\n", SC_STATE,h_forecast_start,h_forecast_finish);
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 110+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(111) : cplan.sql(21323) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 113+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(114) : cplan.sql(21326) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, id, True);
   End If;
   
   If Coalesce(h_mlst_percent, 0) = 10000 Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 120+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(121) : cplan.sql(21333) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ASSIGNMENT_ALREADY_CLOSED), 'NULL') || ' ID: ' || '''' || id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 123+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(124) : cplan.sql(21336) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.ASSIGNMENT_ALREADY_CLOSED, id, True);
   End If;
   
   If ((start_date001 IS NOT NULL)) AND ((finish_date001 IS NOT NULL)) Then
      If PMO_ORASTD.strncmp(h_finish_date, h_start_date, 10) < 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 131+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(132) : cplan.sql(21344) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE), 'NULL') || ' ID: ' || '''' || id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 134+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(135) : cplan.sql(21347) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.F_DATE_EARLIER_THAN_S_DATE, id, True);
      End If;
   End If;
   
   h_week_of := NULL;
   BEGIN
      SELECT MAX(WEEK_OF) + 7      INTO h_week_of FROM TIMESHEET_APPROVAL_STATUS
      WHERE ELEMENT_ID = h_project_id AND RESOURCE_ID = h_res_id
      AND CURRENT_STATE IN (PMO_CNST.SC_APV_STATE_RM_APPROVED,PMO_CNST.SC_APV_STATE_PM_APPROVED,PMO_CNST.SC_APV_STATE_SUBMITTED);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 151+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(152) : cplan.sql(21364) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   --trace_printf("h_week_of: %d %s\n", h_week_of_Ind,h_week_of);
   
   h_rec_user := rec_user001;
   If (PMO_ORASTD.strcmp(h_res_id, h_rec_user) = 0) AND (Coalesce(
         h_acc_rej_bitflag, 0) <> PMO_CNST.SC_ACCEPT_REJECT_FORECAST) Then
      h_alert_id := 'SCETCFORCASTCHANGED';
      h_alert_flag := 1;
      ora_temp_sc_state := PMO_DB2M.SC_STATE;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
      PMO_DB2M.SC_STATE := ora_temp_sc_state;
      
      h_alert_id := 'SCREDUCEDETC';
      h_alert_flag := 0;
      ora_temp_sc_state := PMO_DB2M.SC_STATE;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
      PMO_DB2M.SC_STATE := ora_temp_sc_state;
   End If;
   
   -- if client doesn't sent date  time
   If ((h_start_date IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) = 0) Then
      h_start_date := h_start_DT;
   End If;
   
   If ((start_date001 IS NULL)) AND ((finish_date001 IS NULL)) Then
      BEGIN
         UPDATE TASK_ASSIGNMENTS SET
         FORECAST_START  = start_date001,
         FORECAST_FINISH = finish_date001,
         FORECAST_DURATION = 0
         WHERE TSK_ASSIGNMENT_ID = id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 195+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(196) : cplan.sql(21408) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      --trace_printf("SC_STATE: %d, h_start_date: %d %s, h_finish_date: %d %s\n", SC_STATE,h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
   Else
      --if one of the foreast dates is cleared, null, then clear it
      If (start_date001 IS NULL) AND (h_forecast_start IS NOT NULL) Then
         --trace_printf("CLear the FS\n");
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            FORECAST_START  = start_date001,
            FORECAST_DURATION = 0
            WHERE TSK_ASSIGNMENT_ID = id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 220+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(221) : cplan.sql(21433) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --when clear FS but not chnaged FF then just call rollup, else let itcontinue withupdate
         --the same when clear FF, but change or not change FS
         If PMO_ORASTD.strncmp(h_finish_date, h_forecast_finish, 10) = 0 Then
            --trace_printf("FF dates didn't change\n");
            h_continue := 0;
         End If;
      End If;
      
      If (finish_date001 IS NULL) AND (h_forecast_finish IS NOT NULL) Then
         --trace_printf("Clear FF date\n");
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET
            FORECAST_FINISH = finish_date001,
            FORECAST_DURATION = 0
            WHERE TSK_ASSIGNMENT_ID = id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 250+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(251) : cplan.sql(21463) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --when clear FS but not chnaged FF then just call rollup, else let itcontinue withupdate
         --the same when clear FF, but change or not change FS
         
         If PMO_ORASTD.strncmp(h_start_date, h_forecast_start, 10) = 0 Then
            --trace_printf("FS didn't change\n");
            h_continue := 0;
         End If;
      End If;
      
      If Coalesce(h_continue, 0) = 1 Then
         --trace_printf("one of the forecast dates chnaged\n");
         -- if client doesn't sent date  time
         If ((h_finish_date IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_finish_DT, 10) = 0) Then
            h_finish_date := h_finish_DT;
         End If;
         
         If ((h_week_of IS NOT NULL)) AND ((h_finish_date IS NOT NULL)) AND (PMO_ORASTD.strcmp(h_week_of
               , h_finish_date) > 0) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 274+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(275) : cplan.sql(21487) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.FORECAST_APPROVAL_EXISTS), 'NULL') || ' ID: ' || '''' || h_res_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 277+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(278) : cplan.sql(21490) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.FORECAST_APPROVAL_EXISTS, h_res_id, True);
         End If;
         
         If (start_date001 IS NOT NULL) Then
            If (h_actual_start IS NOT NULL) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 285+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(286) : cplan.sql(21498) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 288+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(289) : cplan.sql(21501) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL, id, True);
            End If;
            
            --trace_printf("h_ta_start_DT: %d %s, h_start_date: %d %s, h_start_DT: %d %s\n", h_ta_start_DT_Ind,h_ta_start_DT,h_start_date_Ind,h_start_date,h_start_DT_Ind,h_start_DT);
            h_effort := h_etc;
            If PMO_ORASTD.strncmp(h_start_date, h_ta_start_DT, 10) = 0 Then
               h_start_date := NULL;
            ElsIf PMO_ORASTD.strncmp(h_start_date, h_start_DT, 10) <> 0 Then
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.FIRST_AVAIL_TIME(BIT_SCHED))) INTO h_start_date
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_project_id
                  AND DAY_YEAR = TO_DATE(SubStr(start_date001, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 309+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(310) : cplan.sql(21522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  --Should be replace with c function
                  BEGIN
                     SELECT  (PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(start_date001, 1, 10)), PMO_ORASTD.TIME('08:00')))  INTO  h_start_date FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 324+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(325) : cplan.sql(21537) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf (finish_date001 IS NULL) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 331+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(332) : cplan.sql(21544) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 334+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(335) : cplan.sql(21547) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(0, id, True);
            End If;
         Else
            h_effort := 0;
            h_start_date := NULL;
         End If;
         
         If (finish_date001 IS NOT NULL) Then
            --trace_printf("h_start_date: %d %s, h_finish_date: %d %s\n", h_start_date_Ind,h_start_date,h_finish_date_Ind,h_finish_date);
            If ((start_date001 IS NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_ta_start_DT, 10) < 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 347+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(348) : cplan.sql(21560) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 350+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(351) : cplan.sql(21563) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, id, True);
            ElsIf ((h_last_submitted IS NOT NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_last_submitted
                  , 10) <= 0) Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WP_FORECAST', 356+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WP_FORECAST(357) : cplan.sql(21569) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 359+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(360) : cplan.sql(21572) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.CANNOT_UPDATE_HAS_ACTUAL, True);
            End If;
            
            If ((start_date001 IS NULL)) AND (PMO_ORASTD.strncmp(h_finish_date, h_finish_DT, 10) < 0) Then
               h_FF := 1;
            Else
               h_FF := 0;
            End If;
            --trace_printf("h_FF: %d, h_ta_finish_DT: %d %s, h_finish_date: %d %s, h_finish_DT: %d %s\n", h_FF,h_ta_finish_DT_Ind,h_ta_finish_DT,h_finish_date_Ind,h_finish_date,h_finish_DT_Ind,h_finish_DT);
            
            If PMO_ORASTD.strncmp(h_finish_date, h_ta_finish_DT, 10) = 0 Then
               h_finish_date := NULL;
            ElsIf PMO_ORASTD.strcmp(h_finish_date, h_finish_DT) <> 0 Then
               --trace_printf("when finish_date and finish_DT are the same\n");
               BEGIN
                  SELECT PMO_ORASTD.DB2TIMESTAMP(DAY_YEAR, PMO_ORASTD.TIME(PMO_STPU.LAST_AVAIL_TIME(BIT_SCHED))) INTO h_finish_date
                  FROM CALENDAR_DAY_OF_YEAR
                  WHERE ELEMENT_ID = h_project_id
                  AND DAY_YEAR = TO_DATE(SubStr(finish_date001, 1, 10));
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 386+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(387) : cplan.sql(21599) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               --trace_printf("SC_STATE: %d, h_finish_date: %d %s\n", SC_STATE, h_finish_date_Ind, h_finish_date);
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  --Should be replace with c function
                  BEGIN
                     SELECT  (PMO_ORASTD.DB2TIMESTAMP(TO_DATE(SubStr(finish_date001, 1, 10)), PMO_ORASTD.TIME('17:00')))  INTO  h_finish_date FROM DUAL;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 402+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(403) : cplan.sql(21615) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
            ElsIf PMO_ORASTD.strncmp(h_finish_date, h_ta_start_DT, 10) = 0 Then
               --trace_printf("Forecast finish = Sched finish\n");
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 410+1) THEN 
                  LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(411) : cplan.sql(21623) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || id || '''');
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 413+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(414) : cplan.sql(21626) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_ERROR_SP(0, id, True);
            End If;
         Else
            h_finish_date := NULL;
         End If;
         
         h_rec_user := rec_user001;
         h_temp_var := 0;
         
         --trace_printf("**** h_acc_rej_bitflag: %d, h_res_id: %s, h_rec_user: %s\n",h_acc_rej_bitflag,h_res_id, h_rec_user);
         If Coalesce(h_acc_rej_bitflag, 0) <> PMO_CNST.SC_ACCEPT_REJECT_FORECAST Then
            BEGIN
               UPDATE TASK_ASSIGNMENTS SET
               FORECAST_START  = h_start_date,
               FORECAST_FINISH = h_finish_date,
               FORECAST_DURATION = 0
               WHERE TSK_ASSIGNMENT_ID = id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 442+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(443) : cplan.sql(21655) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            --check forecast dates against assignmnet dates if project assignmnet exists before calling accept forecast
            
            If (h_prj_assignment_id IS NOT NULL) Then
               --trace_printf("project assignmnet is not null\n");
               If (finish_date001 IS NOT NULL) Then
                  BEGIN
                     SELECT COUNT('X') INTO h_temp_var
                     FROM DUAL
                     WHERE EXISTS (SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND ((TSK_ASSIGNMENT_ID = h_prj_assignment_id) OR (TSK_ASSIGNMENT_ID = h_tsk_assignment_id))
                     AND DAY_YEAR = TO_DATE(SubStr(finish_date001, 1, 10)));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 464+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(465) : cplan.sql(21677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  If Coalesce(h_temp_var, 0) = 0 Then
                     --trace_printf("FF is outside the assignmnet datse\n");
                     
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 472+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(473) : cplan.sql(21685) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 475+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(476) : cplan.sql(21688) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING, h_tsk_assignment_id
                           , True);
                  End If;
               ElsIf (start_date001 IS NOT NULL) Then
                  --if FS is outside project affected dates return an error
                  BEGIN
                     SELECT COUNT('X') INTO h_temp_var
                     FROM DUAL
                     WHERE EXISTS (SELECT DAY_YEAR FROM TASK_DAY_OF_YEAR
                     WHERE PROJECT_ID = h_project_id AND ((TSK_ASSIGNMENT_ID = h_prj_assignment_id) OR (TSK_ASSIGNMENT_ID = h_tsk_assignment_id))
                     AND DAY_YEAR = TO_DATE(SubStr(start_date001, 1, 10)));
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 494+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(495) : cplan.sql(21707) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If Coalesce(h_temp_var, 0) = 0 Then
                     --trace_printf("FS is outside Project affected dates: h_start_date: %s, h_projaffected_start: %s\n", h_start_date, h_projaffected_start);
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 502+1) THEN 
                        LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(503) : cplan.sql(21715) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                     END IF; 
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 505+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(506) : cplan.sql(21718) : ' || SYSTIMESTAMP);
                     END IF; 
                     PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING, h_tsk_assignment_id
                           , True);
                  Else
                     --here we nedd to fetch the data by cehcking prj_assign_id against parent_id, otherwise
                     --in case of changing the start date one day later when we have only 8 hours remaining at project,and shoud acept the start date
                     --doesn't accept if we go by tsk_assignmnet_id = to h_prj_assignmnet_id
                     
                     BEGIN
                        SELECT SUM(SCHED_MIN) INTO h_total FROM TASK_DAY_OF_YEAR
                        WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_prj_assignment_id AND DAY_YEAR >= TO_DATE(SubStr(start_date001, 1, 10));
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 523+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(524) : cplan.sql(21736) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     If Coalesce(h_total, 0) < Coalesce(h_etc, 0) Then
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 529+1) THEN 
                           LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(530) : cplan.sql(21742) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                        END IF; 
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 532+1) THEN 
                           LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(533) : cplan.sql(21745) : ' || SYSTIMESTAMP);
                        END IF; 
                        PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.PROJECT_ASSIGNMENT_REMAINING, h_tsk_assignment_id
                              , True);
                     End If;
                  End If;
               Else
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WP_FORECAST', 540+1) THEN 
                     LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WP_FORECAST(541) : cplan.sql(21753) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.TASK_ASSIGNMENT_ERROR), 'NULL') || ' ID: ' || '''' || h_tsk_assignment_id || '''');
                  END IF; 
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WP_FORECAST', 543+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_U_WP_FORECAST(544) : cplan.sql(21756) : ' || SYSTIMESTAMP);
                  END IF; 
                  PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.TASK_ASSIGNMENT_ERROR, h_tsk_assignment_id
                        , True);
               End If;
               
            End If;
            -- call accept forcast dates
            h_bitflag := 2;
            h_tsk_assignment_id := id;
            
            If (h_start_date IS NULL) AND ((h_last_submitted IS NOT NULL)) Then
               h_start_date := h_start_DT;
            End If;
            
            h_temp_sc_state := FC_ACPT_ETC_ASSIGN(h_project_id, h_task_id, h_tsk_assignment_id
                                      , h_prj_assignment_id, h_start_date, h_finish_date
                                      , h_last_submitted, h_bit_flag, h_effort, h_remaining
                                      , h_sched_work, h_etc, h_contour, h_FF, h_bitflag
                                      , h_ta_start_DT, h_ta_finish_DT, h_reg_cost, h_reg_sale
                                      , h_billable, h_capital_expense, h_rec_user, 0);
            
            h_temp_var := 2;
            h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_tsk_assignment_id, h_temp_var, 0, 0);
            
            If (h_prj_assignment_id IS NOT NULL) Then
               h_temp_var := 3;
               h_temp_sc_state := U_ASSIGN_STRING(h_project_id, h_prj_assignment_id, h_temp_var, 0, 0);
            End If;
         End If;
      End If;
   End If;
   
   If (PMO_ORASTD.strcmp(h_res_id, h_rec_user) = 0) AND (Coalesce(
         h_acc_rej_bitflag, 0) <> PMO_CNST.SC_ACCEPT_REJECT_FORECAST) Then
      h_alert_id := 'SCETCFORCASTCHANGED';
      h_alert_flag := 2;
      ora_temp_sc_state := PMO_DB2M.SC_STATE;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
      PMO_DB2M.SC_STATE := ora_temp_sc_state;
      
      h_alert_id := 'SCREDUCEDETC';
      h_alert_flag := 2;
      ora_temp_sc_state := PMO_DB2M.SC_STATE;
      null_return := FC_ISSUE_ALERT(h_alert_id, h_tsk_assignment_id, h_rec_user, h_alert_flag, 0);
      PMO_DB2M.SC_STATE := ora_temp_sc_state;
   End If;
   
   h_temp_sc_state := FC_Rollup(h_project_id);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WP_FORECAST', 594+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WP_FORECAST(595) : cplan.sql(21807) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, id, True);
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 602+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(603) : cplan.sql(21815) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WP_FORECAST', 614+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WP_FORECAST(615) : cplan.sql(21827) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WP_FORECAST;
ALTER FUNCTION SP_U_WP_FORECAST COMPILE DEBUG;

/*****************************************************************************/

Prompt == FUNCTION SP_U_ASSIGN_DESCR ==
CREATE OR REPLACE FUNCTION SP_U_ASSIGN_DESCR
(
     prj_id VARCHAR2
   , res_id001 VARCHAR2
   , description001 VARCHAR2
   , rec_user001 VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_ASSIGN_DESCR') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_ASSIGN_DESCR'), '-- ' || SYSTIMESTAMP, ':r := SP_U_ASSIGN_DESCR(' || 
       '''' || prj_id || ''''
          || ', ' || '''' || res_id001 || ''''
          || ', ' || '''' || description001 || ''''
          || ', ' || '''' || rec_user001 || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ASSIGN_DESCR', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_ASSIGN_DESCR(21) : cplan.sql(21864) : ' || SYSTIMESTAMP);
   END IF; 

   If (prj_id IS NULL) OR (res_id001 IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_ASSIGN_DESCR', 25+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_ASSIGN_DESCR(26) : cplan.sql(21869) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL') || ' ID: ' || '''' || prj_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_ASSIGN_DESCR', 28+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_ASSIGN_DESCR(29) : cplan.sql(21872) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_PARAMS, prj_id, True);
   End If;
   
   --Execute update Statement
   BEGIN
      UPDATE PROJECT_ASSIGNMENTS
      SET DESCRIPTION = description001
      WHERE RESOURCE_ID = res_id001
      AND PROJECT_ID = prj_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ASSIGN_DESCR', 49+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ASSIGN_DESCR(50) : cplan.sql(21893) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_ASSIGN_DESCR', 55+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_ASSIGN_DESCR(56) : cplan.sql(21899) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || prj_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, prj_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ASSIGN_DESCR', 62+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ASSIGN_DESCR(63) : cplan.sql(21906) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_ASSIGN_DESCR', 74+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_ASSIGN_DESCR(75) : cplan.sql(21918) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_ASSIGN_DESCR;
ALTER FUNCTION SP_U_ASSIGN_DESCR COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_TEST_ASSIGN ==
CREATE OR REPLACE FUNCTION SP_TEST_ASSIGN
(
     rank001 VARCHAR2
   , bitflag001 PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
   h_full_name VARCHAR2(255);
   h_project_id VARCHAR2(32);
   h_res_assignment_id VARCHAR2(32);
   h_tsk_assignment_id VARCHAR2(32);
   h_security_id VARCHAR2(32);
   h_res_id VARCHAR2(32);
   h_res_type VARCHAR2(1);
   h_participant PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C_RPA0 IS
         SELECT ELEMENT_ID FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND RANK LIKE h_rank || '%';

      CURSOR C_RPA1 IS
         SELECT RES_ASSIGNMENT_ID, TSK_ASSIGNMENT_ID, RESOURCE_ID, RESOURCE_TYPE
         FROM TASK_ASSIGNMENTS
         WHERE PROJECT_ID = h_project_id AND TASK_ID = h_project_id;

      CURSOR C_FPA1 IS
         SELECT ra.RESOURCE_ID, ra.RESOURCE_TYPE, ra.PARTICIPANT, ra.SECURITY_ID
         FROM RESOURCE_ASSIGNMENTS ra
         WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id AND NOT EXISTS (
         SELECT 'X' FROM PROJECT_ASSIGNMENTS pa WHERE pa.PROJECT_ID = h_project_id AND pa.RESOURCE_ID = ra.RESOURCE_ID);

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_TEST_ASSIGN') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_TEST_ASSIGN'), '-- ' || SYSTIMESTAMP, ':r := SP_TEST_ASSIGN(' || 
       '''' || rank001 || ''''
          || ', ' || COALESCE(TO_CHAR(bitflag001), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_TEST_ASSIGN', 41+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_TEST_ASSIGN(42) : cplan.sql(21976) : ' || SYSTIMESTAMP);
   END IF; 

   h_rank := rank001;
   h_bitflag := bitflag001;
   
   OPEN C_RPA0;
   
   LOOP
      FETCH C_RPA0 INTO h_project_id;
      IF C_RPA0%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         EXIT;
      End If;
      --trace_printf("1__ SC_STATE = %d, h_project_id = %s\n", SC_STATE, h_project_id);
      
      If Coalesce(h_bitflag, 0) = 0 Then
         OPEN C_RPA1;
         
         LOOP
            FETCH C_RPA1 INTO h_res_assignment_id, h_tsk_assignment_id, h_res_id, h_res_type;
            IF C_RPA1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            BEGIN
               SELECT COUNT('X') INTO h_bitflag FROM DUAL WHERE EXISTS (SELECT 'X' FROM TASK_ASSIGNMENTS
               WHERE PROJECT_ASSIGNMENT_ID = h_tsk_assignment_id AND RESOURCE_ID = h_res_id);
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 84+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(85) : cplan.sql(22019) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_bitflag, 0) <> 0 Then
               BEGIN
                  UPDATE TASK_ASSIGNMENTS SET PROJECT_ASSIGNMENT_ID = NuLL
                  WHERE PROJECT_ASSIGNMENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 103+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(104) : cplan.sql(22038) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               BEGIN
                  UPDATE TASK_DAY_OF_YEAR SET PARENT_ID = TSK_ASSIGNMENT_ID
                  WHERE PROJECT_ID = h_project_id AND PARENT_ID = h_tsk_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 121+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(122) : cplan.sql(22056) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               DELETE FROM RESOURCE_ASSIGNMENTS
               WHERE RES_ASSIGNMENT_ID = h_res_assignment_id
               AND SECURITY_ID = 'SCPROJECTASSIGNMENT';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 141+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(142) : cplan.sql(22076) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               BEGIN
                  DELETE FROM PROJECT_ASSIGNMENTS WHERE PROJECT_ID = h_project_id AND RESOURCE_ID = h_res_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 159+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(160) : cplan.sql(22094) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If SubStr(h_res_type, 1, 1) = '0' Then
                  BEGIN
                     SELECT NICK_NAME || ';' INTO h_full_name
                     FROM RESOURCES
                     WHERE RESOURCE_ID = h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 175+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(176) : cplan.sql(22110) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               Else
                  BEGIN
                     SELECT PROFILE_NAME || ';' INTO h_full_name
                     FROM PROFILES
                     WHERE PROFILE_ID = h_res_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 190+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(191) : cplan.sql(22125) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
               End If;
               
               BEGIN
                  UPDATE TMT_WBS SET ASSIGN_NAMES = REPLACE (ASSIGN_NAMES, h_full_name, '')
                  WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 209+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(210) : cplan.sql(22144) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE RESOURCE_ASSIGNMENTS SET PARTICIPANT = PMO_STPU.BIT_ANDNOT(PARTICIPANT,4096)
                  WHERE RES_ASSIGNMENT_ID = h_res_assignment_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 227+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(228) : cplan.sql(22162) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               DELETE FROM TASK_ASSIGNMENTS WHERE TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 245+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(246) : cplan.sql(22180) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               DELETE FROM TASK_DAY_OF_YEAR WHERE PROJECT_ID = h_project_id AND TSK_ASSIGNMENT_ID = h_tsk_assignment_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 262+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(263) : cplan.sql(22197) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         END LOOP;
         
         BEGIN
            CLOSE C_RPA1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 276+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(277) : cplan.sql(22211) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         COMMIT;
      ElsIf Coalesce(h_bitflag, 0) = 1 Then
         OPEN C_FPA1;
         
         LOOP
            FETCH C_FPA1 INTO h_res_id, h_res_type, h_participant, h_security_id;
            IF C_FPA1%FOUND THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               EXIT;
            End If;
            
            If SubStr(h_res_type, 1, 1) = '0' Then
               BEGIN
                  SELECT NICK_NAME || ';' INTO h_full_name
                  FROM RESOURCES
                  WHERE RESOURCE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 307+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(308) : cplan.sql(22242) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  SELECT PROFILE_NAME || ';' INTO h_full_name
                  FROM PROFILES
                  WHERE PROFILE_ID = h_res_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 322+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(323) : cplan.sql(22257) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            BEGIN
               UPDATE TMT_WBS SET ASSIGN_NAMES = CASE
               WHEN InStr(ASSIGN_NAMES, h_full_name) > 0 THEN ASSIGN_NAMES
               ELSE ASSIGN_NAMES || h_full_name END
               WHERE PROJECT_ID = h_project_id AND ELEMENT_ID = h_project_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 343+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(344) : cplan.sql(22278) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               INSERT INTO PROJECT_ASSIGNMENTS (PROJECT_ID, RESOURCE_ID, PARTICIPANT, SECURITY_ID, RESOURCE_TYPE, ELEMENT_TYPE_BITS,ROLLUP_BITS)
               VALUES(h_project_id, h_res_id, h_participant, h_security_id, h_res_type, 1, PMO_CNST.SC_ASSIGNMENT_BIT);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 361+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(362) : cplan.sql(22296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            --trace_printf("INSERT_* SC_STATE: %d, h_participant: %d, h_security_id: %s\n",SC_STATE,h_participant,h_security_id);
         END LOOP;
         
         BEGIN
            CLOSE C_FPA1 ;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 376+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(377) : cplan.sql(22311) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         COMMIT;
      End If;
   END LOOP;
   
   BEGIN
      CLOSE C_RPA0 ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 393+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(394) : cplan.sql(22328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_TEST_ASSIGN', 399+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_TEST_ASSIGN(400) : cplan.sql(22334) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_rank || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_rank, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 406+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(407) : cplan.sql(22341) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_TEST_ASSIGN', 418+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_TEST_ASSIGN(419) : cplan.sql(22353) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_TEST_ASSIGN;
ALTER FUNCTION SP_TEST_ASSIGN COMPILE DEBUG;

/*********************************************************************************************************************************************/

Prompt == FUNCTION FC_MIGRATION ==
CREATE OR REPLACE FUNCTION FC_MIGRATION
(
     h_project_id IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_level PLS_INTEGER := 0;
   h_rollup_bits PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
-- Declare Cursors
      CURSOR C01 IS
         SELECT PROJECT_ID, LEVEL_ID 
         FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         ORDER BY RANK DESC;

      CURSOR C02 IS
         SELECT PROJECT_ID, LEVEL_ID 
         FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3)
         ORDER BY RANK DESC;

      CURSOR C03 IS
         SELECT LEVEL_ID 
         FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) 
         AND ELEMENT_ID = h_project_id
         ORDER BY RANK DESC;

      CURSOR C04 IS
         SELECT /*PROJECT_ID,*/ LEVEL_ID
         FROM TMT_WBS
         WHERE TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND ELEMENT_ID = h_project_id
         ORDER BY RANK DESC;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_MIGRATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_MIGRATION'), '-- ' || SYSTIMESTAMP, ':r := FC_MIGRATION(' || 
       '''' || h_project_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MIGRATION', 41+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_MIGRATION(42) : cplan.sql(22411) : ' || SYSTIMESTAMP);
   END IF; 

   If (h_project_id IS NULL) Then
      -- Step 1: fix plane and schedule only
      
      OPEN C01;
      
      LOOP
         FETCH C01 INTO h_project_id, h_level;
         IF C01%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         h_rollup_bits := 3269638;
         -- all bits except: assign, sched and planned
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = CASE WHEN ACTUAL_START IS NuLL THEN PMO_CNST.SC_SET_ALL_BITS ELSE h_rollup_bits END
            WHERE PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 76+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(77) : cplan.sql(22446) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_rollup_bits := PMO_FUNC.BIT_OR(PMO_CNST.SC_PLANNED_BIT, PMO_CNST.SC_SCHED_BIT);
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL,
            ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND (PLANNED_EFFORT + SCHED_EFFORT > 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 98+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(99) : cplan.sql(22468) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = h_rollup_bits, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 116+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(117) : cplan.sql(22486) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_CRI SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND TYPE_ID <> 14;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 134+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(135) : cplan.sql(22504) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C01 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 154+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(155) : cplan.sql(22524) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Step 2: fix all for duration based
      
      OPEN C02;
      
      LOOP
         FETCH C02 INTO h_project_id, h_level;
         IF C02%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 189+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(190) : cplan.sql(22559) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C02 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 209+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(210) : cplan.sql(22579) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      -- Step 1: fix plane and schedule only
      
      OPEN C03;
      
      LOOP
         FETCH C03 INTO h_level;
         IF C03%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         h_rollup_bits := PMO_FUNC.BIT_OR(PMO_CNST.SC_PLANNED_BIT, PMO_CNST.SC_SCHED_BIT);
         
         BEGIN
            UPDATE TASK_ASSIGNMENTS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS WHERE PROJECT_ID = h_project_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 244+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(245) : cplan.sql(22614) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL,
            ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND (PLANNED_EFFORT + SCHED_EFFORT > 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 264+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(265) : cplan.sql(22634) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = h_rollup_bits, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 282+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(283) : cplan.sql(22652) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_CRI SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, PLANNED_FINISH = NuLL, SCHED_FINISH = NuLL
            WHERE PROJECT_ID = h_project_id AND TYPE_ID <> 14;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 300+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(301) : cplan.sql(22670) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C03 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 320+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(321) : cplan.sql(22690) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      -- Step 2: fix all for duration based
      
      OPEN C04;
      
      LOOP
         FETCH C04 INTO h_level;
         IF C04%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         BEGIN
            UPDATE TMT_WBS SET ROLLUP_BITS = PMO_CNST.SC_SET_ALL_BITS, ACTUAL_FINISH = NuLL,
            FORECAST_FINISH = CASE WHEN FORECAST_START IS NuLL THEN FORECAST_FINISH ELSE NuLL END
            WHERE PROJECT_ID = h_project_id AND ((PLANNED_EFFORT + SCHED_EFFORT) = 0);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 355+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(356) : cplan.sql(22725) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level, 0) = 0 Then
            h_temp_sc_state := FC_Rollup(h_project_id);
            
            COMMIT;
         End If;
      END LOOP;
      
      BEGIN
         CLOSE C04 ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_MIGRATION', 375+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_MIGRATION(376) : cplan.sql(22745) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_MIGRATION', 382+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_MIGRATION(383) : cplan.sql(22752) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_MIGRATION;
ALTER FUNCTION FC_MIGRATION COMPILE DEBUG;

