-- already in pmo_sqlprep.h : #include "pmodefines.h"

/*****************************************************************************************
    Create a new row (process, step, or outcome) in tmt_workflow_cycles.
******************************************************************************************/

Prompt == FUNCTION SP_I_WKFCYCLES ==
CREATE OR REPLACE FUNCTION SP_I_WKFCYCLES
(
     p_wkf_name VARCHAR2
   , p_parent_id VARCHAR2
   , p_wkf_type PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_uniqueID VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_max_rank PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_start_with PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_parent_id VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_I_WKFCYCLES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_I_WKFCYCLES'), '-- ' || SYSTIMESTAMP, ':r := SP_I_WKFCYCLES(' || 
       '''' || p_wkf_name || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_wkf_type), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_WKFCYCLES', 27+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_I_WKFCYCLES(28) : workflow.sql(35) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (p_parent_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_WKFCYCLES', 34+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_WKFCYCLES(35) : workflow.sql(42) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_WKFCYCLES', 37+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_WKFCYCLES(38) : workflow.sql(45) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   h_parent_id := p_parent_id;
   
   BEGIN
      SELECT PARENT_ID INTO h_uniqueID
      FROM TMT_WORKFLOW_CYCLES
      WHERE ELEMENT_ID = h_parent_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_WKFCYCLES', 54+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_WKFCYCLES(55) : workflow.sql(62) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_I_WKFCYCLES', 61+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_I_WKFCYCLES(62) : workflow.sql(69) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_I_WKFCYCLES', 64+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_I_WKFCYCLES(65) : workflow.sql(72) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   Else
      PMODEFINES.PMO_GET_MAX_RANK('TMT_WORKFLOW_CYCLES', h_parent_id, h_max_rank);
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         SELECT RTRIM(RANK) || h_rankBase64, LEVEL_ID+1 INTO h_rank, h_level_id
         FROM TMT_WORKFLOW_CYCLES
         WHERE ELEMENT_ID = p_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_WKFCYCLES', 81+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_WKFCYCLES(82) : workflow.sql(89) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If (h_uniqueID IS NOT NULL) Then
      h_start_with := Coalesce(h_max_rank, 0);
   Else
      h_start_with := 0;
   End If;
   
   h_uniqueID := '';
   h_uniqueID := SYS_GUID();
   
   BEGIN
      INSERT INTO TMT_WORKFLOW_CYCLES
      (ELEMENT_ID, ELEMENT_NAME, PARENT_ID, START_WITH, TYPE_ID, LEVEL_ID, RANK, REC_USER)
      VALUES
      (h_uniqueID, p_wkf_name, p_parent_id,
      h_start_with, p_wkf_type, h_level_id, h_rank,
      p_rec_user);
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_WKFCYCLES', 113+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_WKFCYCLES(114) : workflow.sql(121) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_I_WKFCYCLES', 119+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_I_WKFCYCLES(120) : workflow.sql(127) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_WKFCYCLES', 126+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_WKFCYCLES(127) : workflow.sql(134) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_I_WKFCYCLES', 138+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_I_WKFCYCLES(139) : workflow.sql(146) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_I_WKFCYCLES;
ALTER FUNCTION SP_I_WKFCYCLES COMPILE DEBUG;

/*****************************************************************************************
    Update element_name of a row (process, step, or outcome) in tmt_workflow_cycles.
******************************************************************************************/

Prompt == FUNCTION SP_U_WORKFLOW_DEF ==
CREATE OR REPLACE FUNCTION SP_U_WORKFLOW_DEF
(
     p_id VARCHAR2
   , p_wkf_name VARCHAR2
   , p_type PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WORKFLOW_DEF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WORKFLOW_DEF'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WORKFLOW_DEF(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_wkf_name || ''''
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WORKFLOW_DEF', 20+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WORKFLOW_DEF(21) : workflow.sql(185) : ' || SYSTIMESTAMP);
   END IF; 

   --Execute update Statement
   BEGIN
      UPDATE TMT_WORKFLOW_CYCLES
      SET ELEMENT_NAME = p_wkf_name,
      TYPE_ID = p_type,
      REC_USER = p_rec_user,
      REC_STATUS='U'
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WORKFLOW_DEF', 41+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WORKFLOW_DEF(42) : workflow.sql(206) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WORKFLOW_DEF', 47+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WORKFLOW_DEF(48) : workflow.sql(212) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WORKFLOW_DEF', 54+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WORKFLOW_DEF(55) : workflow.sql(219) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WORKFLOW_DEF', 66+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WORKFLOW_DEF(67) : workflow.sql(231) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WORKFLOW_DEF;
ALTER FUNCTION SP_U_WORKFLOW_DEF COMPILE DEBUG;

/*****************************************************************************************
    Delete a row and its children from tmt_workflow_cycles.
******************************************************************************************/

Prompt == FUNCTION SP_D_WKFCYCLES ==
CREATE OR REPLACE FUNCTION SP_D_WKFCYCLES
(
     p_wkf_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rec_status VARCHAR2(1);
   h_rank VARCHAR2(200);
   h_level_id PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_parent_id VARCHAR2(32);
   h_target_rank VARCHAR2(200);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_WKFCYCLES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_WKFCYCLES'), '-- ' || SYSTIMESTAMP, ':r := SP_D_WKFCYCLES(' || 
       '''' || p_wkf_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_WKFCYCLES', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_WKFCYCLES(27) : workflow.sql(276) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_parent_id := p_parent_id;
   
   If (p_wkf_id IS NULL) OR (p_parent_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_WKFCYCLES', 35+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_WKFCYCLES(36) : workflow.sql(285) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_WKFCYCLES', 38+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_WKFCYCLES(39) : workflow.sql(288) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   BEGIN
      SELECT LEVEL_ID, REC_STATUS, RANK INTO h_level_id, h_rec_status, h_rank
      FROM TMT_WORKFLOW_CYCLES
      WHERE ELEMENT_ID = p_wkf_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(54) : workflow.sql(303) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_WKFCYCLES', 60+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_WKFCYCLES(61) : workflow.sql(310) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_wkf_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_WKFCYCLES', 63+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_WKFCYCLES(64) : workflow.sql(313) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_wkf_id, True);
   Else
      If Coalesce(h_level_id, 0) = 0 Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_D_WKFCYCLES', 69+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_D_WKFCYCLES(70) : workflow.sql(319) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.CANNOT_DELETE_RECORD), 'NULL') || ' ID: ' || '''' || p_wkf_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_WKFCYCLES', 72+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_WKFCYCLES(73) : workflow.sql(322) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.CANNOT_DELETE_RECORD, p_wkf_id, True);
      ElsIf SubStr(h_rec_status, 1, 1) = 'D' AND (SubStr(p_del_flag, 1, 1) = 'D') Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_WKFCYCLES', 77+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_WKFCYCLES(78) : workflow.sql(327) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_ALREADY_DELETED), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_WKFCYCLES', 80+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_D_WKFCYCLES(81) : workflow.sql(330) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.RECORD_ALREADY_DELETED, True);
      End If;
      
      h_rec_status := p_del_flag;
      -- allways do logical delete
      h_rec_status := 'D';
      
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      If SubStr(h_rec_status, 1, 1) = 'X' Then
         BEGIN
            DELETE FROM TMT_WORKFLOW_CYCLES WHERE RANK LIKE h_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 115+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(116) : workflow.sql(365) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level_id, 0) = 2 Then
            BEGIN
               UPDATE TMT_WORKFLOW_CYCLES
               SET NEXT_PROCESS_ID = Null
               WHERE NEXT_PROCESS_ID = p_wkf_id
               AND RANK LIKE SUBSTR(h_rank, 1, LENGTH(h_rank)-4) || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 136+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(137) : workflow.sql(386) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         BEGIN
            UPDATE TMT_WORKFLOW_CYCLES SET CHILD_COUNT  = CHILD_COUNT - 1
            WHERE ELEMENT_ID = p_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 154+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(155) : workflow.sql(404) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_WORKFLOW_CYCLES
            SET REC_STATUS = h_rec_status,
            REC_DATETIME = SYSTIMESTAMP,
            REC_USER = p_rec_user
            WHERE RANK LIKE h_rank || '%'  AND REC_STATUS <> 'D';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 175+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(176) : workflow.sql(425) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WORKFLOW_CYCLES
            SET CHILD_COUNT = CHILD_COUNT - 1,
            DELETED_COUNT = DELETED_COUNT + 1
            WHERE ELEMENT_ID = p_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 195+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(196) : workflow.sql(445) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_level_id, 0) = 2 Then
            BEGIN
               UPDATE TMT_WORKFLOW_CYCLES
               SET NEXT_PROCESS_ID = Null
               WHERE NEXT_PROCESS_ID = p_wkf_id
               AND RANK LIKE SUBSTR(h_rank, 1, LENGTH(h_rank)-4) || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 216+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(217) : workflow.sql(466) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_WKFCYCLES', 229+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_WKFCYCLES(230) : workflow.sql(479) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_wkf_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_wkf_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 236+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(237) : workflow.sql(486) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WKFCYCLES', 248+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WKFCYCLES(249) : workflow.sql(498) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_WKFCYCLES;
ALTER FUNCTION SP_D_WKFCYCLES COMPILE DEBUG;

/*****************************************************************************************
    Update process, step, or outcome in tmt_workflow_cycles.
******************************************************************************************/

Prompt == FUNCTION SP_U_WKFCYCLES ==
CREATE OR REPLACE FUNCTION SP_U_WKFCYCLES
(
     p_id VARCHAR2
   , p_request_id PLS_INTEGER
   , p_outcome_id PLS_INTEGER
   , p_next_state_id VARCHAR2
   , p_next_stage_id PLS_INTEGER
   , p_must_obey PLS_INTEGER
   , p_participants VARCHAR2
   , p_checklist VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WKFCYCLES') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WKFCYCLES'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WKFCYCLES(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_request_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_outcome_id), 'NULL')
          || ', ' || '''' || p_next_state_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_next_stage_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_must_obey), 'NULL')
          || ', ' || '''' || p_participants || ''''
          || ', ' || '''' || p_checklist || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKFCYCLES', 30+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WKFCYCLES(31) : workflow.sql(547) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      UPDATE TMT_WORKFLOW_CYCLES
      SET REQUEST_ID = p_request_id,
      OUTCOME_ID = p_outcome_id,
      NEXT_PROCESS_ID = p_next_state_id,
      CHECKLIST = p_checklist,
      PARTICIPANTS = p_participants,
      NEXT_STAGE_ID = p_next_stage_id,
      MUST_OBEY = p_must_obey
      WHERE ELEMENT_ID = p_id;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKFCYCLES', 53+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKFCYCLES(54) : workflow.sql(570) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WKFCYCLES', 59+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WKFCYCLES(60) : workflow.sql(576) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKFCYCLES', 66+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKFCYCLES(67) : workflow.sql(583) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKFCYCLES', 78+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKFCYCLES(79) : workflow.sql(595) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WKFCYCLES;
ALTER FUNCTION SP_U_WKFCYCLES COMPILE DEBUG;

/*****************************************************************************************
    Create new workflow, notification, or custom workflow.
******************************************************************************************/

Prompt == FUNCTION FC_A_NOTIFICATION ==
CREATE OR REPLACE FUNCTION FC_A_NOTIFICATION
(
     sh_h_table_type PLS_INTEGER
   , h_in_parent_id IN OUT NOCOPY VARCHAR2
   , h_in_wkf_def_id IN OUT NOCOPY VARCHAR2
   , sh_h_in_type PLS_INTEGER
   , h_in_rec_user IN OUT NOCOPY VARCHAR2
   , h_uniqueID IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_in_type PLS_INTEGER := sh_h_in_type;
   h_attached_name VARCHAR2(255);
   h_project_name VARCHAR2(255);
   h_rank VARCHAR2(200);
   h_max_rank PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_parent_type PLS_INTEGER := 0;
   h_attached_type PLS_INTEGER := 0;
   h_rankBase64 VARCHAR2(8);
   h_parent_id VARCHAR2(32);
   h_name VARCHAR2(100);
   h_email VARCHAR2(255);
   h_rank2 VARCHAR2(200);
   h_wkf_def_id VARCHAR2(32);
   h_wkf_prcess_id VARCHAR2(32);
   h_process_level_id PLS_INTEGER := 0;
   newNotification PLS_INTEGER NOT NULL := 1;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_A_NOTIFICATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_A_NOTIFICATION'), '-- ' || SYSTIMESTAMP, ':r := FC_A_NOTIFICATION(' || 
       COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || '''' || h_in_parent_id || ''''
          || ', ' || '''' || h_in_wkf_def_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_in_type), 'NULL')
          || ', ' || '''' || h_in_rec_user || ''''
          || ', ' || '''' || h_uniqueID || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_NOTIFICATION', 41+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_A_NOTIFICATION(42) : workflow.sql(655) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_parent_id := h_in_parent_id;
   
   BEGIN
      SELECT PARENT_ID, LEVEL_ID  INTO h_wkf_prcess_id, h_process_level_id
      FROM TMT_WORKFLOW_CYCLES
      WHERE ELEMENT_ID = h_in_wkf_def_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 58+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(59) : workflow.sql(672) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_process_level_id, 0) > 1 Then
      h_wkf_def_id := h_in_wkf_def_id;
      h_in_wkf_def_id := h_wkf_prcess_id;
   Else
      h_wkf_def_id := NULL;
   End If;
   --
   
   -- get workflow defination (root level in tmt_workflow_cycles) name and type
   BEGIN
      SELECT P.ELEMENT_NAME, P.RANK, P.TYPE_ID INTO h_name, h_rank, h_type
      FROM TMT_WORKFLOW_CYCLES P, TMT_WORKFLOW_CYCLES C
      WHERE C.ELEMENT_ID = h_in_wkf_def_id AND P.ELEMENT_ID = C.PARENT_ID;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 82+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(83) : workflow.sql(696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_parent_type := Coalesce(h_table_type, 0);
   If Coalesce(h_type, 0) = PMO_CNST.SC_NOTIFICATION Then
      If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
         h_type := PMO_CNST.SC_WBS_NOTIFICATION;
      ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CRI Then
         h_type := PMO_CNST.SC_CRI_NOTIFICATION;
      ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_DOC Then
         h_type := PMO_CNST.SC_DOC_NOTIFICATION;
      ElsIf (Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_DOC) OR (Coalesce(
                h_table_type, 0) = PMO_CNST.SC_TABLE_POL_DOC) OR (Coalesce(
                h_table_type, 0) = PMO_CNST.SC_TABLE_RES_DOC) OR (Coalesce(
                h_table_type, 0) = PMO_CNST.SC_TABLE_AST_DOC) OR (Coalesce(
                h_table_type, 0) = PMO_CNST.SC_TABLE_CLN_DOC) OR (Coalesce(
                h_table_type, 0) = PMO_CNST.SC_TABLE_VDR_DOC) Then
         h_type := PMO_CNST.SC_DOC_NOTIFICATION;
      End If;
      
      h_attached_type := h_in_type;
   End If;
   
   If Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_WBS Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, PRJ.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_WBS C, TMT_WBS PRJ
         WHERE C.ELEMENT_ID = h_parent_id AND PRJ.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 119+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(120) : workflow.sql(733) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_PORTFOLIO Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, C.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_PORTFOLIOS C
         WHERE C.ELEMENT_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 135+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(136) : workflow.sql(749) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_attached_type := PMO_CNST.SC_PORTFOLIO;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_CRI Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, PRJ.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_CRI C, TMT_WBS PRJ
         WHERE C.ELEMENT_ID = h_parent_id AND PRJ.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 153+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(154) : workflow.sql(767) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_DOC Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, PRJ.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_DOCUMENTS C, TMT_WBS PRJ
         WHERE C.ELEMENT_ID = h_parent_id AND PRJ.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 169+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(170) : workflow.sql(783) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_attached_type, 0) = PMO_CNST.SC_FOLDER Then
         h_attached_type := PMO_CNST.SC_HIDDEN_DOC_FOLDER;
      End If;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_SCD Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, PRJ.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_ASSIGNED_SCORECARDS C, TMT_WBS PRJ
         WHERE C.ELEMENT_ID = h_parent_id AND PRJ.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 189+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(190) : workflow.sql(803) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_RES_DOC Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, P.FULL_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_RESOURCE_DOCUMENTS C ,RESOURCES P
         WHERE C.ELEMENT_ID = h_parent_id AND
         P.RESOURCE_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 206+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(207) : workflow.sql(820) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --trace_printf("\n h_attached_type:%d", h_attached_type);
      --trace_printf("\n SC_FOLDER:%d", SC_FOLDER);
      
      If Coalesce(h_attached_type, 0) = PMO_CNST.SC_FOLDER Then
         h_attached_type := PMO_CNST.SC_HIDDEN_DOC_FOLDER;
      End If;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_POL_DOC Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, P.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_POOL_DOCUMENTS C ,TMT_POOLS P
         WHERE C.ELEMENT_ID = h_parent_id AND
         P.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 230+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(231) : workflow.sql(844) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_attached_type, 0) = PMO_CNST.SC_FOLDER Then
         h_attached_type := PMO_CNST.SC_HIDDEN_DOC_FOLDER;
      End If;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_AST_DOC Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, P.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_ASSET_DOCUMENTS C ,TMT_ASSETS P
         WHERE C.ELEMENT_ID = h_parent_id
         AND P.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 251+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(252) : workflow.sql(865) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_attached_type, 0) = PMO_CNST.SC_FOLDER Then
         h_attached_type := PMO_CNST.SC_HIDDEN_DOC_FOLDER;
      End If;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_CLN_DOC Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, P.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_CLIENT_DOCUMENTS C,TMT_CLIENTS P
         WHERE C.ELEMENT_ID = h_parent_id
         AND P.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 272+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(273) : workflow.sql(886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_attached_type, 0) = PMO_CNST.SC_FOLDER Then
         h_attached_type := PMO_CNST.SC_HIDDEN_DOC_FOLDER;
      End If;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_VDR_DOC Then
      BEGIN
         SELECT C.ELEMENT_NAME, C.TYPE_ID, P.ELEMENT_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM TMT_VENDOR_DOCUMENTS C ,TMT_VENDORS P
         WHERE C.ELEMENT_ID = h_parent_id
         AND P.ELEMENT_ID = C.PROJECT_ID;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 293+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(294) : workflow.sql(907) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_attached_type, 0) = PMO_CNST.SC_FOLDER Then
         h_attached_type := PMO_CNST.SC_HIDDEN_DOC_FOLDER;
      End If;
   ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_RES Then
      BEGIN
         SELECT FULL_NAME, PMO_CNST.SC_RESOURCE, FULL_NAME
         INTO h_attached_name, h_attached_type, h_project_name
         FROM RESOURCES
         WHERE RESOURCE_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 313+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(314) : workflow.sql(927) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      h_attached_name := NULL;
      h_project_name := NULL;
   End If;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         SELECT LEVEL_ID
         INTO h_level_id
         FROM TMT_WORKFLOWS
         WHERE ELEMENT_ID = h_in_parent_id AND TYPE_ID = h_type;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 334+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(335) : workflow.sql(948) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
         PMODEFINES.PMO_GET_ROOT_MAX_RANK('TMT_WORKFLOWS', h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         BEGIN
            INSERT INTO TMT_WORKFLOWS
            (ELEMENT_ID, ELEMENT_NAME, CREATED_DATE, AUTHOR_ID, LEVEL_ID, TYPE_ID, REC_USER, RANK, PARENT_TYPE)
            SELECT h_parent_id, h_name, SYSTIMESTAMP, h_in_rec_user, LEVEL_ID + 1, h_type,
            h_in_rec_user, RTRIM(RANK) || h_rankBase64, h_attached_type
            FROM TMT_WORKFLOWS WHERE ELEMENT_ID IS NuLL;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 359+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(360) : workflow.sql(973) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_A_NOTIFICATION', 366+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_A_NOTIFICATION(367) : workflow.sql(980) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_in_parent_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_NOTIFICATION', 369+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_A_NOTIFICATION(370) : workflow.sql(983) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_DB2M.SC_STATE, h_in_parent_id, False);
         End If;
      End If;
      
      BEGIN
         SELECT MAX_RANK + 1 INTO h_max_rank
         FROM TMT_WORKFLOWS
         WHERE TYPE_ID = h_type AND ELEMENT_ID = h_parent_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 385+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(386) : workflow.sql(999) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         UPDATE TMT_WORKFLOWS SET MAX_RANK = MAX_RANK + 1, CHILD_COUNT = CHILD_COUNT + 1
         WHERE TYPE_ID = h_type AND ELEMENT_ID = h_parent_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 403+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(404) : workflow.sql(1017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      COMMIT;
      
      --Get Base 64
      PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
      
      BEGIN
         SELECT RTRIM(RANK) || h_rankBase64, LEVEL_ID+1 INTO h_rank, h_level_id
         FROM TMT_WORKFLOWS
         WHERE ELEMENT_ID = h_parent_id AND TYPE_ID = h_type;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 423+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(424) : workflow.sql(1037) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         BEGIN
            SELECT ELEMENT_NAME, TYPE_ID INTO h_name, h_type
            FROM TMT_WORKFLOW_CYCLES
            WHERE ELEMENT_ID = h_in_wkf_def_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 439+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(440) : workflow.sql(1053) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_A_NOTIFICATION', 446+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_A_NOTIFICATION(447) : workflow.sql(1060) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_in_wkf_def_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_NOTIFICATION', 449+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: FC_A_NOTIFICATION(450) : workflow.sql(1063) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_in_wkf_def_id, False);
         End If;
         
         BEGIN
            SELECT E_MAIL_ADDRESS INTO h_email
            FROM RESOURCES
            WHERE RESOURCE_ID = h_in_rec_user;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 464+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(465) : workflow.sql(1078) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If (h_wkf_def_id IS NULL) Then
            BEGIN
               SELECT ELEMENT_ID INTO h_wkf_def_id
               FROM TMT_WORKFLOW_CYCLES
               WHERE RANK = (SELECT MIN(RANK) FROM TMT_WORKFLOW_CYCLES
               WHERE PARENT_ID = h_in_wkf_def_id AND REC_STATUS <> 'D');
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 481+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(482) : workflow.sql(1095) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               h_wkf_def_id := h_in_wkf_def_id;
            End If;
         End If;
         --Get a unique SC_H_ID byte id
         h_uniqueID := SYS_GUID();
         
         BEGIN
            INSERT INTO TMT_WORKFLOWS
            (ELEMENT_ID, WKF_DEF_ID, ELEMENT_NAME, PARENT_ID, PARENT_TYPE,
            EMAIL_ADDRESS, CREATED_DATE, AUTHOR_ID, TYPE_ID, LEVEL_ID, REC_USER, RANK,
            PROJECT_NAME, ATTACHED_NAME)
            VALUES
            (h_uniqueID, h_wkf_def_id, h_name, h_parent_id, h_parent_type,
            h_email, SYSTIMESTAMP, h_in_rec_user, h_type, h_level_id, h_in_rec_user, h_rank,
            h_project_name, h_attached_name );
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_A_NOTIFICATION', 512+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_A_NOTIFICATION(513) : workflow.sql(1126) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_A_NOTIFICATION', 519+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_A_NOTIFICATION(520) : workflow.sql(1133) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_A_NOTIFICATION;
ALTER FUNCTION FC_A_NOTIFICATION COMPILE DEBUG;

/*****************************************************************************************
    Create a new notification
******************************************************************************************/

Prompt == FUNCTION SP_A_NOTIFICATION ==
CREATE OR REPLACE FUNCTION SP_A_NOTIFICATION
(
     p_table_type PLS_INTEGER
   , p_parent_id VARCHAR2
   , p_wkf_def_id VARCHAR2
   , p_type PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_wkf_def_id VARCHAR2(32);
   h_type PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_table_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_NOTIFICATION') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_NOTIFICATION'), '-- ' || SYSTIMESTAMP, ':r := SP_A_NOTIFICATION(' || 
       COALESCE(TO_CHAR(p_table_type), 'NULL')
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_wkf_def_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_NOTIFICATION', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_NOTIFICATION(29) : workflow.sql(1174) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_wkf_def_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_NOTIFICATION', 33+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_NOTIFICATION(34) : workflow.sql(1179) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_NOTIFICATION', 36+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_NOTIFICATION(37) : workflow.sql(1182) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   Else
      If (p_parent_id IS NULL) Then
         h_parent_id := 'CUSTOMWORKFLOWS';
         h_table_type := 0;
      Else
         h_parent_id := p_parent_id;
         h_table_type := p_table_type;
      End If;
      -- Copy input parameters in host variables
      h_wkf_def_id := p_wkf_def_id;
      h_type := p_type;
      h_rec_user := p_rec_user;
      
      PMO_DB2M.SC_STATE := FC_A_NOTIFICATION(h_table_type, h_parent_id, h_wkf_def_id, h_type, h_rec_user
                                       , h_uniqueID);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_NOTIFICATION', 57+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_NOTIFICATION(58) : workflow.sql(1203) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_NOTIFICATION', 64+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_NOTIFICATION(65) : workflow.sql(1210) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_NOTIFICATION', 76+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_NOTIFICATION(77) : workflow.sql(1222) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_NOTIFICATION;
ALTER FUNCTION SP_A_NOTIFICATION COMPILE DEBUG;

/*****************************************************************************************
    Create a new workflow or custom workflow.
******************************************************************************************/

Prompt == FUNCTION SP_A_WORKFLOW ==
CREATE OR REPLACE FUNCTION SP_A_WORKFLOW
(
     p_table_type PLS_INTEGER
   , p_parent_id VARCHAR2
   , p_wkf_def_id VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_parent_id VARCHAR2(32);
   h_wkf_def_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_type PLS_INTEGER := 0;
   h_table_type PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_A_WORKFLOW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_A_WORKFLOW'), '-- ' || SYSTIMESTAMP, ':r := SP_A_WORKFLOW(' || 
       COALESCE(TO_CHAR(p_table_type), 'NULL')
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_wkf_def_id || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_WORKFLOW', 26+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_A_WORKFLOW(27) : workflow.sql(1267) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_wkf_def_id IS NULL) OR (p_parent_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_A_WORKFLOW', 31+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_A_WORKFLOW(32) : workflow.sql(1272) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_A_WORKFLOW', 34+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_A_WORKFLOW(35) : workflow.sql(1275) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   Else
      -- Copy input parameters in host variables
      h_parent_id := p_parent_id;
      h_wkf_def_id := p_wkf_def_id;
      h_rec_user := p_rec_user;
      h_table_type := p_table_type;
      
      PMO_DB2M.SC_STATE := FC_A_NOTIFICATION(h_table_type, h_parent_id, h_wkf_def_id, h_type, h_rec_user
                                       , h_uniqueID);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_A_WORKFLOW', 49+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_A_WORKFLOW(50) : workflow.sql(1290) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_uniqueID || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_uniqueID, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_WORKFLOW', 56+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_WORKFLOW(57) : workflow.sql(1297) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_A_WORKFLOW', 68+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_A_WORKFLOW(69) : workflow.sql(1309) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_A_WORKFLOW;
ALTER FUNCTION SP_A_WORKFLOW COMPILE DEBUG;

/*****************************************************************************************
    Continue an existing workflow or custom workflow.
    p_id is id of current workflow
    p_next_state_id is id of outcome
    and if we close the cycle p_next_state_id will be null and  p_end_cycle will be '1'
******************************************************************************************/

Prompt == FUNCTION SP_U_WKF_OUTCOME ==
CREATE OR REPLACE FUNCTION SP_U_WKF_OUTCOME
(
     p_id VARCHAR2
   , p_outcome_element_id VARCHAR2
   , p_next_state_id VARCHAR2
   , p_end_cycle VARCHAR2
   , p_expires_date VARCHAR2
   , p_comment VARCHAR2
   , p_subject VARCHAR2
   , p_checklist VARCHAR2
   , p_group_to VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
   h_max_rank PLS_INTEGER := 0;
   h_completed VARCHAR2(1);
   h_rankBase64 VARCHAR2(8);
   h_parent_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_res_id VARCHAR2(4000);
   h_res VARCHAR2(32);
   h_workflow_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_type PLS_INTEGER := 0;
   h_request_id PLS_INTEGER := 0;
   h_next_state_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_e_mail VARCHAR2(100);
   h_element_id VARCHAR2(32);
   h_outcome_element_id VARCHAR2(32);
   h_prev_element_id VARCHAR2(32);
   h_parent_rank VARCHAR2(200);
   h_prev_res_ids VARCHAR2(4000);
   h_count PLS_INTEGER := 0;
   h_parent_type PLS_INTEGER := 0;
   h_project_name VARCHAR2(255);
   h_bitflag PLS_INTEGER := 0;
   token LONG;
   pos LONG;
   isFirst PLS_INTEGER NOT NULL := 1;
   h_next_wkf_id VARCHAR2(32);
   h_revision VARCHAR2(32);
   h_priority PLS_INTEGER := 0;
   h_expires_date TIMESTAMP;
   h_status PLS_INTEGER := 0;
   h_subject VARCHAR2(255);
   h_message VARCHAR2(16384);
   h_participants VARCHAR2(2048);
   h_group_to VARCHAR2(2048);
   h_attachments VARCHAR2(255);
   h_checklist VARCHAR2(4000);
   h_outcome_name VARCHAR2(255);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WKF_OUTCOME') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WKF_OUTCOME'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WKF_OUTCOME(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_outcome_element_id || ''''
          || ', ' || '''' || p_next_state_id || ''''
          || ', ' || '''' || p_end_cycle || ''''
          || ', ' || '''' || p_expires_date || ''''
          || ', ' || '''' || p_comment || ''''
          || ', ' || '''' || p_subject || ''''
          || ', ' || '''' || p_checklist || ''''
          || ', ' || '''' || p_group_to || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_OUTCOME', 74+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WKF_OUTCOME(75) : workflow.sql(1405) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   h_rec_user := p_rec_user;
   
   BEGIN
      SELECT END_CYCLE, RANK, WKF_DEF_ID, PARENT_TYPE
      INTO h_rec_status, h_parent_rank, h_next_state_id, h_parent_type
      FROM TMT_WORKFLOWS
      WHERE ELEMENT_ID = p_id
      AND AUTHOR_ID = p_rec_user;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 93+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(94) : workflow.sql(1424) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WKF_OUTCOME', 100+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WKF_OUTCOME(101) : workflow.sql(1431) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_OUTCOME', 103+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WKF_OUTCOME(104) : workflow.sql(1434) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_id, True);
   ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If SubStr(h_rec_status, 1, 1) = '1' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WKF_OUTCOME', 109+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WKF_OUTCOME(110) : workflow.sql(1440) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.WORKFLOW_ALREADY_CLOSED), 'NULL') || ' ID: ' || '''' || p_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_OUTCOME', 112+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WKF_OUTCOME(113) : workflow.sql(1443) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.WORKFLOW_ALREADY_CLOSED, p_id, True);
      End If;
      
      h_completed := p_end_cycle;
      h_workflow_id := p_id;
      h_outcome_element_id := p_outcome_element_id;
      If SubStr(h_completed, 1, 1) = '1' Then
         BEGIN
            SELECT RTRIM(ELEMENT_NAME) INTO h_outcome_name
            FROM   TMT_WORKFLOW_CYCLES
            WHERE ELEMENT_ID = h_outcome_element_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 131+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(132) : workflow.sql(1462) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --strcpy(h_prev_res_ids, "");
         h_prev_res_ids := '';
         UPDATE TMT_WORKFLOWS SET
                 DESCRIPTION = NULL
            WHERE ELEMENT_ID = P_ID;

         BEGIN
            UPDATE TMT_WORKFLOWS
            SET OUTCOME_ID = 0 ,--p_outcome_id,
            END_CYCLE = '1',
            CLOSED_DATE = SYSTIMESTAMP,
            REC_STATUS = 'U',
            OUTCOME_NAME = h_outcome_name,
            DESCRIPTION = CASE WHEN DESCRIPTION IS NOT NuLL THEN DESCRIPTION ELSE (SELECT CLOB_DATA FROM EMPTYLOB) END
             || CHR(13) || CHR(10) || '----------' || CHR(13) || CHR(10) || CASE WHEN p_comment is null THEN '' ELSE p_comment END,
            REC_USER = p_rec_user,
            REC_DATETIME = SYSTIMESTAMP
            WHERE ELEMENT_ID = p_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 164+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(165) : workflow.sql(1495) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE WORKFLOW_REPLIES
            SET REC_DATETIME = SYSTIMESTAMP,
            MARKED_READ = '0'
            WHERE ELEMENT_ID = p_id AND RESOURCE_ID <> p_rec_user;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 184+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(185) : workflow.sql(1515) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_id := p_id;
         h_bitflag := 4;
         PMO_DB2M.SC_STATE := FC_INSERT_ALERT(h_id, h_bitflag, h_rec_user);
      ElsIf SubStr(h_completed, 1, 1) = '0' Then
         If (p_next_state_id IS NULL) Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WKF_OUTCOME', 195+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WKF_OUTCOME(196) : workflow.sql(1526) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_OUTCOME', 198+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_WKF_OUTCOME(199) : workflow.sql(1529) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_id, True);
         End If;
         
         UPDATE TMT_WORKFLOWS SET
                 DESCRIPTION = NULL
            WHERE ELEMENT_ID = P_ID;

         BEGIN
            UPDATE TMT_WORKFLOWS
            SET OUTCOME_ID = 0 ,--p_outcome_id,
            END_CYCLE = '1',
            CLOSED_DATE = SYSTIMESTAMP,
            REC_STATUS = 'U',
            EXPIRES_DATE = p_expires_date,
            DESCRIPTION = CASE WHEN DESCRIPTION IS NOT NuLL THEN DESCRIPTION ELSE (SELECT CLOB_DATA FROM EMPTYLOB) END
             || CHR(13) || CHR(10) || '----------' || CHR(13) || CHR(10) || (SELECT ELEMENT_NAME FROM TMT_WORKFLOW_CYCLES WHERE ELEMENT_ID = p_next_state_id),
            REC_USER = p_rec_user,
            REC_DATETIME = SYSTIMESTAMP
            WHERE ELEMENT_ID = p_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 229+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(230) : workflow.sql(1560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         PMODEFINES.PMO_GET_MAX_RANK('TMT_WORKFLOWS', p_id, h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         h_workflow_id := SYS_GUID();
         
         BEGIN
            SELECT REQUEST_ID,CHECKLIST INTO h_request_id,h_checklist
            FROM TMT_WORKFLOW_CYCLES
            WHERE ELEMENT_ID = p_next_state_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 248+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(249) : workflow.sql(1579) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            INSERT INTO TMT_WORKFLOWS
            (ELEMENT_ID, ELEMENT_NAME, PARENT_ID, PARENT_TYPE,
            WKF_DEF_ID, REVISION, CREATED_DATE, AUTHOR_ID, LEVEL_ID,
            TYPE_ID, SUBJECT, REC_USER, RANK, DESCRIPTION, REQUEST_ID,
            PROJECT_NAME, ATTACHED_NAME,CHECKLIST)
            SELECT h_workflow_id, T.ELEMENT_NAME, p_id, PARENT_TYPE,
            p_next_state_id, T.REVISION, SYSTIMESTAMP, T.AUTHOR_ID, T.LEVEL_ID+1,
            T.TYPE_ID, p_subject, p_rec_user,
            RTRIM(RANK) || h_rankBase64, p_comment, h_request_id,
            PROJECT_NAME, ATTACHED_NAME,h_checklist
            FROM TMT_WORKFLOWS T
            WHERE T.ELEMENT_ID  = p_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 276+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(277) : workflow.sql(1607) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --trace_printf(" h_workflow_id :%s\n",h_workflow_id);
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
            h_revision := NULL;
            h_priority := NULL;
            h_expires_date := p_expires_date;
            h_status := 0;
            h_subject := p_subject;
            h_message := p_comment;
            h_participants := p_checklist;
            h_group_to := p_group_to;
            h_attachments := NULL;
            h_bitflag := p_bitflag;
            h_prev_element_id := p_id;
            
            --trace_printf("1-----------> p_id : %s,h_prev_element_id :%s \n",p_id, h_prev_element_id);
            --trace_printf("2-----------> p_outcome_element_id : %s,h_outcome_element_id :%s \n",p_outcome_element_id, h_outcome_element_id);
            PMO_DB2M.SC_STATE := FC_U_WORKFLOW(h_workflow_id, h_revision, h_request_id, h_priority
                                   , h_expires_date, h_status, h_subject, h_message, h_participants
                                   , h_group_to, h_attachments, h_bitflag, h_outcome_element_id
                                   , h_prev_element_id, h_rec_user);
         End If;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_WKF_OUTCOME', 304+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_WKF_OUTCOME(305) : workflow.sql(1635) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_OUTCOME', 307+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_WKF_OUTCOME(308) : workflow.sql(1638) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WKF_OUTCOME', 315+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WKF_OUTCOME(316) : workflow.sql(1646) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_workflow_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_workflow_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 322+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(323) : workflow.sql(1653) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_OUTCOME', 334+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_OUTCOME(335) : workflow.sql(1665) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WKF_OUTCOME;
ALTER FUNCTION SP_U_WKF_OUTCOME COMPILE DEBUG;

/*************************************************************************************************************
    Create workflow replies for a new workflow, notification, or custom workflow created by FC_A_NOTIFICATION
**************************************************************************************************************/

Prompt == FUNCTION FC_U_WORKFLOW ==
CREATE OR REPLACE FUNCTION FC_U_WORKFLOW
(
     h_in_id IN OUT NOCOPY VARCHAR2
   , h_in_revision IN OUT NOCOPY VARCHAR2
   , sh_h_in_request_id PLS_INTEGER
   , sh_h_in_priority PLS_INTEGER
   , h_in_expires_date IN OUT NOCOPY TIMESTAMP
   , sh_h_in_status PLS_INTEGER
   , h_in_subject IN OUT NOCOPY VARCHAR2
   , h_in_message IN OUT NOCOPY VARCHAR2
   , h_in_participants IN OUT NOCOPY VARCHAR2
   , h_in_group_to IN OUT NOCOPY VARCHAR2
   , h_in_attachments IN OUT NOCOPY VARCHAR2
   , sh_h_in_bitflag PLS_INTEGER
   , h_in_outcome_element_id IN OUT NOCOPY VARCHAR2
   , h_in_prev_element_id IN OUT NOCOPY VARCHAR2
   , h_in_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_in_request_id PLS_INTEGER := sh_h_in_request_id;
   h_in_priority PLS_INTEGER := sh_h_in_priority;
   h_in_status PLS_INTEGER := sh_h_in_status;
   h_in_bitflag PLS_INTEGER := sh_h_in_bitflag;
   h_workflow_id VARCHAR2(32);
   h_uniqueID VARCHAR2(32);
   h_res_id VARCHAR2(4000);
   h_res VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_parent_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_type PLS_INTEGER := 0;
   h_parent_type PLS_INTEGER := 0;
   h_e_mail VARCHAR2(100);
   h_element_id VARCHAR2(32);
   h_count PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_project_name VARCHAR2(255);
   h_bitflag PLS_INTEGER := 0;
   h_last_wkf_type VARCHAR2(255);
   h_last_wkf_by VARCHAR2(255);
   h_last_wkf_dt TIMESTAMP;
   h_previous_workflow_step VARCHAR2(255);
   h_outcome_name VARCHAR2(255);
   h_stage_id PLS_INTEGER := 0;
   h_exists VARCHAR2(32);
   token LONG;
   pos LONG;
   isFirst PLS_INTEGER;
-- Declare Cursors
      CURSOR C25 IS
         SELECT R.E_MAIL_ADDRESS, ELEMENT_ID
         FROM TMT_CONTACTS T LEFT JOIN RESOURCES R ON T.ELEMENT_ID=R.RESOURCE_ID
         WHERE T.TYPE_ID = PMO_CNST.SC_RESOURCE AND RANK LIKE h_rank || '%';

      CURSOR C16 IS
         SELECT R.E_MAIL_ADDRESS, ELEMENT_ID
         FROM TMT_CONTACTS T LEFT JOIN RESOURCES R ON T.ELEMENT_ID = R.RESOURCE_ID
         WHERE T.TYPE_ID = PMO_CNST.SC_RESOURCE AND RANK LIKE h_rank || '%'
         AND NOT EXISTS(SELECT W.RESOURCE_ID FROM WORKFLOW_REPLIES W
         WHERE W.RESOURCE_ID = R.RESOURCE_ID AND W.ELEMENT_ID = h_workflow_id);

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_U_WORKFLOW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_U_WORKFLOW'), '-- ' || SYSTIMESTAMP, ':r := FC_U_WORKFLOW(' || 
       '''' || h_in_id || ''''
          || ', ' || '''' || h_in_revision || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_in_request_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_in_priority), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_in_expires_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_in_status), 'NULL')
          || ', ' || '''' || h_in_subject || ''''
          || ', ' || '''' || h_in_message || ''''
          || ', ' || '''' || h_in_participants || ''''
          || ', ' || '''' || h_in_group_to || ''''
          || ', ' || '''' || h_in_attachments || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_in_bitflag), 'NULL')
          || ', ' || '''' || h_in_outcome_element_id || ''''
          || ', ' || '''' || h_in_prev_element_id || ''''
          || ', ' || '''' || h_in_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WORKFLOW', 82+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_U_WORKFLOW(83) : workflow.sql(1766) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT END_CYCLE, RANK, PARENT_TYPE, TYPE_ID
      INTO h_rec_status, h_rank, h_parent_type, h_type
      FROM TMT_WORKFLOWS
      WHERE ELEMENT_ID = h_in_id
      AND AUTHOR_ID = h_in_rec_user;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 97+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(98) : workflow.sql(1781) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_U_WORKFLOW', 104+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_U_WORKFLOW(105) : workflow.sql(1788) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_in_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WORKFLOW', 107+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: FC_U_WORKFLOW(108) : workflow.sql(1791) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.RECORD_NOT_FOUND, h_in_id, False);
   ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If SubStr(h_rec_status, 1, 1) = '1' Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'FC_U_WORKFLOW', 113+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: FC_U_WORKFLOW(114) : workflow.sql(1797) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.WORKFLOW_ALREADY_CLOSED), 'NULL') || ' ID: ' || '''' || h_in_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WORKFLOW', 116+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: FC_U_WORKFLOW(117) : workflow.sql(1800) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.WORKFLOW_ALREADY_CLOSED, h_in_id, False);
      End If;
      
      h_workflow_id := h_in_id;
      
      -- get attachment_id (h_parent_id)
      BEGIN
         SELECT ELEMENT_ID INTO h_parent_id
         FROM TMT_WORKFLOWS
         WHERE LEVEL_ID > -1 AND PARENT_ID IS NuLL
         AND (SUBSTR(h_rank, 1, CASE WHEN LENGTH(RANK) > LENGTH(h_rank) THEN 1 ELSE LENGTH(RANK) END) = RANK 
         AND (RANK LIKE SUBSTR(h_rank, 1, 8) || '%'));
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 136+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(137) : workflow.sql(1820) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         BEGIN
            SELECT PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat(PC.ELEMENT_NAME, ', '), C.ELEMENT_NAME), R.FULL_NAME, W.CREATED_DATE
            INTO h_last_wkf_type, h_last_wkf_by, h_last_wkf_dt
            FROM TMT_WORKFLOWS W, TMT_WORKFLOW_CYCLES C, TMT_WORKFLOW_CYCLES PC, RESOURCES R
            WHERE W.ELEMENT_ID = h_in_id AND W.WKF_DEF_ID = C.ELEMENT_ID
            AND PC.ELEMENT_ID = C.PARENT_ID AND W.AUTHOR_ID = R.RESOURCE_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 154+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(155) : workflow.sql(1838) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_previous_workflow_step := NULL;
         
         --trace_printf("h_in_outcome_element_id : %s\n",h_in_outcome_element_id);
         --trace_printf("h_in_prev_element_id : %s\n",h_in_prev_element_id);
         If (h_in_outcome_element_id IS NOT NULL) AND ((h_in_prev_element_id IS NOT NULL)) Then
            BEGIN
               SELECT RTRIM(ELEMENT_NAME) INTO h_outcome_name
               FROM   TMT_WORKFLOW_CYCLES
               WHERE ELEMENT_ID = h_in_outcome_element_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 174+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(175) : workflow.sql(1858) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT C.ELEMENT_NAME || ';' || h_outcome_name || ';' || 
               SUBSTR(TO_CHAR(W.REC_DATETIME),1,10) || ' ' || SUBSTR(TO_CHAR(W.REC_DATETIME),12,2) || '' || SUBSTR(TO_CHAR(W.REC_DATETIME),15,2)
               INTO h_previous_workflow_step
               FROM TMT_WORKFLOWS W, TMT_WORKFLOW_CYCLES C
               WHERE W.ELEMENT_ID = h_in_prev_element_id AND W.WKF_DEF_ID = C.ELEMENT_ID;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 191+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(192) : workflow.sql(1875) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_WORKFLOWS
               SET OUTCOME_NAME = h_outcome_name
               WHERE ELEMENT_ID = h_in_prev_element_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 210+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(211) : workflow.sql(1894) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         -- Update Parent's Hierarchy Info
         If Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_WBS Then
            BEGIN
               UPDATE TMT_WBS
               SET LAST_WF_TYPE     = h_last_wkf_type,
               LAST_WF_BY       = h_last_wkf_by,
               LAST_WF_DT       = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 234+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(235) : workflow.sql(1918) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID, RTRIM(TO_CHAR(MAJOR_REVISION)) || '.' || RTRIM(TO_CHAR(MINOR_REVISION))
               INTO h_stage_id, h_in_revision
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 250+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(251) : workflow.sql(1934) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_CRI Then
            BEGIN
               UPDATE TMT_CRI
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DATETIME= h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 272+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(273) : workflow.sql(1956) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_CRI
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 287+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(288) : workflow.sql(1971) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_DOC Then
            BEGIN
               UPDATE TMT_DOCUMENTS
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DT      = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 309+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(310) : workflow.sql(1993) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_DOCUMENTS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 324+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(325) : workflow.sql(2008) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_SCD Then
            BEGIN
               UPDATE TMT_ASSIGNED_SCORECARDS
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DT      = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 346+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(347) : workflow.sql(2030) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_ASSIGNED_SCORECARDS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 361+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(362) : workflow.sql(2045) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_RES_DOC Then
            BEGIN
               UPDATE TMT_RESOURCE_DOCUMENTS
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DT      = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 383+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(384) : workflow.sql(2067) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_RESOURCE_DOCUMENTS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 398+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(399) : workflow.sql(2082) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_POL_DOC Then
            BEGIN
               UPDATE TMT_POOL_DOCUMENTS
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DT      = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 420+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(421) : workflow.sql(2104) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_POOL_DOCUMENTS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 435+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(436) : workflow.sql(2119) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_AST_DOC Then
            BEGIN
               UPDATE TMT_ASSET_DOCUMENTS
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DT      = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_parent_id
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 457+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(458) : workflow.sql(2141) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_ASSET_DOCUMENTS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 472+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(473) : workflow.sql(2156) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_CLN_DOC Then
            BEGIN
               UPDATE TMT_CLIENT_DOCUMENTS
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DT      = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 494+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(495) : workflow.sql(2178) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_CLIENT_DOCUMENTS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 509+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(510) : workflow.sql(2193) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_parent_type, 0) = PMO_CNST.SC_TABLE_VDR_DOC Then
            BEGIN
               UPDATE TMT_VENDOR_DOCUMENTS
               SET LAST_WF_TYPE    = h_last_wkf_type,
               LAST_WF_BY      = h_last_wkf_by,
               LAST_WF_DT      = h_last_wkf_dt,
               PREVIOUS_WKF_STEP = h_previous_workflow_step
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 531+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(532) : workflow.sql(2215) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT STAGE_ID INTO h_stage_id
               FROM TMT_VENDOR_DOCUMENTS
               WHERE ELEMENT_ID = h_parent_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 546+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(547) : workflow.sql(2230) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         UPDATE TMT_WORKFLOWS SET
                 DESCRIPTION = NULL
            WHERE ELEMENT_ID = H_IN_ID;

         BEGIN
            UPDATE TMT_WORKFLOWS
            SET REVISION = h_in_revision,
            REQUEST_ID = h_in_request_id,
            WKF_PRIORITY = h_in_priority,
            EXPIRES_DATE = h_in_expires_date,
            WKF_STATUS =  h_in_status,
            SUBJECT = h_in_subject,
            DESCRIPTION = h_in_message,
            REC_STATUS = 'U',
            STAGE_ID = h_stage_id
            WHERE ELEMENT_ID = h_in_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 578+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(579) : workflow.sql(2262) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_res_id := h_in_group_to;
         If Coalesce(Length(h_res_id), 0) > 0 Then
            token := h_res_id;
            pos := h_res_id;
            token := PMO_STPU.REstrtok(token, PMO_CNST.sep, pos);
            h_res := token;
            
            BEGIN
               SELECT RANK INTO h_rank FROM TMT_CONTACTS WHERE ELEMENT_ID = h_res;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 598+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(599) : workflow.sql(2282) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN C25;
            LOOP
               FETCH C25 INTO h_e_mail, h_element_id;
               IF C25%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> 0 Then
                  EXIT;
               End If;
               
               h_uniqueID := SYS_GUID();
               BEGIN
                  INSERT INTO WORKFLOW_REPLIES(WKF_REPLY_ID, ELEMENT_ID, EMAIL_ADDRESS, GROUP_ID, RESOURCE_ID, REPLY_TYPE)
                  VALUES(h_uniqueID, h_workflow_id, h_e_mail, h_res, h_element_id, PMO_CNST.SC_TO);
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 629+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(630) : workflow.sql(2313) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If Coalesce(h_type, 0) = PMO_CNST.SC_STAFFING_REQUESTS Then
                  BEGIN
                     SELECT '1' INTO h_exists FROM RM_ASSIGNMENTS
                     WHERE PROJECT_ID = h_parent_id AND MANAGER_ID = h_element_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 644+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(645) : workflow.sql(2328) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     BEGIN
                        INSERT INTO RM_ASSIGNMENTS (PROJECT_ID, MANAGER_ID, SECURITY_ID, REC_USER, REC_STATUS)
                        VALUES (h_parent_id, h_element_id, 'SCRESOURCEMANAGER', h_in_rec_user, 'U');
                        IF SQL%FOUND = TRUE THEN
                           PMO_DB2M.SC_STATE := 0;
                        ELSE
                           PMO_DB2M.SC_STATE := 100;
                        END IF;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 663+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(664) : workflow.sql(2347) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                  End If;
               End If;
            END LOOP;
            
            BEGIN
               CLOSE C25 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 679+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(680) : workflow.sql(2363) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            LOOP
               token := PMO_STPU.REstrtok(NULL, PMO_CNST.sep, pos);
               If (token IS NULL) Then
                  EXIT;
               End If;
               --  strcpy(h_res,token);
               h_res := token;
               BEGIN
                  SELECT RANK INTO h_rank FROM TMT_CONTACTS WHERE ELEMENT_ID = h_res;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 699+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(700) : workflow.sql(2383) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               OPEN C16;
               
               LOOP
                  FETCH C16 INTO h_e_mail, h_element_id;
                  IF C16%FOUND THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
                  
                  If PMO_DB2M.SC_STATE <> 0 Then
                     EXIT;
                  End If;
                  
                  h_uniqueID := SYS_GUID();
                  
                  BEGIN
                     INSERT INTO WORKFLOW_REPLIES(WKF_REPLY_ID, ELEMENT_ID, EMAIL_ADDRESS, GROUP_ID, RESOURCE_ID, REPLY_TYPE)
                     VALUES(h_uniqueID, h_workflow_id, h_e_mail, h_res, h_element_id, PMO_CNST.SC_TO);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 733+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(734) : workflow.sql(2417) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If Coalesce(h_type, 0) = PMO_CNST.SC_STAFFING_REQUESTS Then
                     BEGIN
                        SELECT '1' INTO h_exists FROM RM_ASSIGNMENTS
                        WHERE PROJECT_ID = h_parent_id AND MANAGER_ID = h_element_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 748+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(749) : workflow.sql(2432) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                        BEGIN
                           INSERT INTO RM_ASSIGNMENTS (PROJECT_ID, MANAGER_ID, SECURITY_ID, REC_USER, REC_STATUS)
                           VALUES (h_parent_id, h_element_id, 'SCRESOURCEMANAGER', h_in_rec_user, 'U');
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 767+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(768) : workflow.sql(2451) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
               END LOOP;
               
               BEGIN
                  CLOSE C16 ;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 783+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(784) : workflow.sql(2467) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            END LOOP;
         End If;
         
         h_res_id := h_in_participants;
         If Coalesce(Length(h_res_id), 0) > 0 Then
            token := h_res_id;
            pos := h_res_id;
            isFirst := 1;
            
            LOOP
               If isFirst = 1 Then
                  token := PMO_STPU.REstrtok(token, PMO_CNST.sep, pos);
                  isFirst := 0;
               Else
                  token := PMO_STPU.REstrtok(NULL, PMO_CNST.sep, pos);
               End If;
               
               If (token IS NULL) Then
                  EXIT;
               End If;
               
               h_res := token;
               
               BEGIN
                  SELECT E_MAIL_ADDRESS INTO h_e_mail
                  FROM RESOURCES WHERE RESOURCE_ID = h_res AND NOT EXISTS(
                  SELECT RESOURCE_ID FROM WORKFLOW_REPLIES WHERE RESOURCE_ID = h_res AND ELEMENT_ID = h_workflow_id);
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 820+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(821) : workflow.sql(2504) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  h_uniqueID := SYS_GUID();
                  BEGIN
                     INSERT INTO WORKFLOW_REPLIES(WKF_REPLY_ID, ELEMENT_ID, RESOURCE_ID, REPLY_TYPE, EMAIL_ADDRESS)
                     VALUES( h_uniqueID, h_workflow_id, h_res, PMO_CNST.SC_TO, h_e_mail);
                     IF SQL%FOUND = TRUE THEN
                        PMO_DB2M.SC_STATE := 0;
                     ELSE
                        PMO_DB2M.SC_STATE := 100;
                     END IF;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 840+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(841) : workflow.sql(2524) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If Coalesce(h_type, 0) = PMO_CNST.SC_STAFFING_REQUESTS Then
                     BEGIN
                        SELECT '1' INTO h_exists FROM RM_ASSIGNMENTS
                        WHERE PROJECT_ID = h_parent_id
                        AND MANAGER_ID = h_res
                        AND PROFILE_ID IS NuLL;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 857+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(858) : workflow.sql(2541) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If NOT PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                        BEGIN
                           INSERT INTO RM_ASSIGNMENTS (PROJECT_ID, MANAGER_ID, SECURITY_ID, REC_USER, REC_STATUS)
                           VALUES (h_parent_id, h_res, 'SCRESOURCEMANAGER', h_in_rec_user, 'U');
                           IF SQL%FOUND = TRUE THEN
                              PMO_DB2M.SC_STATE := 0;
                           ELSE
                              PMO_DB2M.SC_STATE := 100;
                           END IF;
                        EXCEPTION
                           WHEN OTHERS THEN
                           BEGIN
                              PMO_DB2M.SC_STATE := SQLCODE;
                              IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 876+1) THEN 
                                 LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(877) : workflow.sql(2560) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                              END IF; 
                           END;
                        END;
                     End If;
                  End If;
               End If;
            END LOOP;
         End If;
         
         BEGIN
            SELECT COUNT(ELEMENT_ID) INTO h_count
            FROM WORKFLOW_REPLIES
            WHERE ELEMENT_ID = h_workflow_id AND RESOURCE_ID = h_in_rec_user;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 896+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(897) : workflow.sql(2580) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) = 0 Then
            h_uniqueID := SYS_GUID();
            
            BEGIN
               INSERT INTO WORKFLOW_REPLIES(WKF_REPLY_ID, ELEMENT_ID, RESOURCE_ID, REPLY_TYPE, COMPLETED, MARKED_READ)
               VALUES(h_uniqueID, h_workflow_id, h_in_rec_user, PMO_CNST.SC_HIDEN_REPLY, '1', '1');
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_U_WORKFLOW', 917+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: FC_U_WORKFLOW(918) : workflow.sql(2601) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
      
      h_rec_user := h_in_rec_user;
      h_bitflag := 1;
      If Coalesce(h_in_bitflag, 0) <> -2 Then
         PMO_DB2M.SC_STATE := FC_INSERT_ALERT(h_workflow_id, h_bitflag, h_rec_user);
      End If;
      PMO_DB2M.SC_STATE := 0;
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_U_WORKFLOW', 932+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_U_WORKFLOW(933) : workflow.sql(2616) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_U_WORKFLOW;
ALTER FUNCTION FC_U_WORKFLOW COMPILE DEBUG;

/*************************************************************************************************************
    Create workflow replies for a new workflow, notification, or custom workflow created by FC_A_NOTIFICATION
**************************************************************************************************************/

Prompt == FUNCTION SP_U_WORKFLOW ==
CREATE OR REPLACE FUNCTION SP_U_WORKFLOW
(
     p_id VARCHAR2
   , p_revision VARCHAR2
   , p_request_id PLS_INTEGER
   , p_priority PLS_INTEGER
   , p_expires_date VARCHAR2
   , p_status PLS_INTEGER
   , p_subject VARCHAR2
   , p_message VARCHAR2
   , p_participants VARCHAR2
   , p_group_to VARCHAR2
   , p_attachments VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_revision VARCHAR2(32);
   h_request_id PLS_INTEGER := 0;
   h_priority PLS_INTEGER := 0;
   h_expires_date TIMESTAMP;
   h_status PLS_INTEGER := 0;
   h_subject VARCHAR2(255);
   h_message VARCHAR2(16384);
   h_participants VARCHAR2(2048);
   h_group_to VARCHAR2(2048);
   h_attachments VARCHAR2(255);
   h_bitflag PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_temp_id VARCHAR2(32);
   h_temp_id2 VARCHAR2(32);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WORKFLOW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WORKFLOW'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WORKFLOW(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_revision || ''''
          || ', ' || COALESCE(TO_CHAR(p_request_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_priority), 'NULL')
          || ', ' || '''' || p_expires_date || ''''
          || ', ' || COALESCE(TO_CHAR(p_status), 'NULL')
          || ', ' || '''' || p_subject || ''''
          || ', ' || '''' || p_message || ''''
          || ', ' || '''' || p_participants || ''''
          || ', ' || '''' || p_group_to || ''''
          || ', ' || '''' || p_attachments || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WORKFLOW', 53+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WORKFLOW(54) : workflow.sql(2682) : ' || SYSTIMESTAMP);
   END IF; 

   -- 16 ?
   -- ?
   -- Initialize parameters
   h_id := p_id;
   h_revision := p_revision;
   h_request_id := p_request_id;
   h_priority := p_priority;
   h_expires_date := p_expires_date;
   h_status := p_status;
   h_subject := p_subject;
   h_message := SubStr(p_message, 1, 4000);
   h_participants := p_participants;
   h_group_to := p_group_to;
   h_attachments := p_attachments;
   h_bitflag := p_bitflag;
   h_rec_user := p_rec_user;
   
   h_temp_id := NULL;
   h_temp_id2 := NULL;
   
   --  if (strlen(h_message) > 4000)
   --      h_message[4000] = '\0';
   
   PMO_DB2M.SC_STATE := FC_U_WORKFLOW(h_id, h_revision, h_request_id, h_priority, h_expires_date
                                   , h_status, h_subject, h_message, h_participants, h_group_to
                                   , h_attachments, h_bitflag, h_temp_id, h_temp_id2, h_rec_user);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WORKFLOW', 84+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WORKFLOW(85) : workflow.sql(2713) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WORKFLOW', 91+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WORKFLOW(92) : workflow.sql(2720) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WORKFLOW', 103+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WORKFLOW(104) : workflow.sql(2732) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WORKFLOW;
ALTER FUNCTION SP_U_WORKFLOW COMPILE DEBUG;

/*****************************************************************************************
    Update workflow_replies status (reply_status)
    0:  open (in inbox and/or send box)
    1:  move to deleted box
    2:  move to archived box
******************************************************************************************/

Prompt == FUNCTION SP_U_WKF_STATUS ==
CREATE OR REPLACE FUNCTION SP_U_WKF_STATUS
(
     p_id VARCHAR2
   , p_status PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_rank VARCHAR2(200);
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WKF_STATUS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WKF_STATUS'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WKF_STATUS(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_status), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_STATUS', 19+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WKF_STATUS(20) : workflow.sql(2773) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT RANK INTO h_rank
      FROM TMT_WORKFLOWS
      WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_STATUS', 32+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_STATUS(33) : workflow.sql(2786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_WKF_STATUS', 39+1) THEN 
         LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_WKF_STATUS(40) : workflow.sql(2793) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || p_id || '''');
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_STATUS', 42+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_WKF_STATUS(43) : workflow.sql(2796) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, p_id, True);
   End If;
   
   DECLARE 
   	nLen pls_integer :=0; 
   BEGIN 
   	If (h_rank IS NOT NULL) Then 
   		nLen := Coalesce(Length(h_rank), 0); 
   		If 0 <> 0 AND nLen <> 0 Then 
   		nLen := nLen - 1; 
   		End If; 
   	End If; 
   	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
   		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
   	End If; 
   END;
   --Execute Update Statement
   BEGIN
      UPDATE WORKFLOW_REPLIES
      SET REPLY_STATUS = p_status
      WHERE RESOURCE_ID = p_rec_user
      AND ELEMENT_ID IN (SELECT ELEMENT_ID FROM TMT_WORKFLOWS
      WHERE RANK LIKE SUBSTR(h_rank, 1, 12) || '%' AND LENGTH(RANK) <= LENGTH(h_rank));
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_STATUS', 77+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_STATUS(78) : workflow.sql(2831) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WKF_STATUS', 84+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WKF_STATUS(85) : workflow.sql(2838) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_STATUS', 91+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_STATUS(92) : workflow.sql(2845) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_STATUS', 103+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_STATUS(104) : workflow.sql(2857) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WKF_STATUS;
ALTER FUNCTION SP_U_WKF_STATUS COMPILE DEBUG;

/*****************************************************************************************
    Update workflow_replies status (completed and marked_read for notification, marked_read for workflow)
    completed;
    '0':    not reply
    '1':    yes
    '2':    no

    marked_read:
    '0':    not read
    '1':    read
******************************************************************************************/

Prompt == FUNCTION SP_U_REPLY_READ ==
CREATE OR REPLACE FUNCTION SP_U_REPLY_READ
(
     p_id VARCHAR2
   , p_completed VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_type PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_completed VARCHAR2(1);
   h_author_id VARCHAR2(32);
   h_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_REPLY_READ') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_REPLY_READ'), '-- ' || SYSTIMESTAMP, ':r := SP_U_REPLY_READ(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_completed || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_REPLY_READ', 25+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_REPLY_READ(26) : workflow.sql(2909) : ' || SYSTIMESTAMP);
   END IF; 

   h_completed := p_completed;
   
   -- Validate parent_id
   BEGIN
      SELECT TYPE_ID, AUTHOR_ID INTO h_type, h_author_id
      FROM TMT_WORKFLOWS
      WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 41+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(42) : workflow.sql(2925) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      If Coalesce(h_type, 0) = PMO_CNST.SC_NOTIFICATION Then
         If SubStr(h_completed, 1, 1) = '1' Then
            BEGIN
               UPDATE WORKFLOW_REPLIES
               SET REPLY_DATE = SYSTIMESTAMP,
               COMPLETED = p_completed,
               MARKED_READ = p_completed
               WHERE ELEMENT_ID = p_id
               AND RESOURCE_ID = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 66+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(67) : workflow.sql(2950) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               SELECT COUNT(ELEMENT_ID) INTO h_count
               FROM WORKFLOW_REPLIES
               WHERE ELEMENT_ID = p_id AND COMPLETED = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 81+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(82) : workflow.sql(2965) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) = 0 Then
               BEGIN
                  UPDATE TMT_WORKFLOWS
                  SET WKF_STATUS = 1          -- completed
                  WHERE ELEMENT_ID = p_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 101+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(102) : workflow.sql(2985) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               --                  Jaime wants remove comfirmation for notification. June 26, 2003.
               --                  InitializeHost(SC_PARAM(h_rec_user),p_rec_user,ind_u_read->p_rec_user);
               --                  if (strncmp(h_rec_user, h_author_id, 32) != SC_MATCH)
               --                  {
               --                      BEGIN_SQL
               --                          UPDATE WORKFLOW_REPLIES
               --                          SET REC_DATETIME = SC_NOW,
               --                              MARKED_READ = '0'
               --                          WHERE ELEMENT_ID = HOST_DB(p_id,ind_u_read) AND RESOURCE_ID = HOST(h_author_id);
               --                      END_SQL;
               
               --                      InitializeHost(SC_PARAM(h_id),p_id,ind_u_read->p_id);
               --                      h_bitflag = 2;
               --                      SC_STATE = FC_INSERT_ALERT(SC_PARAM(h_id), SC_PARAM(h_bitflag), SC_PARAM(h_rec_user));
               --                  }
            End If;
         Else
            BEGIN
               SELECT COUNT(ELEMENT_ID) INTO h_count
               FROM WORKFLOW_REPLIES
               WHERE ELEMENT_ID = p_id AND COMPLETED = '0';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 133+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(134) : workflow.sql(3017) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE WORKFLOW_REPLIES
               SET REPLY_DATE = NuLL,
               MARKED_READ = p_completed,
               COMPLETED = p_completed
               WHERE ELEMENT_ID = p_id
               AND RESOURCE_ID = p_rec_user;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 155+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(156) : workflow.sql(3039) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_count, 0) = 0 Then
               BEGIN
                  UPDATE TMT_WORKFLOWS
                  SET WKF_STATUS = 0          -- open
                  WHERE ELEMENT_ID = p_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 175+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(176) : workflow.sql(3059) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      Else
         BEGIN
            UPDATE WORKFLOW_REPLIES
            SET MARKED_READ = p_completed
            WHERE ELEMENT_ID = p_id AND RESOURCE_ID = p_rec_user;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 196+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(197) : workflow.sql(3080) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_REPLY_READ', 203+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_REPLY_READ(204) : workflow.sql(3087) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(0), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(0, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 210+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(211) : workflow.sql(3094) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_READ', 222+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_READ(223) : workflow.sql(3106) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_REPLY_READ;
ALTER FUNCTION SP_U_REPLY_READ COMPILE DEBUG;

/*****************************************************************************************
    Update workflow_replies status (reply_status)
    0:  open (in inbox and/or send box)
    1:  move to deleted box
    2:  move to archived box
******************************************************************************************/

Prompt == FUNCTION SP_U_REPLY_STATUS ==
CREATE OR REPLACE FUNCTION SP_U_REPLY_STATUS
(
     p_id VARCHAR2
   , p_status PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_REPLY_STATUS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_REPLY_STATUS'), '-- ' || SYSTIMESTAMP, ':r := SP_U_REPLY_STATUS(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_status), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_REPLY_STATUS', 18+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_REPLY_STATUS(19) : workflow.sql(3146) : ' || SYSTIMESTAMP);
   END IF; 

   --Execute Update Statement
   BEGIN
      UPDATE WORKFLOW_REPLIES
      SET REPLY_STATUS = p_status
      WHERE ELEMENT_ID = p_id
      AND RESOURCE_ID = p_rec_user;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_STATUS', 37+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_STATUS(38) : workflow.sql(3165) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
      BEGIN
         UPDATE WORKFLOW_REPLIES
         SET REPLY_STATUS = p_status
         WHERE WKF_REPLY_ID = p_id
         AND RESOURCE_ID = p_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_STATUS', 58+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_STATUS(59) : workflow.sql(3186) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_REPLY_STATUS', 65+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_REPLY_STATUS(66) : workflow.sql(3193) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_STATUS', 72+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_STATUS(73) : workflow.sql(3200) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_REPLY_STATUS', 84+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_REPLY_STATUS(85) : workflow.sql(3212) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_REPLY_STATUS;
ALTER FUNCTION SP_U_REPLY_STATUS COMPILE DEBUG;

/*****************************************************************************************
    Update workflow_replies status (completed)
******************************************************************************************/

Prompt == FUNCTION SP_U_WKF_REPLY ==
CREATE OR REPLACE FUNCTION SP_U_WKF_REPLY
(
     p_id VARCHAR2
   , p_comment VARCHAR2
   , p_completed VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_count PLS_INTEGER := 0;
   h_author_id VARCHAR2(32);
   h_workflow_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_status PLS_INTEGER := 0;
   h_rec_user VARCHAR2(32);
   h_type_id PLS_INTEGER := 0;
   h_bitflag PLS_INTEGER := 0;
   lcount PLS_INTEGER NOT NULL := 0;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_WKF_REPLY') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_WKF_REPLY'), '-- ' || SYSTIMESTAMP, ':r := SP_U_WKF_REPLY(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_comment || ''''
          || ', ' || '''' || p_completed || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_WKF_REPLY', 29+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_WKF_REPLY(30) : workflow.sql(3260) : ' || SYSTIMESTAMP);
   END IF; 

   h_status := 0;
   
   BEGIN
      SELECT ELEMENT_ID, AUTHOR_ID, TYPE_ID, PARENT_ID,
      CASE WHEN AUTHOR_ID = p_rec_user THEN 0 ELSE 1 END
      INTO h_workflow_id, h_author_id, h_type_id, h_parent_id, h_status
      FROM TMT_WORKFLOWS
      WHERE ELEMENT_ID = (SELECT ELEMENT_ID FROM WORKFLOW_REPLIES
      WHERE WKF_REPLY_ID = p_id AND RESOURCE_ID = p_rec_user);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 47+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(48) : workflow.sql(3278) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      SELECT COUNT(WKF_REPLY_ID) INTO h_count
      FROM WORKFLOW_REPLIES
      WHERE COMPLETED = '0' AND ELEMENT_ID = h_workflow_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 62+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(63) : workflow.sql(3293) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   BEGIN
      UPDATE WORKFLOW_REPLIES
      SET REPLY_DATE = SYSTIMESTAMP,
      DESCRIPTION = p_comment,
      COMPLETED = p_completed
      WHERE WKF_REPLY_ID = p_id AND RESOURCE_ID = p_rec_user;
      IF SQL%FOUND = TRUE THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 83+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(84) : workflow.sql(3314) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   lcount := Coalesce(h_count, 0);
   BEGIN
      SELECT COUNT(WKF_REPLY_ID) INTO h_count
      FROM WORKFLOW_REPLIES
      WHERE COMPLETED = '0' AND ELEMENT_ID = h_workflow_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 99+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(100) : workflow.sql(3330) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_count, 0) = 0 Then
      If (lcount > 0) AND (Coalesce(h_type_id, 0) = PMO_CNST.SC_STAFFING_REQUESTS) Then
         BEGIN
            UPDATE TMT_WORKFLOWS
            SET WKF_STATUS = 1,
            SUBJECT = 'Suggestion Process is Completed'
            WHERE ELEMENT_ID = h_workflow_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 121+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(122) : workflow.sql(3352) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf (lcount > 0) AND (Coalesce(h_type_id, 0) = PMO_CNST.SC_ACCEPTED_SUGGEST) Then
         BEGIN
            UPDATE TMT_WORKFLOWS
            SET WKF_STATUS = 1,
            SUBJECT = 'Resource Assignment Process has been Completed'
            WHERE ELEMENT_ID = h_workflow_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 141+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(142) : workflow.sql(3372) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         -- Archive staffing requests
         BEGIN
            UPDATE RM_ASSIGNMENTS
            SET REC_STATUS = 'A'
            WHERE PROJECT_ID = h_parent_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 161+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(162) : workflow.sql(3392) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         BEGIN
            UPDATE TMT_WORKFLOWS
            SET WKF_STATUS = 1
            WHERE ELEMENT_ID = h_workflow_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 180+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(181) : workflow.sql(3411) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If Coalesce(h_status, 0) = 1 Then
         BEGIN
            UPDATE WORKFLOW_REPLIES
            SET REC_DATETIME = SYSTIMESTAMP,
            MARKED_READ = '0'
            WHERE ELEMENT_ID = h_workflow_id AND RESOURCE_ID = h_author_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 202+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(203) : workflow.sql(3433) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         h_rec_user := p_rec_user;
         If lcount = 0 Then
            h_bitflag := 3;
         Else
            h_bitflag := 2;
         End If;
         
         PMO_DB2M.SC_STATE := FC_INSERT_ALERT(h_workflow_id, h_bitflag, h_rec_user);
      End If;
   End If;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_WKF_REPLY', 218+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_WKF_REPLY(219) : workflow.sql(3449) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 225+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(226) : workflow.sql(3456) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_WKF_REPLY', 237+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_WKF_REPLY(238) : workflow.sql(3468) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_WKF_REPLY;
ALTER FUNCTION SP_U_WKF_REPLY COMPILE DEBUG;

/*****************************************************************************************
    Delete a workflow and its children
******************************************************************************************/

Prompt == FUNCTION SP_D_WOKFLOW ==
CREATE OR REPLACE FUNCTION SP_D_WOKFLOW
(
     p_id VARCHAR2
   , p_parent_id VARCHAR2
   , p_del_flag VARCHAR2
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rec_status VARCHAR2(1);
   h_element_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_wkf_status PLS_INTEGER := 0;
   h_count PLS_INTEGER := 0;
   h_parent_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR DELETED_WKF IS
         SELECT ELEMENT_ID, WKF_STATUS, REC_STATUS, PARENT_ID, SUBSTR(RANK, 1, LENGTH(RANK)-4)
         FROM TMT_WORKFLOWS
         WHERE RANK LIKE SUBSTR(h_rank, 1, 12) || '%'
         ORDER BY RANK DESC;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_D_WOKFLOW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_D_WOKFLOW'), '-- ' || SYSTIMESTAMP, ':r := SP_D_WOKFLOW(' || 
       '''' || p_id || ''''
          || ', ' || '''' || p_parent_id || ''''
          || ', ' || '''' || p_del_flag || ''''
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_WOKFLOW', 34+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_D_WOKFLOW(35) : workflow.sql(3521) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   If (p_id IS NULL) OR (p_del_flag IS NULL) OR (p_rec_user IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_D_WOKFLOW', 41+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_D_WOKFLOW(42) : workflow.sql(3528) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_D_WOKFLOW', 44+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_D_WOKFLOW(45) : workflow.sql(3531) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   -- Validate id
   h_id := p_id;
   h_rec_status := p_del_flag;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(256, 'SP_D_WOKFLOW', 54+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_DELETE: SP_D_WOKFLOW(55) : workflow.sql(3541) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_WORKFLOWS' || '''' || ' ID: ' || '''' || h_id || '''');
   END IF; 
   BEGIN 
      IF h_rec_status = 'X' THEN 
         EXECUTE IMMEDIATE 'SELECT a.REC_STATUS FROM ' || 'TMT_WORKFLOWS' || ' a 
         WHERE a.ELEMENT_ID = :i AND LENGTH(a.RANK) > 4 AND NOT a.REC_STATUS = '' L '' AND NOT EXISTS 
         (SELECT b.REC_STATUS 
                  FROM TMT_ORGANIZATIONS b 
                 WHERE b.REC_STATUS = '' L '' AND instr(b.RANK, a.RANK) = 1)' 
            INTO h_rec_status 
            Using h_id; 
      ELSE 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS FROM ' || 'TMT_WORKFLOWS' || ' 
              WHERE ELEMENT_ID = :i AND REC_STATUS NOT IN (''L'',''D'') ' 
            INTO h_rec_status 
            Using h_id; 
      END IF; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND_OR_UNAVAIL, h_id, True); 
      WHEN OTHERS THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id , True); 
   END;
   
   BEGIN
      SELECT PARENT_ID, RANK INTO h_parent_id, h_rank
      FROM TMT_WORKFLOWS
      WHERE ELEMENT_ID = p_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 88+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(89) : workflow.sql(3575) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If SubStr(p_del_flag, 1, 1) = 'X' Then
      DECLARE 
      	nLen pls_integer :=0; 
      BEGIN 
      	If (h_rank IS NOT NULL) Then 
      		nLen := Coalesce(Length(h_rank), 0); 
      		If 0 <> 0 AND nLen <> 0 Then 
      		nLen := nLen - 1; 
      		End If; 
      	End If; 
      	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
      		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_rank, True); 
      	End If; 
      END;
      BEGIN
         UPDATE WORKFLOW_REPLIES SET REPLY_STATUS = 3
         WHERE RESOURCE_ID = p_rec_user
         AND ELEMENT_ID IN (SELECT ELEMENT_ID FROM TMT_WORKFLOWS
         WHERE RANK LIKE SUBSTR(h_rank, 1, 12) || '%' AND LENGTH(RANK) <= LENGTH(h_rank));
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 122+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(123) : workflow.sql(3609) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      OPEN DELETED_WKF;
      
      LOOP
         FETCH DELETED_WKF
         INTO h_element_id, h_wkf_status, h_rec_status, h_parent_id, h_parent_rank;
         IF DELETED_WKF
         %FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         BEGIN
            SELECT COUNT(*) INTO h_count
            FROM WORKFLOW_REPLIES
            WHERE ELEMENT_ID = h_element_id AND REPLY_STATUS <> 3;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 152+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(153) : workflow.sql(3639) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) > 0 Then
            EXIT;
         End If;
         
         BEGIN
            DELETE FROM TMT_WORKFLOWS WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 173+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(174) : workflow.sql(3660) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM WORKFLOW_REPLIES WHERE ELEMENT_ID = h_element_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 190+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(191) : workflow.sql(3677) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If SubStr(h_rec_status, 1, 1) = 'D' Then
            BEGIN
               UPDATE TMT_WORKFLOWS SET DELETED_COUNT = DELETED_COUNT - 1
               WHERE RANK = h_parent_rank;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 209+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(210) : workflow.sql(3696) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WORKFLOWS SET CHILD_COUNT = CHILD_COUNT - 1
               WHERE RANK = h_parent_rank;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 227+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(228) : workflow.sql(3714) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      END LOOP;
      BEGIN
         CLOSE DELETED_WKF ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 241+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(242) : workflow.sql(3728) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         UPDATE TMT_WORKFLOWS SET REC_STATUS = p_del_flag, REC_USER = p_rec_user
         WHERE ELEMENT_ID = p_id AND AUTHOR_ID = p_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 259+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(260) : workflow.sql(3746) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
         -- Update Parent's Hierarchy Info
         If (h_parent_id IS NOT NULL) Then
            BEGIN
               UPDATE TMT_WORKFLOWS
               SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE ELEMENT_ID = h_parent_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 281+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(282) : workflow.sql(3768) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               UPDATE TMT_WORKFLOWS
               SET CHILD_COUNT = CHILD_COUNT - 1, DELETED_COUNT = DELETED_COUNT + 1
               WHERE RANK = '0000';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 300+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(301) : workflow.sql(3787) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_D_WOKFLOW', 309+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_D_WOKFLOW(310) : workflow.sql(3796) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 316+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(317) : workflow.sql(3803) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_D_WOKFLOW', 328+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_D_WOKFLOW(329) : workflow.sql(3815) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_D_WOKFLOW;
ALTER FUNCTION SP_D_WOKFLOW COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_WORKFLOW_DEF ==
CREATE OR REPLACE FUNCTION SP_S_WORKFLOW_DEF
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_count PLS_INTEGER := 0;
   h_exists VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_must_obey PLS_INTEGER := 0;
   h_level_id PLS_INTEGER := 0;
   h_rank VARCHAR2(200);
   h_id VARCHAR2(32);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C6667a PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C6667b PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C6667c PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C6667d PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C6667e PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      refresh_wkf PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C6667g PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C6667h PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_WORKFLOW_DEF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_WORKFLOW_DEF'), '-- ' || SYSTIMESTAMP, ':r := SP_S_WORKFLOW_DEF(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 47+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_WORKFLOW_DEF(48) : workflow.sql(3879) : ' || SYSTIMESTAMP);
   END IF; 

   h_id := p_id;
   h_bitflag := p_bitflag;
   
   If (p_id IS NULL) Then
      If Coalesce(h_bitflag, 0) = 0 Then
         OPEN C6667a FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         CHILD_COUNT, DELETED_COUNT, REC_STATUS, START_WITH, REQUEST_ID, OUTCOME_ID,
         NEXT_PROCESS_ID as NEXT_STATE_ID, PARTICIPANTS, CHECKLIST,
         '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
         FROM TMT_WORKFLOW_CYCLES
         WHERE PARENT_ID IS NuLL AND REC_STATUS<>'D' ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 62+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(63) : workflow.sql(3894) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C6667a;
      ElsIf Coalesce(h_bitflag, 0) = 1 Then
         OPEN C6667b FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         CHILD_COUNT, DELETED_COUNT, REC_STATUS, START_WITH, REQUEST_ID, OUTCOME_ID,
         NEXT_PROCESS_ID as NEXT_STATE_ID, PARTICIPANTS, CHECKLIST,
         '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
         FROM TMT_WORKFLOW_CYCLES
         WHERE PARENT_ID = 'HIDDEN' AND REC_STATUS<>'D'
         UNION ALL
         SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         CHILD_COUNT, DELETED_COUNT, REC_STATUS, START_WITH, REQUEST_ID, OUTCOME_ID,
         NEXT_PROCESS_ID as NEXT_STATE_ID, PARTICIPANTS, CHECKLIST,
         '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
         FROM TMT_WORKFLOW_CYCLES
         WHERE PARENT_ID IS NuLL AND REC_STATUS<>'D'
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 81+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(82) : workflow.sql(3913) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C6667b;
      ElsIf Coalesce(h_bitflag, 0) = -1 Then
         OPEN C6667c FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         CHILD_COUNT, DELETED_COUNT, REC_STATUS, START_WITH, REQUEST_ID, OUTCOME_ID,
         NEXT_PROCESS_ID as NEXT_STATE_ID, PARTICIPANTS, CHECKLIST,
         '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
         FROM TMT_WORKFLOW_CYCLES
         WHERE REC_STATUS<>'D' ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 92+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(93) : workflow.sql(3924) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C6667c;
      Else
         BEGIN
            SELECT ELEMENT_ID INTO h_id
            FROM TMT_WORKFLOW_CYCLES
            WHERE TYPE_ID = h_bitflag AND PARENT_ID IS NuLL AND REC_STATUS<>'D';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 106+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(107) : workflow.sql(3938) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW_DEF', 113+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW_DEF(114) : workflow.sql(3945) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 116+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(117) : workflow.sql(3948) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.RECORD_NOT_FOUND, True);
         Else
            OPEN C6667d FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
            CHILD_COUNT, DELETED_COUNT, REC_STATUS, START_WITH, REQUEST_ID, OUTCOME_ID,
            NEXT_PROCESS_ID as NEXT_STATE_ID, PARTICIPANTS, CHECKLIST,
            '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
            FROM TMT_WORKFLOW_CYCLES
            WHERE PARENT_ID = h_id AND REC_STATUS<>'D' ORDER BY RANK;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 127+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(128) : workflow.sql(3959) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN C6667d;
         End If;
      End If;
   Else
      If Coalesce(h_bitflag, 0) = 2 Then
         BEGIN
            SELECT C.RANK, P.MUST_OBEY INTO h_rank, h_must_obey
            FROM TMT_WORKFLOW_CYCLES C,  TMT_WORKFLOW_CYCLES P
            WHERE C.ELEMENT_ID = h_id AND C.REC_STATUS<>'D' AND P.ELEMENT_ID = C.PARENT_ID;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 144+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(145) : workflow.sql(3976) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW_DEF', 151+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW_DEF(152) : workflow.sql(3983) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 154+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(155) : workflow.sql(3986) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.RECORD_NOT_FOUND, True);
         Else
            --trace_printf("\n C6667e \n");
            
            OPEN C6667e FOR SELECT C.LEVEL_ID, C.TYPE_ID, C.RANK, C.PARENT_ID, C.ELEMENT_ID, C.ELEMENT_NAME,
            C.CHILD_COUNT, C.DELETED_COUNT, C.REC_STATUS, C.START_WITH, C.REQUEST_ID, C.OUTCOME_ID,
            C.NEXT_PROCESS_ID as NEXT_STATE_ID, N.PARTICIPANTS, N.CHECKLIST,
            N.ELEMENT_NAME AS NEXT_STATE_NAME, D.DESCRIPTION AS OUTCOME_NAME,C.NEXT_STAGE_ID, h_must_obey as MUST_OBEY
            FROM TMT_WORKFLOW_CYCLES C
            LEFT JOIN TMT_WORKFLOW_CYCLES N ON N.ELEMENT_ID = C.NEXT_PROCESS_ID
            LEFT JOIN TMT_DATAFIELDS D ON C.OUTCOME_ID = D.ELEMENT_ID
            WHERE C.RANK LIKE h_rank || '%' AND C.REC_STATUS<>'D'
            ORDER BY RANK;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 170+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(171) : workflow.sql(4002) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN C6667e;
         End If;
      ElsIf Coalesce(h_bitflag, 0) = PMO_CNST.SC_REFRESH Then
         BEGIN
            SELECT ELEMENT_ID
            INTO h_exists
            FROM TMT_WORKFLOW_CYCLES
            WHERE ELEMENT_ID = h_id AND REC_STATUS<>'D';
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 186+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(187) : workflow.sql(4018) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW_DEF', 193+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW_DEF(194) : workflow.sql(4025) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 196+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(197) : workflow.sql(4028) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.RECORD_NOT_FOUND, True);
         End If;
         
         --trace_printf("\n C6667h \n");
         OPEN refresh_wkf FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
         CHILD_COUNT, DELETED_COUNT,REC_STATUS, START_WITH, REQUEST_ID,
         OUTCOME_ID, NEXT_PROCESS_ID as NEXT_STATE_ID, PARTICIPANTS, CHECKLIST,
         '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
         FROM TMT_WORKFLOW_CYCLES
         WHERE ELEMENT_ID = h_id AND REC_STATUS<>'D';
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 209+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(210) : workflow.sql(4041) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN refresh_wkf;
      Else
         BEGIN
            SELECT COUNT(ELEMENT_ID)
            INTO h_count
            FROM TMT_WORKFLOW_CYCLES
            WHERE PARENT_ID = h_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 224+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(225) : workflow.sql(4056) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_count, 0) > 0 Then
            BEGIN
               SELECT LEVEL_ID ,RANK INTO h_level_id, h_rank
               FROM TMT_WORKFLOW_CYCLES
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 240+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(241) : workflow.sql(4072) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("\n C6667g \n");
            
            OPEN C6667g FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
            CHILD_COUNT, DELETED_COUNT, REC_STATUS, START_WITH, REQUEST_ID, OUTCOME_ID,
            NEXT_PROCESS_ID as NEXT_STATE_ID,
            PARTICIPANTS, CHECKLIST, '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
            FROM TMT_WORKFLOW_CYCLES
            WHERE RANK LIKE h_rank || '%'
            AND LEVEL_ID < h_level_id + 2
            AND REC_STATUS<>'D'
            ORDER BY RANK;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 257+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(258) : workflow.sql(4089) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN C6667g;
         Else
            BEGIN
               SELECT ELEMENT_ID INTO h_exists
               FROM TMT_WORKFLOW_CYCLES
               WHERE ELEMENT_ID = h_id AND REC_STATUS<>'D';
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 271+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(272) : workflow.sql(4103) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW_DEF', 278+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW_DEF(279) : workflow.sql(4110) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 281+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(282) : workflow.sql(4113) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.RECORD_NOT_FOUND, True);
            End If;
            
            --trace_printf("\n C6667h \n");
            OPEN C6667h FOR SELECT LEVEL_ID, TYPE_ID, RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,
            CHILD_COUNT, DELETED_COUNT,REC_STATUS, START_WITH, REQUEST_ID,
            OUTCOME_ID, NEXT_PROCESS_ID as NEXT_STATE_ID, PARTICIPANTS, CHECKLIST,
            '' AS NEXT_STATE_NAME, '' AS OUTCOME_NAME,NEXT_STAGE_ID,MUST_OBEY
            FROM TMT_WORKFLOW_CYCLES
            WHERE ELEMENT_ID = h_id AND REC_STATUS<>'D';
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW_DEF', 294+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW_DEF(295) : workflow.sql(4126) : ' || SYSTIMESTAMP);
            END IF; 
            RETURN C6667h;
         End If;
      End If;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 305+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(306) : workflow.sql(4137) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW_DEF', 317+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW_DEF(318) : workflow.sql(4149) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_WORKFLOW_DEF;
ALTER FUNCTION SP_S_WORKFLOW_DEF COMPILE DEBUG;

/****************************************************************************************/

Prompt == FUNCTION SP_S_ESETTINGS ==
CREATE OR REPLACE FUNCTION SP_S_ESETTINGS RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
      -- Resultset Cursor Declaration
      C66671 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ESETTINGS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ESETTINGS'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ESETTINGS();');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ESETTINGS', 13+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ESETTINGS(14) : workflow.sql(4179) : ' || SYSTIMESTAMP);
   END IF; 

   OPEN C66671 FOR SELECT ELEMENT_NAME, CASE WHEN SETTING_VALUE IS NuLL THEN DESCRIPTION ELSE TO_CHAR(SETTING_VALUE) END
   FROM TMT_SETTINGS WHERE TYPE_ID = PMO_CNST.SC_EMAIL;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ESETTINGS', 19+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: SP_S_ESETTINGS(20) : workflow.sql(4185) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN C66671;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ESETTINGS', 27+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ESETTINGS(28) : workflow.sql(4193) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ESETTINGS', 39+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ESETTINGS(40) : workflow.sql(4205) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ESETTINGS;
ALTER FUNCTION SP_S_ESETTINGS COMPILE DEBUG;

/*******************************************************************************************/

Prompt == FUNCTION FC_CREATE_DATE_LIMITS ==
CREATE OR REPLACE FUNCTION FC_CREATE_DATE_LIMITS
(
     h_date IN OUT NOCOPY DATE
   , sh_h_date_limit_bitflag PLS_INTEGER
   , h_starting_date IN OUT NOCOPY DATE
   , h_ending_date IN OUT NOCOPY DATE
) RETURN PLS_INTEGER
AS
   h_date_limit_bitflag PLS_INTEGER := sh_h_date_limit_bitflag;
   h_week_starting_day PLS_INTEGER := 0;
   h_today_day_of_week PLS_INTEGER := 0;
   h_number_of_week PLS_INTEGER := 0;
   h_delta PLS_INTEGER := 0;
   h_month PLS_INTEGER := 0;
   h_year PLS_INTEGER := 0;
   h_temp_bitflag PLS_INTEGER := 0;
   h_calculate_week PLS_INTEGER := 0;
   h_temp_str VARCHAR2(49);
   h_day_name VARCHAR2(19);
   null_return LONG;
   ora_temp_sc_state PLS_INTEGER NOT NULL := 0;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_CREATE_DATE_LIMITS') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_CREATE_DATE_LIMITS'), '-- ' || SYSTIMESTAMP, ':r := FC_CREATE_DATE_LIMITS(' || 
       COALESCE(NULLIF('''' || h_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(TO_CHAR(sh_h_date_limit_bitflag), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_starting_date || '''',''''''), 'NULL')
          || ', ' || COALESCE(NULLIF('''' || h_ending_date || '''',''''''), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_DATE_LIMITS', 31+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_CREATE_DATE_LIMITS(32) : workflow.sql(4253) : ' || SYSTIMESTAMP);
   END IF; 

   h_number_of_week := 0;
   h_calculate_week := 0;
   
   If Coalesce(h_date_limit_bitflag, 0) = 0 Then
      BEGIN
         SELECT TO_DATE(SubStr(TO_TIMESTAMP('1899-12-30 8:00:00'), 1, 10)), TO_DATE(SubStr(TO_TIMESTAMP('2300-12-12 12:00:00'), 1, 10))
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 48+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(49) : workflow.sql(4270) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_LAST_7_DAYS Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)), TO_DATE(SubStr(h_date, 1, 10)) - 6     
         INTO h_ending_date, h_starting_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 63+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(64) : workflow.sql(4285) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_SIX_DAYS_BEFORE_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) - 6     , TO_DATE(SubStr(h_date, 1, 10)) - 6     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 78+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(79) : workflow.sql(4300) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_FIVE_DAYS_BEFORE_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) - 5     , TO_DATE(SubStr(h_date, 1, 10)) - 5     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 93+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(94) : workflow.sql(4315) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_FOUR_DAYS_BEFORE_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) - 4     , TO_DATE(SubStr(h_date, 1, 10)) - 4     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 108+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(109) : workflow.sql(4330) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_THREE_DAYS_BEFORE_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) - 3     , TO_DATE(SubStr(h_date, 1, 10)) - 3     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 123+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(124) : workflow.sql(4345) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_TWO_DAYS_BEFORE_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) - 2     , TO_DATE(SubStr(h_date, 1, 10)) - 2     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 138+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(139) : workflow.sql(4360) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_TWO_DAYS_AFTER_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) + 2     , TO_DATE(SubStr(h_date, 1, 10)) + 2     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 153+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(154) : workflow.sql(4375) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_THREE_DAYS_AFTER_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) + 3     , TO_DATE(SubStr(h_date, 1, 10)) + 3     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 168+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(169) : workflow.sql(4390) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_FOUR_DAYS_AFTER_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) + 4     , TO_DATE(SubStr(h_date, 1, 10)) + 4     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 183+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(184) : workflow.sql(4405) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_FIVE_DAYS_AFTER_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) + 5     , TO_DATE(SubStr(h_date, 1, 10)) + 5     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 198+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(199) : workflow.sql(4420) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_SIX_DAYS_AFTER_TODAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) + 6     , TO_DATE(SubStr(h_date, 1, 10)) + 6     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 213+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(214) : workflow.sql(4435) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_CURRENT_DAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)), TO_DATE(SubStr(h_date, 1, 10))
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 228+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(229) : workflow.sql(4450) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_YESTERDAY Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) - 1     , TO_DATE(SubStr(h_date, 1, 10)) - 1     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 243+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(244) : workflow.sql(4465) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_LAST_WEEK Then
      h_calculate_week := 1;
      h_number_of_week := -1;
      -- last week
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_LAST_TWO_WEEKS Then
      h_calculate_week := 1;
      h_number_of_week := -2;
      -- last two weeks
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_LAST_THREE_WEEKS Then
      h_calculate_week := 1;
      h_number_of_week := -3;
      -- last three weeks
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_EARLIER_THIS_MONTH Then
      h_calculate_week := 1;
      h_number_of_week := -4;
      -- last four weeks or earlier this month
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_LAST_MONTH Then
      BEGIN
         SELECT PMO_ORASTD.MONTH(TO_DATE(SubStr(h_date, 1, 10))), PMO_ORASTD.YEAR(TO_DATE(SYSDATE))
         INTO h_month, h_year
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 274+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(275) : workflow.sql(4496) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If Coalesce(h_month, 0) = 1 Then
         h_month := 12;
         h_year := Coalesce(h_year, 0) - 1;
      Else
         h_month := Coalesce(h_month, 0) - 1;
      End If;
      
      h_starting_date := LPad(TO_CHAR(Coalesce(h_year, 0)), 4) || '-' || LPad(TO_CHAR(Coalesce(
                   h_month, 0)), 2, '0') || '-01';
      h_starting_date := h_starting_date;
      
      BEGIN
         SELECT TO_DATE(SubStr(h_starting_date, 1, 10)), (Add_Months(TO_DATE(SubStr(h_starting_date, 1, 10)), 1)) - 1     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 300+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(301) : workflow.sql(4522) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_OLDER_LAST_MONTH Then
      -- means older than last month workflows,
      
      -- first fetch the last month start and finish dates
      h_temp_bitflag := PMO_CNST.SC_LAST_MONTH;
      ora_temp_sc_state := PMO_DB2M.SC_STATE;
      null_return := FC_CREATE_DATE_LIMITS(h_date, h_temp_bitflag, h_starting_date, h_ending_date);
      PMO_DB2M.SC_STATE := ora_temp_sc_state;
      
      h_date_limit_bitflag := PMO_CNST.SC_ALL_PERIOD;
      BEGIN
         SELECT TO_DATE(SubStr(TO_TIMESTAMP('1899-12-30 8:00:00'), 1, 10)), TO_DATE(SubStr(h_starting_date, 1, 10)) - 1     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 324+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(325) : workflow.sql(4546) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_TOMORROW Then
      BEGIN
         SELECT TO_DATE(SubStr(h_date, 1, 10)) + 1     , TO_DATE(SubStr(h_date, 1, 10)) + 1     
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 339+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(340) : workflow.sql(4561) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_NEXT_WEEK Then
      h_calculate_week := 1;
      h_number_of_week := 1;
      -- next week
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_THIS_WEEK Then
      h_calculate_week := 1;
      h_number_of_week := 0;
      -- current week
   ElsIf Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_ALL_PERIOD Then
      h_date_limit_bitflag := PMO_CNST.SC_ALL_PERIOD;
      BEGIN
         SELECT TO_DATE(SubStr(TO_TIMESTAMP('1899-12-30 8:00:00'), 1, 10)), TO_DATE(SubStr(TO_TIMESTAMP('2300-12-12 12:00:00'), 1, 10))
         INTO h_starting_date, h_ending_date
         FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 363+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(364) : workflow.sql(4585) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If Coalesce(h_calculate_week, 0) = 1 Then
      -- Fetch week starting day from tmt_settings
      BEGIN
         SELECT SETTING_VALUE
         INTO h_week_starting_day
         FROM TMT_SETTINGS
         WHERE ELEMENT_ID  ='TIMESHEET_START_WEEKDAY';
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 382+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(383) : workflow.sql(4604) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_today_day_of_week := GET_DAY_OF_WEEK(h_date, h_day_name);
      
      If Coalesce(h_number_of_week, 0) < 0 Then
         h_number_of_week := -1 * Coalesce(h_number_of_week, 0);
         If Coalesce(h_today_day_of_week, 0) < Coalesce(h_week_starting_day, 0) Then
            h_delta := 8 - (Coalesce(h_week_starting_day, 0) - Coalesce(h_today_day_of_week, 0));
         Else
            h_delta := (Coalesce(h_today_day_of_week, 0) - Coalesce(h_week_starting_day, 0)) + 1;
         End If;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_date, 1, 10)) - (h_delta + (7 * h_number_of_week - 1))     
            INTO h_starting_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 407+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(408) : workflow.sql(4629) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_starting_date, 1, 10)) + 6     
            INTO h_ending_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 422+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(423) : workflow.sql(4644) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If Coalesce(h_date_limit_bitflag, 0) = PMO_CNST.SC_EARLIER_THIS_MONTH Then
            If PMO_ORASTD.strncmp(h_date, h_ending_date, 7) = 0 Then
               --pmo_strncpy(h_starting_date, h_ending_date, 0, 7);
               --strcat(h_starting_date, "-01");
               BEGIN
                  SELECT TO_DATE(SubStr(SUBSTR(h_ending_date,1,7) || '-01', 1, 10))
                  INTO h_starting_date
                  FROM DUAL;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 441+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(442) : workflow.sql(4663) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
      ElsIf Coalesce(h_number_of_week, 0) = 0 Then
         If Coalesce(h_today_day_of_week, 0) < Coalesce(h_week_starting_day, 0) Then
            h_delta := (Coalesce(h_week_starting_day, 0) - Coalesce(h_today_day_of_week, 0)) + 1;
         Else
            h_delta := (Coalesce(h_today_day_of_week, 0) - Coalesce(h_week_starting_day, 0));
         End If;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_date, 1, 10)) - h_delta     
            INTO h_starting_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 464+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(465) : workflow.sql(4686) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_starting_date, 1, 10)) + 6     
            INTO h_ending_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 479+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(480) : workflow.sql(4701) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         If Coalesce(h_today_day_of_week, 0) < Coalesce(h_week_starting_day, 0) Then
            h_delta := Coalesce(h_week_starting_day, 0) - Coalesce(h_today_day_of_week, 0);
         Else
            h_delta := 7 - (Coalesce(h_today_day_of_week, 0) - Coalesce(h_week_starting_day, 0));
         End If;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_date, 1, 10)) + (h_delta + (7 * (h_number_of_week - 1)))     
            INTO h_starting_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 500+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(501) : workflow.sql(4722) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            SELECT TO_DATE(SubStr(h_starting_date, 1, 10)) + 6     
            INTO h_ending_date
            FROM DUAL;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_CREATE_DATE_LIMITS', 515+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_CREATE_DATE_LIMITS(516) : workflow.sql(4737) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   End If;
   
   --trace_printf("\n h_starting_date====>{%s}", h_starting_date);
   --trace_printf("\n h_ending_date====>{%s}\n", h_ending_date);
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_DATE_LIMITS', 526+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_DATE_LIMITS(527) : workflow.sql(4748) : ' || SYSTIMESTAMP);
   END IF; 
   RETURN Coalesce(h_date_limit_bitflag, 0);
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_CREATE_DATE_LIMITS', 530+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_CREATE_DATE_LIMITS(531) : workflow.sql(4752) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_CREATE_DATE_LIMITS;
ALTER FUNCTION FC_CREATE_DATE_LIMITS COMPILE DEBUG;

/*******************************************************************************************************************************************************************/

Prompt == FUNCTION SP_S_WORKFLOW ==
CREATE OR REPLACE FUNCTION SP_S_WORKFLOW
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_id2 VARCHAR2
   , p_author_id VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_author_id VARCHAR2(32);
   h_parent_id VARCHAR2(32);
   h_rank VARCHAR2(200);
   h_rankp VARCHAR2(200);
   h_workflow_id VARCHAR2(32);
   h_bitflag PLS_INTEGER := 0;
   h_parent_type PLS_INTEGER := 0;
   h_type PLS_INTEGER := 0;
   h_status PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_attached_id VARCHAR2(32);
   h_attached_type PLS_INTEGER := 0;
   h_attached_name VARCHAR2(255);
   h_project_name VARCHAR2(255);
   h_stage_id PLS_INTEGER := 0;
   h_temp_sc_state PLS_INTEGER := 0;
   h_starting_date DATE;
   h_ending_date DATE;
   h_date_limit_bitflag PLS_INTEGER := 0;
   h_date DATE;
   stmt VARCHAR2(8000);
-- Declare Cursors
      -- Resultset Cursor Declaration
      CR0001 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CRN001 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CR0002 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CR0003 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CR0004 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CR0005 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CR0006 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CR0007 PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      CR0008 PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_WORKFLOW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_WORKFLOW'), '-- ' || SYSTIMESTAMP, ':r := SP_S_WORKFLOW(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_id2 || ''''
          || ', ' || '''' || p_author_id || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 69+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_WORKFLOW(70) : workflow.sql(4832) : ' || SYSTIMESTAMP);
   END IF; 

   h_date_limit_bitflag := 0;
   
   -- Parse parameters to local variables
   h_id := p_id;
   h_author_id := p_author_id;
   h_workflow_id := p_id2;
   h_bitflag := p_bitflag;
   
   --trace_printf("\n h_bitflag===>%d", h_bitflag);
   
   -- PLEASE UPDATE THE FOLLOWING COMMENT IF YOU CHANGE SOMETHING : November 03, 2003
   
   -- the first 16 bits of bitflag are used for main operation
   
   --  h_bitflag values are    updated on November 03, 2003
   --  value         meaning
   -------------------------------------------
   -- 0            -> Send Items(for root level nodes)
   -- 2            -> Inbox(for root level nodes)
   -- 4            -> as before???
   -- 8            -> late response items
   -- 16           -> expand process or notification node(other levels)
   -- 32           -> deleted workflows(for root level nodes)
   -- 64           -> Archived workflows(for root level nodes)
   -- 128          -> unread items in inbox folder
   
   -- the second 16 bits are used for date limit options -> REFER TO PMO_CNST.H WORKFLOW CONSTANT MACROS
   
   --trace_printf("\n 1-h_bitflag:%d", h_bitflag);
   
   If Coalesce(h_bitflag, 0) > 0 Then
      h_date_limit_bitflag := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_FFFF0000);
      h_bitflag := BITAND(Coalesce(h_bitflag, 0), PMO_CNST.SC_0000FFFF);
   Else
      h_date_limit_bitflag := PMO_CNST.SC_ALL_PERIOD;
      h_bitflag := 0;
   End If;
   
   --trace_printf("\n 4-h_date_limit_bitflag:%d", h_date_limit_bitflag);
   --trace_printf("\n 5-h_bitflag:%d", h_bitflag);
   
   BEGIN
      SELECT TO_DATE(SYSDATE)
      INTO h_date
      FROM DUAL;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 123+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(124) : workflow.sql(4886) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   h_temp_sc_state := FC_CREATE_DATE_LIMITS(h_date, h_date_limit_bitflag, h_starting_date
                                         , h_ending_date);
   
   h_type_id := 0;
   
   h_status := 0;
   If Coalesce(h_bitflag, 0) = 32 Then
      h_status := 1;
   ElsIf Coalesce(h_bitflag, 0) = 64 Then
      h_status := 2;
   End If;
   
   If (Coalesce(h_bitflag, 0) > 16) AND (Coalesce(h_bitflag, 0) <> 128) Then
      h_bitflag := 0;
   End If;
   
   If (p_id IS NULL) Then
      -- Temporary Table Declaration has been changed to Truncate
      EXECUTE IMMEDIATE 'DELETE FROM PMO_TMP_WKF_0006';
      PMO_DB2M.SC_STATE := 0;
      
      If (p_author_id IS NULL) Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW', 151+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW(152) : workflow.sql(4914) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 154+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(155) : workflow.sql(4917) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
      
      If Coalesce(h_status, 0) = 0 Then
         If Coalesce(h_bitflag, 0) = 0 Then
            BEGIN
               INSERT INTO PMO_TMP_WKF_0006
               SELECT DISTINCT A.ELEMENT_ID, A.RANK, P.ELEMENT_ID AS ATTACHED_ID, P.PARENT_TYPE
               FROM TMT_WORKFLOWS A, TMT_WORKFLOWS P, WORKFLOW_REPLIES WR
               WHERE A.LEVEL_ID>0
               AND A.AUTHOR_ID = h_author_id
               AND WR.ELEMENT_ID = A.ELEMENT_ID
               AND WR.RESOURCE_ID = A.AUTHOR_ID
               AND WR.REPLY_STATUS = 0
               AND P.RANK = SUBSTR(A.RANK, 1, 8)
               AND (TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) >= h_starting_date AND TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) <= h_ending_date);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 182+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(183) : workflow.sql(4945) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("\n h_starting_date:%s", h_starting_date);
            --trace_printf("\n h_ending_date:%s", h_ending_date);
         ElsIf Coalesce(h_bitflag, 0) = 2 Then
            BEGIN
               INSERT INTO PMO_TMP_WKF_0006
               SELECT DISTINCT A.ELEMENT_ID, A.RANK, P.ELEMENT_ID AS ATTACHED_ID, P.PARENT_TYPE
               FROM TMT_WORKFLOWS A, TMT_WORKFLOWS P, WORKFLOW_REPLIES WR
               WHERE WR.RESOURCE_ID = h_author_id
               AND A.ELEMENT_ID = WR.ELEMENT_ID
               AND A.LEVEL_ID>0
               AND P.RANK = SUBSTR(A.RANK, 1, 8)
               AND WR.REPLY_STATUS = 0
               AND (WR.REPLY_TYPE <> 0 OR (WR.REPLY_TYPE = 0 AND A.WKF_STATUS = 1))
               AND (TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) >= h_starting_date AND TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) <= h_ending_date);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 211+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(212) : workflow.sql(4974) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         ElsIf Coalesce(h_bitflag, 0) = 8 Then
            BEGIN
               INSERT INTO PMO_TMP_WKF_0006
               SELECT DISTINCT A.ELEMENT_ID, A.RANK, P.ELEMENT_ID AS ATTACHED_ID, P.PARENT_TYPE
               FROM TMT_WORKFLOWS A, TMT_WORKFLOWS P, WORKFLOW_REPLIES WR
               WHERE WR.RESOURCE_ID = h_author_id
               AND A.ELEMENT_ID = WR.ELEMENT_ID
               AND A.LEVEL_ID>0
               AND P.RANK = SUBSTR(A.RANK, 1, 8)
               AND WR.REPLY_STATUS = 0
               AND A.TYPE_ID IN (PMO_CNST.SC_WORKFLOW_CYCLE, PMO_CNST.SC_STAFFING_REQUESTS, PMO_CNST.SC_ACCEPTED_SUGGEST)
               AND A.END_CYCLE = '0'
               AND WR.COMPLETED = '0'
               AND (WR.REPLY_TYPE <> 0 OR (WR.REPLY_TYPE = 0 AND A.WKF_STATUS = 1))
               AND TO_DATE(SubStr(A.EXPIRES_DATE, 1, 10)) >= h_starting_date
               AND TO_DATE(SubStr(A.EXPIRES_DATE, 1, 10)) <= h_ending_date;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 241+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(242) : workflow.sql(5004) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            --trace_printf("\n h_starting_date:%s", h_starting_date);
            --trace_printf("\n h_ending_date:%s", h_ending_date);
            --trace_printf("\n h_author_id:%s", h_author_id);
         ElsIf Coalesce(h_bitflag, 0) = 128 Then
            BEGIN
               INSERT INTO PMO_TMP_WKF_0006
               SELECT DISTINCT A.ELEMENT_ID, A.RANK, P.ELEMENT_ID AS ATTACHED_ID, P.PARENT_TYPE
               FROM TMT_WORKFLOWS A, TMT_WORKFLOWS P, WORKFLOW_REPLIES WR
               WHERE WR.RESOURCE_ID = h_author_id
               AND A.ELEMENT_ID = WR.ELEMENT_ID
               AND A.LEVEL_ID>0
               AND P.RANK = SUBSTR(A.RANK, 1, 8)
               AND WR.REPLY_STATUS = 0
               AND WR.MARKED_READ = '0'
               AND (WR.REPLY_TYPE <> 0 OR (WR.REPLY_TYPE = 0 AND A.WKF_STATUS = 1))
               AND (TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) >= h_starting_date
               AND TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) <= h_ending_date);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 273+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(274) : workflow.sql(5036) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW', 279+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW(280) : workflow.sql(5042) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 282+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(283) : workflow.sql(5045) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
      Else
         BEGIN
            INSERT INTO PMO_TMP_WKF_0006
            SELECT DISTINCT A.ELEMENT_ID, A.RANK, P.ELEMENT_ID AS ATTACHED_ID, P.PARENT_TYPE
            FROM TMT_WORKFLOWS A, TMT_WORKFLOWS P, WORKFLOW_REPLIES WR
            WHERE A.LEVEL_ID>0 AND WR.RESOURCE_ID = h_author_id
            AND WR.ELEMENT_ID = A.ELEMENT_ID
            AND WR.REPLY_STATUS = h_status
            AND P.RANK = SUBSTR(A.RANK, 1, 8)
            AND (TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) >= h_starting_date AND TO_DATE(SubStr(A.CREATED_DATE, 1, 10)) <= h_ending_date);
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 306+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(307) : workflow.sql(5069) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      --trace_printf("PPP11111111111111111");
      
      If Coalesce(h_type_id, 0) = 16 Then
         --trace_printf("\n CR0001");
         
         OPEN CR0001 FOR SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, TMP.ATTACHED_ID AS PARENT_ID, T.ELEMENT_ID,D.ELEMENT_NAME,
         1 AS CHILD_COUNT, T.DELETED_COUNT,
         CASE WHEN T.TYPE_ID = PMO_CNST.SC_NOTIFICATION THEN T.SUBJECT
         WHEN T.TYPE_ID IN (PMO_CNST.SC_STAFFING_REQUESTS, PMO_CNST.SC_ACCEPTED_SUGGEST) THEN T.ELEMENT_NAME
         ELSE T.ELEMENT_NAME || ' (' || D.ELEMENT_NAME || ')' END AS CYCLE_NAME,
         T.REVISION, T.REQUEST_ID, B.REC_DATETIME AS CREATED_DATE,
         T.CLOSED_DATE, T.AUTHOR_ID, R.FULL_NAME AS AUTHOR_NAME,
         CASE WHEN T.AUTHOR_ID IS NuLL THEN T.EMAIL_ADDRESS ELSE R.E_MAIL_ADDRESS END AS EMAIL,
         T.WKF_STATUS AS STATUS, T.SUBJECT,
         T.OUTCOME_ID, T.END_CYCLE, T.DESCRIPTION AS WKF_COMMENT,'' AS USER_COMMENT, T.PROJECT_NAME, T.WKF_DEF_ID,
         CASE WHEN T.END_CYCLE = '1' THEN 'L'
         WHEN EXISTS(SELECT ELEMENT_ID FROM WORKFLOW_REPLIES WHERE ELEMENT_ID = T.ELEMENT_ID AND COMPLETED = '0') THEN 'O' ELSE 'C' END AS ALL_VOTED,
         COALESCE(B.MARKED_READ, '1') AS READ, '' AS LAST_WKF, '' AS DOCUMENTS, '' AS CRI, T.PARENT_TYPE,
         TMP.ATTACHED_ID AS ATTACHED_ID, TMP.ATTACHED_TYPE AS ATTACHED_TYPE, T.ATTACHED_NAME,
         D.CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, P.MUST_OBEY, T.EXPIRES_DATE
         FROM TMT_WORKFLOWS T, WORKFLOW_REPLIES B, TMT_WORKFLOW_CYCLES D, TMT_WORKFLOW_CYCLES P,
         RESOURCES R, PMO_TMP_WKF_0006 TMP
         WHERE T.ELEMENT_ID = TMP.ELEMENT_ID
         AND NOT EXISTS
         (
         SELECT TP.ELEMENT_ID FROM PMO_TMP_WKF_0006 TP
         WHERE TP.ELEMENT_ID != T.ELEMENT_ID
         AND T.RANK >= SUBSTR(TP.RANK, 1, 8) AND T.RANK <= TP.RANK
         AND ((T.RANK >= SUBSTR(TP.RANK, 1, 8) AND T.RANK <= TP.RANK) 
         AND SUBSTR(TP.RANK,1, CASE WHEN LENGTH(T.RANK) > LENGTH(TP.RANK) THEN 1 ELSE LENGTH(T.RANK) END ) = T.RANK)
         )
         AND T.ELEMENT_ID = B.ELEMENT_ID AND B.RESOURCE_ID = h_author_id
         AND T.WKF_DEF_ID = D.ELEMENT_ID AND T.AUTHOR_ID = R.RESOURCE_ID
         AND D.PARENT_ID = P.ELEMENT_ID AND B.MARKED_READ = '0'
         ORDER BY CREATED_DATE DESC;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 348+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(349) : workflow.sql(5111) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN CR0001;
      Else
         --trace_printf("\n CRN001");
         
         OPEN CRN001 FOR SELECT 0 AS LEVEL_ID, T.TYPE_ID, T.RANK, TMP.ATTACHED_ID AS PARENT_ID, T.ELEMENT_ID,D.ELEMENT_NAME,
         1 AS CHILD_COUNT, T.DELETED_COUNT,
         CASE WHEN T.TYPE_ID = PMO_CNST.SC_NOTIFICATION THEN T.SUBJECT
         WHEN T.TYPE_ID IN (PMO_CNST.SC_STAFFING_REQUESTS, PMO_CNST.SC_ACCEPTED_SUGGEST) THEN T.ELEMENT_NAME
         ELSE T.ELEMENT_NAME || ' (' || D.ELEMENT_NAME || ')' END AS CYCLE_NAME,
         T.REVISION, T.REQUEST_ID, B.REC_DATETIME AS CREATED_DATE,
         T.CLOSED_DATE, T.AUTHOR_ID, R.FULL_NAME AS AUTHOR_NAME,
         CASE WHEN T.AUTHOR_ID IS NuLL THEN T.EMAIL_ADDRESS ELSE R.E_MAIL_ADDRESS END AS EMAIL,
         T.WKF_STATUS AS STATUS, T.SUBJECT,
         T.OUTCOME_ID, T.END_CYCLE, T.DESCRIPTION AS WKF_COMMENT,'' AS USER_COMMENT, T.PROJECT_NAME, T.WKF_DEF_ID,
         CASE WHEN T.END_CYCLE = '1' THEN 'L'
         WHEN EXISTS(SELECT ELEMENT_ID FROM WORKFLOW_REPLIES WHERE ELEMENT_ID = T.ELEMENT_ID AND COMPLETED = '0') THEN 'O' ELSE 'C' END AS ALL_VOTED,
         COALESCE(B.MARKED_READ, '1') AS READ, '' AS LAST_WKF, '' AS DOCUMENTS, '' AS CRI, T.PARENT_TYPE,
         TMP.ATTACHED_ID AS ATTACHED_ID,
         TMP.ATTACHED_TYPE AS ATTACHED_TYPE,
         T.ATTACHED_NAME,
         D.CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, P.MUST_OBEY , T.EXPIRES_DATE
         FROM TMT_WORKFLOWS T, WORKFLOW_REPLIES B, TMT_WORKFLOW_CYCLES D, TMT_WORKFLOW_CYCLES P,
         RESOURCES R, PMO_TMP_WKF_0006 TMP
         WHERE T.ELEMENT_ID = TMP.ELEMENT_ID
         AND NOT EXISTS
         (
         SELECT TP.ELEMENT_ID FROM PMO_TMP_WKF_0006 TP
         WHERE TP.ELEMENT_ID != T.ELEMENT_ID
         AND T.RANK >= SUBSTR(TP.RANK, 1, 8) AND T.RANK <= TP.RANK
         AND ((TP.RANK >= SUBSTR(T.RANK, 1, 8) AND TP.RANK <= T.RANK) 
         AND SUBSTR(T.RANK,1, CASE WHEN LENGTH(TP.RANK) > LENGTH(T.RANK) THEN 1 ELSE LENGTH(TP.RANK) END ) = TP.RANK)
         )
         AND T.ELEMENT_ID = B.ELEMENT_ID AND B.RESOURCE_ID = h_author_id
         AND T.WKF_DEF_ID = D.ELEMENT_ID AND T.AUTHOR_ID = R.RESOURCE_ID
         AND D.PARENT_ID = P.ELEMENT_ID
         ORDER BY CREATED_DATE DESC;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 387+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(388) : workflow.sql(5150) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN CRN001;
      End If;
   Else
      If Coalesce(h_bitflag, 0) = 16 Then
         BEGIN
            SELECT SUBSTR(RANK, 1, 8) INTO h_rankp
            FROM TMT_WORKFLOWS
            WHERE ELEMENT_ID = p_id;
            PMO_DB2M.SC_STATE := SQLCODE;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 403+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(404) : workflow.sql(5166) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW', 410+1) THEN 
               LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW(411) : workflow.sql(5173) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 413+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(414) : workflow.sql(5176) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
         End If;
         
         If (p_id2 IS NULL) Then
            BEGIN
               SELECT PARENT_TYPE, ELEMENT_ID INTO h_type, h_parent_id
               FROM TMT_WORKFLOWS
               WHERE RANK = h_rankp;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 429+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(430) : workflow.sql(5192) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW', 436+1) THEN 
                  LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW(437) : workflow.sql(5199) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
               END IF; 
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 439+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(440) : workflow.sql(5202) : ' || SYSTIMESTAMP);
               END IF; 
               PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
            End If;
            
            If Coalesce(h_type, 0) = PMO_CNST.SC_CUSTOMWORKFLOWS Then
               --trace_printf("PPP22222222222222222222");
               
               --trace_printf("\n CR0002");
               
               OPEN CR0002 FOR SELECT CASE WHEN a.ELEMENT_ID = h_id THEN 0 ELSE 1 END AS LEVEL_ID,
               a.TYPE_ID, a.RANK, a.PARENT_ID, a.ELEMENT_ID, D.ELEMENT_NAME,
               CASE WHEN a.ELEMENT_ID = h_id THEN 1 ELSE 0 END AS CHILD_COUNT, a.DELETED_COUNT,
               CASE WHEN a.TYPE_ID = PMO_CNST.SC_NOTIFICATION THEN a.SUBJECT
               WHEN a.TYPE_ID IN (PMO_CNST.SC_STAFFING_REQUESTS, PMO_CNST.SC_ACCEPTED_SUGGEST) THEN a.ELEMENT_NAME
               ELSE a.ELEMENT_NAME || ' (' || D.ELEMENT_NAME || ')' END AS CYCLE_NAME,
               a.REVISION,a.REQUEST_ID, a.CREATED_DATE, a.CLOSED_DATE,
               a.AUTHOR_ID, R.FULL_NAME AS AUTHOR_NAME,'' AS EMAIL, a.WKF_STATUS AS STATUS, a.SUBJECT, a.OUTCOME_ID, a.END_CYCLE, a.DESCRIPTION AS COMMENT, a.PROJECT_NAME,
               a.WKF_DEF_ID,
               CASE WHEN a.END_CYCLE = '1' THEN 'L'
               WHEN EXISTS(SELECT ELEMENT_ID FROM WORKFLOW_REPLIES WHERE ELEMENT_ID = a.ELEMENT_ID AND COMPLETED = '0') THEN 'O' ELSE 'C' END AS ALL_VOTED,
               WR.MARKED_READ AS READ, 'N' AS LAST_WKF,'' AS DOCUMENTS, '' AS CRI, a.PARENT_TYPE,
               '' AS ATTACHED_ID, 0 AS ATTACHED_TYPE, '' AS ATTACHED_NAME, '' AS CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, P.MUST_OBEY , a.EXPIRES_DATE
               FROM (
               (
               (
               (
               TMT_WORKFLOWS a JOIN TMT_WORKFLOWS b ON b.WKF_DEF_ID=a.WKF_DEF_ID
               AND a.RANK >= SUBSTR(b.RANK, 1, 8) AND a.RANK <= b.RANK
               AND substr(b.RANK, 1, case when length(a.RANK) > length(b.RANK) then 1 else length(a.RANK) end) = a.RANK
               )
               JOIN WORKFLOW_REPLIES WR ON a.ELEMENT_ID=WR.ELEMENT_ID AND WR.RESOURCE_ID=h_author_id
               )
               LEFT JOIN TMT_WORKFLOW_CYCLES D ON a.WKF_DEF_ID = d.ELEMENT_ID
               )
               LEFT JOIN TMT_WORKFLOW_CYCLES P ON P.ELEMENT_ID = D.PARENT_ID
               )
               JOIN RESOURCES R ON a.AUTHOR_ID = R.RESOURCE_ID
               WHERE b.ELEMENT_ID = h_id
               ORDER BY END_CYCLE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 480+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(481) : workflow.sql(5243) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN CR0002;
            Else
               h_attached_type := Coalesce(h_type, 0);
               h_stage_id := 0;
               h_attached_id := h_parent_id;
               
               --trace_printf("\n CR0003");
               
               OPEN CR0003 FOR SELECT 0 AS LEVEL_ID,T.TYPE_ID,T.RANK, T.PARENT_ID,T.ELEMENT_ID,
               D.ELEMENT_NAME, 1 AS CHILD_COUNT,T.DELETED_COUNT,
               CASE WHEN T.TYPE_ID = PMO_CNST.SC_NOTIFICATION THEN T.SUBJECT
               WHEN T.TYPE_ID IN (PMO_CNST.SC_STAFFING_REQUESTS, PMO_CNST.SC_ACCEPTED_SUGGEST) THEN T.ELEMENT_NAME
               ELSE T.ELEMENT_NAME || ' (' || D.ELEMENT_NAME || ')' END AS CYCLE_NAME,
               T.REVISION,T.REQUEST_ID,T.CREATED_DATE,T.CLOSED_DATE,T.AUTHOR_ID, R.FULL_NAME AS AUTHOR_NAME,
               CASE WHEN T.AUTHOR_ID IS NuLL THEN T.EMAIL_ADDRESS ELSE R.E_MAIL_ADDRESS END AS EMAIL,T.WKF_STATUS AS STATUS,
               T.SUBJECT,
               T.OUTCOME_ID, T.END_CYCLE,T.DESCRIPTION AS WKF_COMMENT,'' AS USER_COMMENT,T.PROJECT_NAME,T.WKF_DEF_ID,
               CASE WHEN T.END_CYCLE = '1' THEN 'L'
               WHEN EXISTS(SELECT ELEMENT_ID FROM WORKFLOW_REPLIES WHERE ELEMENT_ID = T.ELEMENT_ID AND COMPLETED = '0') THEN 'O' ELSE 'C' END AS ALL_VOTED,
               COALESCE(B.MARKED_READ, '1') AS READ, '' AS LAST_WKF, '' AS DOCUMENTS, '' AS CRI, T.PARENT_TYPE,
               h_attached_id AS ATTACHED_ID,
               h_attached_type AS ATTACHED_TYPE, T.ATTACHED_NAME,
               D.CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, P.MUST_OBEY , T.EXPIRES_DATE
               FROM TMT_WORKFLOWS T, WORKFLOW_REPLIES B, TMT_WORKFLOW_CYCLES D, TMT_WORKFLOW_CYCLES P, RESOURCES R
               WHERE T.ELEMENT_ID = h_id AND T.ELEMENT_ID=B.ELEMENT_ID
               AND B.RESOURCE_ID= h_author_id
               AND T.WKF_DEF_ID=D.ELEMENT_ID AND T.AUTHOR_ID = R.RESOURCE_ID AND P.ELEMENT_ID = D.PARENT_ID
               
               UNION ALL
               
               SELECT 1 AS LEVEL_ID, A.REPLY_TYPE AS TYPE_ID, '00' AS RANK,
               A.ELEMENT_ID AS PARENT_ID, A.WKF_REPLY_ID AS ELEMENT_ID, '' AS ELEMENT_NAME,
               0 AS CHILD_COUNT, 0 AS DELETED_COUNT, R.FULL_NAME AS CYCLE_NAME, '' AS REVISION,
               0 AS REQUEST_ID, A.REPLY_DATE AS CREATED_DATE, a.reply_date AS CLOSED_DATE, W.AUTHOR_ID,
               '' AS AUTHOR_NAME, R.E_MAIL_ADDRESS AS EMAIL,
               A.REPLY_STATUS AS STATUS, W.SUBJECT, 1 AS OUTCOME_ID, '' AS END_CYCLE,W.DESCRIPTION AS WKF_COMMENT ,A.DESCRIPTION AS USER_COMMENT,
               W.PROJECT_NAME AS PROJECT_NAME, '' AS WKF_DEF_ID,
               CASE WHEN A.COMPLETED = '1' THEN 'A' WHEN A.COMPLETED = '2' THEN 'B' ELSE '' END AS ALL_VOTED,
               CASE WHEN A.RESOURCE_ID = h_author_id THEN 'Y' ELSE 'N' END AS READ, '' AS LAST_WKF, '' AS DOCUMENT,
               '' AS CRI, W.PARENT_TYPE AS PARENT_TYPE,
               h_attached_id AS ATTACHED_ID,
               h_attached_type AS ATTACHED_TYPE, W.ATTACHED_NAME,
               C.CHECKLIST, Trunc(A.COMPLETED) AS COMPLETED, A.RESOURCE_ID, 0 AS MUST_OBEY , W.EXPIRES_DATE
               FROM WORKFLOW_REPLIES A, TMT_WORKFLOWS W, TMT_WORKFLOW_CYCLES C, RESOURCES R
               WHERE A.ELEMENT_ID = h_id AND A.REPLY_TYPE <> 0 AND A.ELEMENT_ID = W.ELEMENT_ID
               AND C.ELEMENT_ID=W.WKF_DEF_ID AND A.RESOURCE_ID = R.RESOURCE_ID
               
               UNION ALL
               
               SELECT 1 AS LEVEL_ID,T.TYPE_ID,T.RANK, T.PARENT_ID,T.ELEMENT_ID,
               D.ELEMENT_NAME, 1 AS CHILD_COUNT,T.DELETED_COUNT,
               CASE WHEN T.TYPE_ID = PMO_CNST.SC_NOTIFICATION THEN T.SUBJECT
               WHEN T.TYPE_ID IN (PMO_CNST.SC_STAFFING_REQUESTS, PMO_CNST.SC_ACCEPTED_SUGGEST) THEN T.ELEMENT_NAME
               ELSE T.ELEMENT_NAME || ' (' || D.ELEMENT_NAME || ')' END AS CYCLE_NAME,
               T.REVISION,T.REQUEST_ID,T.CREATED_DATE,T.CLOSED_DATE,T.AUTHOR_ID,R.FULL_NAME AS AUTHOR_NAME,
               CASE WHEN T.AUTHOR_ID IS NuLL THEN T.EMAIL_ADDRESS ELSE R.E_MAIL_ADDRESS END AS EMAIL,T.WKF_STATUS AS STATUS,
               T.SUBJECT,
               T.OUTCOME_ID, T.END_CYCLE,T.DESCRIPTION AS WKF_COMMENT,'' AS USER_COMMENT,T.PROJECT_NAME,T.WKF_DEF_ID, CASE WHEN T.END_CYCLE = '1' THEN 'L'
               WHEN EXISTS(SELECT ELEMENT_ID FROM WORKFLOW_REPLIES WHERE ELEMENT_ID = T.ELEMENT_ID AND COMPLETED = '0') THEN 'O' ELSE 'C' END AS ALL_VOTED,
               COALESCE(B.MARKED_READ, '1') AS READ, '' AS LAST_WKF, '' AS DOCUMENTS, '' AS CRI, T.PARENT_TYPE,
               h_attached_id AS ATTACHED_ID,
               h_attached_type AS ATTACHED_TYPE, T.ATTACHED_NAME,
               D.CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, P.MUST_OBEY  , T.EXPIRES_DATE
               FROM TMT_WORKFLOWS T, WORKFLOW_REPLIES B, TMT_WORKFLOW_CYCLES D, TMT_WORKFLOW_CYCLES P, RESOURCES R
               WHERE T.LEVEL_ID > 0
               AND T.ELEMENT_ID = ( SELECT P.PARENT_ID FROM TMT_WORKFLOWS P WHERE P.ELEMENT_ID = h_id )
               AND T.ELEMENT_ID = B.ELEMENT_ID AND B.RESOURCE_ID = h_author_id AND T.WKF_DEF_ID = D.ELEMENT_ID
               AND P.ELEMENT_ID = D.PARENT_ID AND T.AUTHOR_ID = R.RESOURCE_ID
               ORDER BY LEVEL_ID, RANK, CYCLE_NAME;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 552+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(553) : workflow.sql(5315) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN CR0003;
            End If;
         Else
            BEGIN
               SELECT LEVEL_ID,RANK INTO h_type ,h_rank
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_workflow_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 567+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(568) : workflow.sql(5330) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               --trace_printf("\n CR0004");
               
               OPEN CR0004 FOR SELECT (1 + T.LEVEL_ID) AS LEVEL_ID, T.TYPE_ID,
               (h_rankp || T.RANK) AS RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,  T.CHILD_COUNT,
               T.DELETED_COUNT, '' AS CYCLE_NAME, '' AS REVISION, '' AS REQUEST_ID,
               TO_TIMESTAMP('1899-12-31 00:00:00')  AS CREATED_DATE,
               TO_TIMESTAMP('1899-12-31 00:00:00') AS CLOSED_DATE, '' AS AUTHOR_ID, '' AS AUTHOR_NAME,'' AS EMAIL,
               0 AS STATUS, '' AS SUBJECT, '' AS OUTCOME_ID, '' AS END_CYCLE, '' AS COMMENT,'' AS PROJECT_NAME,'' AS WKF_DEF_ID, '' ALL_VOTED, '' AS READ, 'N' AS LAST_WKF,
               CASE WHEN EXISTS(SELECT PARENT_ID FROM TMT_DOCUMENTS WHERE PARENT_ID = T.ELEMENT_ID) THEN 'Y' ELSE 'N' END AS DOCUMENTS, '' AS CRI, 1 AS PARENT_TYPE,
               '' AS ATTACHED_ID, 0 AS ATTACHED_TYPE, '' AS ATTACHED_NAME, '' AS CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, 0 AS MUST_OBEY
               FROM TMT_WBS T
               WHERE T.RANK LIKE h_rank || '%'
               AND T.LEVEL_ID <= h_type + 1
               ORDER BY RANK;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 588+1) THEN 
                  LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(589) : workflow.sql(5351) : ' || SYSTIMESTAMP);
               END IF; 
               RETURN CR0004;
            ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
               BEGIN
                  SELECT RANK INTO h_rank FROM TMT_CRI WHERE ELEMENT_ID = h_workflow_id;
                  PMO_DB2M.SC_STATE := SQLCODE;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 600+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(601) : workflow.sql(5363) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
               
               If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                  --trace_printf("PPP555555555555555555555");
                  
                  --trace_printf("\n CR0005");
                  
                  OPEN CR0005 FOR SELECT (1+T.LEVEL_ID) AS LEVEL_ID, T.TYPE_ID, (h_rankp || T.RANK) AS RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME,  T.CHILD_COUNT,
                  T.DELETED_COUNT,'' AS CYCLE_NAME, '' AS REVISION, '' AS REQUEST_ID,
                  TO_TIMESTAMP('1899-12-31 00:00:00') AS CREATED_DATE,
                  TO_TIMESTAMP('1899-12-31 00:00:00') AS CLOSED_DATE, '' AS AUTHOR_ID, '' AS AUTHOR_NAME,'' AS EMAIL,
                  0 AS STATUS, '' AS SUBJECT, '' AS OUTCOME_ID, '' AS END_CYCLE, '' AS COMMENT,'' AS PROJECT_NAME,'' AS WKF_DEF_ID, '' ALL_VOTED, '' AS READ, 'N' AS LAST_WKF,
                  CASE WHEN EXISTS(SELECT PARENT_ID FROM TMT_DOCUMENTS WHERE PARENT_ID = T.ELEMENT_ID) THEN 'Y' ELSE 'N' END AS DOCUMENTS, '' AS CRI, 2 AS PARENT_TYPE,
                  '' AS ATTACHED_ID, 0 AS ATTACHED_TYPE, '' AS ATTACHED_NAME, '' AS CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, 0 AS MUST_OBEY
                  FROM TMT_CRI T WHERE T.ELEMENT_ID = h_workflow_id OR
                  (T.LEVEL_ID = h_type + 1 AND T.PARENT_ID = h_workflow_id ) ORDER BY RANK;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 620+1) THEN 
                     LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(621) : workflow.sql(5383) : ' || SYSTIMESTAMP);
                  END IF; 
                  RETURN CR0005;
               ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                  BEGIN
                     SELECT RANK INTO h_rank FROM TMT_DOCUMENTS WHERE ELEMENT_ID = h_workflow_id;
                     PMO_DB2M.SC_STATE := SQLCODE;
                  EXCEPTION
                     WHEN OTHERS THEN
                     BEGIN
                        PMO_DB2M.SC_STATE := SQLCODE;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 632+1) THEN 
                           LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(633) : workflow.sql(5395) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                        END IF; 
                     END;
                  END;
                  
                  If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                     --trace_printf("PPP66666666666666666666");
                     
                     --trace_printf("\n CR0006");
                     
                     OPEN CR0006 FOR SELECT (1+LEVEL_ID) AS LEVEL_ID, TYPE_ID, (h_rankp || RANK) AS RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,  CHILD_COUNT,
                     DELETED_COUNT,'' AS CYCLE_NAME, '' AS REVISION, '' AS REQUEST_ID,
                     TO_TIMESTAMP('1899-12-31 00:00:00') AS CREATED_DATE,
                     TO_TIMESTAMP('1899-12-31 00:00:00') AS CLOSED_DATE, '' AS AUTHOR_ID, '' AS AUTHOR_NAME,'' AS EMAIL,
                     0 AS STATUS, '' AS SUBJECT, '' AS OUTCOME_ID, '' AS END_CYCLE, '' AS COMMENT,'' AS PROJECT_NAME,'' AS WKF_DEF_ID, '' ALL_VOTED, '' AS READ, 'N' AS LAST_WKF, '' AS DOCUMENTS, '' AS CRI, 3 AS PARENT_TYPE,
                     '' AS ATTACHED_ID, 0 AS ATTACHED_TYPE, '' AS ATTACHED_NAME, '' AS CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, 0 AS MUST_OBEY
                     FROM TMT_DOCUMENTS WHERE ELEMENT_ID = h_workflow_id OR
                     (LEVEL_ID = h_type + 1 AND
                     PARENT_ID = h_workflow_id ) ORDER BY RANK;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 652+1) THEN 
                        LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(653) : workflow.sql(5415) : ' || SYSTIMESTAMP);
                     END IF; 
                     RETURN CR0006;
                  ElsIf PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_NO_DATA Then
                     BEGIN
                        SELECT RANK INTO h_rank FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID = h_workflow_id;
                        PMO_DB2M.SC_STATE := SQLCODE;
                     EXCEPTION
                        WHEN OTHERS THEN
                        BEGIN
                           PMO_DB2M.SC_STATE := SQLCODE;
                           IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 664+1) THEN 
                              LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(665) : workflow.sql(5427) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                           END IF; 
                        END;
                     END;
                     
                     If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
                        --trace_printf("PPP77777777777777777777777");
                        
                        --trace_printf("\n CR0007");
                        
                        OPEN CR0007 FOR SELECT (1+LEVEL_ID) AS LEVEL_ID, TYPE_ID, (h_rankp || RANK) AS RANK, PARENT_ID, ELEMENT_ID, ELEMENT_NAME,  CHILD_COUNT,
                        DELETED_COUNT,'' AS CYCLE_NAME, '' AS REVISION, '' AS REQUEST_ID,
                        TO_TIMESTAMP('1899-12-31 00:00:00') AS CREATED_DATE,
                        TO_TIMESTAMP('1899-12-31 00:00:00') AS CLOSED_DATE, '' AS AUTHOR_ID, '' AS AUTHOR_NAME,'' AS EMAIL,
                        0 AS STATUS, '' AS SUBJECT, '' AS OUTCOME_ID, '' AS END_CYCLE, '' AS COMMENT,'' AS PROJECT_NAME,'' AS WKF_DEF_ID, '' ALL_VOTED, '' AS READ, 'N' AS LAST_WKF, '' AS DOCUMENTS, '' AS CRI, 10 AS PARENT_TYPE,
                        '' AS ATTACHED_ID, 0 AS ATTACHED_TYPE, '' AS ATTACHED_NAME, '' AS CHECKLIST, 0 AS COMPLETED, '' AS RESOURCE_ID, 0 AS MUST_OBEY
                        FROM TMT_ASSIGNED_SCORECARDS WHERE ELEMENT_ID = h_workflow_id OR
                        (LEVEL_ID = h_type + 1 AND
                        PARENT_ID = h_workflow_id ) ORDER BY RANK;
                        IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 684+1) THEN 
                           LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(685) : workflow.sql(5447) : ' || SYSTIMESTAMP);
                        END IF; 
                        RETURN CR0007;
                     End If;
                  End If;
               End If;
            End If;
         End If;
      ElsIf Coalesce(h_bitflag, 0) = 4 Then
         --trace_printf("PPP88888888888888888888888");
         
         --trace_printf("\n CR0008");
         
         OPEN CR0008 FOR SELECT A.WKF_REPLY_ID AS ELEMENT_ID, A.ELEMENT_ID AS PARENT_ID, A.REPLY_TYPE AS TYPE_ID, A.REPLY_DATE, A.RESOURCE_ID, R.FULL_NAME, R.E_MAIL_ADDRESS,
         A.REPLY_STATUS, Trunc(A.COMPLETED) AS COMPLETED, W.SUBJECT, W.CHECKLIST, A.DESCRIPTION AS USER_COMMENT,
         CASE WHEN A.RESOURCE_ID = h_author_id THEN 'Y' ELSE 'N' END AS ME, '1' AS ICON
         FROM ((WORKFLOW_REPLIES A JOIN TMT_WORKFLOWS W ON A.ELEMENT_ID=W.ELEMENT_ID) LEFT JOIN TMT_WORKFLOW_CYCLES C ON
         C.PARENT_ID=W.WKF_DEF_ID AND C.REQUEST_ID=W.REQUEST_ID) JOIN RESOURCES R ON A.RESOURCE_ID = R.RESOURCE_ID
         WHERE A.ELEMENT_ID = h_id ORDER BY R.FULL_NAME;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 704+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(705) : workflow.sql(5467) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN CR0008;
      Else
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_WORKFLOW', 709+1) THEN 
            LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_WORKFLOW(710) : workflow.sql(5472) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_WORKFLOW', 712+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_WORKFLOW(713) : workflow.sql(5475) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
      End If;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 722+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(723) : workflow.sql(5485) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_WORKFLOW', 734+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_WORKFLOW(735) : workflow.sql(5497) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_WORKFLOW;
ALTER FUNCTION SP_S_WORKFLOW COMPILE DEBUG;

/*****************************************************************************************
    It is not used
******************************************************************************************/

Prompt == FUNCTION SP_M_WORKFLOW ==
CREATE OR REPLACE FUNCTION SP_M_WORKFLOW
(
     p_target_id VARCHAR2
   , p_source_id VARCHAR2
   , p_bitflag PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_target_id VARCHAR2(32);
   h_source_id VARCHAR2(32);
   h_target_level PLS_INTEGER := 0;
   h_source_level PLS_INTEGER := 0;
   h_len PLS_INTEGER := 0;
   h_target_rank VARCHAR2(200);
   h_source_rank VARCHAR2(200);
   h_parent_rank VARCHAR2(200);
   h_target_parent_id VARCHAR2(32);
   h_source_parent_id VARCHAR2(32);
   h_cur_rank VARCHAR2(200);
   h_rankBase64 VARCHAR2(8);
   h_cur_date TIMESTAMP;
   h_max_rank PLS_INTEGER := 0;
   h_rec_status VARCHAR2(1);
   result PLS_INTEGER NOT NULL := -1;
   iparent PLS_INTEGER NOT NULL := -1;
-- Declare Cursors
      CURSOR C2 IS
         SELECT RANK FROM TMT_WORKFLOWS WHERE RANK LIKE h_parent_rank || '%' AND RANK >= h_target_rank FOR UPDATE OF RANK, REC_USER, REC_DATETIME, REC_STATUS;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_M_WORKFLOW') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_M_WORKFLOW'), '-- ' || SYSTIMESTAMP, ':r := SP_M_WORKFLOW(' || 
       '''' || p_target_id || ''''
          || ', ' || '''' || p_source_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_WORKFLOW', 40+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_M_WORKFLOW(41) : workflow.sql(5556) : ' || SYSTIMESTAMP);
   END IF; 

   -- Declare host variables
   
   -- Validate target  source id
   If (p_target_id IS NULL) OR (p_source_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_WORKFLOW', 48+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_WORKFLOW(49) : workflow.sql(5564) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_WORKFLOW', 51+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_WORKFLOW(52) : workflow.sql(5567) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Parse parameters to local variables
   h_target_id := p_target_id;
   h_source_id := p_source_id;
   
   If NOT ((Coalesce(p_bitflag, 0) = 0) OR (Coalesce(p_bitflag, 0) = 1)) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_M_WORKFLOW', 62+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_M_WORKFLOW(63) : workflow.sql(5578) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.BITFLAG_OUT_OF_RANGE), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_M_WORKFLOW', 65+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_M_WORKFLOW(66) : workflow.sql(5581) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.BITFLAG_OUT_OF_RANGE, True);
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_M_WORKFLOW', 71+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_M_WORKFLOW(72) : workflow.sql(5587) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_WORKFLOWS' || '''' || ' ID: ' || '''' || h_target_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_WORKFLOWS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_target_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_target_id, True); 
   END;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1024, 'SP_M_WORKFLOW', 83+1) THEN 
      LFILE.FPUT('AllTrace', '     VALIDATE_RECORD: SP_M_WORKFLOW(84) : workflow.sql(5599) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_WORKFLOWS' || '''' || ' ID: ' || '''' || h_source_id || '''');
   END IF; 
   BEGIN 
      EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_WORKFLOWS' || ' WHERE ELEMENT_ID = :i' 
         INTO h_rec_status 
         Using h_source_id; 
   EXCEPTION 
      WHEN NO_DATA_FOUND THEN 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_source_id, True); 
   END;
   
   -- Declare local variable
   
   h_target_rank := '';
   h_target_parent_id := '';
   h_source_parent_id := '';
   BEGIN
      SELECT RANK,LEVEL_ID,PARENT_ID INTO h_target_rank,h_target_level,h_target_parent_id
      FROM TMT_WORKFLOWS WHERE ELEMENT_ID = h_target_id;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 108+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(109) : workflow.sql(5624) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
      BEGIN
         SELECT RANK,LENGTH(RANK),LEVEL_ID,PARENT_ID INTO h_source_rank,h_len,h_source_level,h_source_parent_id FROM TMT_WORKFLOWS WHERE ELEMENT_ID = h_source_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 122+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(123) : workflow.sql(5638) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      BEGIN
         SELECT (SYSTIMESTAMP)  INTO  h_cur_date FROM DUAL;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 135+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(136) : workflow.sql(5651) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      --h_bitflag = 0, the target is the new parent for the source, copy the source as last child of the target
      --h_bitflag = 1, insert the source to the target position, new source's parent is the target's parent
      If Coalesce(p_bitflag, 0) = 1 Then
         iparent := PMO_ORASTD.strcmp(h_target_parent_id, h_source_parent_id);
         If (iparent = 0) AND (Coalesce(h_source_level, 0) = Coalesce(h_target_level, 0)) Then
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_target_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_target_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_target_rank, True); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_WORKFLOWS SET REC_STATUS='U', REC_USER = p_rec_user, REC_DATETIME = h_cur_date,
               RANK =
               CASE
               WHEN LENGTH(RANK) = h_len THEN RTRIM(h_source_rank)
               ELSE RTRIM(h_source_rank) || SUBSTR(RANK,h_len +1,LENGTH(RANK)-h_len)
               END WHERE RANK LIKE h_target_rank || '%';
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 175+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(176) : workflow.sql(5691) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_source_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_source_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, True); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_WORKFLOWS SET REC_STATUS='U', REC_USER = p_rec_user, REC_DATETIME = h_cur_date,
               RANK =
               CASE
               WHEN LENGTH(RANK) = h_len THEN RTRIM(h_target_rank)
               ELSE RTRIM(h_target_rank) || SUBSTR(RANK,h_len +1,LENGTH(RANK)-h_len)
               END WHERE RANK LIKE h_source_rank || '%' AND (REC_DATETIME IS NuLL OR NOT REC_DATETIME = h_cur_date);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 210+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(211) : workflow.sql(5726) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         Else
            BEGIN
               SELECT SUBSTR(RANK,1,LENGTH(RANK)-4) INTO h_parent_rank FROM TMT_WORKFLOWS
               WHERE ELEMENT_ID = h_target_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 224+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(225) : workflow.sql(5740) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            OPEN C2;
            LOOP
               FETCH C2 INTO h_cur_rank;
               IF C2%FOUND THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
               If PMO_DB2M.SC_STATE <> 0 Then
                  EXIT;
               End If;
               
               --increments by 1
               PMO_STPU.incrBase64(h_cur_rank, Coalesce(h_target_level, 0) + 1);
               
               BEGIN
                  UPDATE TMT_WORKFLOWS SET REC_STATUS='U', REC_USER = p_rec_user, RANK = h_cur_rank,
                  REC_DATETIME = h_cur_date WHERE CURRENT OF C2;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 257+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(258) : workflow.sql(5773) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            END LOOP;
            BEGIN
               CLOSE C2 ;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 270+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(271) : workflow.sql(5786) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            DECLARE 
            	nLen pls_integer :=0; 
            BEGIN 
            	If (h_source_rank IS NOT NULL) Then 
            		nLen := Coalesce(Length(h_source_rank), 0); 
            		If 0 <> 0 AND nLen <> 0 Then 
            		nLen := nLen - 1; 
            		End If; 
            	End If; 
            	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
            		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, True); 
            	End If; 
            END;
            BEGIN
               UPDATE TMT_WORKFLOWS SET REC_STATUS='U', REC_USER = p_rec_user, REC_DATETIME = h_cur_date,
               LEVEL_ID = LEVEL_ID + (h_target_level - h_source_level) - 1, RANK =
               CASE
               WHEN LENGTH(RANK) = h_len THEN RTRIM(h_target_rank)
               ELSE RTRIM(h_target_rank) || SUBSTR(RANK,h_len +1,LENGTH(RANK)-h_len)
               END
               WHERE RANK LIKE h_source_rank || '%' AND (REC_DATETIME IS NuLL OR NOT REC_DATETIME = h_cur_date);
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 306+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(307) : workflow.sql(5822) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
         
         -- update the parent id field
         BEGIN
            UPDATE TMT_WORKFLOWS SET PARENT_ID = h_target_parent_id WHERE ELEMENT_ID = p_source_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 325+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(326) : workflow.sql(5841) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WORKFLOWS SET CHILD_COUNT = CHILD_COUNT + 1 WHERE ELEMENT_ID = p_target_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 342+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(343) : workflow.sql(5858) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WORKFLOWS SET CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID = p_source_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 359+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(360) : workflow.sql(5875) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      Else
         PMODEFINES.PMO_GET_MAX_RANK('TMT_WORKFLOWS', h_target_id, h_max_rank);
         PMO_STPU.toBase64(Coalesce(h_max_rank, 0), h_rankBase64);
         
         -- update the level, rec_user, rec_datetime, rec_status, rank
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_source_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_source_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.INVALID_RANK_LIKE, h_source_rank, True); 
         	End If; 
         END;
         BEGIN
            UPDATE TMT_WORKFLOWS SET LEVEL_ID = LEVEL_ID + (h_target_level - h_source_level) - 1,
            REC_USER = p_rec_user, REC_DATETIME = h_cur_date, REC_STATUS = 'U', RANK =
            CASE
            WHEN LENGTH(RANK) = h_len THEN RTRIM(h_target_rank) || h_rankBase64
            ELSE RTRIM(h_target_rank) || h_rankBase64 || SUBSTR(RANK,h_len +1,LENGTH(RANK)-h_len)
            END
            WHERE RANK LIKE h_source_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 399+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(400) : workflow.sql(5915) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         --update new parent for the source
         BEGIN
            UPDATE TMT_WORKFLOWS SET PARENT_ID = p_target_id WHERE ELEMENT_ID = p_source_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 417+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(418) : workflow.sql(5933) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         result := PMO_ORASTD.strcmp(h_target_id, h_source_parent_id);
         If result <> 0 Then
            BEGIN
               UPDATE TMT_WORKFLOWS SET CHILD_COUNT = CHILD_COUNT + 1 WHERE ELEMENT_ID = p_target_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 436+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(437) : workflow.sql(5952) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_WORKFLOWS SET CHILD_COUNT = CHILD_COUNT - 1 WHERE ELEMENT_ID = p_source_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 453+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(454) : workflow.sql(5969) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
         End If;
      End If;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_M_WORKFLOW', 462+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_M_WORKFLOW(463) : workflow.sql(5978) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || h_target_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, h_target_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 469+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(470) : workflow.sql(5985) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_M_WORKFLOW', 481+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_M_WORKFLOW(482) : workflow.sql(5997) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_M_WORKFLOW;
ALTER FUNCTION SP_M_WORKFLOW COMPILE DEBUG;

/*****************************************************************************************/

Prompt == FUNCTION SP_S_ADDR_PROJECT ==
CREATE OR REPLACE FUNCTION SP_S_ADDR_PROJECT
(
     p_id VARCHAR2
   , p_bitflag PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_id VARCHAR2(32);
   h_sql VARCHAR2(4000);
   h_rec_status VARCHAR2(1);
-- Declare Cursors
      -- Resultset Cursor Declaration
      C66678a PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C66678b PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C66678c PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_ADDR_PROJECT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_ADDR_PROJECT'), '-- ' || SYSTIMESTAMP, ':r := SP_S_ADDR_PROJECT(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_bitflag), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADDR_PROJECT', 28+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_ADDR_PROJECT(29) : workflow.sql(6042) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_ADDR_PROJECT', 33+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_ADDR_PROJECT(34) : workflow.sql(6047) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADDR_PROJECT', 36+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ADDR_PROJECT(37) : workflow.sql(6050) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   -- Declare host variables
   
   h_id := p_id;
   
   If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 2) Then
      BEGIN 
         SELECT REC_STATUS INTO h_rec_status FROM RESOURCES WHERE RESOURCE_ID = h_id; 
         IF SubStr(h_rec_status, 1, 1) IN ('D', 'L', 'A') THEN 
            PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.PARENT_MARKED_AS_DELETED, True); 
         END IF; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      OPEN C66678a FOR SELECT 0 AS LEVEL_ID, PMO_CNST.SC_RESOURCE AS TYPE_ID, '00000000' || R.FULL_NAME AS RANK, '' AS PARENT_ID, R.RESOURCE_ID AS ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME,
      (SELECT COUNT(RES_ASSIGNMENT_ID) FROM RESOURCE_ASSIGNMENTS WHERE RESOURCE_ID = h_id) AS CHILD_COUNT,
      0 AS DELETED_COUNT, R.REC_STATUS, R.TITLE AS TITLE,
      (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
      (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
      (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = R.RESOURCE_ID AND
      PARENT_ID = R.PRIMARY_ROLE))), ', '), 
      (SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
      (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = R.RESOURCE_ID AND
      PARENT_ID = R.PRIMARY_ROLE)))) AS COMPETENCY
      FROM RESOURCES R WHERE R.RESOURCE_ID = h_id
      UNION ALL
      SELECT 1 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, 1 AS CHILD_COUNT,
      T.DELETED_COUNT, T.REC_STATUS, '' AS TITLE, '' AS COMPETENCY
      FROM TMT_WBS T, PROJECT_ASSIGNMENTS R
      WHERE T.TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3) AND T.ELEMENT_ID = T.PROJECT_ID AND T.REC_STATUS IN ('I', 'O', 'U')
      AND T.PROJECT_ID = R.PROJECT_ID AND R.RESOURCE_ID = h_id ORDER BY RANK;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADDR_PROJECT', 74+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_ADDR_PROJECT(75) : workflow.sql(6088) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C66678a;
   Else
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(64, 'SP_S_ADDR_PROJECT', 79+1) THEN 
         LFILE.FPUT('AllTrace', '  VALIDATE_EXISTENCE: SP_S_ADDR_PROJECT(80) : workflow.sql(6093) : ' || SYSTIMESTAMP || ' ' || 'TABLE: ' || '''' || 'TMT_WBS' || '''' || ' ID: ' || '''' || h_id || '''');
      END IF; 
      BEGIN 
         EXECUTE IMMEDIATE 'SELECT REC_STATUS INTO :r FROM ' || 'TMT_WBS' || ' WHERE ELEMENT_ID = :i' 
            INTO h_rec_status 
            Using h_id; 
      EXCEPTION 
         WHEN NO_DATA_FOUND THEN 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True); 
      END;
      
      If (NOT ((p_bitflag IS NULL))) AND (Coalesce(p_bitflag, 0) = 1) Then
         OPEN C66678b FOR SELECT 1 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, '' AS TITLE, '' AS COMPETENCY
         FROM TMT_WBS T WHERE T.ELEMENT_ID = h_id
         UNION ALL
         SELECT DISTINCT 2 AS LEVEL_ID, PMO_CNST.SC_RESOURCE AS TYPE_ID, R.FULL_NAME AS RANK, T.PROJECT_ID AS PARENT_ID, R.RESOURCE_ID AS ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME, 0 AS CHILD_COUNT, 0 AS DELETED_COUNT, R.REC_STATUS, R.TITLE AS TITLE,
         (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = R.RESOURCE_ID AND
         PARENT_ID = R.PRIMARY_ROLE))), ', '), 
         (SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = R.RESOURCE_ID AND
         PARENT_ID = R.PRIMARY_ROLE)))) AS COMPETENCY
         FROM (TMT_WBS T JOIN RESOURCE_ASSIGNMENTS A ON T.ELEMENT_ID = A.ELEMENT_ID) /*LEFT*/ JOIN RESOURCES R ON A.RESOURCE_ID = R.RESOURCE_ID
         WHERE T.PROJECT_ID = h_id AND A.RESOURCE_TYPE = '0' AND T.REC_STATUS IN ('I', 'O', 'U')
         ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADDR_PROJECT', 106+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_ADDR_PROJECT(107) : workflow.sql(6120) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C66678b;
         h_sql := h_sql || ' ';
      Else
         OPEN C66678c FOR SELECT 1 AS LEVEL_ID, T.TYPE_ID, T.RANK, T.PARENT_ID, T.ELEMENT_ID, T.ELEMENT_NAME, T.CHILD_COUNT, T.DELETED_COUNT, T.REC_STATUS, '' AS TITLE, '' AS COMPETENCY
         FROM TMT_WBS T WHERE T.ELEMENT_ID = h_id
         UNION ALL
         SELECT DISTINCT 2 AS LEVEL_ID, PMO_CNST.SC_RESOURCE AS TYPE_ID, R.FULL_NAME AS RANK, T.PROJECT_ID AS PARENT_ID, R.RESOURCE_ID AS ELEMENT_ID, R.FULL_NAME AS ELEMENT_NAME, 0 AS CHILD_COUNT, 0 AS DELETED_COUNT, R.REC_STATUS, R.TITLE AS TITLE,
         (PMO_ORASTD.StringConcat(PMO_ORASTD.StringConcat((SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = R.RESOURCE_ID AND
         PARENT_ID = R.PRIMARY_ROLE))), ', '), 
         (SELECT RTRIM(ELEMENT_NAME) FROM TMT_COMPETENCIES WHERE ELEMENT_ID =
         (SELECT PARENT_ID FROM TMT_COMPETENCIES WHERE RESOURCE_ID = R.RESOURCE_ID AND
         PARENT_ID = R.PRIMARY_ROLE)))) AS COMPETENCY
         FROM (TMT_WBS T /*LEFT*/ JOIN RESOURCE_ASSIGNMENTS A ON T.ELEMENT_ID = A.ELEMENT_ID) /*LEFT*/ JOIN RESOURCES R ON A.RESOURCE_ID = R.RESOURCE_ID
         WHERE T.PROJECT_ID = h_id AND A.RESOURCE_TYPE = '0' ORDER BY RANK;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_ADDR_PROJECT', 125+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_S_ADDR_PROJECT(126) : workflow.sql(6139) : ' || SYSTIMESTAMP);
         END IF; 
         RETURN C66678c;
      End If;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ADDR_PROJECT', 135+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ADDR_PROJECT(136) : workflow.sql(6149) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_ADDR_PROJECT', 147+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_ADDR_PROJECT(148) : workflow.sql(6161) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_ADDR_PROJECT;
ALTER FUNCTION SP_S_ADDR_PROJECT COMPILE DEBUG;

/**********************************************************************************************************/

Prompt == FUNCTION SP_S_STAGE_ID ==
CREATE OR REPLACE FUNCTION SP_S_STAGE_ID
(
     p_id VARCHAR2
   , p_type_id PLS_INTEGER
) RETURN PMO_DB2M.REF_CURSOR
AS
-- Declare Cursors
      -- Resultset Cursor Declaration
      C_SW_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SC_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SD_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SCD_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SRD_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SPD_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SAD_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SCLND_STAGE PMO_DB2M.REF_CURSOR;

      -- Resultset Cursor Declaration
      C_SVD_STAGE PMO_DB2M.REF_CURSOR;

   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_S_STAGE_ID') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_S_STAGE_ID'), '-- ' || SYSTIMESTAMP, ':r := SP_S_STAGE_ID(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL') || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 43+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_S_STAGE_ID(44) : workflow.sql(6221) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_STAGE_ID', 48+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_STAGE_ID(49) : workflow.sql(6226) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 51+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(52) : workflow.sql(6229) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If (p_type_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_S_STAGE_ID', 58+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_S_STAGE_ID(59) : workflow.sql(6236) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 61+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(62) : workflow.sql(6239) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   If Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_WBS Then
      OPEN C_SW_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_WBS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 71+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(72) : workflow.sql(6249) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SW_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_CRI Then
      OPEN C_SC_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_CRI
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 79+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(80) : workflow.sql(6257) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SC_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_DOC Then
      OPEN C_SD_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_DOCUMENTS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 87+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(88) : workflow.sql(6265) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SD_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_SCD Then
      OPEN C_SCD_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_ASSIGNED_SCORECARDS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 95+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(96) : workflow.sql(6273) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SCD_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_RES_DOC Then
      OPEN C_SRD_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_RESOURCE_DOCUMENTS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 103+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(104) : workflow.sql(6281) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SRD_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_POL_DOC Then
      OPEN C_SPD_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_POOL_DOCUMENTS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 111+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(112) : workflow.sql(6289) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SPD_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_AST_DOC Then
      OPEN C_SAD_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_ASSET_DOCUMENTS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 119+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(120) : workflow.sql(6297) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SAD_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_CLN_DOC Then
      OPEN C_SCLND_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_CLIENT_DOCUMENTS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 127+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(128) : workflow.sql(6305) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SCLND_STAGE;
   ElsIf Coalesce(p_type_id, 0) = PMO_CNST.SC_TABLE_VDR_DOC Then
      OPEN C_SVD_STAGE FOR SELECT ELEMENT_ID, STAGE_ID
      FROM TMT_VENDOR_DOCUMENTS
      WHERE ELEMENT_ID = p_id;
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_S_STAGE_ID', 135+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_S_STAGE_ID(136) : workflow.sql(6313) : ' || SYSTIMESTAMP);
      END IF; 
      RETURN C_SVD_STAGE;
   End If;
   
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_STAGE_ID', 144+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_STAGE_ID(145) : workflow.sql(6322) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_S_STAGE_ID', 156+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_S_STAGE_ID(157) : workflow.sql(6334) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_S_STAGE_ID;
ALTER FUNCTION SP_S_STAGE_ID COMPILE DEBUG;

/**********************************************************************************************************/

Prompt == FUNCTION SP_U_STAGE_ID ==
CREATE OR REPLACE FUNCTION SP_U_STAGE_ID
(
     p_id VARCHAR2
   , p_type_id PLS_INTEGER
   , p_stage_id PLS_INTEGER
   , p_rec_user VARCHAR2
) RETURN PMO_DB2M.REF_CURSOR
AS
   h_name VARCHAR2(100);
   h_id VARCHAR2(32);
   h_wbs_id VARCHAR2(32);
   h_project_id VARCHAR2(32);
   h_rec_user VARCHAR2(32);
   h_effort_driven PLS_INTEGER := 0;
   h_temp_var PLS_INTEGER := 0;
   h_type_id PLS_INTEGER := 0;
   h_transfer_flag VARCHAR2(1);
   h_a_cri_count PLS_INTEGER := 0;
   h_close_stage_id PLS_INTEGER := 0;
   h_fd PLS_INTEGER := 0;
   h_stage_id PLS_INTEGER := 0;
   h_item_type_id PLS_INTEGER := 0;
   h_wbs_transfered_task PLS_INTEGER;
-- Declare Cursors
   resultSet PMO_DB2M.REF_CURSOR;
BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('SP_U_STAGE_ID') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('SP_U_STAGE_ID'), '-- ' || SYSTIMESTAMP, ':r := SP_U_STAGE_ID(' || 
       '''' || p_id || ''''
          || ', ' || COALESCE(TO_CHAR(p_type_id), 'NULL')
          || ', ' || COALESCE(TO_CHAR(p_stage_id), 'NULL')
          || ', ' || '''' || p_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_STAGE_ID', 35+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: SP_U_STAGE_ID(36) : workflow.sql(6386) : ' || SYSTIMESTAMP);
   END IF; 

   If (p_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_STAGE_ID', 40+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_STAGE_ID(41) : workflow.sql(6391) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.ELEMENT_ID_REQUIRED), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_STAGE_ID', 43+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_STAGE_ID(44) : workflow.sql(6394) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.ELEMENT_ID_REQUIRED, True);
   End If;
   
   If (p_stage_id IS NULL) OR (p_type_id IS NULL) Then
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(8, 'SP_U_STAGE_ID', 50+1) THEN 
         LFILE.FPUT('AllTrace', '      RETURN_INVALID: SP_U_STAGE_ID(51) : workflow.sql(6401) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.INVALID_PARAMS), 'NULL'));
      END IF; 
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_STAGE_ID', 53+1) THEN 
         LFILE.FPUT('AllTrace', '                EXIT: SP_U_STAGE_ID(54) : workflow.sql(6404) : ' || SYSTIMESTAMP);
      END IF; 
      PMODEFINES.PMO_RETURN_INVALID_SP(PMO_CNST.INVALID_PARAMS, True);
   End If;
   
   h_id := p_id;
   h_rec_user := p_rec_user;
   h_stage_id := p_stage_id;
   h_item_type_id := p_type_id;
   
   -- for TRANSFERED CRI_TASKs we should process like wbs-task elements
   h_wbs_transfered_task := 0;
   If Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_CRI Then
      BEGIN
         SELECT PROJECT_ID, TYPE_ID, PLANNED_EFFORT + SCHED_EFFORT, WBS_ID
         INTO h_project_id, h_type_id, h_effort_driven, h_wbs_id
         FROM TMT_CRI
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 77+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(78) : workflow.sql(6428) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_STAGE_ID', 84+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_STAGE_ID(85) : workflow.sql(6435) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_STAGE_ID', 87+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_STAGE_ID(88) : workflow.sql(6438) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
      Else
         If ((h_wbs_id IS NOT NULL)) AND (isCRIConvertTask(Coalesce(h_type_id, 0)) <> 0 
               OR isWBSTransferedTask(Coalesce(h_type_id, 0)) <> 0) Then
            h_item_type_id := PMO_CNST.SC_TABLE_WBS;
            h_wbs_transfered_task := 1;
         End If;
      End If;
   End If;
   
   If Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_WBS Then
      BEGIN
         SELECT PROJECT_ID, PLANNED_EFFORT + SCHED_EFFORT, TRANSFER_FLAG, A_CRI_COUNT, TYPE_ID
         INTO h_project_id, h_effort_driven, h_transfer_flag, h_a_cri_count, h_type_id
         FROM TMT_WBS
         WHERE ELEMENT_ID = h_id;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 111+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(112) : workflow.sql(6462) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
      
      h_close_stage_id := getCloseStage(Coalesce(h_type_id, 0));
      
      If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_STAGE_ID', 120+1) THEN 
            LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_STAGE_ID(121) : workflow.sql(6471) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
         END IF; 
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_STAGE_ID', 123+1) THEN 
            LFILE.FPUT('AllTrace', '                EXIT: SP_U_STAGE_ID(124) : workflow.sql(6474) : ' || SYSTIMESTAMP);
         END IF; 
         PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
      End If;
      
      --trace_printf("\n h_stage_id:%d", h_stage_id);
      --trace_printf("\n h_close_stage_id:%d", h_close_stage_id);
      
      If Coalesce(h_stage_id, 0) = Coalesce(h_close_stage_id, 0) Then
         If (isTask(Coalesce(h_type_id, 0))) <> 0 AND (Coalesce(h_effort_driven, 0) > 0) Then
            h_temp_var := FC_CLOSE_EFFORT(h_id, h_rec_user);
         Else
            h_temp_var := 0;
         End If;
         
         If Coalesce(h_temp_var, 0) = 0 Then
            --trace_printf("\n SP_U_STAGE_ID (1) ");
            BEGIN
               UPDATE TMT_WBS
               SET FD = PMO_CNST.SC_AF,
               FORECAST_START = Null,
               FORECAST_FINISH = Null,
               FORECAST_DURATION = 0,
               STAGE_ID = h_stage_id,
               REC_USER = h_rec_user
               WHERE ELEMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 159+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(160) : workflow.sql(6510) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_temp_var := FC_Rollup(h_project_id);
            End If;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_STAGE_ID', 169+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_STAGE_ID(170) : workflow.sql(6520) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_STAGE_ID', 172+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_STAGE_ID(173) : workflow.sql(6523) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
         End If;
      Else
         --trace_printf("\n SP_U_STAGE_ID (2) ");
         BEGIN
            UPDATE TMT_WBS
            SET STAGE_ID = h_stage_id,
            REC_USER = h_rec_user
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 193+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(194) : workflow.sql(6544) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
      
      If (h_transfer_flag IS NOT NULL) Then
         BEGIN
            UPDATE TMT_CRI
            SET STAGE_ID = h_stage_id,
            REC_USER = p_rec_user
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 215+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(216) : workflow.sql(6566) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      ElsIf Coalesce(h_a_cri_count, 0) > 0 Then
         BEGIN
            UPDATE TMT_CRI
            SET STAGE_ID = h_stage_id,
            REC_USER = h_rec_user
            WHERE WBS_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 235+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(236) : workflow.sql(6586) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_CRI Then
      h_close_stage_id := getCloseStage(Coalesce(h_type_id, 0));
      If Coalesce(h_stage_id, 0) = Coalesce(h_close_stage_id, 0) Then
         h_id := p_id;
         h_rec_user := p_rec_user;
         
         If (isCRITask(Coalesce(h_type_id, 0))) <> 0 AND (Coalesce(h_effort_driven, 0) > 0) Then
            h_temp_var := FC_CLOSE_EFFORT(h_id, h_rec_user);
         Else
            h_temp_var := 0;
         End If;
         
         If Coalesce(h_temp_var, 0) = 0 Then
            BEGIN
               SELECT FULL_NAME INTO h_name
               FROM RESOURCES WHERE RESOURCE_ID = h_rec_user;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 262+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(263) : workflow.sql(6613) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            BEGIN
               UPDATE TMT_CRI
               SET CLOSED_DATETIME = SYSTIMESTAMP,
               CLOSED_BY_NAME = h_name,
               STAGE_ID = h_stage_id,
               STAGE_LAST_DT = SYSTIMESTAMP,
               REC_USER = p_rec_user,
               REC_DATETIME = SYSTIMESTAMP
               WHERE ELEMENT_ID = h_id;
               IF SQL%FOUND = TRUE THEN
                  PMO_DB2M.SC_STATE := 0;
               ELSE
                  PMO_DB2M.SC_STATE := 100;
               END IF;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 286+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(287) : workflow.sql(6637) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If (h_wbs_id IS NOT NULL) Then
               BEGIN
                  UPDATE TMT_WBS
                  SET STAGE_ID = h_stage_id,
                  REC_USER = h_rec_user
                  WHERE ELEMENT_ID = h_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 307+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(308) : workflow.sql(6658) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
            
            If PMO_DB2M.SC_STATE = PMO_DB2M.SC_SQL_SUCCESS Then
               h_temp_var := FC_Rollup(h_project_id);
            End If;
         Else
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(32, 'SP_U_STAGE_ID', 318+1) THEN 
               LFILE.FPUT('AllTrace', '        RETURN_ERROR: SP_U_STAGE_ID(319) : workflow.sql(6669) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_CNST.RECORD_NOT_FOUND), 'NULL') || ' ID: ' || '''' || h_id || '''');
            END IF; 
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'SP_U_STAGE_ID', 321+1) THEN 
               LFILE.FPUT('AllTrace', '                EXIT: SP_U_STAGE_ID(322) : workflow.sql(6672) : ' || SYSTIMESTAMP);
            END IF; 
            PMODEFINES.PMO_RETURN_ERROR_SP(PMO_CNST.RECORD_NOT_FOUND, h_id, True);
         End If;
      Else
         If (h_wbs_id IS NOT NULL) Then
            BEGIN
               SELECT FD INTO h_fd
               FROM TMT_WBS
               WHERE ELEMENT_ID = h_id;
               PMO_DB2M.SC_STATE := SQLCODE;
            EXCEPTION
               WHEN OTHERS THEN
               BEGIN
                  PMO_DB2M.SC_STATE := SQLCODE;
                  IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 337+1) THEN 
                     LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(338) : workflow.sql(6688) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                  END IF; 
               END;
            END;
            
            If Coalesce(h_fd, 0) = PMO_CNST.SC_AF Then
               BEGIN
                  UPDATE TMT_WBS
                  SET FD = PMO_CNST.SC_UAF,
                  STAGE_ID = h_stage_id,
                  REC_USER = h_rec_user
                  WHERE ELEMENT_ID = h_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 359+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(360) : workflow.sql(6710) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            Else
               BEGIN
                  UPDATE TMT_WBS
                  SET STAGE_ID = h_stage_id,
                  REC_USER = h_rec_user
                  WHERE ELEMENT_ID = h_id;
                  IF SQL%FOUND = TRUE THEN
                     PMO_DB2M.SC_STATE := 0;
                  ELSE
                     PMO_DB2M.SC_STATE := 100;
                  END IF;
               EXCEPTION
                  WHEN OTHERS THEN
                  BEGIN
                     PMO_DB2M.SC_STATE := SQLCODE;
                     IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 379+1) THEN 
                        LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(380) : workflow.sql(6730) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
                     END IF; 
                  END;
               END;
            End If;
         End If;
         
         BEGIN
            UPDATE TMT_CRI
            SET STAGE_ID = h_stage_id,
            STAGE_LAST_DT = SYSTIMESTAMP,
            CLOSED_DATETIME = NuLL,
            CLOSED_BY_NAME = NuLL,
            REC_USER = h_rec_user
            WHERE ELEMENT_ID = h_id;
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 404+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(405) : workflow.sql(6755) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      End If;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_DOC Then
      BEGIN
         UPDATE TMT_DOCUMENTS
         SET STAGE_ID = h_stage_id,
         REC_USER = h_rec_user
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 425+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(426) : workflow.sql(6776) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_SCD Then
      BEGIN
         UPDATE TMT_ASSIGNED_SCORECARDS
         SET
         STAGE_ID = h_stage_id,
         REC_USER = h_rec_user
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 446+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(447) : workflow.sql(6797) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_RES_DOC Then
      BEGIN
         UPDATE TMT_RESOURCE_DOCUMENTS
         SET STAGE_ID = h_stage_id,
         REC_USER = h_rec_user
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 466+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(467) : workflow.sql(6817) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_POL_DOC Then
      BEGIN
         UPDATE TMT_POOL_DOCUMENTS
         SET STAGE_ID = h_stage_id,
         REC_USER = h_rec_user
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 486+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(487) : workflow.sql(6837) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_AST_DOC Then
      BEGIN
         UPDATE TMT_ASSET_DOCUMENTS
         SET STAGE_ID = h_stage_id,
         REC_USER = h_rec_user
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 506+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(507) : workflow.sql(6857) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_CLN_DOC Then
      BEGIN
         UPDATE TMT_CLIENT_DOCUMENTS
         SET STAGE_ID = h_stage_id,
         REC_USER = h_rec_user
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 526+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(527) : workflow.sql(6877) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_item_type_id, 0) = PMO_CNST.SC_TABLE_VDR_DOC Then
      BEGIN
         UPDATE TMT_VENDOR_DOCUMENTS
         SET STAGE_ID = h_stage_id,
         REC_USER = h_rec_user
         WHERE ELEMENT_ID = h_id;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 546+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(547) : workflow.sql(6897) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   If PMO_DB2M.SC_STATE = 100 Then
      PMO_DB2M.SC_STATE := 0;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(4, 'SP_U_STAGE_ID', 557+1) THEN 
      LFILE.FPUT('AllTrace', '       RETURN_STATUS: SP_U_STAGE_ID(558) : workflow.sql(6908) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || COALESCE(TO_CHAR(PMO_DB2M.SC_STATE), 'NULL') || ' ID: ' || '''' || p_id || '''');
   END IF; 
   PMODEFINES.PMO_RETURN_STATUS(PMO_DB2M.SC_STATE, p_id, True);
   
EXCEPTION
   WHEN PMO_DB2M.RETURN_ERROR_EXCEPTION THEN
      IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 564+1) THEN 
         LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(565) : workflow.sql(6915) : ' || SYSTIMESTAMP);
         LFILE.FPUT('AllTrace',                         'RETURN_ERROR_EXCEPTION : STATE: ' || PMO_DB2M.CURRENT_STATE || ' ID: ' || PMO_DB2M.CURRENT_ID);
      END IF; 
      OPEN resultSet
         FOR SELECT PMO_DB2M.CURRENT_STATE "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   WHEN OTHERS THEN
   BEGIN
      PMO_DB2M.SC_STATE := SQLCODE;
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'SP_U_STAGE_ID', 576+1) THEN 
      LFILE.FPUT('AllTrace', '           EXCEPTION: SP_U_STAGE_ID(577) : workflow.sql(6927) : ' || SYSTIMESTAMP);
      LFILE.FPUT('AllTrace',                         'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
   END IF; 
      OPEN resultSet
         FOR SELECT -9999 "ERROR" ,
         PMO_DB2M.CURRENT_ID "ID"
         FROM DUAL;
      RETURN resultSet;
   END;
END;
/
SHOW ERRORS FUNCTION SP_U_STAGE_ID;
ALTER FUNCTION SP_U_STAGE_ID COMPILE DEBUG;

/**********************************************************************************************************
    h_element_id:   workflow_id
    h_bitflag:      1: creating new workflow;
                    2: completing workflow
                    3: changing workflow
                    4: closing workflow
    h_rec_user:     who makes the change
**********************************************************************************************************/

Prompt == FUNCTION FC_INSERT_ALERT ==
CREATE OR REPLACE FUNCTION FC_INSERT_ALERT
(
     h_element_id IN OUT NOCOPY VARCHAR2
   , sh_h_bitflag PLS_INTEGER
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_bitflag PLS_INTEGER := sh_h_bitflag;
   h_sql VARCHAR2(4000);
   h_comment VARCHAR2(4000);
   h_full_name VARCHAR2(255);
   h_completed VARCHAR2(1);
   stmt VARCHAR2(4000);
   h_workflow_subject VARCHAR2(255);
   strTemp VARCHAR2(3);
-- Declare Cursors
      CURSOR C_ALERTS IS
         SELECT R.FULL_NAME, W.COMPLETED, W.DESCRIPTION AS COMMENT_STR
         FROM WORKFLOW_REPLIES W, RESOURCES R
         WHERE W.ELEMENT_ID = h_element_id AND REPLY_TYPE != PMO_CNST.SC_HIDEN_REPLY AND W.RESOURCE_ID = R.RESOURCE_ID;

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_INSERT_ALERT') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_INSERT_ALERT'), '-- ' || SYSTIMESTAMP, ':r := FC_INSERT_ALERT(' || 
       '''' || h_element_id || ''''
          || ', ' || COALESCE(TO_CHAR(sh_h_bitflag), 'NULL')
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INSERT_ALERT', 30+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_INSERT_ALERT(31) : workflow.sql(6981) : ' || SYSTIMESTAMP);
   END IF; 

   BEGIN
      SELECT R.FULL_NAME || '#!' || CASE WHEN T.SUBJECT IS NuLL THEN '' ELSE T.SUBJECT END || '#!' || CASE WHEN D.ELEMENT_NAME IS NuLL THEN '' ELSE D.ELEMENT_NAME END || '#!' || TO_CHAR(TO_DATE(SubStr(T.CREATED_DATE, 1, 10))) || '#!' || 
      CASE
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_WBS THEN (SELECT W.ELEMENT_NAME FROM TMT_WBS W WHERE W.ELEMENT_ID = P.ELEMENT_ID)
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_CRI THEN (SELECT C.ELEMENT_NAME FROM TMT_CRI C WHERE C.ELEMENT_ID = P.ELEMENT_ID)
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_DOC THEN (SELECT DOC.ELEMENT_NAME FROM TMT_DOCUMENTS DOC WHERE DOC.ELEMENT_ID = P.ELEMENT_ID)
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_SCD THEN (SELECT SCD.ELEMENT_NAME FROM TMT_ASSIGNED_SCORECARDS SCD WHERE SCD.ELEMENT_ID = P.ELEMENT_ID)
      ELSE '' END
       || '#!' || 
      CASE
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_WBS THEN (SELECT W1.ELEMENT_NAME FROM TMT_WBS W1 WHERE W1.ELEMENT_ID = (SELECT W2.PROJECT_ID FROM TMT_WBS W2 WHERE W2.ELEMENT_ID = P.ELEMENT_ID))
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_CRI THEN (SELECT W1.ELEMENT_NAME FROM TMT_WBS W1 WHERE W1.ELEMENT_ID = (SELECT C.PROJECT_ID FROM TMT_CRI C WHERE C.ELEMENT_ID = P.ELEMENT_ID))
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_DOC THEN (SELECT W1.ELEMENT_NAME FROM TMT_WBS W1 WHERE W1.ELEMENT_ID = (SELECT DOC.PROJECT_ID FROM TMT_DOCUMENTS DOC WHERE DOC.ELEMENT_ID = P.ELEMENT_ID))
      WHEN T.PARENT_TYPE = PMO_CNST.SC_TABLE_SCD THEN (SELECT W1.ELEMENT_NAME FROM TMT_WBS W1 WHERE W1.ELEMENT_ID = (SELECT SCD.PROJECT_ID FROM TMT_ASSIGNED_SCORECARDS SCD WHERE SCD.ELEMENT_ID = P.ELEMENT_ID))
      ELSE '' END
       || '#!', T.DESCRIPTION, T.SUBJECT
      INTO h_sql, h_comment, h_workflow_subject
      FROM TMT_WORKFLOWS T LEFT JOIN TMT_WORKFLOW_CYCLES D ON T.WKF_DEF_ID=D.ELEMENT_ID, RESOURCES R, TMT_WORKFLOWS P
      WHERE T.ELEMENT_ID = h_element_id
      AND T.AUTHOR_ID = R.RESOURCE_ID
      AND P.RANK = SUBSTR(T.RANK, 1, 8);
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_ALERT', 60+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_ALERT(61) : workflow.sql(7011) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If (h_comment IS NULL) Then
      h_sql := h_sql || ' #!';
   Else
      h_sql := h_sql || h_comment || '#!';
   End If;
   
   OPEN C_ALERTS;
   
   stmt := h_sql;
   LOOP
      FETCH C_ALERTS INTO h_full_name, h_completed, h_comment;
      IF C_ALERTS%FOUND THEN
         PMO_DB2M.SC_STATE := 0;
      ELSE
         PMO_DB2M.SC_STATE := 100;
      END IF;
      
      If PMO_DB2M.SC_STATE <> 0 Then
         EXIT;
      End If;
      
      If SubStr(h_completed, 1, 1) = '0' Then
         strTemp := '(-)';
      ElsIf SubStr(h_completed, 1, 1) = '1' Then
         strTemp := '(+)';
      Else
         strTemp := '(x)';
      End If;
      
      If (h_comment IS NULL) Then
         h_comment := '';
      End If;
      
      h_sql := RPad(h_full_name, 30) || RPad(strTemp, 5) || Chr(10) || Chr(9) || h_comment || Chr(10);
      stmt := stmt || h_sql;
   END LOOP;
   BEGIN
      CLOSE C_ALERTS ;
      PMO_DB2M.SC_STATE := SQLCODE;
   EXCEPTION
      WHEN OTHERS THEN
      BEGIN
         PMO_DB2M.SC_STATE := SQLCODE;
         IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_ALERT', 109+1) THEN 
            LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_ALERT(110) : workflow.sql(7060) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
         END IF; 
      END;
   END;
   
   If Coalesce(h_bitflag, 0) = 1 Then
      BEGIN
         INSERT INTO ALERTS (ELEMENT_ID, RESOURCE_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, FLAG, REC_USER, ALERT_SUBJECT)
         SELECT h_element_id, WR.RESOURCE_ID, 'SCNEWWORKFLOW', 'TMT_WORKFLOWS', stmt, 'I', h_rec_user, h_workflow_subject
         FROM WORKFLOW_REPLIES WR
         WHERE WR.ELEMENT_ID = h_element_id
         AND WR.RESOURCE_ID <> h_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_ALERT', 131+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_ALERT(132) : workflow.sql(7082) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 2 Then
      BEGIN
         INSERT INTO ALERTS (ELEMENT_ID, RESOURCE_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, FLAG, REC_USER)
         SELECT h_element_id, AUTHOR_ID, 'SCCOMPLETEDWORKFLOW', 'TMT_WORKFLOWS', stmt, 'I', h_rec_user
         FROM TMT_WORKFLOWS
         WHERE ELEMENT_ID = h_element_id AND AUTHOR_ID <> h_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_ALERT', 151+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_ALERT(152) : workflow.sql(7102) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_bitflag, 0) = 3 Then
      BEGIN
         INSERT INTO ALERTS (ELEMENT_ID, RESOURCE_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, FLAG, REC_USER)
         SELECT h_element_id, AUTHOR_ID, 'SCREVOTEDWORKFLOW', 'TMT_WORKFLOWS', stmt, 'I', h_rec_user
         FROM TMT_WORKFLOWS
         WHERE ELEMENT_ID = h_element_id AND AUTHOR_ID <> h_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_ALERT', 171+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_ALERT(172) : workflow.sql(7122) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   Else
      BEGIN
         INSERT INTO ALERTS (ELEMENT_ID, RESOURCE_ID, ALERT_ID, PARENT_TABLE, VALUE_LIST, FLAG, REC_USER)
         SELECT h_element_id, RESOURCE_ID, 'SCCLOSEDWORKFLOW', 'TMT_WORKFLOWS', stmt, 'I', h_rec_user
         FROM WORKFLOW_REPLIES
         WHERE ELEMENT_ID = h_element_id AND RESOURCE_ID <> h_rec_user;
         IF SQL%FOUND = TRUE THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_INSERT_ALERT', 191+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_INSERT_ALERT(192) : workflow.sql(7142) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   PMO_DB2M.SC_STATE := 0;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_INSERT_ALERT', 199+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_INSERT_ALERT(200) : workflow.sql(7150) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_INSERT_ALERT;
ALTER FUNCTION FC_INSERT_ALERT COMPILE DEBUG;

/**************************************************************************************************/

Prompt == FUNCTION FC_DELETE_ATTACHED_WKF ==
CREATE OR REPLACE FUNCTION FC_DELETE_ATTACHED_WKF
(
     sh_h_table_type PLS_INTEGER
   , h_element_id IN OUT NOCOPY VARCHAR2
   , h_rank IN OUT NOCOPY VARCHAR2
   , h_rec_user IN OUT NOCOPY VARCHAR2
) RETURN PLS_INTEGER
AS
   h_table_type PLS_INTEGER := sh_h_table_type;
   h_wkf_rank VARCHAR2(200);
-- Declare Cursors
      CURSOR wbs_wkf IS
         SELECT RANK FROM TMT_WORKFLOWS
         WHERE ELEMENT_ID = h_element_id
         AND TYPE_ID IN (PMO_CNST.SC_PROJECT,PMO_CNST.SC_PROPOSAL,PMO_CNST.SC_PROJ_TYPE1,PMO_CNST.SC_PROJ_TYPE2,PMO_CNST.SC_PROJ_TYPE3 , PMO_CNST.SC_DELIVERABLE, PMO_CNST.SC_TASK, PMO_CNST.SC_WBS_NOTIFICATION, PMO_CNST.SC_STAFFING_REQUESTS);

      CURSOR cri_wkf IS
         SELECT RANK FROM TMT_WORKFLOWS
         WHERE ELEMENT_ID = h_element_id;

      CURSOR doc_wkf IS
         SELECT RANK
         FROM TMT_WORKFLOWS
         WHERE ELEMENT_ID IN (
         SELECT ELEMENT_ID
         FROM TMT_DOCUMENTS
         WHERE RANK LIKE h_rank || '%'
         AND LAST_WF_DT IS NOT NuLL)
         AND TYPE_ID IN (PMO_CNST.SC_DOCUMENT, PMO_CNST.SC_DOC_NOTIFICATION);

BEGIN
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_PARAMS('FC_DELETE_ATTACHED_WKF') THEN 
      LFILE.FPUT_PARAMS('AllTrace', ALLTRACES.LINES_BEFORE_PARAMS('FC_DELETE_ATTACHED_WKF'), '-- ' || SYSTIMESTAMP, ':r := FC_DELETE_ATTACHED_WKF(' || 
       COALESCE(TO_CHAR(sh_h_table_type), 'NULL')
          || ', ' || '''' || h_element_id || ''''
          || ', ' || '''' || h_rank || ''''
          || ', ' || '''' || h_rec_user || '''' || ');');
   END IF; 

   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DELETE_ATTACHED_WKF', 40+1) THEN 
      LFILE.FPUT('AllTrace', '               ENTER: FC_DELETE_ATTACHED_WKF(41) : workflow.sql(7201) : ' || SYSTIMESTAMP);
   END IF; 

   If Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_WBS Then
      -- delete related workflows
      
      OPEN wbs_wkf;
      LOOP
         FETCH wbs_wkf INTO h_wkf_rank;
         IF wbs_wkf%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_wkf_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_wkf_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_wkf_rank, False); 
         	End If; 
         END;
         BEGIN
            DELETE FROM WORKFLOW_REPLIES
            WHERE ELEMENT_ID IN
            (SELECT ELEMENT_ID FROM TMT_WORKFLOWS WHERE RANK LIKE h_wkf_rank || '%');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 85+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(86) : workflow.sql(7246) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TMT_WORKFLOWS
            WHERE RANK LIKE h_wkf_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 103+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(104) : workflow.sql(7264) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WORKFLOWS
            SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE RANK = '0000';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 122+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(123) : workflow.sql(7283) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      BEGIN
         CLOSE wbs_wkf ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 135+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(136) : workflow.sql(7296) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_CRI Then
      OPEN cri_wkf;
      
      LOOP
         FETCH cri_wkf INTO h_wkf_rank;
         IF cri_wkf%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_wkf_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_wkf_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_wkf_rank, False); 
         	End If; 
         END;
         BEGIN
            DELETE FROM WORKFLOW_REPLIES
            WHERE ELEMENT_ID IN (
            SELECT ELEMENT_ID
            FROM TMT_WORKFLOWS
            WHERE RANK LIKE h_wkf_rank || '%');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 182+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(183) : workflow.sql(7343) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TMT_WORKFLOWS
            WHERE RANK LIKE h_wkf_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 200+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(201) : workflow.sql(7361) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            UPDATE TMT_WORKFLOWS
            SET CHILD_COUNT = CHILD_COUNT - 1
            WHERE RANK = '0000';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 219+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(220) : workflow.sql(7380) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      BEGIN
         CLOSE cri_wkf ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 232+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(233) : workflow.sql(7393) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   ElsIf Coalesce(h_table_type, 0) = PMO_CNST.SC_TABLE_DOC Then
      OPEN doc_wkf;
      
      LOOP
         FETCH doc_wkf INTO h_wkf_rank;
         IF doc_wkf%FOUND THEN
            PMO_DB2M.SC_STATE := 0;
         ELSE
            PMO_DB2M.SC_STATE := 100;
         END IF;
         
         If PMO_DB2M.SC_STATE <> PMO_DB2M.SC_SQL_SUCCESS Then
            EXIT;
         End If;
         
         DECLARE 
         	nLen pls_integer :=0; 
         BEGIN 
         	If (h_wkf_rank IS NOT NULL) Then 
         		nLen := Coalesce(Length(h_wkf_rank), 0); 
         		If 0 <> 0 AND nLen <> 0 Then 
         		nLen := nLen - 1; 
         		End If; 
         	End If; 
         	If NOT nLen <> 0 OR (nLen MOD 4) <> 0 Then 
         		return PMODEFINES.PMO_RETURN_ERROR(PMO_CNST.INVALID_RANK_LIKE, h_wkf_rank, False); 
         	End If; 
         END;
         BEGIN
            DELETE FROM WORKFLOW_REPLIES
            WHERE ELEMENT_ID IN (
            SELECT ELEMENT_ID
            FROM TMT_WORKFLOWS
            WHERE RANK LIKE h_wkf_rank || '%');
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 280+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(281) : workflow.sql(7441) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
         
         BEGIN
            DELETE FROM TMT_WORKFLOWS
            WHERE RANK LIKE h_wkf_rank || '%';
            IF SQL%FOUND = TRUE THEN
               PMO_DB2M.SC_STATE := 0;
            ELSE
               PMO_DB2M.SC_STATE := 100;
            END IF;
         EXCEPTION
            WHEN OTHERS THEN
            BEGIN
               PMO_DB2M.SC_STATE := SQLCODE;
               IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 298+1) THEN 
                  LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(299) : workflow.sql(7459) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
               END IF; 
            END;
         END;
      END LOOP;
      
      BEGIN
         CLOSE doc_wkf ;
         PMO_DB2M.SC_STATE := SQLCODE;
      EXCEPTION
         WHEN OTHERS THEN
         BEGIN
            PMO_DB2M.SC_STATE := SQLCODE;
            IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(16, 'FC_DELETE_ATTACHED_WKF', 312+1) THEN 
               LFILE.FPUT('AllTrace', '           EXCEPTION: FC_DELETE_ATTACHED_WKF(313) : workflow.sql(7473) : ' || SYSTIMESTAMP || ' ' || 'SQLCODE: ' || SQLCODE || ' SQLERR: ' || SubStr(SQLERRM, 1, 500));
            END IF; 
         END;
      END;
   End If;
   
   IF ALLTRACES.TRACE_ENABLED AND ALLTRACES.CHECK_TRACE(1, 'FC_DELETE_ATTACHED_WKF', 319+1) THEN 
      LFILE.FPUT('AllTrace', '                EXIT: FC_DELETE_ATTACHED_WKF(320) : workflow.sql(7480) : ' || SYSTIMESTAMP || ' ' || 'STATE: ' || PMO_DB2M.SC_STATE);
   END IF; 
   RETURN PMO_DB2M.SC_STATE;
END;
/
SHOW ERRORS FUNCTION FC_DELETE_ATTACHED_WKF;
ALTER FUNCTION FC_DELETE_ATTACHED_WKF COMPILE DEBUG;


